//==================================================================
//
//	File name	:	button.js
//
//	Summary		:	Button class
//
//	Ceated		:	2014/09/23
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (button) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	button.STYLE_DEFAULT = 0;
	button.STYLE_FLAT = 1;
	button.STYLE_TEXT = 2;
	button.STYLE_IMAGE = 4;
	button.STYLE_ICON = 8;
	
	// Grobal variables
	button.buttons = {};
	
	
	//==========================================================
	//
	//	Class		:	Button
	//
	//	Summary		:	Class of button
	//
	//==========================================================
	button.Button = function(sID,			// * ID[string]
							oRect,			// * Size and position[FixedRect]
							iStyle,			// * Item style[int]
							bEnabled,		// * Enabled or not[boolean]
							sContents,		// Label text or image URL or icon name[string]
							sTooltip,		// Tool tip text[string]
							iTabIndex,		// Tab index[int]
							fpClick)		// Function on click[function]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"button.js", "jaswin.button.Button.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"button.js", "jaswin.button.Button.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"button.js", "jaswin.button.Button.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"button.js", "jaswin.button.Button.constructor");
		
		if(sContents)
		{
			if(typeof sContents != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_CONTENTS, sID),
											"button.js", "jaswin.button.Button.constructor");
		}
		
		if(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
											"button.js", "jaswin.button.Button.constructor");
		}
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"button.js", "jaswin.button.Button.constructor");
		}
		
		if(fpClick)
		{
			if(typeof fpClick != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CLICK, sID),
											"button.js", "jaswin.button.Button.constructor");
		}
		
		// Set properties
		this.id = sID;					// ID[String]
		this.rect = oRect;				// Size and position[FixedRect]
		this.style = iStyle;			// Button style[int]
		this.enabled = bEnabled;		// Enabled or not[boolean]
		this.parentEnabled = true;		// Enabled or not[boolean]
		this.onClick = fpClick;			// Function on click[function(Argument:Button class)]
		this.element = null;			// This element[DOM Element]
		this.focusElement = null;		// Focus element[DOM Element]
		this.content = null;			// Content element for text or image[DOM Element]
		this.tabIndex = -1;				// Tab index[int]
		this.parent = null;				// Parent class[global / window / panel class]
		
		// Create element
		this.element = document.createElement("div");
		if(this.style & button.STYLE_FLAT)
			this.element.className = "button-flat";
		else
			this.element.className = "button";
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		
		this.element.style.width = this.rect.width - 2 + "px";		// 2:border
		this.element.style.height = this.rect.height - 2 + "px";	// 2:border
		this.focusElement = this.element;
		
		if(!this.enabled)
			this.element.style.cursor = "default";
		
		// Create contents
		if(this.style & button.STYLE_TEXT)
		{
			// Text label
			this.content = document.createElement("span");
			if(this.enabled)
				this.content.className = "button-text";
			else
				this.content.className = "button-text-disabled";
			
			this.content.style.lineHeight = this.rect.height - 2 - 2 + "px";	// 2:border 2:margin
			this.content.innerHTML = sContents;
			this.element.appendChild(this.content);
		}
		else if(this.style & button.STYLE_IMAGE)
		{
			// Image
			this.content = document.createElement("img");
			if(this.enabled)
				this.content.className = "button-image";
			else
				this.content.className = "button-image-disabled";
			this.content.src = sContents;
			this.element.appendChild(this.content);
		}
		else if(this.style & button.STYLE_ICON)
		{
			// Icon
			this.content = document.createElement("span");
			if(this.enabled)
				this.content.className = "button-icon";
			else
				this.content.className = "button-icon-disabled";
			this.content.style.width = this.rect.width - 2 - 2 + "px";		// 2:border 2:margin
			this.content.style.height = this.rect.height - 2 - 2 + "px";	// 2:border 2:margin
			
			var oIconSpan = document.createElement("span");
			oIconSpan.className = sContents;
			
			this.content.appendChild(oIconSpan);
			this.element.appendChild(this.content);
		}
		
		// Tooltip
		if((sTooltip) && (sTooltip.length > 0))
			this.element.title = sTooltip;
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Event binding
		if(this.enabled)
		{
			this.element.addEventListener("mousemove", mouseMoveEvent, false);
			this.element.addEventListener("mouseout", mouseOutEvent, false);
			this.element.addEventListener("mousedown", mouseDownEvent, false);
			this.element.addEventListener("mouseup", mouseUpEvent, false);
			this.element.addEventListener("click", clickEvent, false);
			this.element.addEventListener("keyup", keyUpEvent, false);
			this.focusElement.addEventListener("focus", focusEvent, false);
			this.focusElement.addEventListener("blur", blurEvent, false);
		}
	}
	button.Button.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			delete button.buttons[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (button.buttons[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, this.id),
											"button.js", "jaswin.button.Button.setParent");
			this.element.id = sID;
			button.buttons[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this button
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.element.tabIndex = this.tabIndex;
				
				if(this.content)
				{
					if(this.style & button.STYLE_TEXT)
						this.content.className = "button-text";
					else if(this.style & button.STYLE_IMAGE)
						this.content.className = "button-image";
					else if(this.style & button.STYLE_ICON)
						this.content.className = "button-icon";
				}
				
				this.element.style.cursor = "pointer";
				
				this.element.addEventListener("mousemove", mouseMoveEvent, false);
				this.element.addEventListener("mouseout", mouseOutEvent, false);
				this.element.addEventListener("mousedown", mouseDownEvent, false);
				this.element.addEventListener("mouseup", mouseUpEvent, false);
				this.element.addEventListener("click", clickEvent, false);
				this.element.addEventListener("keyup", keyUpEvent, false);
				this.focusElement.addEventListener("focus", focusEvent, false);
				this.focusElement.addEventListener("blur", blurEvent, false);
			}
		},
		
		//==================================================
		//	Disable this button
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.element.tabIndex = -1;
			
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat";
			else
				this.element.className = "button";
			
			if(this.content)
			{
				if(this.style & button.STYLE_TEXT)
					this.content.className = "button-text-disabled";
				else if(this.style & button.STYLE_IMAGE)
					this.content.className = "button-image-disabled";
				else if(this.style & button.STYLE_ICON)
					this.content.className = "button-icon-disabled";
			}
			
			this.element.style.cursor = "default";
			
			this.element.removeEventListener("mousemove", mouseMoveEvent, false);
			this.element.removeEventListener("mouseout", mouseOutEvent, false);
			this.element.removeEventListener("mousedown", mouseDownEvent, false);
			this.element.removeEventListener("mouseup", mouseUpEvent, false);
			this.element.removeEventListener("click", clickEvent, false);
			this.element.removeEventListener("keyup", keyUpEvent, false);
			this.focusElement.removeEventListener("focus", focusEvent, false);
			this.focusElement.removeEventListener("blur", blurEvent, false);
		},
		
		//==================================================
		//	Show this button
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this button
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Set label text
		//==================================================
		setLabel: function(sText)
		{
			if(!this.content)
				return;
			
			if(!this.style & button.STYLE_TEXT)
				return;
			
			if(typeof sText != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"button.js", "jaswin.button.Button.setLabel");
			
			this.content.innerHTML = sText;
		},
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip: function(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"button.js", "jaswin.button.Button.setTooltip");
			this.element.title = sTooltip;
		},
		
		//==================================================
		//	Set this button hover
		//==================================================
		setHover: function()
		{
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat-focus";
			else
				this.element.className = "button-focus";
		},
		
		//==================================================
		//	Set this button mouse out
		//==================================================
		setOut: function()
		{
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat";
			else
				this.element.className = "button";
			
			if(this.content)
			{
				if(this.style & button.STYLE_TEXT)
					this.content.className = "button-text";
				else if(this.style & button.STYLE_IMAGE)
					this.content.className = "button-image";
				else if(this.style & button.STYLE_ICON)
					this.content.className = "button-icon";
			}
		},
		
		//==================================================
		//	Set this button mouse down
		//==================================================
		setDown: function()
		{
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat-click";
			else
				this.element.className = "button-click";
			
			if(this.content)
			{
				if(this.style & button.STYLE_TEXT)
					this.content.className = "button-text-click";
				else if(this.style & button.STYLE_IMAGE)
					this.content.className = "button-image-click";
				else if(this.style & button.STYLE_ICON)
					this.content.className = "button-icon-click";
			}
		},
		
		//==================================================
		//	Set this button mouse up
		//==================================================
		setUp: function()
		{
			if(this.style & button.STYLE_FLAT)
				this.element.className = "button-flat-focus";
			else
				this.element.className = "button-focus";
			
			if(this.content)
			{
				if(this.style & button.STYLE_TEXT)
					this.content.className = "button-text";
				else if(this.style & button.STYLE_IMAGE)
					this.content.className = "button-image";
				else if(this.style & button.STYLE_ICON)
					this.content.className = "button-icon";
			}
		},
		
		//==================================================
		//	Click this button
		//==================================================
		click: function()
		{
			if((this.enabled) && (this.parentEnabled) && (this.onClick))
				this.onClick(this);
		},
		
		//==================================================
		//	Set click event
		//==================================================
		setClickEvent: function(fpClick)
		{
			if(typeof fpClick != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CLICK, this.id),
											"button.js", "jaswin.button.Button.setClickEvent");
			this.onClick = fpClick;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetButton
	//
	//	Summary		:	Get target button element from event
	//
	//==========================================================
	function getTargetButton(e)
	{
		var oElement = e.target || e.srcElement;
		
		while(true)
		{
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
				break;
			
			oElement = oElement.parentNode;
		}
		
		if(!oElement.id)
			return null;
		
		return button.buttons[oElement.id];
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseMoveEvent
	//
	//	Summary		:	Event handler of mouse move
	//
	//==========================================================
	function mouseMoveEvent(e)
	{
		var oButton = null;
		
		oButton = getTargetButton(e);
		
		if(oButton)
			oButton.setHover();
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseOutEvent
	//
	//	Summary		:	Event handler of mouse out
	//
	//==========================================================
	function mouseOutEvent(e)
	{
		var oButton = null;
		
		oButton = getTargetButton(e);
		
		if(oButton)
			oButton.setOut();
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseDownEvent
	//
	//	Summary		:	Event handler of mouse down
	//
	//==========================================================
	function mouseDownEvent(e)
	{
		var oButton = null;
		
		oButton = getTargetButton(e);
		if(oButton)
			oButton.setDown();
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseUpEvent
	//
	//	Summary		:	Event handler of mouse up
	//
	//==========================================================
	function mouseUpEvent(e)
	{
		var oButton = null;
		
		oButton = getTargetButton(e);
		if(oButton)
			oButton.setUp();
	}
	
	
	//==========================================================
	//
	//	Function	:	clickEvent
	//
	//	Summary		:	Event handler of click
	//
	//==========================================================
	function clickEvent(e)
	{
		var oButton = null;
		
		oButton = getTargetButton(e);
		if(oButton)
			oButton.click();
	}
	
	
	//==========================================================
	//
	//	Function	:	keyUpEvent
	//
	//	Summary		:	Event handler of key up
	//
	//==========================================================
	function keyUpEvent(e)
	{
		var oButton = null;
		
		if((e.keyCode != 13) && (e.keyCode != 32))
			return true;
		
		oButton = getTargetButton(e);
		if(!oButton)
			return true;
		
		oButton.click();
	}
	
	
	//==========================================================
	//
	//	Function	:	focusEvent
	//
	//	Summary		:	Event handler of focus to a button
	//
	//==========================================================
	function focusEvent(e)
	{
		var oButton = null;
		
		oButton = getTargetButton(e);
		if(!oButton)
			return;
		
		if((!oButton.enabled) || (!oButton.parentEnabled))
		{
			e.preventDefault();
			return;
		}
		
		oButton.setHover();
	}
	
	
	//==========================================================
	//
	//	Function	:	blurEvent
	//
	//	Summary		:	Event handler of blur from a button
	//
	//==========================================================
	function blurEvent(e)
	{
		var oButton = null;
		
		oButton = getTargetButton(e);
		if(!oButton)
			return;
		
		if((!oButton.enabled) || (!oButton.parentEnabled))
		{
			e.preventDefault();
			return;
		}
		
		oButton.setOut();
	}
	
})(jaswin.button = jaswin.button || {});
