//==================================================================
//
//	File name	:	calendar.js
//
//	Summary		:	Calendar class
//
//	Ceated		:	2018/02/05
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (calendar) {
	//==========================================================
	//	Global
	//==========================================================
	// Grobal variables
	calendar.Calendars = {};
	
	// Private variables
	var oPopUp = null;
	var iaLastDay = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	
	
	//==========================================================
	//
	//	Class		:	Calendar
	//
	//	Summary		:	Class of calendar
	//
	//==========================================================
	calendar.Calendar = function(sID,			// * ID[String]
								oRect,			// * Size and position[FixedRect]
								oTarget,		// Target textbox[TextBox]
								bEnabled,		// * Enabled or not[boolean]
								sFormat,		// Format of output [String]
								oDate,			// Initial date[Date]
								sTooltip,		// Tool tip text[String]
								iTabIndex,		// Tab index[int]
								fpChange)		// Function on change[function]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check included files
		if(!jaswin.button)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_INCLUDE, "button.js"),
										"calendar.js", "jaswin.calendar.Calendar.constructor");
		
		if(!jaswin.calendar.DEFAULT_FORMAT)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_INCLUDE, "calendar_const.js/calendar_const_ja.js"),
										"calendar.js", "jaswin.calendar.Calendar.constructor");
		
		if(oTarget)
		{
			if(!jaswin.textbox)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "textbox.js"),
											"calendar.js", "jaswin.calendar.Calendar.constructor");
		}
		
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"calendar.js", "jaswin.calendar.Calendar.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"calendar.js", "jaswin.calendar.Calendar.constructor");
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"calendar.js", "jaswin.calendar.Calendar.constructor");
		
		if(oTarget)
		{
			if((typeof oTarget != "object")
			|| (!oTarget instanceof jaswin.textbox.TextBox))
			{
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_TEXTBOX, sID),
												"calendar.js", "jaswin.calendar.Calendar.constructor");
			}
		}
		
		if(sFormat)
		{
			if(typeof sFormat != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FORMAT, sID),
											"calendar.js", "jaswin.calendar.Calendar.constructor");
		}
		
		if(oDate)
		{
			if((typeof oDate != "object")
			|| (!oDate instanceof Date))
			{
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_DATE, sID),
											"calendar.js", "jaswin.calendar.Calendar.constructor");
			}
		}
		
		if(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
											"calendar.js", "jaswin.calendar.Calendar.constructor");
		}
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"calendar.js", "jaswin.calendar.Calendar.constructor");
		}
		
		if(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
											"calendar.js", "jaswin.button.Calendar.constructor");
		}
		if((!oTarget) && (!fpChange))
		{
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_TEXTBOX, sID),
										"calendar.js", "jaswin.calendar.Calendar.constructor");
		}
		
		// Set properties
		this.id = sID;						// ID[String]
		this.rect = oRect;					// Size and position[FixedRect]
		this.enabled = bEnabled;			// Enabled or not[boolean]
		this.parentEnabled = true;			// Enabled or not[boolean]
		this.element = null;				// This element[DOM Element]
		this.sheet = null;					// Calendar sheet element[DOM Element]
		this.prevMonth = null;				// Button for previous month[Button]
		this.nextMonth = null;				// Button for next month[Button]
		this.monthLabel = null;				// Label for current month[Label]
		this.container = null;				// Container for client area[DOM Element]
		this.focusElement = null;			// Focus element[DOM Element]
		this.parent = null;					// Parent class[global / window / panel class]
		this.target = oTarget;				// Target textbox[TextBox]
		this.onChange = fpChange;			// Function on change[function(Argument:Calendar class)]
		this.currentYear = null;
		this.currentMonth = null;
		
		if(sFormat)
			this.format = sFormat;
		else
			this.format = calendar.DEFAULT_FORMAT;
		
		if(oDate)
			this.date = oDate;
		else
			this.date = new Date();
		this.tempDate = new Date(this.date.getTime());
		
		// Create element
		this.element = document.createElement("div");
		
		if(this.enabled)
			this.element.className = "ico-calendar";
		else
			this.element.className = "ico-calendar-disabled";
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		
		this.focusElement = this.element;
		
		// Tooltip
		if((sTooltip) && (sTooltip.length > 0))
			this.element.title = sTooltip;
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Calendar sheet
		this.sheet = document.createElement("div");
		this.sheet.className = "calendar";
		this.sheet.style.display = "none";
		this.element.appendChild(this.sheet);
		
		var oRect = new jaswin.FixedRect(jaswin.ANCHOR_LEFT, jaswin.ANCHOR_TOP,
										2, 2, 16, 16);
		this.prevMonth = new jaswin.button.Button("Btn" + this.id + "_PrevMonth", oRect,
													jaswin.button.STYLE_FLAT | jaswin.button.STYLE_ICON,
													true,
													jaswin.ICON_TRIANGLE_LEFT, null,
													0, clickPrevMonth);
		this.sheet.appendChild(this.prevMonth.element);
		
		oRect = new jaswin.FixedRect(jaswin.ANCHOR_RIGHT, jaswin.ANCHOR_TOP,
										2, 2, 16, 16);
		this.nextMonth = new jaswin.button.Button("Btn" + this.id + "_NextMonth", oRect,
													jaswin.button.STYLE_FLAT | jaswin.button.STYLE_ICON,
													true,
													jaswin.ICON_TRIANGLE_RIGHT, null,
													0, clickNextMonth);
		this.sheet.appendChild(this.nextMonth.element);
		
		this.monthLabel = document.createElement("span");
		this.monthLabel.className = "calendar-month";
		this.sheet.appendChild(this.monthLabel);
		
		var oInner;
		
		oInner = document.createElement("div");
		oInner.className = "calendar-inner";
		this.sheet.appendChild(oInner);
		
		this.container = document.createElement("div");
		this.container.className = "calendar-container";
		oInner.appendChild(this.container);
		
		var iCounter;
		var oDiv;
		
		oDiv = document.createElement("div");
		oDiv.className = "calendar-sun";
		oDiv.innerHTML = calendar.WEEK[0];
		oInner.appendChild(oDiv);
		for(iCounter = 1; iCounter < 6; iCounter++)
		{
			oDiv = document.createElement("div");
			oDiv.className = "calendar-week";
			oDiv.innerHTML = calendar.WEEK[iCounter];
			oInner.appendChild(oDiv);
		}
		oDiv = document.createElement("div");
		oDiv.className = "calendar-sat";
		oDiv.innerHTML = calendar.WEEK[6];
		oInner.appendChild(oDiv);
		
		oDiv = document.createElement("div");
		oDiv.style.clear = "both";
		oInner.appendChild(oDiv);
		
		this.setMonth(this.date.getFullYear(), this.date.getMonth());
		
		// Event binding
		if(this.enabled)
		{
			this.element.addEventListener("click", clickEvent, false);
			this.element.addEventListener("keydown", keyDownEvent, false);
		}
	}
	calendar.Calendar.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			this.prevMonth.destroy();
			this.nextMonth.destroy();
			
			delete calendar.Calendars[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
			{
				sID = oParentWnd.id + "_" + this.id;
				this.sheet.style.zIndex = 50 + jaswin.frame.windowCount + oParentWnd.modalLevel;
			}
			
			this.prevMonth.setParent(this);
			this.nextMonth.setParent(this);
			
			// Duplication check
			if((document.getElementById(sID)) || (calendar.Calendars[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"calendar.js", "jaswin.calendar.Calendar.setParent");
			this.element.id = sID;
			calendar.Calendars[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this calendar
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.element.tabIndex = this.tabIndex;
				this.element.className = "ico-calendar";
				this.element.addEventListener("click", clickEvent, false);
				this.element.addEventListener("keydown", keyDownEvent, false);
			}
		},
		
		//==================================================
		//	Disable this calendar
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			this.sheet.style.display = "none";
			oPopUp = null;
			
			this.element.tabIndex = -1;
			this.element.className = "ico-calendar-disabled";
			this.element.removeEventListener("click", clickEvent, false);
			this.element.removeEventListener("keydown", keyDownEvent, false);
		},
		
		//==================================================
		//	Show this calendar
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this calendar
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip: function(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setTooltip");
			this.element.title = sTooltip;
		},
		
		//==================================================
		//	Click this calendar
		//==================================================
		clickCalendar: function()
		{
			if(this.sheet.style.display == "none")
			{
				var oRect = this.element.getBoundingClientRect();
				var iLeft;
				var iTop;
				
				if(oRect.left + 16 - 160 - 2 < 0)
					iLeft = oRect.left * (-1);
				else
					iLeft = 16 - 160 - 2;
				this.sheet.style.left = iLeft + "px";
				
				if(oRect.top + 16 + 164 + 2 > window.innerHeight)
					iTop = -164 - 8;
				else
					iTop = 16;
				this.sheet.style.top = iTop + "px";
				
				this.tempDate = new Date(this.date.getTime());
				this.setMonth(this.date.getFullYear(), this.date.getMonth());
				
				this.element.className = "ico-calendar-on";
				this.sheet.style.display = "block";
				oPopUp = this;
				
				global.element.addEventListener("click", calendarCancel, false);
			}
			else
			{
				this.element.className = "ico-calendar";
				this.sheet.style.display = "none";
				oPopUp = null;
				global.element.removeEventListener("click", calendarCancel, false);
			}
		},
		
		//==================================================
		//	Set Month
		//==================================================
		setMonth: function(iYear, iMonth)
		{
			if(typeof iYear != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_YEAR, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setMonth");
			if(typeof iMonth != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MONTH, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setMonth");
			if(iMonth < 0 || iMonth > 11)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_MONTH, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setMonth");
			
			var sMonthStr;
			var iLastDay;
			var oDate;
			var iCounter;
			var oDiv;
			
			this.currentYear = iYear;
			this.currentMonth = iMonth;
			
			sMonthStr = calendar.MONTH_FORMAT.replace("YYYY", "" + iYear);
			sMonthStr = sMonthStr.replace("MONTH", calendar.LONG_MONTH[iMonth]);
			this.monthLabel.innerHTML = sMonthStr;
			
			if(iMonth == 1)
			{
				if(iYear % 400 == 0 || (iYear % 100 != 0 && iYear % 4 == 0))
					iLastDay = 29;
				else
					iLastDay = 28;
			}
			else
				iLastDay = iaLastDay[iMonth];
			
			this.container.innerHTML = "";
			oDate = new Date(iYear, iMonth, 1);
			for(iCounter = 0; iCounter < oDate.getDay(); iCounter++)
			{
				oDiv = document.createElement("div");
				oDiv.className = "calendar-blank";
				this.container.appendChild(oDiv);
			}
			for(iCounter = 1; iCounter <= iLastDay; iCounter++)
			{
				oDiv = document.createElement("div");
				oDiv.innerHTML = "" + iCounter;
				
				if((this.date.getFullYear() == this.currentYear)
				&& (this.date.getMonth() == this.currentMonth)
				&& (this.date.getDate() == iCounter))
					oDiv.className = "calendar-selected";
				else if((this.tempDate.getFullYear() == this.currentYear)
				&& (this.tempDate.getMonth() == this.currentMonth)
				&& (this.tempDate.getDate() == iCounter))
					oDiv.className = "calendar-focus";
				else
					oDiv.className = "calendar-day";
				this.container.appendChild(oDiv);
				oDiv.addEventListener("click", clickDay, false);
			}
			
			oDiv = document.createElement("div");
			oDiv.style.clear = "both";
			this.container.appendChild(oDiv);
		},
		
		//==================================================
		//	Set Day
		//==================================================
		selectDay: function(iDay)
		{
			if(typeof iDay != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_DATE, this.id),
											"calendar.js", "jaswin.calendar.Calendar.selectDay");
			if(!this.container.childNodes.length)
				return;
			if(iDay < 1)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_DATE, this.id),
											"calendar.js", "jaswin.calendar.Calendar.selectDay");
			var iCounter;
			var bFound = false;
			
			for(iCounter = 0; iCounter < this.container.childNodes.length - 1; iCounter++)
			{
				if(this.container.childNodes[iCounter].innerHTML == "" + iDay)
				{
					this.date = new Date(this.currentYear, this.currentMonth, iDay);
					this.container.childNodes[iCounter].className = "calendar-selected";
					this.tempDate = new Date(this.date.getTime());
					bFound = true;
				}
				else
				{
					if(this.container.childNodes[iCounter].className != "calendar-blank")
						this.container.childNodes[iCounter].className = "calendar-day";
				}
			}
			
			this.element.className = "ico-calendar";
			this.sheet.style.display = "none";
			oPopUp = null;
			global.element.removeEventListener("click", calendarCancel, false);
			
			if(!bFound)
				return;
			
			var sOutput = this.format.replace("YYYY", "" + this.currentYear);
			var sTmp;
			
			if(this.currentMonth + 1 < 10)
				sTmp = "0" + (this.currentMonth + 1);
			else
				sTmp = "" + (this.currentMonth + 1);
			
			sOutput = sOutput.replace("MM", sTmp);
			
			if(iDay < 10)
				sTmp = "0" + iDay;
			else
				sTmp = "" + iDay;
			sOutput = sOutput.replace("DD", sTmp);
			
			if(this.target)
				this.target.setValue(sOutput);
			if(this.onChange)
				this.onChange(this);
		},
		
		//==================================================
		//	Get selected value
		//==================================================
		getValue: function()
		{
			var sOutput = this.format.replace("YYYY", "" + this.currentYear);
			var sTmp;
			var iDay = this.date.getDate();
			
			if(this.currentMonth + 1 < 10)
				sTmp = "0" + (this.currentMonth + 1);
			else
				sTmp = "" + (this.currentMonth + 1);
			
			sOutput = sOutput.replace("MM", sTmp);
			
			if(iDay < 10)
				sTmp = "0" + iDay;
			else
				sTmp = "" + iDay;
			sOutput = sOutput.replace("DD", sTmp);
			
			return sOutput;
		},
		
		//==================================================
		//	Key down on this calendar
		//==================================================
		keyDown: function(e)
		{
			var oElement = e.target || e.srcElement;
			
			if((e.keyCode == 13) || (e.keyCode == 32))
			{
				// Enter or Space
				if(this.sheet.style.display == "none")
					this.clickCalendar();
				else
					this.selectDay(this.tempDate.getDate());
				
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			else if(e.keyCode == 27)
			{
				// Esc
				if(this.sheet.style.display == "block")
					this.clickCalendar();
			}
			else
			{
				if((e.keyCode < 37) || (e.keyCode > 40))
					return true;
				
				if((this.tempDate.getFullYear() != this.currentYear)
				|| (this.tempDate.getMonth() != this.currentMonth))
					return true;
				
				if(e.keyCode == 37)
				{
					// Left
					this.tempDate.setDate(this.tempDate.getDate() - 1);
				}
				else if(e.keyCode == 39)
				{
					// Right
					this.tempDate.setDate(this.tempDate.getDate() + 1);
				}
				else if(e.keyCode == 38)
				{
					// Up
					this.tempDate.setDate(this.tempDate.getDate() - 7);
				}
				else if(e.keyCode == 40)
				{
					// Down
					this.tempDate.setDate(this.tempDate.getDate() + 7);
				}
				this.setMonth(this.tempDate.getFullYear(), this.tempDate.getMonth());
				
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			return true;
		},
		
		//==================================================
		//	Set change event
		//==================================================
		setChangeEvent: function(fpChange)
		{
			if(typeof fpClick != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, this.id),
											"calendar.js", "jaswin.calendar.Calendar.setChangeEvent");
			this.onClick = fpClick;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetCalendar
	//
	//	Summary		:	Get target Calendar element from event
	//
	//==========================================================
	function getTargetCalendar(e)
	{
		var oElement = e.target || e.srcElement;
		var oCalendar = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oCalendar = calendar.Calendars[oElement.id];
				if(!oCalendar)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oCalendar;
	}
	
	
	//==========================================================
	//
	//	Function	:	clickEvent
	//
	//	Summary		:	Event handler of click a calendar
	//
	//==========================================================
	function clickEvent(e)
	{
		var oCalendar = null;
		var oElement = e.target || e.srcElement;
		
		oCalendar = getTargetCalendar(e);
		if(oCalendar)
		{
			if(!oCalendar.sheet.contains(oElement))
				oCalendar.clickCalendar();
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	keyDownEvent
	//
	//	Summary		:	Event handler of key down
	//
	//==========================================================
	function keyDownEvent(e)
	{
		var oCalendar = getTargetCalendar(e);
		
		if(oCalendar)
			return oCalendar.keyDown(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	calendarCancel
	//
	//	Summary		:	Event handler of cancel a calendar
	//
	//==========================================================
	function calendarCancel(e)
	{
		if(!oPopUp)
			return;
		
		var oElement = e.target || e.srcElement;
		
		if(!oPopUp.element.contains(oElement))
		{
			oPopUp.clickCalendar();
			return;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	clickPrevMonth
	//
	//	Summary		:	Event handler of click a previous month
	//
	//==========================================================
	function clickPrevMonth(oButton)
	{
		var oCalendar = null;
		var iYear;
		var iMonth;
		
		oCalendar = oButton.parent;
		if(oCalendar.currentMonth == 0)
		{
			iYear = oCalendar.currentYear - 1;
			iMonth = 11;
		}
		else
		{
			iYear = oCalendar.currentYear;
			iMonth = oCalendar.currentMonth - 1;
		}
		oCalendar.setMonth(iYear, iMonth);
	}
	
	//==========================================================
	//
	//	Function	:	clickNextMonth
	//
	//	Summary		:	Event handler of click a next month
	//
	//==========================================================
	function clickNextMonth(oButton)
	{
		oCalendar = oButton.parent;
		if(oCalendar.currentMonth == 11)
		{
			iYear = oCalendar.currentYear + 1;
			iMonth = 0;
		}
		else
		{
			iYear = oCalendar.currentYear;
			iMonth = oCalendar.currentMonth + 1;
		}
		oCalendar.setMonth(iYear, iMonth);
	}
	
	
	//==========================================================
	//
	//	Function	:	clickDay
	//
	//	Summary		:	Event handler of click a calendar day
	//
	//==========================================================
	function clickDay(e)
	{
		var oElement = e.target || e.srcElement;
		
		oCalendar = getTargetCalendar(e);
		
		if(oCalendar)
			oCalendar.selectDay(parseInt(oElement.innerHTML));
	}
	
})(jaswin.calendar = jaswin.calendar || {});
