//==================================================================
//
//	File name	:	checkbox.js
//
//	Summary		:	CheckBox, radio, toggle classes
//
//	Ceated		:	2014/09/26
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (checkbox) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	checkbox.STYLE_DEFAULT = 0;
	checkbox.STYLE_BUTTON = 1;
	checkbox.STYLE_FLAT_BUTTON = 2;
	checkbox.STYLE_IMAGE = 4;
	checkbox.STYLE_ICON = 8;
	
	// Grobal variables
	checkbox.checkBoxes = {};
	
	
	//==========================================================
	//
	//	Class		:	CheckBox
	//
	//	Summary		:	Class of checkbox
	//
	//==========================================================
	checkbox.CheckBox = function(sID,			// * ID[String]
								oRect,			// * Size and position[FixedRect]
								iStyle,			// * Item style[int]
								bEnabled,		// * Enabled or not[boolean]
								bChecked,		// * Checked or not[boolean]
								sGroup,			// Group for radio button[String]
								sContents,		// Label text or image URL or icon name[string]
								sTooltip,		// Tool tip text[String]
								iTabIndex,		// Tab index[int]
								fpChange)		// Function on change
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		
		if(typeof bChecked != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_CHECKED, sID),
										"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		
		if(sGroup)
		{
			if(typeof sGroup != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_GROUP, sID),
											"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		}
		
		if(sContents)
		{
			if(typeof sContents != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_CONTENTS, sID),
											"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		}
		
		if(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
											"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		}
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		}
		
		if(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
											"checkbox.js", "jaswin.checkbox.CheckBox.constructor");
		}
		
		// Set properties
		this.id = sID;					// ID[String]
		this.rect = oRect;				// Size and position[FixedRect]
		this.style = iStyle;			// Button style[int]
		this.group = null;				// Group for radio button[string]
		this.enabled = bEnabled;		// Enabled or not[boolean]
		this.parentEnabled = true;		// Enabled or not[boolean]
		this.checked = bChecked;		// Checked or not[boolean]
		this.onChange = fpChange;		// Function on change[function(Argument:CheckBox class)]
		this.element = null;			// This element[DOM Element]
		this.focusElement = null;		// Focus element[DOM Element]
		this.box = null;				// Box for checkbox or radio button[DOM Element]
		this.content = null;			// Content element for text or image[DOM Element]
		this.tabIndex = -1;				// Tab index[int]
		this.parent = null;				// Parent class[global / window / panel class]
		
		if((sGroup) && (sGroup != ""))
			this.group = sGroup;
		
		// Create element
		this.element = document.createElement("div");
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		this.focusElement = this.element;
		
		if((this.style & checkbox.STYLE_BUTTON)
		|| (this.style & checkbox.STYLE_FLAT_BUTTON))
		{
			// Toggle button
			if(this.enabled)
			{
				if(this.checked)
				{
					if(this.style & checkbox.STYLE_FLAT_BUTTON)
						this.element.className = "button-flat-click";
					else
						this.element.className = "button-click";
				}
				else
				{
					if(this.style & checkbox.STYLE_FLAT_BUTTON)
						this.element.className = "button-flat";
					else
						this.element.className = "button";
				}
			}
			else
			{
				if(this.checked)
				{
					if(this.style & checkbox.STYLE_FLAT_BUTTON)
						this.element.className = "button-flat-click-disabled";
					else
						this.element.className = "button-click-disabled";
					this.element.style.cursor = "default";
				}
				else
				{
					if(this.style & checkbox.STYLE_FLAT_BUTTON)
						this.element.className = "button-flat";
					else
						this.element.className = "button";
					this.element.style.cursor = "default";
				}
			}
			
			this.element.style.width = this.rect.width - 2 + "px";		// 2:border
			this.element.style.height = this.rect.height - 2 + "px";	// 2:border
		}
		else
		{
			// CheckBox or Radio button
			this.element.className = "checkbox";
			this.element.style.width = this.rect.width + "px";
			this.element.style.height = this.rect.height + "px";
			
			if(!this.enabled)
				this.element.style.cursor = "default";
		}
		
		// Create contents
		if((this.style & checkbox.STYLE_BUTTON)
		|| (this.style & checkbox.STYLE_FLAT_BUTTON))
		{
			// Toggle button
			if(this.style & checkbox.STYLE_IMAGE)
			{
				// Image button
				this.content = document.createElement("img");
				if(this.enabled)
				{
					if(this.checked)
						this.content.className = "button-image-click";
					else
						this.content.className = "button-image";
				}
				else
				{
					if(this.checked)
						this.content.className = "button-image-click-disabled";
					else
						this.content.className = "button-image-disabled";
				}
				this.content.src = sContents;
			}
			else if(this.style & checkbox.STYLE_ICON)
			{
				// Icon button
				this.content = document.createElement("span");
				if(this.enabled)
				{
					if(this.checked)
						this.content.className = "button-icon-click";
					else
						this.content.className = "button-icon";
				}
				else
				{
					if(this.checked)
						this.content.className = "button-icon-click-disabled";
					else
						this.content.className = "button-icon-disabled";
				}
				
				this.content.style.width = this.rect.width - 2 - 2 + "px";		// 2:border 2:margin
				this.content.style.height = this.rect.height - 2 - 2 + "px";	// 2:border 2:margin
				
				var oIconSpan = document.createElement("span");
				oIconSpan.className = sContents;
				this.content.appendChild(oIconSpan);
			}
			else if((sContents) && (sContents.length > 0))
			{
				// Text label
				this.content = document.createElement("span");
				if(this.enabled)
				{
					if(this.checked)
						this.content.className = "button-text-click";
					else
						this.content.className = "button-text";
				}
				else
				{
					if(this.checked)
						this.content.className = "button-text-click-disabled";
					else
						this.content.className = "button-text-disabled";
				}
				
				this.content.style.lineHeight = this.rect.height - 2 - 2 + "px";	// 2:border 2:margin
				this.content.innerHTML = sContents;
			}
			this.element.appendChild(this.content);
		}
		else
		{
			if(this.group)
			{
				// Radio button
				this.box = document.createElement("span");
				if(this.enabled)
				{
					if(this.checked)
						this.box.className = jaswin.ICON_RADIO_CHECKED;
					else
						this.box.className = jaswin.ICON_RADIO;
				}
				else
				{
					if(this.checked)
						this.box.className = jaswin.ICON_RADIO_CHECKED_DISABLED;
					else
						this.box.className = jaswin.ICON_RADIO;
				}
				
				this.box.style.left = "2px";
				this.box.style.top = "5px";
			}
			else
			{
				// Check box
				this.box = document.createElement("span");
				if(this.enabled)
				{
					if(this.checked)
						this.box.className = jaswin.ICON_CHECKBOX_CHECKED;
					else
						this.box.className = jaswin.ICON_CHECKBOX;
				}
				else
				{
					if(this.checked)
						this.box.className = ICON_CHECKBOX_CHECKED_DISABLED;
					else
						this.box.className = jaswin.ICON_CHECKBOX;
				}
				
				this.box.style.left = "1px";
				this.box.style.top = "4px";
			}
			this.box.style.postion = "absolute";
			this.element.appendChild(this.box);
			this.content = document.createElement("span");
			if(this.enabled)
				this.content.className = "checkbox-label";
			else
			{
				this.content.className = "checkbox-label-disabled";
				this.box.style.cursor = "default";
			}
			this.content.style.left = "16px";
			this.content.style.top = "0px";
			this.content.style.width = this.rect.width - 16 + "px";
			this.content.style.height = this.rect.height + "px";
			
			this.content.innerHTML = sContents;
			this.element.appendChild(this.content);
		}
		
		if((sTooltip) && (sTooltip.length > 0))
			this.element.title = sTooltip;
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		if(this.enabled)
		{
			this.element.addEventListener("mousemove", mouseMoveEvent, false);
			this.element.addEventListener("mouseout", mouseOutEvent, false);
			this.focusElement.addEventListener("focus", focusEvent, false);
			this.focusElement.addEventListener("blur", blurEvent, false);
			this.element.addEventListener("mousedown", mouseDownEvent, false);
			this.element.addEventListener("keyup", keyUpEvent, false);
		}
	}
	checkbox.CheckBox.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			delete checkbox.checkBoxes[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (checkbox.checkBoxes[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"checkbox.js", "jaswin.checkbox.CheckBox.setParent");
			this.element.id = sID;
			checkbox.checkBoxes[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Set label text
		//==================================================
		setLabel: function(sText)
		{
			if(!this.content)
				return;
			
			if((this.style & checkbox.STYLE_IMAGE)
			|| (this.style & checkbox.STYLE_ICON))
				return;
			
			if(typeof sText != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"checkbox.js", "jaswin.checkbox.CheckBox.setLabel");
			
			this.content.innerHTML = sText;
		},
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip: function(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"checkbox.js", "jaswin.checkbox.CheckBox.setTooltip");
			this.element.title = sTooltip;
		},
		
		//==================================================
		//	Enable this checkbox
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.element.tabIndex = this.tabIndex;
				
				if((this.style & checkbox.STYLE_BUTTON)
				|| (this.style & checkbox.STYLE_FLAT_BUTTON))
				{
					if(this.checked)
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat-click";
						else
							this.element.className = "button-click";
						
						if(this.content)
						{
							if(this.style & checkbox.STYLE_IMAGE)
								this.content.className = "button-image-click";
							else if(this.style & checkbox.STYLE_ICON)
								this.content.className = "button-icon-click";
							else
								this.content.className = "button-text-click";
						}
					}
					else
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat";
						else
							this.element.className = "button";
						
						if(this.content)
						{
							if(this.style & checkbox.STYLE_IMAGE)
								this.content.className = "button-image";
							else if(this.style & checkbox.STYLE_ICON)
								this.content.className = "button-icon";
							else
								this.content.className = "button-text";
						}
					}
				}
				else
				{
					if(this.group)
					{
						if(this.checked)
							this.box.className = jaswin.ICON_RADIO_CHECKED;
						else
							this.box.className = jaswin.ICON_RADIO;
					}
					else
					{
						if(this.checked)
							this.box.className = jaswin.ICON_CHECKBOX_CHECKED;
						else
							this.box.className = jaswin.ICON_CHECKBOX;
					}
					this.box.style.cursor = "pointer";
					
					if(this.content)
						this.content.className = "checkbox-label";
				}
				
				this.element.style.cursor = "pointer";
				
				this.element.addEventListener("mousemove", mouseMoveEvent, false);
				this.element.addEventListener("mouseout", mouseOutEvent, false);
				this.focusElement.addEventListener("focus", focusEvent, false);
				this.focusElement.addEventListener("blur", blurEvent, false);
				this.element.addEventListener("mousedown", mouseDownEvent, false);
				this.element.addEventListener("keyup", keyUpEvent, false);
			}
		},
		
		//==================================================
		//	Disable this checkbox
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.element.tabIndex = -1;
			
			if((this.style & checkbox.STYLE_BUTTON)
			|| (this.style & checkbox.STYLE_FLAT_BUTTON))
			{
				if(this.checked)
				{
					if(this.style & checkbox.STYLE_FLAT_BUTTON)
						this.element.className = "button-flat-click-disabled";
					else
						this.element.className = "button-click-disabled";
					
					if(this.content)
					{
						if(this.style & checkbox.STYLE_IMAGE)
							this.content.className = "button-image-click-disabled";
						else if(this.style & checkbox.STYLE_ICON)
							this.content.className = "button-icon-click-disabled";
						else
							this.content.className = "button-text-click-disabled";
					}
				}
				else
				{
					if(this.style & checkbox.STYLE_FLAT_BUTTON)
						this.element.className = "button-flat";
					else
						this.element.className = "button";
					
					if(this.content)
					{
						if(this.style & checkbox.STYLE_IMAGE)
							this.content.className = "button-image-disabled";
						else if(this.style & checkbox.STYLE_ICON)
							this.content.className = "button-icon-disabled";
						else
							this.content.className = "button-text-disabled";
					}
				}
			}
			else
			{
				if(this.content)
					this.content.className = "checkbox-label-disabled";
				if(this.group)
				{
					if(this.checked)
						this.box.className = jaswin.ICON_RADIO_CHECKED_DISABLED;
					else
						this.box.className = jaswin.ICON_RADIO;
				}
				else
				{
					if(this.checked)
						this.box.className = jaswin.ICON_CHECKBOX_CHECKED_DISABLED;
					else
						this.box.className = jaswin.ICON_CHECKBOX;
				}
				
				this.box.style.cursor = "default";
			}
			this.element.style.cursor = "default";
			
			this.element.removeEventListener("mousemove", mouseMoveEvent, false);
			this.element.removeEventListener("mouseout", mouseOutEvent, false);
			this.focusElement.removeEventListener("focus", focusEvent, false);
			this.focusElement.removeEventListener("blur", blurEvent, false);
			this.element.removeEventListener("mousedown", mouseDownEvent, false);
			this.element.removeEventListener("keyup", keyUpEvent, false);
		},
		
		//==================================================
		//	Show this checkbox
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this checkbox
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Set this checkbox hover
		//==================================================
		setHover: function()
		{
			if(this.checked)
				return;
			
			if(this.style & checkbox.STYLE_FLAT_BUTTON)
				this.element.className = "button-flat-focus";
			else if(this.style & checkbox.STYLE_BUTTON)
				this.element.className = "button-focus";
			else
			{
				if(this.group)
					this.box.className = "icon-radio-hover";
				else
					this.box.className = "icon-check-hover";
			}
		},
		
		//==================================================
		//	Set this checkbox mouse out
		//==================================================
		setOut: function()
		{
			if(this.checked)
				return;
			
			if(this.style & checkbox.STYLE_FLAT_BUTTON)
				this.element.className = "button-flat";
			else if(this.style & checkbox.STYLE_BUTTON)
				this.element.className = "button";
			else
			{
				if(this.group)
					this.box.className = jaswin.ICON_RADIO;
				else
					this.box.className = jaswin.ICON_CHECKBOX;
			}
		},
		
		//==================================================
		//	Change status of this checkbox
		//==================================================
		changeStatus: function()
		{
			if(this.group)
			{
				// Radio button
				if(this.checked)
					return;
				
				var oRadio;
				
				for(var sKey in checkbox.checkBoxes)
				{
					oRadio = checkbox.checkBoxes[sKey];
					if(!oRadio)
						continue;
					if((oRadio.group) && (oRadio.group == this.group))
					{
						// Only for same group
						if(oRadio == this)
						{
							this.checked = true;
							if((this.style & checkbox.STYLE_BUTTON)
							|| (this.style & checkbox.STYLE_FLAT_BUTTON))
							{
								if(this.style & checkbox.STYLE_FLAT_BUTTON)
									this.element.className = "button-flat-click";
								else
									this.element.className = "button-click";
								
								if(this.style & checkbox.STYLE_IMAGE)
									this.content.className = "button-image-click";
								else if(this.style & checkbox.STYLE_ICON)
									this.content.className = "button-icon-click";
								else
									this.content.className = "button-text-click";
							}
							else
								this.box.className = jaswin.ICON_RADIO_CHECKED;
						}
						else
						{
							oRadio.checked = false;
							if((this.style & checkbox.STYLE_BUTTON)
							|| (this.style & checkbox.STYLE_FLAT_BUTTON))
							{
								if(this.style & checkbox.STYLE_FLAT_BUTTON)
									oRadio.element.className = "button-flat";
								else
									oRadio.element.className = "button";
								
								if(this.style & checkbox.STYLE_IMAGE)
									oRadio.content.className = "button-image";
								else if(this.style & checkbox.STYLE_ICON)
									oRadio.content.className = "button-icon";
								else
									oRadio.content.className = "button-text";
							}
							else
								oRadio.box.className = jaswin.ICON_RADIO;
						}
					}
				}
			}
			else
			{
				// Checkbox
				if(this.checked)
				{
					this.checked = false;
					if((this.style & checkbox.STYLE_BUTTON)
					|| (this.style & checkbox.STYLE_FLAT_BUTTON))
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat";
						else
							this.element.className = "button";
						
						if(this.style & checkbox.STYLE_IMAGE)
							this.content.className = "button-image";
						else if(this.style & checkbox.STYLE_ICON)
							this.content.className = "button-icon";
						else
							this.content.className = "button-text";
					}
					else
						this.box.className = jaswin.ICON_CHECKBOX;
				}
				else
				{
					this.checked = true;
					if((this.style & checkbox.STYLE_BUTTON)
					|| (this.style & checkbox.STYLE_FLAT_BUTTON))
					{
						if(this.style & checkbox.STYLE_FLAT_BUTTON)
							this.element.className = "button-flat-click";
						else
							this.element.className = "button-click";
						
						if(this.style & checkbox.STYLE_IMAGE)
							this.content.className = "button-image-click";
						else if(this.style & checkbox.STYLE_ICON)
							this.content.className = "button-icon-click";
						else
							this.content.className = "button-text-click";
					}
					else
						this.box.className = jaswin.ICON_CHECKBOX_CHECKED;
				}
			}
			
			if(this.onChange)
				this.onChange(this);
		},
		
		//==================================================
		//	Set change event
		//==================================================
		setChangeEvent: function(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, this.id),
											"checkbox.js", "jaswin.checkbox.CheckBox.setChangeEvent");
			
			this.onChange = fpChange;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetCheckBox
	//
	//	Summary		:	Get target checkbox element from event
	//
	//==========================================================
	function getTargetCheckBox(e)
	{
		var oElement = e.target || e.srcElement;
		
		if((oElement.tagName == "span")
		|| (oElement.tagName == "SPAN")
		|| (oElement.tagName == "img")
		|| (oElement.tagName == "IMG"))
		{
			oElement = oElement.parentNode;
			if((oElement.tagName == "span")
			|| (oElement.tagName == "SPAN"))
				oElement = oElement.parentNode;
		}
		
		if(!oElement.id)
			return null;
		
		return checkbox.checkBoxes[oElement.id];
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseMoveEvent
	//
	//	Summary		:	Event handler of mouse move
	//
	//==========================================================
	function mouseMoveEvent(e)
	{
		var oCheckBox = null;
		
		oCheckBox = getTargetCheckBox(e);
		
		if(oCheckBox)
			oCheckBox.setHover();
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseOutEvent
	//
	//	Summary		:	Event handler of mouse out
	//
	//==========================================================
	function mouseOutEvent(e)
	{
		var oCheckBox = null;
		
		oCheckBox = getTargetCheckBox(e);
		
		if(oCheckBox)
			oCheckBox.setOut();
	}
	
	
	//==========================================================
	//
	//	Function	:	focusEvent
	//
	//	Summary		:	Event handler of focus
	//
	//==========================================================
	function focusEvent(e)
	{
		var oCheckBox = null;
		
		oCheckBox = getTargetCheckBox(e);
		if(!oCheckBox)
			return;
		
		if((!oCheckBox.enabled) || (!oCheckBox.parentEnabled))
		{
			e.preventDefault();
			return;
		}
		
		oCheckBox.setHover();
	}
	
	
	//==========================================================
	//
	//	Function	:	blurEvent
	//
	//	Summary		:	Event handler of blur
	//
	//==========================================================
	function blurEvent(e)
	{
		var oCheckBox = null;
		
		oCheckBox = getTargetCheckBox(e);
		if(!oCheckBox)
			return;
		
		if((!oCheckBox.enabled) || (!oCheckBox.parentEnabled))
		{
			e.preventDefault();
			return;
		}
		
		oCheckBox.setOut();
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseDownEvent
	//
	//	Summary		:	Event handler of mouse down
	//
	//==========================================================
	function mouseDownEvent(e)
	{
		var oCheckBox = null;
		
		oCheckBox = getTargetCheckBox(e);
		
		if(oCheckBox)
			oCheckBox.changeStatus();
	}
	
	
	//==========================================================
	//
	//	Function	:	keyUpEvent
	//
	//	Summary		:	Event handler of key up
	//
	//==========================================================
	function keyUpEvent(e)
	{
		var oCheckBox = null;
		
		if(e.keyCode != 32)
			return true;
		
		oCheckBox = getTargetCheckBox(e);
		if(!oCheckBox)
			return true;
		
		oCheckBox.changeStatus();
	}
	
})(jaswin.checkbox = jaswin.checkbox || {});