//==================================================================
//
//	File name	:	frame.js
//
//	Summary		:	Window frame class
//
//	Ceated		:	2014/09/16
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (frame) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	frame.STYLE_TITLEBAR = 1;
	frame.STYLE_CLOSEBUTTON = 2;
	frame.STYLE_DESTROY_ON_CLOSE = 4;
	frame.STYLE_CLIENTEDGE = 8;
	frame.STYLE_RESIZABLE = 16;
	frame.STYLE_MOBABLE = 32;
	frame.STYLE_SCROLLABLE = 64;
	frame.STYLE_IMAGE_ICON = 128;
	frame.STYLE_CSS_ICON = 256;
	frame.RESIZE_NONE = 0;
	frame.RESIZE_LEFT = 1;
	frame.RESIZE_RIGHT = 2;
	frame.RESIZE_TOP = 3;
	frame.RESIZE_BOTTOM = 4;
	frame.RESIZE_BOTH = 5;
	frame.CENTER_POSITION = -999999;
	
	// Grobal variables
	frame.windows = {};
	frame.windowCount = 0;
	frame.currentWindow = null;
	
	// Private variables
	var iResizeMode = frame.RESIZE_NONE;
	var iMovingX = 0;
	var iMovingY = 0;
	var currentModalLevel = 0;
	
	
	//==========================================================
	//
	//	Class		:	Window
	//
	//	Summary		:	Class of window
	//
	//==========================================================
	frame.Window = function(sID,			// * ID[String]
							oRect,			// * Size and position[ExpandableRect]
							iMinWidth,		// * Minimum width[int]
							iMinHeight,		// * Minimum height[int]
							iStyle,			// * Window style[int:bit flag]
							sTitle,			// Title[String]
							sIcon,			// Icon URL or icon class[String]
							bHideOnCreate)	// Hide on create or not[boolean]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"frame.js", "jaswin.frame.Window.constructor");
		
		if((document.getElementById(sID)) || (frame.windows[sID]))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
										"frame.js", "jaswin.frame.Window.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.ExpandableRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"frame.js", "jaswin.frame.Window.constructor");
		
		if(typeof iMinWidth != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_MIN_WIDTH, sID),
										"frame.js", "jaswin.frame.Window.constructor");
		
		if(iMinWidth < 0)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_MIN_WIDTH, sID),
										"frame.js", "jaswin.frame.Window.constructor");
		
		if(typeof iMinHeight != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_MIN_HEIGHT, sID),
										"frame.js", "jaswin.frame.Window.constructor");
		
		if(iMinHeight < 0)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_MIN_HEIGHT, sID),
										"frame.js", "jaswin.frame.Window.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"frame.js", "jaswin.frame.Window.constructor");
		
		if(sTitle)
		{
			if(typeof sTitle != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TITLE, sID),
											"frame.js", "jaswin.frame.Window.constructor");
		}
		
		if(sIcon)
		{
			if(typeof sIcon != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ICON_URL, sID),
											"frame.js", "jaswin.frame.Window.constructor");
		}
		
		if(typeof bHideOnCreate != "undefined")
		{
			if(typeof bHideOnCreate != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_HIDE_ON_CREATE, sID),
											"frame.js", "jaswin.frame.Window.constructor");
		}
		
		// Set properties
		this.id = sID;						// ID[String]
		this.rect = oRect;					// Size and position[ExpandableRect]
		this.minWidth = iMinWidth;			// Minimum width[int]
		this.minHeight = iMinHeight;		// Minimum height[int]
		this.style = iStyle;				// Window style[int:bit flag]
		this.element = null;				// Window element[DOM Element]
		this.center = null;					// Center element[DOM Element]
		this.titleBar = null;				// Title bar element[DOM Element]
		this.icon = null;					// Icon image element[DOM Element]
		this.titleSpan = null;				// Title span element[DOM Element]
		this.closeButton = null;			// Close button element[DOM Element]
		this.container = null;				// Container for client area[DOM Element]
		this.controls = new Array();		// Controls on this window[Array of Control classes]
		this.tabstops = new Array();		// Controls for tabstop[Array of Control classes]
		this.menu = null;					// Menu header[MenuBar class]
		this.toolbar = null;				// Toolbar[Toolbar class]
		this.contextMenu = null;			// Context menu on this window[Menu class]
		this.onContextMenu = null;			// Function on right click[function(Argument:Window class)]
		this.active = true;					// Active window or not[boolean]
		this.parent = null;					// Parent class[global / panel]
		this.enabled = true;				// Enable only[boolean]
		this.zIndex = 0;					// Z-index[int]
		this.modalLevel = currentModalLevel	// Modal level[int]
		this.modalPanel = null;				// Modal base panel element[DOM Element]
		this.returnValue = null;			// Return value[any]
		this.returnWindow = null;			// Return window[Window]
		this.callBack = null;				// Call back function[function]
		
		// Create element
		this.element = document.createElement("div");
		this.element.id = this.id;
		this.element.className = "window";
		
		// Horizontal rect
		if(this.rect.anchorX == jaswin.ANCHOR_BOTH)
		{
			this.element.style.left = this.rect.left + "px";
			this.element.style.right = this.rect.right + "px";
		}
		else
		{
			if(this.rect.width < this.minWidth)
				this.rect.width = this.minWidth;
			
			if(this.rect.anchorX == jaswin.ANCHOR_LEFT)
				this.element.style.left = this.rect.left + "px";
			else if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.right + "px";
			this.element.style.width = this.rect.width + "px";
		}
		
		// Vertical rect
		if(this.rect.anchorY == jaswin.ANCHOR_BOTH)
		{
			this.element.style.top = this.rect.top + "px";
			this.element.style.bottom = this.rect.bottom + "px";
		}
		else
		{
			if(this.rect.height < this.iMinHeight)
				this.rect.height = this.iMinHeight;
			
			if(this.rect.anchorY == jaswin.ANCHOR_TOP)
				this.element.style.top = this.rect.top + "px";
			else if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.bottom + "px";
			this.element.style.height = this.rect.height + "px";
		}
		
		// Top Edge
		var oTopEdge = document.createElement("div");
		oTopEdge.className = "edge";
		oTopEdge.style.height = "2px";
		oTopEdge.style.top = "0px";
		oTopEdge.style.left = "0px";
		oTopEdge.style.right = "0px";
		this.element.appendChild(oTopEdge);
		
		// Left Edge
		var oLeftEdge = document.createElement("div");
		oLeftEdge.className = "edge";
		oLeftEdge.style.left = "0px";
		oLeftEdge.style.top = "2px";
		oLeftEdge.style.bottom = "2px";
		oLeftEdge.style.width = "2px";
		this.element.appendChild(oLeftEdge);
		
		// Center(For client area)
		this.center = document.createElement("div");
		this.center.style.position = "absolute";
		this.center.style.left = "2px";
		this.center.style.right = "2px";
		this.center.style.top = "2px";
		this.center.style.bottom = "2px";
		this.element.appendChild(this.center);
		
		// Right edge
		var oRightEdge = document.createElement("div");
		oRightEdge.className = "edge";
		oRightEdge.style.right = "0px";
		oRightEdge.style.top = "2px";
		oRightEdge.style.bottom = "2px";
		oRightEdge.style.width = "2px";
		this.element.appendChild(oRightEdge);
		
		// Bottom edge
		var oBottomEdge = document.createElement("div");
		oBottomEdge.className = "edge";
		oBottomEdge.style.left = "0px";
		oBottomEdge.style.right = "2px";
		oBottomEdge.style.bottom = "0px";
		oBottomEdge.style.height = "2px";
		this.element.appendChild(oBottomEdge);
		
		// Right bottom corner edge
		var oCornerEdge = document.createElement("div");
		oCornerEdge.className = "edge";
		oCornerEdge.style.right = "0px";
		oCornerEdge.style.bottom = "0px";
		oCornerEdge.style.width = "2px";
		oCornerEdge.style.height = "2px";
		this.element.appendChild(oCornerEdge);
		
		// Title bar
		if(this.style & frame.STYLE_TITLEBAR)
		{
			this.titleBar = document.createElement("div");
			this.titleBar.className = "title";
			
			if(sIcon)
			{
				if(this.style & frame.STYLE_IMAGE_ICON)
				{
					// Image icon
					this.icon = document.createElement("img");
					this.icon.src = sIcon;
					this.icon.width = 16;
					this.icon.height = 16;
					this.icon.style.position = "absolute";
					this.icon.style.left = "1px";
					this.icon.style.top = "1px";
					this.titleBar.appendChild(this.icon);
				}
				else if(this.style & frame.STYLE_CSS_ICON)
				{
					// Span icon
					this.icon = document.createElement("span");
					this.icon.style.position = "absolute";
					this.icon.style.width = "16px";
					this.icon.style.height = "16px";
					this.icon.style.left = "1px";
					this.icon.style.top = "1px";
					
					var oSpan = document.createElement("span");
					oSpan.className = sIcon;
					this.icon.appendChild(oSpan);
					this.titleBar.appendChild(this.icon);
				}
			}
			
			this.titleSpan = document.createElement("span");
			this.titleSpan.className = "title-text";
			if(sTitle)
				this.titleSpan.innerHTML = sTitle;
			
			if(sIcon)
				this.titleSpan.style.left = "22px";
			else
				this.titleSpan.style.left = "4px";
			
			if(this.style & frame.STYLE_CLOSEBUTTON)
				this.titleSpan.style.right = "36px";
			else
				this.titleSpan.style.right = "4px";
			this.titleBar.appendChild(this.titleSpan);
			
			if(this.style & frame.STYLE_CLOSEBUTTON)
			{
				// Close button
				this.closeButton = document.createElement("span");
				this.closeButton.className = "close";
				this.closeButton.innerHTML = "x";
				this.closeButton.addEventListener("click", closeWindowEvent, false);
				
				this.titleBar.appendChild(this.closeButton);
			}
			this.center.appendChild(this.titleBar);
		}
		
		// Client area
		this.container = document.createElement("div");
		
		if(this.style & frame.STYLE_CLIENTEDGE)
			this.container.className = "clientedge";
		else
			this.container.className = "clieantflat";
		
		if(this.style & frame.STYLE_SCROLLABLE)
			this.container.style.overflow = "auto";
		
		if(this.style & frame.STYLE_TITLEBAR)
			this.container.style.top = "18px";
		else
			this.container.style.top = "0px";
		this.center.appendChild(this.container);
		
		// Window forcus event
		this.element.addEventListener("mousedown", selectWindowEvent, false);
		
		if(this.style & frame.STYLE_RESIZABLE)
		{
			// Resize event
			if(this.rect.anchorX != jaswin.ANCHOR_BOTH)
			{
				oLeftEdge.addEventListener("mousedown", startResizeWindow);
				oLeftEdge.style.cursor = "w-resize";
				oRightEdge.addEventListener("mousedown", startResizeWindow);
				oRightEdge.style.cursor = "e-resize";
			}
			if(this.rect.anchorY != jaswin.ANCHOR_BOTH)
			{
				oTopEdge.addEventListener("mousedown", startResizeWindow);
				oTopEdge.style.cursor = "n-resize";
				oBottomEdge.addEventListener("mousedown", startResizeWindow);
				oBottomEdge.style.cursor = "s-resize";
			}
			
			if((this.rect.anchorX != jaswin.ANCHOR_BOTH)
			&& (this.rect.anchorY != jaswin.ANCHOR_BOTH))
			{
				oCornerEdge.addEventListener("mousedown", startResizeWindow);
				oCornerEdge.style.cursor = "se-resize";
			}
		}
		
		if(this.style & frame.STYLE_MOBABLE)
		{
			// Move event
			if((this.rect.anchorX == jaswin.ANCHOR_LEFT)
			&& (this.rect.anchorY == jaswin.ANCHOR_TOP))
				this.titleBar.addEventListener("mousedown", startDragWindow);
		}
		
		frame.windows[this.id] = this;
		frame.windowCount++;
		
		if((typeof bHideOnCreate != "undefined") && bHideOnCreate)
		{
			for(var sKey in frame.windows)
			{
				if(frame.windows[sKey] == this)
				{
					this.zIndex = 1;
					this.element.style.zIndex = 1 + this.modalLevel;
					this.element.style.display = "none";
				}
				else
				{
					frame.windows[sKey].zIndex++;
					frame.windows[sKey].element.style.zIndex = parseInt(frame.windows[sKey].element.style.zIndex) + 1;
				}
			}
		}
		else
		{
			this.zIndex = frame.windowCount;
			this.element.style.zIndex = frame.windowCount + this.modalLevel;
		}
	}
	frame.Window.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
				this.controls[iCounter].destroy();
			
			if(this.menu)
				this.menu.destroy();
			
			if(this.toolbar)
				this.toolbar.destroy();
			
			if(this.contextMenu)
				this.contextMenu.destroy();
			
			delete frame.windows[this.id];
			frame.windowCount--;
			delete global[this.id];
			
			this.parent.notifyDestroyWindow(this);
			
			this.element.parentNode.removeChild(this.element);
		},
		
		//==================================================
		//	Set active
		//==================================================
		setActive: function()
		{
			if(this.modalLevel != currentModalLevel)
				return;
			
			if(this.style & frame.STYLE_TITLEBAR)
			{
				this.titleBar.className = "title-active";
				this.titleSpan.className = "title-text-active";
			}
			this.active = true;
			frame.currentWindow = this;
			
			var iCounter = 0;
			
			for(iCounter = 0; iCounter < global.tabstops.length; iCounter++)
				global.tabstops[iCounter].focusElement.tabIndex = -1;
			
			var bFirst = true;
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if((this.tabstops[iCounter].enabled) && (this.tabstops[iCounter].parentEnabled))
				{
					this.tabstops[iCounter].focusElement.tabIndex = this.tabstops[iCounter].tabIndex;
					if(bFirst)
					{
						this.tabstops[iCounter].focusElement.focus();
						bFirst = false;
					}
				}
			}
			
			var iCurIndex = 0;
			var iTmpIndex = 0;
			
			iCurIndex = this.zIndex;
			
			for(var sKey in frame.windows)
			{
				if(frame.windows[sKey] == this)
				{
					this.zIndex = frame.windowCount;
					this.element.style.zIndex = frame.windowCount + this.modalLevel;
				}
				else
				{
					iTmpIndex = frame.windows[sKey].zIndex;
					if(iTmpIndex > iCurIndex)
					{
						frame.windows[sKey].zIndex = iTmpIndex - 1;
						frame.windows[sKey].element.style.zIndex = iTmpIndex - 1 + frame.windows[sKey].modalLevel;
					}
					frame.windows[sKey].setInactive();
				}
			}
			
			if(global.menu)
				global.menu.setInactive();
			
			this.element.addEventListener("keydown", windowKeyEvent);
		},
		
		//==================================================
		//	Set inctive
		//==================================================
		setInactive: function()
		{
			if(this.style & frame.STYLE_TITLEBAR)
			{
				this.titleBar.className = "title";
				this.titleSpan.className = "title-text";
			}
			this.active = false;
			
			var iCounter = 0;
			
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
				this.tabstops[iCounter].focusElement.tabIndex = -1;
			
			if(this.menu)
				this.menu.setInactive();
			
			this.element.removeEventListener("keydown", windowKeyEvent);
		},
		
		//==================================================
		//	Show this window
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
			this.setActive();
		},
		
		//==================================================
		//	Show this window as dialog box
		//==================================================
		showDialog: function(oWindow,		// Return window on close[Window class]
							fpCallBack)		// Call back function[function]
		{
			if(oWindow)
			{
				if((typeof oWindow != "object")
				|| (!oWindow instanceof frame.Window))
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_WINDOW, this.id),
												"frame.js", "jaswin.frame.Window.showDialog");
				
				this.returnWindow = oWindow;
			}
			
			if(fpCallBack)
			{
				if(typeof fpCallBack != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_CALLBACK, this.id),
												"frame.js", "jaswin.frame.Window.showDialog");
				this.callBack = fpCallBack;
			}
			
			currentModalLevel += 1000;
			this.modalPanel = document.createElement("div");
			this.modalPanel.className = "modal-panel";
			this.modalPanel.style.zIndex = currentModalLevel;
			global.element.appendChild(this.modalPanel);
			this.modalLevel = currentModalLevel;
			this.show();
			this.setActive();
		},
		
		//==================================================
		//	Hide this window
		//==================================================
		hide: function()
		{
			var iIndex;
			var oWnd = null;
			
			this.element.style.display = "none";
			
			for(var sKey in frame.windows)
			{
				if(frame.windows[sKey] == this)
				{
					this.element.style.zIndex = 1;
					this.setInactive();
				}
				else
				{
					iIndex = frame.windows[sKey].zIndex + 1;
					frame.windows[sKey].zIndex = iIndex;
					frame.windows[sKey].element.style.zIndex = iIndex + frame.windows[sKey].modalLevel;
					if((frame.currentWindow) && (iIndex == frame.windowCount))
						oWnd = frame.windows[sKey];
				}
			}
		},
		
		//==================================================
		//	Close this window
		//==================================================
		close: function()
		{
			var iIndex;
			var oWnd = null;
			
			if(this.modalPanel)
			{
				global.element.removeChild(this.modalPanel);
				this.modalLevel = 0;
				currentModalLevel -= 1000;
			}
			
			this.element.style.display = "none";
			
			if(this.returnWindow)
			{
				if(this.returnWindow != global)
					oWnd = this.returnWindow;
				this.element.style.zIndex = 1;
				this.setInactive();
			}
			else
			{
				for(var sKey in frame.windows)
				{
					if(frame.windows[sKey] == this)
					{
						this.element.style.zIndex = 1;
						this.setInactive();
					}
					else
					{
						if(this.style & frame.STYLE_DESTROY_ON_CLOSE)
						{
							iIndex = frame.windows[sKey].zIndex;
							if(iIndex == frame.windowCount - 1)
								oWnd = frame.windows[sKey];
						}
						else
						{
							iIndex = frame.windows[sKey].zIndex + 1;
							frame.windows[sKey].zIndex = iIndex;
							frame.windows[sKey].element.style.zIndex = iIndex + frame.windows[sKey].modalLevel;
							if((frame.currentWindow) && (iIndex == frame.windowCount))
								oWnd = frame.windows[sKey];
						}
					}
				}
			}
			
			if(this.callBack)
				this.callBack(this);
			
			if(this.style & frame.STYLE_DESTROY_ON_CLOSE)
				this.destroy();
			
			if(oWnd)
				oWnd.setActive();
			else
				global.setActive();
		},
		
		//==================================================
		//	Change size of this window
		//==================================================
		setSize: function(iW, iH)
		{
			if(typeof iW != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_WIDTH, this.id),
											"frame.js", "jaswin.frame.Window.setSize");
			
			if(typeof iH != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_HEIGHT, this.id),
											"frame.js", "jaswin.frame.Window.setSize");
			
			if(this.rect.anchorX != jaswin.ANCHOR_BOTH)
				this.rect.width = iW;
			if(this.rect.anchorY != jaswin.ANCHOR_BOTH)
				this.rect.height = iH;
			
			this.element.style.width = iW + "px";
			this.element.style.height = iH + "px";
			
			if((this.toolbar) && (this.toolbar.isVisible()))
			{
				var iTop = 0;
				var iOffset = 0;
				
				if(this.style & frame.STYLE_TITLEBAR)
					iTop += 18;
				if(this.menu)
					iTop += 22;
				
				if(this.toolbar.controls.length > 0)
					iOffset = this.toolbar.controls[this.toolbar.controls.length - 1].element.offsetTop;
				
				this.toolbar.element.style.height = 20 + iOffset + "px";
				iTop += 22 + iOffset;
				
				this.container.style.top = iTop + "px";
			}
		},
		
		//==================================================
		//	Move this window
		//==================================================
		setPosition: function(iX, iY)
		{
			if(typeof iX != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_X, this.id),
											"frame.js", "jaswin.frame.Window.setPosition");
			
			if(typeof iY != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_Y, this.id),
											"frame.js", "jaswin.frame.Window.setPosition");
			
			this.rect.left = iX;
			this.rect.top = iY;
			
			this.element.style.left = iX + "px";
			this.element.style.top = iY + "px";
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			
			if((this.rect.anchorX == jaswin.ANCHOR_LEFT)
			&& (this.rect.anchorY == jaswin.ANCHOR_TOP))
			{
				if(this.rect.left == frame.CENTER_POSITION)
				{
					var iParentWidth = this.parent.container.clientWidth;
					
					if(this.rect.width >= iParentWidth)
						this.rect.left = 0;
					else
						this.rect.left = Math.floor((iParentWidth - this.rect.width) / 2);
					
					this.element.style.left = this.rect.left + "px";
				}
				if(this.rect.top == frame.CENTER_POSITION)
				{
					var iParentHeight = this.parent.container.clientHeight;
					
					if(this.rect.height >= iParentHeight)
						this.rect.top = 0;
					else
						this.rect.top = Math.floor((iParentHeight - this.rect.height) / 2);
					
					this.element.style.top = this.rect.top + "px";
				}
			}
			
			global[this.id] = this;
		},
		
		//==================================================
		//	Add control to this window
		//==================================================
		addControl: function(oControl)
		{
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, this.id),
											"frame.js", "jaswin.frame.Window.addControl");
			
			if(oControl instanceof jaswin.frame.Window)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_WINDOW_ALLOCATION, this.id),
											"frame.js", "jaswin.frame.Window.addControl");
			
			// Append element and set parent
			this.container.appendChild(oControl.element);
			this.controls[this.controls.length] = oControl;
			oControl.setParent(this);
			
			// Set tabstop
			if((oControl.tabIndex) && (oControl.tabIndex > 0))
			{
				this.tabstops[this.tabstops.length] = oControl;
				if(this.tabstops.length > 1)
					this.tabstops.sort(jaswin.compareTabstop);
				
				if(this != frame.currentWindow)
					oControl.focusElement.tabIndex = -1;
			}
		},
		
		//==================================================
		//	Add tabstop to this window
		//==================================================
		addTabstop: function(oControl)
		{
			// Set tabstop
			if((oControl.tabIndex) && (oControl.tabIndex > 0))
			{
				this.tabstops[this.tabstops.length] = oControl;
				if(this.tabstops.length > 1)
					this.tabstops.sort(jaswin.compareTabstop);
			}
		},
		
		//==================================================
		//	Remove control from this window
		//==================================================
		removeControl: function(oControl)
		{
			var iCounter;
			
			// Remove tabstop
			if((oControl.tabIndex) && (oControl.tabIndex > 0))
				this.removeTabstop(oControl);
			
			// Remove control
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
			{
				if(this.controls[iCounter] == oControl)
				{
					this.controls.splice(iCounter, 1);
					oControl.destroy();
					return;
				}
			}
		},
		
		//==================================================
		//	Remove tabstop from this window
		//==================================================
		removeTabstop: function(oControl)
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if(this.tabstops[iCounter] == oControl)
				{
					this.tabstops.splice(iCounter, 1);
					return;
				}
			}
		},
		
		//==================================================
		//	Set main menu
		//==================================================
		setMenu: function(oMenuBar)
		{
			if(!jaswin.menu)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "menu.js"),
											"frame.js", "jaswin.frame.Window.setMenu");
			
			if((typeof oMenuBar != "object")
			|| (!oMenuBar instanceof jaswin.menu.MenuBar))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU_HEADER, this.id),
											"frame.js", "jaswin.frame.Window.setMenu");
			
			this.menu = oMenuBar;
			this.center.appendChild(this.menu.element);
			this.menu.setParent(this);
			
			var iTop = 22;
			
			if(this.style & frame.STYLE_TITLEBAR)
			{
				iTop += 18;
				this.menu.element.style.top = "18px";
			}
			
			if(this.toolbar)
				iTop += parseInt(this.toolbar.style.height) + 2;
			
			this.container.style.top = iTop + "px";
			
			this.element.addEventListener("menu", windowMenuEvent, false);
		},
		
		//==================================================
		//	Set tool bar
		//==================================================
		setToolBar: function(oToolBar)
		{
			if(!jaswin.toolbar)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "toolbar.js"),
											"frame.js", "jaswin.frame.Window.setToolBar");
			
			if((typeof oToolBar != "object")
			|| (!oToolBar instanceof jaswin.toolbar.ToolBar))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLBAR, this.id),
											"frame.js", "jaswin.frame.Window.setToolBar");
			
			var iTop = 0;
			var iOffset = 0;
			
			if(this.style & frame.STYLE_TITLEBAR)
				iTop += 18;
			if(this.menu)
				iTop += 22;
			
			oToolBar.element.style.top = iTop + "px";
			this.center.appendChild(oToolBar.element);
			oToolBar.setParent(this);
			
			if(oToolBar.controls.length > 0)
				iOffset = oToolBar.controls[oToolBar.controls.length - 1].element.offsetTop;
			
			oToolBar.element.style.height = 20 + iOffset + "px";
			iTop += 22 + iOffset;
			
			this.container.style.top = iTop + "px";
			this.toolbar = oToolBar;
		},
		
		//==================================================
		//	Show tool bar
		//==================================================
		showToolBar: function()
		{
			if(!this.toolbar)
				return;
			this.toolbar.show();
			
			var iTop = 0;
			var iOffset = 0;
			
			if(this.style & frame.STYLE_TITLEBAR)
				iTop += 18;
			if(this.menu)
				iTop += 22;
			
			if(this.toolbar.controls.length > 0)
				iOffset = this.toolbar.controls[this.toolbar.controls.length - 1].element.offsetTop;
			
			this.toolbar.element.style.height = 20 + iOffset + "px";
			iTop += 22 + iOffset;
			
			this.container.style.top = iTop + "px";
		},
		
		//==================================================
		//	Hide tool bar
		//==================================================
		hideToolBar: function()
		{
			if(!this.toolbar)
				return;
			this.toolbar.hide();
			
			var iTop = 0;
			
			if(this.style & frame.STYLE_TITLEBAR)
				iTop += 18;
			if(this.menu)
				iTop += 22;
			
			this.container.style.top = iTop + "px";
		},
		
		//==================================================
		//	Set context menu to this window
		//==================================================
		setContextMenu: function(oMenu, fpContext)
		{
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, this.id),
											"frame.js", "jaswin.frame.Window.setContextMenu");
			
			if((typeof oMenu != "object")
			|| (!oMenu instanceof jaswin.menu.Menu))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU, this.id),
											"frame.js", "jaswin.frame.Window.setContextMenu");
			
			if(fpContext)
			{
				if(typeof fpContext != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CONTEXTMENU, this.id),
												"frame.js", "jaswin.frame.Window.setContextMenu");
			}
			
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
			}
			
			this.container.appendChild(oMenu.element);
			this.contextMenu = oMenu;
			oMenu.setParent(this, 0);
			
			if(fpContext)
				this.onContextMenu = fpContext;
			
			this.container.addEventListener("contextmenu", windowContextMenu, false);
		},
		
		//==================================================
		//	Remove context menu from this panel
		//==================================================
		removeContextMenu: function()
		{
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
				this.container.removeEventListener("contextmenu", windowContextMenu, false);
			}
		},
		
		//==================================================
		//	Show context menu
		//==================================================
		showContextMenu: function(e)
		{
			var oElement = e.target || e.srcElement;
			
			if(oElement != this.container)
				return;
			
			if(this.onContextMenu)
			{
				if(!this.onContextMenu(this))
					return;
			}
			
			this.contextMenu.showContext(e);
		},
		
		//==================================================
		//	Set title
		//==================================================
		setTitle: function(sTitle)
		{
			if(this.style & frame.STYLE_TITLEBAR)
			{
				if(typeof sTitle != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_TITLE, this.id),
												"frame.js", "jaswin.frame.Window.setTitle");
				this.titleSpan.innerHTML = sTitle;
			}
		},
		
		//==================================================
		//	Focus previous tabstop element
		//==================================================
		prevTabstop: function(e)
		{
			var iCounter;
			var iIndex = -1;
			var iValidCnt = 0;
			
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if(this.tabstops[iCounter].element.offsetWidth != 0)
					iValidCnt++;
			}
			if(iValidCnt <= 1)
			{
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			
			for(iCounter = this.tabstops.length - 1; iCounter >= 0 ; iCounter--)
			{
				if(iIndex != -1)
				{
					if(this.tabstops[iCounter].element.offsetWidth != 0)
					{
						this.tabstops[iCounter].focusElement.focus();
						e.preventDefault();
						e.stopPropagation();
						return false;
					}
				}
				
				if(document.activeElement == this.tabstops[iCounter].focusElement)
				{
					iIndex = iCounter;
					continue;
				}
			}
			
			if(iIndex == -1)
				iIndex = 0;
			
			for(iCounter = this.tabstops.length - 1; iCounter > iIndex; iCounter--)
			{
				if(this.tabstops[iCounter].element.offsetWidth != 0)
				{
					this.tabstops[iCounter].focusElement.focus();
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
		},
		
		//==================================================
		//	Focus next tabstop element
		//==================================================
		nextTabstop: function(e)
		{
			var iCounter;
			var iIndex = -1;
			var iValidCnt = 0;
			
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if(this.tabstops[iCounter].element.offsetWidth != 0)
					iValidCnt++;
			}
			if(iValidCnt <= 1)
			{
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if(iIndex != -1)
				{
					if(this.tabstops[iCounter].element.offsetWidth != 0)
					{
						this.tabstops[iCounter].focusElement.focus();
						e.preventDefault();
						e.stopPropagation();
						return false;
					}
				}
				
				if(document.activeElement == this.tabstops[iCounter].focusElement)
				{
					iIndex = iCounter;
					continue;
				}
			}
			
			if(iIndex == -1)
				iIndex = this.tabstops.length;
			
			for(iCounter = 0; iCounter < iIndex; iCounter++)
			{
				if(this.tabstops[iCounter].element.offsetWidth != 0)
				{
					this.tabstops[iCounter].focusElement.focus();
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
			
			return true;
		},
		
		//==================================================
		//	Visible or not
		//==================================================
		isVisible: function()
		{
			if(this.element.style.display == "none")
				return false;
			else
				return true;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetWindow
	//
	//	Summary		:	Get target window element from event
	//
	//==========================================================
	function getTargetWindow(e)
	{
		var oElement = e.target || e.srcElement;
		
		if(!oElement)
			return null;
		
		for(var sKey in frame.windows)
		{
			if(frame.windows[sKey].element.contains(oElement))
				return frame.windows[sKey];
		}
		return null;
	}
	
	
	//==========================================================
	//
	//	Function	:	closeWindowEvent
	//
	//	Summary		:	Event handler of closing a window
	//
	//==========================================================
	function closeWindowEvent(e)
	{
		var oWnd;
		
		oWnd = getTargetWindow(e);
		if(oWnd)
			oWnd.close();
	}
	
	
	//==========================================================
	//
	//	Function	:	selectWindowEvent
	//
	//	Summary		:	Event handler of selecting a window
	//
	//==========================================================
	function selectWindowEvent(e)
	{
		var oWnd;
		var iCurIndex = 0;
		var iTmpIndex = 0;
		
		oWnd = getTargetWindow(e);
		if(oWnd)
		{
			oWnd.setActive();
			e.stopPropagation();
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	startResizeWindow
	//
	//	Summary		:	Event handler of start resizing window
	//
	//==========================================================
	function startResizeWindow(e)
	{
		var oElement = e.target || e.srcElement;
		
		// Target not found
		if(!oElement)
			return;
		
		if(oElement.style.cursor == "w-resize")
		{
			// For left direction
			iResizeMode = frame.RESIZE_LEFT;
			frame.currentWindow = getTargetWindow(e);
		}
		else if(oElement.style.cursor == "e-resize")
		{
			// For right direction
			iResizeMode = frame.RESIZE_RIGHT;
			frame.currentWindow = getTargetWindow(e);
		}
		else if(oElement.style.cursor == "n-resize")
		{
			// For top direction
			iResizeMode = frame.RESIZE_TOP;
			frame.currentWindow = getTargetWindow(e);
		}
		else if(oElement.style.cursor == "s-resize")
		{
			// For bottom direction
			iResizeMode = frame.RESIZE_BOTTOM;
			frame.currentWindow = getTargetWindow(e);
		}
		else if(oElement.style.cursor == "se-resize")
		{
			// For both direction
			iResizeMode = frame.RESIZE_BOTH;
			frame.currentWindow = getTargetWindow(e);
		}
		else
		{
			frame.currentWindow = null;
			return;
		}
		
		global.element.addEventListener("mouseup", endResizeWindow, false);
		global.element.addEventListener("mousemove", resizeWindow, false);
	}
	
	
	//==========================================================
	//
	//	Function	:	resizeWindow
	//
	//	Summary		:	Event handler of resizing window
	//
	//==========================================================
	function resizeWindow(e)
	{
		var iCurX;
		var iCurY;
		var iParentWidth;
		var iParentHeight;
		var iTmpWidth;
		var iTmpHeight;
		var oSelection = window.getSelection();
		var oParentRect;
		
		oSelection.removeAllRanges();
		
		if(!frame.currentWindow)
		{
			global.element.removeEventListener("mouseup", endResizeWindow, false);
			global.element.removeEventListener("mousemove", resizeWindow, false);
			return;
		}
		
		oParentRect = frame.currentWindow.element.parentElement.getBoundingClientRect();
		iCurX = e.clientX - oParentRect.left;
		iCurY = e.clientY - oParentRect.top;
		
		// Get parent size
		iParentWidth = frame.currentWindow.element.parentElement.scrollWidth;
		iParentHeight = frame.currentWindow.element.parentElement.scrollHeight;
		
		// Prevent from running over
		iTmpWidth = frame.currentWindow.rect.width;
		
		if(iResizeMode == frame.RESIZE_LEFT)
		{
			// For left direction
			if(iCurX <= 0)
			{
				// Out of parent rect
				endResizeWindow(null);
				return;
			}
			
			if(iTmpWidth - (iCurX - frame.currentWindow.rect.left) <= frame.currentWindow.minWidth)
			{
				// Minimum size
				endResizeWindow(null);
				return;
			}
			
			iTmpWidth = iTmpWidth - (iCurX - frame.currentWindow.rect.left);
		}
		else if((iResizeMode == frame.RESIZE_RIGHT)
		|| (iResizeMode == frame.RESIZE_BOTH))
		{
			// For right or both direction
			if(iCurX <= frame.currentWindow.rect.left + frame.currentWindow.minWidth)
			{
				// Minimum size
				endResizeWindow(null);
				return;
			}
			
			if(iCurX >= iParentWidth - 2)
			{
				// Maximum size
				endResizeWindow(null);
				return;
			}
			
			iTmpWidth = iCurX - frame.currentWindow.rect.left;
		}
		
		iTmpHeight = frame.currentWindow.rect.height;
		
		if(iResizeMode == frame.RESIZE_TOP)
		{
			// For top direction
			if(iCurY <= 0)
			{
				// Out of parent rect
				endResizeWindow(null);
				return;
			}
			
			if(iTmpHeight - (iCurY - frame.currentWindow.rect.top) <= frame.currentWindow.minHeight)
			{
				// Minimum size
				endResizeWindow(null);
				return;
			}
			
			iTmpHeight = iTmpHeight - (iCurY - frame.currentWindow.rect.top);
		}
		else if((iResizeMode == frame.RESIZE_BOTTOM)
		|| (iResizeMode == frame.RESIZE_BOTH))
		{
			// For bottom or both direction
			if(iCurY <= frame.currentWindow.rect.top + frame.currentWindow.minHeight)
			{
				// Minimum size
				endResizeWindow(null);
				return;
			}
			if(iCurY >= iParentHeight - 2)
			{
				// Maximum size
				endResizeWindow(null);
				return;
			}
			
			iTmpHeight = iCurY - frame.currentWindow.rect.top;
		}
		
		// Set position
		if(iResizeMode == frame.RESIZE_LEFT)
			frame.currentWindow.setPosition(iCurX, frame.currentWindow.rect.top);
		else if(iResizeMode == frame.RESIZE_TOP)
			frame.currentWindow.setPosition(frame.currentWindow.rect.left, iCurY);
		
		// Resize window
		frame.currentWindow.setSize(iTmpWidth, iTmpHeight);
	}
	
	
	//==========================================================
	//
	//	Function	:	endResizeWindow
	//
	//	Summary		:	Event handler of end resizing window
	//
	//==========================================================
	function endResizeWindow(e)
	{
		global.element.removeEventListener("mouseup", endResizeWindow, false);
		global.element.removeEventListener("mousemove", resizeWindow, false);
	}
	
	
	//==========================================================
	//
	//	Function	:	startDragWindow
	//
	//	Summary		:	Event handler of start dragging window
	//
	//==========================================================
	function startDragWindow(e)
	{
		var oElement = e.target || e.srcElement;
		
		// Target not found
		if(!oElement)
			return;
		
		frame.currentWindow = getTargetWindow(e);
		
		// Escape close button
		if((oElement == frame.currentWindow.closeButton)
		|| (oElement == frame.currentWindow.icon))
			return;
		
		iMovingX = e.clientX;
		iMovingY = e.clientY;
		global.element.addEventListener("mouseup", endDragWindow, false);
		global.element.addEventListener("mousemove", dragWindow, false);
	}
	
	
	//==========================================================
	//
	//	Function	:	dragWindow
	//
	//	Summary		:	Event handler of dragging window
	//
	//==========================================================
	function dragWindow(e)
	{
		var iCurX;
		var iCurY;
		var iParentWidth;
		var iParentHeight;
		var iTmpRight;
		var iTmpBottom;
		var iNewX;
		var iNewY;
		var oSelection = window.getSelection();
		
		oSelection.removeAllRanges();
		
		if(!frame.currentWindow)
		{
			global.element.removeEventListener("mouseup", endDragWindow, false);
			global.element.removeEventListener("mousemove", dragWindow, false);
			return;
		}
		
		iCurX = e.clientX;
		iCurY = e.clientY;
		
		// Get parent size
		iParentWidth = frame.currentWindow.element.parentElement.scrollWidth;
		iParentHeight = frame.currentWindow.element.parentElement.scrollHeight;
		
		// Prevent from running over
		if((frame.currentWindow.rect.left + (iCurX - iMovingX) <= 0)
		&& (iCurX - iMovingX < 0))
		{
			endDragWindow(null);
			return;
		}
		if((frame.currentWindow.rect.top + (iCurY - iMovingY) <= 0)
		&& (iCurY - iMovingY < 0))
		{
			endDragWindow(null);
			return;
		}
		
		iTmpRight = frame.currentWindow.rect.left + (iCurX - iMovingX) + frame.currentWindow.rect.width;
		if((iTmpRight + 2 >= iParentWidth) && (iCurX - iMovingX > 0))
		{
			endDragWindow(null);
			return;
		}
		
		iTmpBottom = frame.currentWindow.rect.top + (iCurY - iMovingY) + frame.currentWindow.rect.height;
		if((iTmpBottom + 2 >= iParentHeight) && (iCurY - iMovingY > 0))
		{
			endDragWindow(null);
			return;
		}
		
		// Move window
		iNewX = frame.currentWindow.rect.left + (iCurX - iMovingX);
		iNewY = frame.currentWindow.rect.top + (iCurY - iMovingY);
		frame.currentWindow.setPosition(iNewX, iNewY);
		iMovingX = iCurX;
		iMovingY = iCurY;
	}
	
	
	//==========================================================
	//
	//	Function	:	endDragWindow
	//
	//	Summary		:	Event handler of end dragging window
	//
	//==========================================================
	function endDragWindow(e)
	{
		global.element.removeEventListener("mouseup", endDragWindow, false);
		global.element.removeEventListener("mousemove", dragWindow, false);
	}
	
	
	//==========================================================
	//
	//	Function	:	windowMenuEvent
	//
	//	Summary		:	Event handler of menu shortcut
	//
	//==========================================================
	function windowMenuEvent(e)
	{
		var oWnd;
		var iKeyCode = 0;
		var sChar = "";
		
		if(!jaswin.menu) return;
		
		oWnd = getTargetWindow(e);
		if(!oWnd) return;
		if(!oWnd.menu) return;
		
		iKeyCode = e.detail;
		
		if(iKeyCode == 18)
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < oWnd.menu.items.length; iCounter++)
			{
				if(oWnd.menu.items[iCounter].menu.isVisible())
				{
					// Window menu
					oWnd.menu.items[iCounter].element.className = "menu-header-item";
					oWnd.menu.items[iCounter].menu.hide();
					oWnd.menu.items[iCounter].status = jaswin.menu.HEADER_ITEM_STATUS_DEFAULT;
					return;
				}
			}
			
			if(jaswin.menu.visibleMenus.length > 0)
			{
				// Context menu
				jaswin.menu.visibleMenus[0].hide();
				return;
			}
		}
		
		sChar = String.fromCharCode(iKeyCode);
		oWnd.menu.shortCut(sChar);
	}
	
	
	//==========================================================
	//
	//	Function	:	windowKeyEvent
	//
	//	Summary		:	Event handler of key down event
	//
	//==========================================================
	function windowKeyEvent(e)
	{
		var oWnd;
		
		oWnd = getTargetWindow(e);
		if(!oWnd) return true;
		
		if(e.keyCode == 9)
		{
			// Tab
			if(e.shiftKey)
				return oWnd.prevTabstop(e);
			else
				return oWnd.nextTabstop(e);
		}
		
		return true;
	}
	
	
	//==========================================================
	//
	//	Function	:	windowContextMenu
	//
	//	Summary		:	Event handler of context menu
	//
	//==========================================================
	function windowContextMenu(e)
	{
		var oWnd = null;
		
		oWnd = getTargetWindow(e);
		
		if(oWnd)
			oWnd.showContextMenu(e);
		
		return false;
	}
	
})(jaswin.frame = jaswin.frame || {});
