//==================================================================
//
//	File name	:	imagebox.js
//
//	Summary		:	ImageBox class
//
//	Ceated		:	2015/04/09
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (imagebox) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	imagebox.STYLE_ORIGINAL = 0;
	imagebox.STYLE_AUTO = 1;
	imagebox.STYLE_FULL = 2;
	
	// Grobal variables
	imagebox.imageBoxes = {};
	
	//==========================================================
	//
	//	Class		:	ImageBox
	//
	//	Summary		:	Class of imagebox
	//
	//==========================================================
	imagebox.ImageBox = function(sID,				// ID[String]
								oRect,				// * Size and position[FixedRect]
								iWidthStyle,		// * Style of width[int]
								iHeightStyle,		// * Style of height[int]
								sSrc,				// Image source[String]
								sTooltip)			// Tool tip text[String]
	{
		// Check parameters
		if(sID)
		{
			if(typeof sID != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
											"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
		}
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT),
										"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
		
		if(typeof iWidthStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE),
										"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
		if((iWidthStyle != imagebox.STYLE_ORIGINAL)
		&& (iWidthStyle != imagebox.STYLE_AUTO)
		&& (iWidthStyle != imagebox.STYLE_FULL))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE),
										"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
		
		if(typeof iHeightStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE),
										"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
		if((iHeightStyle != imagebox.STYLE_ORIGINAL)
		&& (iHeightStyle != imagebox.STYLE_AUTO)
		&& (iHeightStyle != imagebox.STYLE_FULL))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE),
										"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
		
		if(sSrc)
		{
			if(typeof sSrc != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_IMAGE_SOURCE),
											"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
		}
		
		if(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP),
											"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
		}
		
		// Set properties
		this.id = sID;							// ID[String]
		this.rect = oRect;						// Size and position[FixedRect]
		this.element = null;					// This element[DOM Element]
		this.parent = null;						// Parent class[global / window / panel class]
		
		// Create element
		this.element = document.createElement("img");
		this.element.style.position = "absolute";
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		
		if(iWidthStyle == imagebox.STYLE_AUTO)
			this.element.style.width = "auto";
		else if(iWidthStyle == imagebox.STYLE_FULL)
			this.element.style.width = "100%";
		
		if(iHeightStyle == imagebox.STYLE_AUTO)
			this.element.style.height = "auto";
		else if(iHeightStyle == imagebox.STYLE_FULL)
			this.element.style.height = "100%";
		
		if(sSrc)
			this.element.src = sSrc;
		
		if(sTooltip)
			this.element.title = sTooltip;
	}
	imagebox.ImageBox.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			this.element.parentNode.removeChild(this.element);
			
			if(this.id)
			{
				delete imagebox.imageBoxes[this.element.id];
				delete oParentWnd[this.id];
			}
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			
			if(this.id)
			{
				var oParentWnd = jaswin.findParentWindow(this);
				var sID;
				
				if(oParentWnd == global)
					sID = this.id;
				else
					sID = oParentWnd.id + "_" + this.id;
				
				// Duplication check
				if((document.getElementById(sID)) || (imagebox.imageBoxes[sID]))
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
												"imagebox.js", "jaswin.imagebox.ImageBox.setParent");
				this.element.id = sID;
				imagebox.imageBoxes[this.element.id] = this;
				oParentWnd[this.id] = this;
			}
		},
		
		//==================================================
		//	Show this imagebox
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this imagebox
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Get source
		//==================================================
		getSource: function()
		{
			return this.element.src;
		},
		
		//==================================================
		//	Set source
		//==================================================
		setSource: function(sSource)
		{
			if(typeof sSource != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_IMAGE_SOURCE),
											"imagebox.js", "jaswin.imagebox.ImageBox.setSource");
			
			this.element.src = sSource;
		},
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip: function(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"imagebox.js", "jaswin.imagebox.ImageBox.setTooltip");
			this.element.title = sTooltip;
		}
	}
	
})(jaswin.imagebox = jaswin.imagebox || {});
