//==================================================================
//
//	File name	:	jaswin.js
//
//	Summary		:	Jaswin common functions and classes
//
//	Ceated		:	2014/08/16
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

var global;		// Global instance

(function (jaswin) {
	//==========================================================
	//	Global
	//==========================================================
	// Browser
	jaswin.BROWSER = "";
	
	// Error type
	jaswin.ERR_ALERT = 1;
	jaswin.ERR_ABORT = -1;
	
	// Align
	jaswin.ALIGN_LEFT = 0;
	jaswin.ALIGN_RIGHT = 1;
	jaswin.ALIGN_CENTER = 2;
	
	// Anchor
	jaswin.ANCHOR_BOTH = 0;
	jaswin.ANCHOR_LEFT = 1;
	jaswin.ANCHOR_RIGHT = 2;
	jaswin.ANCHOR_TOP = 1;
	jaswin.ANCHOR_BOTTOM = 2;
	
	// Icons
	jaswin.ICON_TRIANGLE_TOP				= "triangle-top";
	jaswin.ICON_TRIANGLE_BOTTOM				= "triangle-bottom";
	jaswin.ICON_TRIANGLE_LEFT				= "triangle-left";
	jaswin.ICON_TRIANGLE_RIGHT				= "triangle-right";
	jaswin.ICON_TRIANGLE_TOP_DISABLED		= "triangle-top-disabled";
	jaswin.ICON_TRIANGLE_BOTTOM_DISABLED	= "triangle-bottom-disabled";
	jaswin.ICON_TRIANGLE_LEFT_DISABLED		= "triangle-left-disabled";
	jaswin.ICON_TRIANGLE_RIGHT_DISABLED		= "triangle-right-disabled";
	jaswin.ICON_INFO						= "icon-info";
	jaswin.ICON_QUESTION					= "icon-question";
	jaswin.ICON_ALERT						= "icon-alert";
	jaswin.ICON_ERROR						= "icon-error";
	jaswin.ICON_FOLDER						= "icon-folder";
	jaswin.ICON_FILE						= "icon-file";
	jaswin.ICON_OPEN						= "icon-open";
	jaswin.ICON_SAVE						= "icon-save";
	jaswin.ICON_CHECKBOX					= "icon-check";
	jaswin.ICON_CHECKBOX_HOVER				= "icon-check-hover";
	jaswin.ICON_CHECKBOX_CHECKED			= "icon-check-checked";
	jaswin.ICON_CHECKBOX_CHECKED_DISABLED	= "icon-check-checked-disabled";
	jaswin.ICON_RADIO						= "icon-radio";
	jaswin.ICON_RADIO_HOVER					= "icon-radio-hover";
	jaswin.ICON_RADIO_CHECKED				= "icon-radio-checked";
	jaswin.ICON_RADIO_CHECKED_DISABLED		= "icon-radio-checked-disabled";
	
	// Message box
	jaswin.MSGBOX_OK						= 0;
	jaswin.MSGBOX_INFO						= 1;
	jaswin.MSGBOX_QUESTION					= 2;
	jaswin.MSGBOX_ALERT						= 4;
	jaswin.MSGBOX_ERROR						= 8;
	jaswin.MSGBOX_OKCANCEL					= 16;
	jaswin.MSGBOX_YESNO						= 32;
	jaswin.MSGBOX_RETURN_CANCEL				= 0;
	jaswin.MSGBOX_RETURN_OKYES				= 1;
	
	// Global size
	jaswin.currentWidth = 0;
	jaswin.currentHeight = 0;
	jaswin.lastWidth = 0;
	jaswin.lastHeight = 0;
	
	
	//==========================================================
	//
	//	Class		:	JaswinError
	//
	//	Summary		:	Error throwed by jaswin
	//
	//==========================================================
	jaswin.JaswinError = function(iType,			// Error type[int]
									sMessage,		// Error message[string]
									sFilename,		// File name[string]
									sfunctionName)	// Function name[string]
	{
		Error.call(this, sMessage);
		this.type = iType;
		this.message = sMessage;
		this.fileName = sFilename;
		this.functionName = sfunctionName;
	}
	
	
	//==========================================================
	//
	//	Class		:	Global
	//
	//	Summary		:	Global class
	//
	//==========================================================
	jaswin.Global = function()
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check browser
		getBrowserName();
		if(jaswin.BROWSER == "")
			return;
		
		// Public member
		this.id = "global";										// ID[String]
		this.element = document.createElement("div");			// Base element[DOM Element]
		this.element.style.position = "absolute";
		this.element.style.left = "0px";
		this.element.style.right = "0px";
		this.element.style.top = "0px";
		this.element.style.bottom = "0px";
		this.container = document.createElement("div");			// Container element[DOM Element]
		this.container.id = this.id;
		this.container.className = "global";
		this.element.appendChild(this.container);
		this.controls = new Array();							// Controls on global[Array of Control classes]
		this.tabstops = new Array();							// Controls for tabstop[Array of Control classes]
		this.menu = null;										// Main menu on global[Menu header class]
		this.toolbar = null;									// Toolbar on global[Toolbar class]
		this.contextMenu = null;								// Context menu on global[Menu class]
		this.onContextMenu = null;								// Function on right click[function(Argument:Global class)]
		this.enabled = true;									// Enable only[boolean]
		this.onResize = null;									// Function on resize[function]
		
		// Event binding
		this.element.addEventListener("mousedown", selectGlobalEvent, false);
		this.element.addEventListener("keydown", globalKeyEvent, false);
		window.addEventListener("resize", resizeEvent, false);
		
		document.body.appendChild(this.element);
		
		jaswin.currentWidth = this.container.clientWidth;
		jaswin.currentHeight = this.container.clientHeight;
		jaswin.lastWidth = this.container.clientWidth;
		jaswin.lastHeight = this.container.clientHeight;
		
		document.body.addEventListener("keydown", keyControl, false);
	}
	jaswin.Global.prototype =
	{
		//==================================================
		//	Set active
		//==================================================
		setActive: function()
		{
			var bFirst = true;
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if((this.tabstops[iCounter].enabled) && (this.tabstops[iCounter].parentEnabled))
				{
					this.tabstops[iCounter].focusElement.tabIndex = this.tabstops[iCounter].tabIndex;
					if(bFirst)
					{
						this.tabstops[iCounter].focusElement.focus();
						bFirst = false;
					}
				}
			}
		},
		
		//==================================================
		//	Set main menu
		//==================================================
		setMenu: function(oMenuBar)
		{
			if(!jaswin.menu)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "menu.js"),
											"jaswin.js", "jaswin.Global.setMenu");
			
			if((typeof oMenuBar != "object")
			|| (!oMenuBar instanceof jaswin.menu.MenuBar))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU_HEADER, this.id),
											"jaswin.js", "jaswin.Global.setMenu");
			
			this.menu = oMenuBar;
			this.element.appendChild(this.menu.element);
			this.menu.setParent(this)
			
			var iTop = 22;
			
			if(this.toolbar)
				iTop += parseInt(this.toolbar.style.height) + 2;
			
			this.container.style.top = iTop + "px";
			
			this.element.addEventListener("menu", globalMenuEvent, false);
			
			this.resize();
		},
		
		//==================================================
		//	Set toolbar
		//==================================================
		setToolBar: function(oToolBar)
		{
			if(!jaswin.toolbar)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "toolbar.js"),
											"jaswin.js", "jaswin.Global.setToolBar");
			
			if((typeof oToolBar != "object")
			|| (!oToolBar instanceof jaswin.toolbar.ToolBar))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLBAR, this.id),
											"jaswin.js", "jaswin.Global.setToolBar");
			
			var iTop = 0;
			var iOffset = 0;
			
			if(this.menu)
				iTop += 22;
			
			oToolBar.element.style.top = iTop + "px";
			this.element.appendChild(oToolBar.element);
			oToolBar.setParent(this);
			
			if(oToolBar.controls.length > 0)
				iOffset = oToolBar.controls[oToolBar.controls.length - 1].element.offsetTop;
			
			oToolBar.element.style.height = 20 + iOffset + "px";
			iTop += 22 + iOffset;
			
			this.container.style.top = iTop + "px";
			this.toolbar = oToolBar;
			
			this.resize();
		},
		
		//==================================================
		//	Show tool bar
		//==================================================
		showToolBar: function()
		{
			if(!this.toolbar)
				return;
			this.toolbar.show();
			
			var iTop = 0;
			var iOffset = 0;
			
			if(this.menu)
				iTop += 22;
			
			if(this.toolbar.controls.length > 0)
				iOffset = this.toolbar.controls[this.toolbar.controls.length - 1].element.offsetTop;
			
			this.toolbar.element.style.height = 20 + iOffset + "px";
			iTop += 22 + iOffset;
			
			this.container.style.top = iTop + "px";
			this.resize();
		},
		
		//==================================================
		//	Hide tool bar
		//==================================================
		hideToolBar: function()
		{
			if(!this.toolbar)
				return;
			this.toolbar.hide();
			
			var iTop = 0;
			
			if(this.menu)
				iTop += 22;
			
			this.container.style.top = iTop + "px";
			this.resize();
		},
		
		//==================================================
		//	Add control to global
		//==================================================
		addControl: function(oControl)
		{
			// Append element and set parent
			this.container.appendChild(oControl.element);
			this.controls[this.controls.length] = oControl;
			oControl.setParent(this);
			
			if(jaswin.frame)
			{
				// Activate window
				if((oControl instanceof jaswin.frame.Window)
				&& (oControl.isVisible()))
					oControl.setActive();
			}
			
			// Set tabstop
			if((oControl.tabIndex) && (oControl.tabIndex > 0))
			{
				this.tabstops[this.tabstops.length] = oControl;
				if(this.tabstops.length > 1)
					this.tabstops.sort(jaswin.compareTabstop);
				
				if((jaswin.frame) && (jaswin.frame.currentWindow))
					oControl.focusElement.tabIndex = -1;
			}
		},
		
		//==================================================
		//	Add tabstop to global
		//==================================================
		addTabstop: function(oControl)
		{
			// Set tabstop
			if((oControl.tabIndex) && (oControl.tabIndex > 0))
			{
				this.tabstops[this.tabstops.length] = oControl;
				if(this.tabstops.length > 1)
					this.tabstops.sort(jaswin.compareTabstop);
			}
		},
		
		//==================================================
		//	Remove control from global
		//==================================================
		removeControl: function(oControl)
		{
			var iCounter;
			
			// Remove tabstop
			if((oControl.tabIndex) && (oControl.tabIndex > 0))
				this.removeTabstop(oControl);
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
			{
				if(this.controls[iCounter] == oControl)
				{
					this.controls.splice(iCounter, 1);
					oControl.destroy();
					return;
				}
			}
		},
		
		//==================================================
		//	Remove tabstop from global
		//==================================================
		removeTabstop: function(oControl)
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if(this.tabstops[iCounter] == oControl)
				{
					this.tabstops.splice(iCounter, 1);
					return;
				}
			}
		},
		
		//==================================================
		//	Add download hole to global
		//==================================================
		addDownloadHole: function(oHole)
		{
			if(!jaswin.net)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "net.js"),
											"jaswin.js", "jaswin.Global.addDownloadHole");
			
			if((typeof oHole != "object")
			|| (!oHole instanceof jaswin.net.DownloadHole))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.ALERT_TYPE_DOWNLOAD_HOLE,
											"jaswin.js", "jaswin.Global.addDownloadHole");
			
			this.element.insertBefore(oHole.element, this.container);
			oHole.setParent();
		},
		
		//==================================================
		//	Remove download hole from global
		//==================================================
		removeDownloadHole: function(oHole)
		{
			if((typeof oHole != "object")
			|| (!oHole instanceof jaswin.net.DownloadHole))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.ALERT_TYPE_DOWNLOAD_HOLE,
											"jaswin.js", "jaswin.Global.removeDownloadHole");
			oHole.destroy();
		},
		
		//==================================================
		//	Notify destroy window
		//==================================================
		notifyDestroyWindow: function(oWindow)
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
			{
				if(this.controls[iCounter] == oWindow)
				{
					this.controls.splice(iCounter, 1);
					return;
				}
			}
		},
		
		//==================================================
		//	Set context menu to global
		//==================================================
		setContextMenu: function(oMenu, fpContext)
		{
			if((typeof oMenu != "object")
			|| (!oMenu instanceof jaswin.menu.Menu))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU, this.id),
											"jaswin.js", "jaswin.Global.setContextMenu");
			
			if(fpContext)
			{
				if(typeof fpContext != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CONTEXTMENU, this.id),
												"frame.js", "jaswin.Global.setContextMenu");
			}
			
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
			}
			
			this.container.appendChild(oMenu.element);
			this.contextMenu = oMenu;
			oMenu.setParent(this, 0);
			
			if(fpContext)
				this.onContextMenu = fpContext;
			
			this.container.addEventListener("contextmenu", globalContextMenu, false);
		},
		
		//==================================================
		//	Remove context menu from global
		//==================================================
		removeContextMenu: function()
		{
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
				this.container.removeEventListener("contextmenu", globalContextMenu, false);
			}
		},
		
		//==================================================
		//	Show context menu
		//==================================================
		showContextMenu: function(e)
		{
			var oElement = e.target || e.srcElement;
			
			if(oElement != this.container)
				return;
			
			if(this.onContextMenu)
			{
				if(!this.onContextMenu(this))
					return;
			}
			
			this.contextMenu.showContext(e);
		},
		
		//==================================================
		//	Focus previous tabstop element
		//==================================================
		prevTabstop: function(e)
		{
			var iCounter;
			var iIndex = -1;
			var iValidCnt = 0;
			
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if(this.tabstops[iCounter].element.offsetWidth != 0)
					iValidCnt++;
			}
			if(iValidCnt <= 1)
			{
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			
			for(iCounter = this.tabstops.length - 1; iCounter >= 0 ; iCounter--)
			{
				if(iIndex != -1)
				{
					if(this.tabstops[iCounter].element.offsetWidth != 0)
					{
						this.tabstops[iCounter].focusElement.focus();
						e.preventDefault();
						e.stopPropagation();
						return false;
					}
				}
				
				if(document.activeElement == this.tabstops[iCounter].focusElement)
				{
					iIndex = iCounter;
					continue;
				}
			}
			
			if(iIndex == -1)
				iIndex = 0;
			
			for(iCounter = this.tabstops.length - 1; iCounter > iIndex; iCounter--)
			{
				if(this.tabstops[iCounter].element.offsetWidth != 0)
				{
					this.tabstops[iCounter].focusElement.focus();
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
		},
		
		//==================================================
		//	Focus next tabstop element
		//==================================================
		nextTabstop: function(e)
		{
			var iCounter;
			var iIndex = -1;
			var iValidCnt = 0;
			
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if(this.tabstops[iCounter].element.offsetWidth != 0)
					iValidCnt++;
			}
			if(iValidCnt <= 1)
			{
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			
			for(iCounter = 0; iCounter < this.tabstops.length; iCounter++)
			{
				if(iIndex != -1)
				{
					if(this.tabstops[iCounter].element.offsetWidth != 0)
					{
						this.tabstops[iCounter].focusElement.focus();
						e.preventDefault();
						e.stopPropagation();
						return false;
					}
				}
				
				if(document.activeElement == this.tabstops[iCounter].focusElement)
				{
					iIndex = iCounter;
					continue;
				}
			}
			
			if(iIndex == -1)
				iIndex = this.tabstops.length;
			
			for(iCounter = 0; iCounter < iIndex; iCounter++)
			{
				if(this.tabstops[iCounter].element.offsetWidth != 0)
				{
					this.tabstops[iCounter].focusElement.focus();
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
			
			return true;
		},
		
		//==================================================
		//	Set resize function
		//==================================================
		setResizeEvent: function(fpResize)
		{
			if(typeof fpResize != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_RESIZE, this.id),
											"jaswin.js", "jaswin.Global.setResizeEvent");
			this.onResize = fpResize;
		},
		
		//==================================================
		//	Resize event
		//==================================================
		resize: function()
		{
			if((this.toolbar) && this.toolbar.isVisible())
			{
				var iTop = 0;
				var iOffset = 0;
				
				if(this.menu)
					iTop += 22;
				
				if(this.toolbar.controls.length > 0)
					iOffset = this.toolbar.controls[this.toolbar.controls.length - 1].element.offsetTop;
				
				this.toolbar.element.style.height = 20 + iOffset + "px";
				iTop += 22 + iOffset;
				
				this.container.style.top = iTop + "px";
			}
			
			jaswin.lastWidth = jaswin.currentWidth;
			jaswin.lastHeight = jaswin.currentHeight;
			jaswin.currentWidth = this.container.clientWidth;
			jaswin.currentHeight = this.container.clientHeight;
			
			if(this.onResize)
				this.onResize();
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getBrowserName
	//
	//	Summary		:	Get name of this browser
	//
	//==========================================================
	function getBrowserName()
	{
		var sUserAgent = window.navigator.userAgent.toLowerCase();
		
		if((sUserAgent.indexOf("opera") != -1) || (sUserAgent.indexOf("opr") != -1))
			jaswin.BROWSER = "opera";
		else if (sUserAgent.indexOf("edge") != -1)
			jaswin.BROWSER = "ie";
		else if (sUserAgent.indexOf("chrome") != -1)
			jaswin.BROWSER = "chrome";
		else if (sUserAgent.indexOf("msie") != -1)
			jaswin.BROWSER = "";
		else if (sUserAgent.indexOf("safari") != -1)
			jaswin.BROWSER = "safari";
		else if (sUserAgent.indexOf("like gecko") != -1)
			jaswin.BROWSER = "ie";
		else if (sUserAgent.indexOf("gecko") != -1)
			jaswin.BROWSER = "gecko";
		else
			jaswin.BROWSER = "";
	}
	
	
	//==========================================================
	//
	//	Function	:	compareTabstop
	//
	//	Summary		:	Compare tab stop for sort
	//
	//==========================================================
	jaswin.compareTabstop = function(oControl1, oControl2)
	{
		if(!oControl2.tabIndex)
			return -1;
		if(!oControl1.tabIndex)
			return 1;
		
		if(oControl2.tabIndex < 0)
			return -1;
		if(oControl1.tabIndex < 0)
			return 1;
		
		if(oControl1.tabIndex < oControl2.tabIndex)
			return -1;
		else if(oControl1.tabIndex > oControl2.tabIndex)
			return 1;
		
		return 0;
	}
	
	
	//==========================================================
	//
	//	Function	:	selectGlobalEvent
	//
	//	Summary		:	Event handler of selecting global
	//
	//==========================================================
	function selectGlobalEvent(e)
	{
		if(jaswin.frame)
		{
			if(jaswin.frame.currentWindow)
			{
				jaswin.frame.currentWindow.setInactive();
				jaswin.frame.currentWindow = null;
			}
		}
		
		var iCounter = 0;
		for(iCounter = 0; iCounter < global.tabstops.length; iCounter++)
		{
			if(global.tabstops[iCounter].enabled)
				global.tabstops[iCounter].element.tabIndex = global.tabstops[iCounter].tabIndex;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	globalMenuEvent
	//
	//	Summary		:	Event handler of menu shortcut
	//
	//==========================================================
	function globalMenuEvent(e)
	{
		var iKeyCode = 0;
		var sChar = "";
		
		if(!global.menu) return;
		
		iKeyCode = e.detail;
		sChar = String.fromCharCode(iKeyCode);
		global.menu.shortCut(sChar);
	}
	
	
	//==========================================================
	//
	//	Function	:	globalContextMenu
	//
	//	Summary		:	Event handler of context menu
	//
	//==========================================================
	function globalContextMenu(e)
	{
		global.showContextMenu(e);
		return false;
	}
	
	
	//==========================================================
	//
	//	Function	:	globalKeyEvent
	//
	//	Summary		:	Event handler of key down event
	//
	//==========================================================
	function globalKeyEvent(e)
	{
		if((jaswin.frame) && (jaswin.frame.currentWindow))
			return true;
		
		if(e.keyCode == 9)
		{
			// Tab
			if(e.shiftKey)
				return global.prevTabstop(e);
			else
				return global.nextTabstop(e);
		}
		
		return true;
	}
	
	
	//==========================================================
	//
	//	Function	:	resizeEvent
	//
	//	Summary		:	Event handler of key down event
	//
	//==========================================================
	function resizeEvent(e)
	{
		global.resize();
	}
	
	
	//==========================================================
	//
	//	Function	:	keyControl
	//
	//	Summary		:	Keyboard  control of html body
	//
	//==========================================================
	function keyControl(e)
	{
		var oTarget = null;
		var oEvent = null;
		
		if(e.altKey)
		{
			e.preventDefault();
			
			if(!jaswin.frame)
				oTarget = global;
			else if(!jaswin.frame.currentWindow)
				oTarget = global;
			else
				oTarget = jaswin.frame.currentWindow;
			
			if(!oTarget.menu)
			{
				if((oTarget != global) && (global.menu))
					oTarget = global;
				else
					return false;
			}
			
			oEvent = document.createEvent("UIEvents");
			oEvent.initUIEvent("menu", false, true, document.defaultView, e.keyCode);
			
			oTarget.element.dispatchEvent(oEvent);
			
			return false;
		}
		else
		{
			if(jaswin.menu)
			{
				var iCnt = jaswin.menu.visibleMenus.length;
				
				if(iCnt > 0)
				{
					// A menu is visible
					e.preventDefault();
					
					oEvent = document.createEvent("UIEvents");
					oEvent.initUIEvent("menuKey", false, true, document.defaultView, e.keyCode);
					jaswin.menu.visibleMenus[iCnt - 1].element.dispatchEvent(oEvent);
					
					return false;
				}
			}
		}
		return true;
	}
	
	
	//==========================================================
	//
	//	Class		:	FixedRect
	//
	//	Summary		:	Fixed position and size
	//
	//==========================================================
	jaswin.FixedRect = function(iAnchorX,		// * Horizontal anchor[int]
								iAnchorY,		// * Vertical anchor[int]
								iPosX,			// * Horizontal position[int]
								iPosY,			// * Vertical position[int]
								iWidth,			// * Width[int]
								iHeight)		// * Height[int]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if((iAnchorX != jaswin.ANCHOR_LEFT)
		&& (iAnchorX != jaswin.ANCHOR_RIGHT))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ANCHOR_X,
										"jaswin.js", "jaswin.FixedRect.constructor");
		
		if((iAnchorY != jaswin.ANCHOR_TOP)
		&& (iAnchorY != jaswin.ANCHOR_BOTTOM))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ANCHOR_Y,
										"jaswin.js", "jaswin.FixedRect.constructor");
		
		if(typeof iPosX != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_X, ""),
										"jaswin.js", "jaswin.FixedRect.constructor");
		
		if(typeof iPosY != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_Y, ""),
										"jaswin.js", "jaswin.FixedRect.constructor");
		
		if(typeof iWidth != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_WIDTH, ""),
										"jaswin.js", "jaswin.FixedRect.constructor");
		if(iWidth < 0)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_WIDTH, ""),
										"jaswin.js", "jaswin.FixedRect.constructor");
		
		if(typeof iHeight != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_HEIGHT, ""),
										"jaswin.js", "jaswin.FixedRect.constructor");
		if(iHeight < 0)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_HEIGHT, ""),
										"jaswin.js", "jaswin.FixedRect.constructor");
		
		// Set properties
		this.anchorX = iAnchorX;
		this.anchorY = iAnchorY;
		this.posX = iPosX;
		this.posY = iPosY;
		this.width = iWidth;
		this.height = iHeight;
	}
	
	
	//==========================================================
	//
	//	Class		:	ExpandableRect
	//
	//	Summary		:	Expandable position and size
	//
	//==========================================================
	jaswin.ExpandableRect = function(iAnchorX,		// * Horizontal anchor[int]
									iAnchorY,		// * Vertical anchor[int]
									iLeft,			// Left position(ANCHOR_BOTH or ANCHOR_LEFT)
									iRight,			// Right position(ANCHOR_BOTH or ANCHOR_RIGHT)
									iTop,			// Top position(ANCHOR_BOTH or ANCHOR_TOP)
									iBottom,		// Bottom position(ANCHOR_BOTH or ANCHOR_BOTTOM)
									iWidth,			// Width(except ANCHOR_BOTH)
									iHeight)		// Height(except ANCHOR_BOTH)
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if((iAnchorX != jaswin.ANCHOR_LEFT)
		&& (iAnchorX != jaswin.ANCHOR_RIGHT)
		&& (iAnchorX != jaswin.ANCHOR_BOTH))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ANCHOR_X,
										"jaswin.js", "jaswin.ExpandableRect.constructor");
		
		if((iAnchorY != jaswin.ANCHOR_TOP)
		&& (iAnchorY != jaswin.ANCHOR_BOTTOM)
		&& (iAnchorY != jaswin.ANCHOR_BOTH))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ANCHOR_Y,
										"jaswin.js", "jaswin.ExpandableRect.constructor");
		
		if((iAnchorX == jaswin.ANCHOR_LEFT) || (iAnchorX == jaswin.ANCHOR_BOTH))
		{
			if(typeof iLeft != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_LEFT,
											"jaswin.js", "jaswin.ExpandableRect.constructor");
		}
		
		if((iAnchorX == jaswin.ANCHOR_RIGHT) || (iAnchorX == jaswin.ANCHOR_BOTH))
		{
			if(typeof iRight != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_RIGHT,
											"jaswin.js", "jaswin.ExpandableRect.constructor");
		}
		
		if((iAnchorY == jaswin.ANCHOR_TOP) || (iAnchorY == jaswin.ANCHOR_BOTH))
		{
			if(typeof iTop != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_TOP,
											"jaswin.js", "jaswin.ExpandableRect.constructor");
		}
		
		if((iAnchorY == jaswin.ANCHOR_BOTTOM) || (iAnchorY == jaswin.ANCHOR_BOTH))
		{
			if(typeof iBottom != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_BOTTOM,
											"jaswin.js", "jaswin.ExpandableRect.constructor");
		}
		
		if(iAnchorX != jaswin.ANCHOR_BOTH)
		{
			if(typeof iWidth != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_WIDTH, ""),
											"jaswin.js", "jaswin.ExpandableRect.constructor");
			if(iWidth <= 0)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_WIDTH, ""),
											"jaswin.js", "jaswin.ExpandableRect.constructor");
		}
		
		if(iAnchorY != jaswin.ANCHOR_BOTH)
		{
			if(typeof iHeight != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_HEIGHT, ""),
											"jaswin.js", "jaswin.ExpandableRect.constructor");
			if(iHeight <= 0)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_HEIGHT, ""),
											"jaswin.js", "jaswin.ExpandableRect.constructor");
		}
		
		// Set properties
		this.anchorX = iAnchorX;
		this.anchorY = iAnchorY;
		this.left = iLeft;
		this.right = iRight;
		this.top = iTop;
		this.bottom = iBottom;
		this.width = iWidth;
		this.height = iHeight;
	}
	
	
	//==========================================================
	//
	//	Function	:	findParentWindow
	//
	//	Summary		:	Find parent window
	//
	//==========================================================
	jaswin.findParentWindow = function(oControl)
	{
		var oCurClass = oControl;
		
		if(!jaswin.frame)
			return global;
		while(true)
		{
			if(!oCurClass.parent)
				return global;
			
			if(oCurClass.parent == global)
				return global;
			
			if(oCurClass.parent instanceof jaswin.frame.Window)
				return oCurClass.parent;
			
			oCurClass = oCurClass.parent;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	formatMessage
	//
	//	Summary		:	Format a message
	//
	//==========================================================
	jaswin.formatMessage = function()
	{
		if((!arguments) || (arguments.length == 0))
			return "";
		
		var sRet = arguments[0];
		var iCounter;
		
		if(!sRet)
			return "";
		
		for(iCounter = 1; iCounter < arguments.length; iCounter++)
			sRet = sRet.replace(new RegExp("\\{" + iCounter + "\\}", "g"), arguments[iCounter]);
		
		sRet = sRet.replace(new RegExp("\\{\\d+\\}", "g"), "");
		
		return sRet;
	}
	
	
	//==========================================================
	//
	//	Function	:	createXML
	//
	//	Summary		:	Create XML DOM object
	//
	//==========================================================
	jaswin.createXML = function(sDocument)	// * Document text[String]
	{
		if(typeof sDocument != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_DOC_TEXT,
										"jaswin.js", "jaswin.createXML");
		
		var oParser = new DOMParser();
		var oXml = null;
		
		try
		{
			oXml = oParser.parseFromString(sDocument, "application/xml");
		}
		catch(e)
		{
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_XML_PARSER,
										"jaswin.js", "jaswin.createXML");
		}
		
		return oXml.documentElement;
	}
	
	
	//==========================================================
	//
	//	Function	:	escapeText
	//
	//	Summary		:	Escape HTML/XML tag characters
	//
	//==========================================================
	jaswin.escapeText = function(sOrg)		// Original text[string]
	{
		var sRet = sOrg.replace(/&/g, "&amp;");
		sRet = sRet.replace(/"/g, "&quot;");
		sRet = sRet.replace(/'/g, "&#039;");
		sRet = sRet.replace(/</g, "&lt;");
		sRet = sRet.replace(/>/g, "&gt;");
		
		return sRet;
	}
	
	
	//==========================================================
	//
	//	Function	:	unescapeText
	//
	//	Summary		:	Unescape HTML/XML tag characters
	//
	//==========================================================
	jaswin.unescapeText = function(sOrg)	// Original text[string]
	{
		var sRet = sOrg.replace(/&amp;/g, "&");
		sRet = sRet.replace(/&quot;/g, "\"");
		sRet = sRet.replace(/&#039;/g, "'");
		sRet = sRet.replace(/&lt;/g, "<");
		sRet = sRet.replace(/&gt;/g, ">");
		
		return sRet;
	}
	
	
	//==========================================================
	//
	//	Function	:	escapeRegExp
	//
	//	Summary		:	Escape letters for regular expression
	//
	//==========================================================
	jaswin.escapeRegExp = function(sOrg)	// Original text[string]
	{
		var sRet = "";
		var iCounter;
		
		for(iCounter = 0; iCounter < sOrg.length; iCounter++)
		{
			if(sOrg.charAt(iCounter) == "\\")
				sRet += "\\\\";
			else if(sOrg.charAt(iCounter) == "*")
				sRet += "\\*";
			else if(sOrg.charAt(iCounter) == "+")
				sRet += "\\+";
			else if(sOrg.charAt(iCounter) == ".")
				sRet += "\\.";
			else if(sOrg.charAt(iCounter) == "?")
				sRet += "\\?";
			else if(sOrg.charAt(iCounter) == "{")
				sRet += "\\{";
			else if(sOrg.charAt(iCounter) == "}")
				sRet += "\\}";
			else if(sOrg.charAt(iCounter) == "(")
				sRet += "\\(";
			else if(sOrg.charAt(iCounter) == ")")
				sRet += "\\)";
			else if(sOrg.charAt(iCounter) == "[")
				sRet += "\\[";
			else if(sOrg.charAt(iCounter) == "]")
				sRet += "\\]";
			else if(sOrg.charAt(iCounter) == "^")
				sRet += "\\^";
			else if(sOrg.charAt(iCounter) == "$")
				sRet += "\\$";
			else if(sOrg.charAt(iCounter) == "|")
				sRet += "\\|";
			else
				sRet += sOrg.charAt(iCounter);
		}
		return sRet;
	}
	
	
	//==========================================================
	//
	//	Function	:	messageBox
	//
	//	Summary		:	Show message box
	//
	//==========================================================
	jaswin.messageBox = function(oWindow,		// Return window on close[Window class]
								sMessage,		// Message text[string]
								sTitle,			// Title text[string]
								iStyle,			// * Style of message box[int]
								fpCallBack)		// Call back function[function]
	{
		// Check parameters
		if(!jaswin.frame)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_INCLUDE, "frame.js"),
										"jaswin.js", "jaswin.messageBox");
		if(!jaswin.button)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_INCLUDE, "button.js"),
										"jaswin.js", "jaswin.messageBox");
		
		var sMsg = "";
		var sCaption = "";
		
		if(oWindow)
		{
			if((typeof oWindow != "object")
			|| (!oWindow instanceof jaswin.frame.Window))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_WINDOW, "WndMessageBox"),
											"jaswin.js", "jaswin.messageBox");
		}
		
		if(sMessage)
		{
			if(typeof sMessage != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MESSAGE, "WndMessageBox"),
											"jaswin.js", "jaswin.messageBox");
			
			sMsg = jaswin.escapeText(sMessage);
			sMsg = sMsg.replace(/\r\n/g, "<br />");
			sMsg = sMsg.replace(/\r/g, "<br />");
			sMsg = sMsg.replace(/\n/g, "<br />");
		}
		
		if(sTitle)
		{
			if(typeof sTitle != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TITLE, "WndMessageBox"),
											"jaswin.js", "jaswin.messageBox");
			
			sCaption = sTitle;
		}
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, "WndMessageBox"),
										"jaswin.js", "jaswin.messageBox");
		
		if(fpCallBack)
		{
			if(typeof fpCallBack != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_CALLBACK, "WND_MESSAGE_BOX"),
											"jaswin.js", "jaswin.messageBox");
		}
		
		// Create message box
		var oRect;
		var oBound;
		var oToolBarBound;
		var iWidth = 160;
		var iHeight = 100;
		var sID = "WndMessageBox";
		var iCnt = 0;
		var iAdjustTop = 0;
		
		if(oWindow)
		{
			oBound = oWindow.element.getBoundingClientRect();
			if(global.menu)
				iAdjustTop -= 22;
			if(global.toolbar)
			{
				oToolBarBound = global.toolbar.element.getBoundingClientRect();
				iAdjustTop -= oToolBarBound.height;
			}
			oRect = new jaswin.ExpandableRect(jaswin.ANCHOR_LEFT, jaswin.ANCHOR_TOP,
												oBound.left + Math.floor((oBound.width - iWidth) / 2), 0,
												oBound.top + Math.floor((oBound.height - iHeight) / 2) + iAdjustTop, 0,
												iWidth, iHeight);
		}
		else
		{
			oBound = global.container.getBoundingClientRect();
			oRect = new jaswin.ExpandableRect(jaswin.ANCHOR_LEFT, jaswin.ANCHOR_TOP,
												Math.floor((oBound.width - iWidth) / 2), 0,
												Math.floor((oBound.height - iHeight) / 2), 0,
												iWidth, iHeight);
		}
		
		while(true)
		{
			if((!document.getElementById(sID)) && (!jaswin.frame.windows[sID]))
				break;
			iCnt++;
			sID = "WndMessageBox" + iCnt;
		}
		
		var oMsgBox = new jaswin.frame.Window(sID, oRect, 0, 0,
												jaswin.frame.STYLE_TITLEBAR |
												jaswin.frame.STYLE_CLOSEBUTTON |
												jaswin.frame.STYLE_DESTROY_ON_CLOSE |
												jaswin.frame.STYLE_MOBABLE,
												sCaption,
												null);
		oMsgBox.returnValue = jaswin.MSGBOX_RETURN_CANCEL;
		
		// Icon
		var oSpan;
		var iMessageLeft = 16;
		
		if((iStyle & jaswin.MSGBOX_INFO)
		|| (iStyle & jaswin.MSGBOX_QUESTION)
		|| (iStyle & jaswin.MSGBOX_ALERT)
		|| (iStyle & jaswin.MSGBOX_ERROR))
		{
			oSpan = document.createElement("span");
			if(iStyle & jaswin.MSGBOX_INFO)
			{
				oSpan.className = jaswin.ICON_INFO;
				oSpan.innerHTML = "i";
			}
			else if(iStyle & jaswin.MSGBOX_QUESTION)
			{
				oSpan.className = jaswin.ICON_QUESTION;
				oSpan.innerHTML = "?";
			}
			else if(iStyle & jaswin.MSGBOX_ALERT)
			{
				oSpan.className = jaswin.ICON_ALERT;
				oSpan.innerHTML = "!";
			}
			else if(iStyle & jaswin.MSGBOX_ERROR)
			{
				oSpan.className = jaswin.ICON_ERROR;
				oSpan.innerHTML = "x";
			}
			oSpan.style.position = "absolute";
			oSpan.style.top = "8px";
			oSpan.style.left = "8px";
			
			oMsgBox.container.appendChild(oSpan);
			
			iMessageLeft = 48;
		}
		
		// Label
		oSpan = document.createElement("span");
		oSpan.className = "label";
		oSpan.style.left = iMessageLeft + "px";
		oSpan.style.top = "8px";
		oSpan.style.marginRight = "8px";
		oSpan.innerHTML = sMsg;
		oMsgBox.container.appendChild(oSpan);
		global.addControl(oMsgBox);
		
		// Button
		var oOkButton;
		var oCancelButton;
		var sButtonOk;
		var sButtonCancel;
		
		if(jaswin.MSGBOX_BUTTON_OK)
			sButtonOk = jaswin.MSGBOX_BUTTON_OK;
		else
			sButtonOk = "OK";
		if(jaswin.MSGBOX_BUTTON_CANCEL)
			sButtonCancel = jaswin.MSGBOX_BUTTON_CANCEL;
		else
			sButtonCancel = "Cancel";
		
		if((iStyle & jaswin.MSGBOX_OKCANCEL)
		|| (iStyle & jaswin.MSGBOX_YESNO))
		{
			if(iStyle & jaswin.MSGBOX_YESNO)
			{
				if(jaswin.MSGBOX_BUTTON_YES)
					sButtonOk = jaswin.MSGBOX_BUTTON_YES;
				else
					sButtonOk = "Yes";
				if(jaswin.MSGBOX_BUTTON_NO)
					sButtonCancel = jaswin.MSGBOX_BUTTON_NO;
				else
					sButtonCancel = "No";
			}
			
			oRect = new jaswin.FixedRect(jaswin.ANCHOR_RIGHT, jaswin.ANCHOR_BOTTOM,
										88, 8, 64, 20);
			oOkButton = new jaswin.button.Button("Btn" + sID + "_Ok", oRect,
												jaswin.button.STYLE_DEFAULT | jaswin.button.STYLE_TEXT,
												true,
												sButtonOk, null,
												1, messageBoxOk);
			oMsgBox.addControl(oOkButton);
			
			oRect = new jaswin.FixedRect(jaswin.ANCHOR_RIGHT, jaswin.ANCHOR_BOTTOM,
										8, 8, 72, 20);
			oCancelButton = new jaswin.button.Button("Btn" + sID + "_Cancel", oRect,
													jaswin.button.STYLE_DEFAULT | jaswin.button.STYLE_TEXT,
													true,
													sButtonCancel, null,
													2, messageBoxCancel);
			oMsgBox.addControl(oCancelButton);
		}
		else
		{
			oRect = new jaswin.FixedRect(jaswin.ANCHOR_RIGHT, jaswin.ANCHOR_BOTTOM,
										8, 8, 64, 20);
			oOkButton = new jaswin.button.Button("Btn" + sID + "_Ok", oRect,
												jaswin.button.STYLE_DEFAULT | jaswin.button.STYLE_TEXT,
												true,
												sButtonOk, null,
												1, messageBoxOk);
			oMsgBox.addControl(oOkButton);
		}
		oMsgBox.showDialog(oWindow, fpCallBack);
		oOkButton.element.focus();
		
		// Resize
		var iCounter;
		var iCurIndex = 0;
		var iLines = 1;
		var iGap = 0;
		
		if(oSpan.offsetWidth > oMsgBox.rect.width - iMessageLeft - 10)
		{
			iGap = oSpan.offsetWidth - (oMsgBox.rect.width - iMessageLeft - 10);
			if(iGap > 200)
			{
				oMsgBox.setSize(oMsgBox.rect.width + 200, oMsgBox.rect.height);
				oSpan.style.width = oSpan.offsetWidth - (iGap - 200) + "px";
				oSpan.style.overflow = "hidden";
			}
			else
				oMsgBox.setSize(oMsgBox.rect.width + iGap, oMsgBox.rect.height);
		}
		
		while(true)
		{
			iCurIndex = sMsg.indexOf("<br />", iCurIndex);
			if(iCurIndex == -1)
				break;
			iCurIndex += 4;
			iLines++;
		}
		
		for(iCounter = 0; iCounter < 6; iCounter++)
		{
			if(oSpan.offsetHeight > iLines * 20)
			{
				oMsgBox.setSize(oMsgBox.rect.width + 40, oMsgBox.rect.height);
				if(oMsgBox.rect.left >= 20)
					oMsgBox.setPosition(oMsgBox.rect.left - 20, oMsgBox.rect.top);
			}
			else
				break;
		}
		
		if(oSpan.offsetHeight > 40)
			oMsgBox.setSize(oMsgBox.rect.width, oMsgBox.rect.height + oSpan.offsetHeight - 40);
	}
	
	
	//==========================================================
	//
	//	Function	:	messageBoxOk
	//
	//	Summary		:	Click OK button of a message box
	//
	//==========================================================
	function messageBoxOk(oButton)
	{
		oButton.parent.returnValue = jaswin.MSGBOX_RETURN_OKYES;
		oButton.parent.close();
	}
	
	
	//==========================================================
	//
	//	Function	:	messageBoxCancel
	//
	//	Summary		:	Click cancel button of a message box
	//
	//==========================================================
	function messageBoxCancel(oButton)
	{
		oButton.parent.close();
	}
	
})(window.jaswin = window.jaswin || {});


//==========================================================
//
//	Function	:	initJaswin
//
//	Summary		:	Initialize jaswin(Call on load html)
//
//==========================================================
function initJaswin()
{
	global = new jaswin.Global();
	
	if(jaswin.BROWSER == "")
	{
		alert("This browser is not supported.");
		return;
	}
	// Cancel default context menu
	document.oncontextmenu = function () {return false;}
	
}
