//==================================================================
//
//	File name	:	label.js
//
//	Summary		:	Label class
//
//	Ceated		:	2015/04/05
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (label) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	label.STYLE_DEFAULT = 0;
	label.STYLE_LINED = 1;
	label.STYLE_INSET = 2;
	label.STYLE_OUTSET = 3;
	label.STYLE_LINK = 4;
	
	// Grobal variables
	label.labels = {};
	
	//==========================================================
	//
	//	Class		:	Label
	//
	//	Summary		:	Class of label
	//
	//==========================================================
	label.Label = function(sID,			// ID[String]
							oRect,		// * Size and position[FixedRect]
							iStyle,		// * Label style[int]
							sText,		// * Content text[string]
							iAlign,		// Text align[int]
							sURL,		// Link URL[string]
							bNewWindow)	// Make new window or not on clicked[boolean]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(sID)
		{
			if(typeof sID != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
											"label.js", "jaswin.label.Label.constructor");
		}
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT),
										"label.js", "jaswin.label.Label.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE),
										"label.js", "jaswin.label.Label.constructor");
		if((iStyle != label.STYLE_DEFAULT)
		&& (iStyle != label.STYLE_LINED)
		&& (iStyle != label.STYLE_INSET)
		&& (iStyle != label.STYLE_OUTSET)
		&& (iStyle != label.STYLE_LINK))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE),
										"label.js", "jaswin.label.Label.constructor");
		
		if(typeof sText != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL),
										"label.js", "jaswin.label.Label.constructor");
		
		if(iAlign)
		{
			if(typeof iAlign != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ALIGN),
										"label.js", "jaswin.label.Label.constructor");
			
			if((iAlign != jaswin.ALIGN_LEFT)
			&& (iAlign != jaswin.ALIGN_RIGHT)
			&& (iAlign != jaswin.ALIGN_CENTER))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_ALIGN),
										"label.js", "jaswin.label.Label.constructor");
		}
		
		if(iStyle == label.STYLE_LINK)
		{
			if(typeof sURL != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_URL),
											"label.js", "jaswin.label.Label.constructor");
			if(bNewWindow)
			{
				if(typeof bNewWindow != "boolean")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_NEW_WINDOW),
												"label.js", "jaswin.label.Label.constructor");
			}
		}
		
		// Set properties
		this.id = sID;							// ID[String]
		this.rect = oRect;						// Size and position[FixedRect]
		this.style = iStyle;					// Label style[int]
		this.align = iAlign;					// Text align[int]
		this.element = null;					// This element[DOM Element]
		this.parent = null;						// Parent class[global / window / panel class]
		
		// Create element
		if(this.style == label.STYLE_LINK)
		{
			this.element = document.createElement("a");
			this.element.className = "label";
			if(bNewWindow)
			{
				this.element.href = "javascript:void(0);";
				this.element.addEventListener("click", function(){window.open(sURL);}, false);
			}
			else
				this.element.href = sURL;
		}
		else
		{
			this.element = document.createElement("span");
			if(this.style == label.STYLE_DEFAULT)
				this.element.className = "label";
			else if(this.style == label.STYLE_LINED)
				this.element.className = "label-lined";
			else if(this.style == label.STYLE_INSET)
				this.element.className = "label-inset";
			else if(this.style == label.STYLE_OUTSET)
				this.element.className = "label-outset";
		}
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		
		if(this.style == label.STYLE_DEFAULT)
		{
			this.element.style.width = this.rect.width + "px";
			this.element.style.height = this.rect.height + "px";
		}
		else
		{
			this.element.style.width = this.rect.width - 2 + "px";
			this.element.style.height = this.rect.height - 2 + "px";
		}
		
		if(!this.align)
			this.align = jaswin.ALIGN_LEFT;
		if(this.align == jaswin.ALIGN_LEFT)
			this.element.style.textAlign = "left";
		else if(this.align == jaswin.ALIGN_RIGHT)
			this.element.style.textAlign = "right";
		else if(this.align == jaswin.ALIGN_CENTER)
			this.element.style.textAlign = "center";
		
		this.element.innerHTML = sText;
	}
	label.Label.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			this.element.parentNode.removeChild(this.element);
			
			if(this.id)
			{
				delete label.labels[this.element.id];
				delete oParentWnd[this.id];
			}
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			
			if(this.id)
			{
				var oParentWnd = jaswin.findParentWindow(this);
				var sID;
				
				if(oParentWnd == global)
					sID = this.id;
				else
					sID = oParentWnd.id + "_" + this.id;
				
				// Duplication check
				if((document.getElementById(sID)) || (label.labels[sID]))
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
												"label.js", "jaswin.label.Label.setParent");
				this.element.id = sID;
				label.labels[this.element.id] = this;
				oParentWnd[this.id] = this;
			}
		},
		
		//==================================================
		//	Show this label
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this label
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Get text
		//==================================================
		getText: function()
		{
			return this.element.textContent;
		},
		
		//==================================================
		//	Set text
		//==================================================
		setText: function(sText)
		{
			if(typeof sText != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL),
											"label.js", "jaswin.label.Label.setText");
			
			this.element.innerHTML = sText;
		},
		
		//==================================================
		//	Set font size
		//==================================================
		setFontSize: function(iFontPoint)
		{
			if(typeof iFontPoint != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_SIZE),
											"label.js", "jaswin.label.Label.setFontSize");
			
			this.element.style.fontSize = iFontPoint + "pt";
		},
		
		//==================================================
		//	Set bold font or not
		//==================================================
		setBold: function(bBold)
		{
			if(typeof bBold != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BOLD),
											"label.js", "jaswin.label.Label.setBold");
			if(bBold)
				this.element.style.fontWeight = "bold";
			else
				this.element.style.fontWeight = "normal";
		},
		
		//==================================================
		//	Set font color
		//==================================================
		setFontColor: function(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_COLOR),
											"label.js", "jaswin.label.Label.setFontColor");
			this.element.style.color = sColor;
		},
		
		//==================================================
		//	Set font family
		//==================================================
		setFontFamily: function(sFontFamily)
		{
			if(typeof sFontFamily != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_FAMILY),
											"label.js", "jaswin.label.Label.setFontFamily");
			this.element.style.fontFamily = sFontFamily;
		},
		
		//==================================================
		//	Set background color
		//==================================================
		setBackgroundColor: function(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BACKGROUND),
											"label.js", "jaswin.label.Label.setBackgroundColor");
			this.element.style.backgroundColor = sColor;
		},
		
		//==================================================
		//	Set border color
		//==================================================
		setBorderColor: function(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BORDERCOLOR),
											"label.js", "jaswin.label.Label.setBorderColor");
			this.element.style.borderColor = sColor;
		}
	}
	
})(jaswin.label = jaswin.label || {});
