//==================================================================
//
//	File name	:	list.js
//
//	Summary		:	ListBox ComboBox classes
//
//	Ceated		:	2014/09/27
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (list) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	list.STYLE_DEFAULT = 0;
	list.STYLE_CHECKBOX = 1;
	list.STYLE_POPUP = 2;
	list.STYLE_EDITABLE = 2;
	list.SORT_BY_TEXT = 0;
	list.SORT_BY_VALUE = 1;
	list.SORT_ASC = 0;
	list.SORT_DESC = 1;
	
	// Grobal variables
	list.listBoxes = {};
	list.comboBoxes = {};
	
	// Private variables
	var oPopUp = null;
	var oDropDown = null;
	var oScrolling = null;
	var bDropDownKeep = false;
	var oCurrentMouseDown = null;
	
	
	//==========================================================
	//
	//	Class		:	ListItem
	//
	//	Summary		:	Class of list item
	//
	//==========================================================
	list.ListItem = function()
	{
		this.value = null;
		this.text = null;
		this.selected = false;
		this.element = null;
		this.box = null;
		this.label = null;
	}
	
	
	//==========================================================
	//
	//	Class		:	ListBox
	//
	//	Summary		:	Class of listbox
	//
	//==========================================================
	list.ListBox = function(sID,			// * ID[String]
							oRect,			// * Size and position[FixedRect]
							iStyle,			// * Listbox style[int]
							bEnabled,		// * Enabled or not[boolean]
							sTooltip,		// Tool tip text[String]
							iTabIndex,		// Tab index[int]
							iAlign,			// Text align[int]
							fpChange)		// Function on change[function](ListItem)
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"list.js", "jaswin.list.ListBox.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"list.js", "jaswin.list.ListBox.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"list.js", "jaswin.list.ListBox.constructor");
		
		if((iStyle != list.STYLE_DEFAULT)
		&& (iStyle != list.STYLE_CHECKBOX)
		&& (iStyle != list.STYLE_POPUP))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sID),
										"list.js", "jaswin.list.ListBox.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"list.js", "jaswin.list.ListBox.constructor");
		
		if(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
											"list.js", "jaswin.list.ListBox.constructor");
		}
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"list.js", "jaswin.list.ListBox.constructor");
		}
		
		if(iAlign)
		{
			if(typeof iAlign != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ALIGN, sID),
											"list.js", "jaswin.list.ListBox.constructor");
			if((iAlign != jaswin.ALIGN_LEFT)
			&& (iAlign != jaswin.ALIGN_RIGHT)
			&& (iAlign != jaswin.ALIGN_CENTER))
				throw new jaswin.JaswinError(jaswin.ALERT_VALUE_ALIGN,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ALIGN, sID),
											"list.js", "jaswin.list.ListBox.constructor");
		}
		
		if(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
											"list.js", "jaswin.list.ListBox.constructor");
		}
		
		// Set properties
		this.id = sID;						// ID[String]
		this.rect = oRect;					// Size and position[FixedRect]
		this.style = iStyle;				// ListBox style[int]
		this.enabled = bEnabled;			// Enabled or not[boolean]
		this.parentEnabled = true;			// Enabled or not[boolean]
		this.onChange = fpChange;			// Function on change[function](ListItem)
		this.element = null;				// This element[DOM Element]
		this.focusElement = null;			// Focus element[DOM Element]
		this.tabIndex = -1;					// Tab index[int]
		this.parent = null;					// Parent class[global / window / panel class]
		this.items = new Array();			// List items[ListItem]
		this.selectedIndex = -1;			// Selected index[int]
		this.selectedIndexes = new Array();	// Selected indexes[Array of int]
		this.currentIndex = -1;				// Current index of item on focus[int]
		if(iAlign)							// Text align of label[int]
			this.align = iAlign;
		else
			this.align = jaswin.ALIGN_LEFT;
		
		// Create element
		this.element = document.createElement("ul");
		if(this.style == list.STYLE_POPUP)
			this.element.className = "listbox-popup";
		else
			this.element.className = "listbox";
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		
		this.element.style.width = this.rect.width - 2 + "px";		// 2:border
		this.element.style.height = this.rect.height - 2 + "px";	// 2:border
		this.focusElement = this.element;
		
		// Tooltip
		if((sTooltip) && (sTooltip.length > 0))
			this.element.title = sTooltip;
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Event binding
		if(this.enabled)
		{
			this.element.addEventListener("keydown", listBoxKeyEvent, false);
			this.element.addEventListener("blur", listBoxBlurEvent, false);
			
			if(this.style == list.STYLE_POPUP)
			{
				oPopUp = this;
				global.element.addEventListener("click", listBoxCancel, false);
			}
		}
	}
	list.ListBox.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			delete list.listBoxes[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
			{
				sID = oParentWnd.id + "_" + this.id;
				if(this.style == list.STYLE_POPUP)
					this.element.style.zIndex = 50 + jaswin.frame.windowCount + oParentWnd.modalLevel;
			}
			
			// Duplication check
			if((document.getElementById(sID)) || (list.listBoxes[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"list.js", "jaswin.list.ListBox.setParent");
			this.element.id = sID;
			list.listBoxes[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this listbox
		//==================================================
		enable: function(bParent)
		{
			var iCounter;
			
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.element.tabIndex = this.tabIndex;
				
				this.element.addEventListener("keydown", listBoxKeyEvent, false);
				this.element.addEventListener("blur", listBoxBlurEvent, false);
				for(iCounter = 0; iCounter < this.items.length; iCounter++)
				{
					if(this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item-selected";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX_CHECKED;
					}
					else
						this.items[iCounter].element.className = "list-item";
					
					if(this.style == list.STYLE_CHECKBOX)
						this.items[iCounter].box.style.cursor = "pointer";
					this.items[iCounter].label.className = "list-item-label";
					
					this.items[iCounter].element.addEventListener("mousemove", listBoxMove, false);
					this.items[iCounter].element.addEventListener("mouseout", listBoxOut, false);
					this.items[iCounter].element.addEventListener("mousedown", listBoxMouseDown, false);
				}
				
				if((this.style == list.STYLE_POPUP) && (this.element.style.display != "none"))
				{
					oPopUp = this;
					global.element.addEventListener("click", listBoxCancel, false);
				}
			}
		},
		
		//==================================================
		//	Disable this listbox
		//==================================================
		disable: function(bParent)
		{
			var iCounter;
			
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.element.tabIndex = -1;
			
			this.element.removeEventListener("keydown", listBoxKeyEvent, false);
			this.element.removeEventListener("blur", listBoxBlurEvent, false);
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].selected)
				{
					this.items[iCounter].element.className = "list-item-selected-disabled";
					if(this.style == list.STYLE_CHECKBOX)
						this.items[iCounter].box.className = jaswin.ICON_CHECKBOX_CHECKED_DISABLED;
				}
				else
					this.items[iCounter].element.className = "list-item-disabled";
				
				if(this.style == list.STYLE_CHECKBOX)
					this.items[iCounter].box.style.cursor = "default";
				this.items[iCounter].label.className = "list-item-label-disabled";
				
				this.items[iCounter].element.removeEventListener("mousemove", listBoxMove, false);
				this.items[iCounter].element.removeEventListener("mouseout", listBoxOut, false);
				this.items[iCounter].element.removeEventListener("mousedown", listBoxMouseDown, false);
			}
			
			if((this.style == list.STYLE_POPUP) && (this.element.style.display != "none"))
			{
				oPopUp = null;
				global.element.removeEventListener("click", listBoxCancel, false);
			}
		},
		
		//==================================================
		//	Show this listbox
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
			if((this.style == list.STYLE_POPUP) && (this.enabled))
			{
				oPopUp = this;
				global.element.addEventListener("click", listBoxCancel, false);
			}
		},
		
		//==================================================
		//	Hide this listbox
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
			if((this.style == list.STYLE_POPUP) && (this.enabled))
			{
				oPopUp = null;
				global.element.removeEventListener("click", listBoxCancel, false);
			}
		},
		
		//==================================================
		//	Add item to this listbox
		//==================================================
		addItem: function(sText, sVal, bSelected)
		{
			if(typeof sText != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"list.js", "jaswin.list.ListBox.addItem");
			
			if(typeof sVal != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"list.js", "jaswin.list.ListBox.addItem");
			
			if(typeof bSelected != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_SELECTED, this.id),
											"list.js", "jaswin.list.ListBox.addItem");
			
			var oItem = new list.ListItem();
			var iCounter;
			var iWidth;
			
			// Adjust list width and height
			iWidth = this.rect.width - 2 - (1 * 2) - (1 * 2);	// 2:padding 1*2:border(frame) 1*2:border(item)
			if(this.style == list.STYLE_CHECKBOX)
				iWidth -= 17;									// 17:checkbox
			
			if(this.style == list.STYLE_POPUP)
			{
				// Height when popup
				if((this.items.length + 1) * 20 < this.rect.height)
					this.element.style.height = (this.items.length + 1) * 20 + "px";
				else
					this.element.style.height = this.rect.height + "px";
			}
			
			if(((this.items.length + 1) * 20) > this.rect.height - 2)
			{
				// Width of label
				iWidth -= 17;									// 17:scrollbar
				for(iCounter = 0; iCounter < this.items.length; iCounter++)
					this.items[iCounter].label.style.width = iWidth + "px";
			}
			
			// Make a new item
			oItem.text = sText;
			oItem.value = sVal;
			oItem.element = document.createElement("li");
			if((this.enabled) && (this.parentEnabled))
				oItem.element.className = "list-item";
			else
				oItem.element.className = "list-item-disabled";
			
			if(this.style == list.STYLE_CHECKBOX)
			{
				// Checkbox
				oItem.box = document.createElement("span");
				oItem.box.className = jaswin.ICON_CHECKBOX;
				oItem.box.style.left = "2px";
				oItem.box.style.top = "3px";
				oItem.element.appendChild(oItem.box);
				if((!this.enabled) || (!this.parentEnabled))
					oItem.box.style.cursor = "default";
			}
			
			// Label
			oItem.label = document.createElement("span");
			if((this.enabled) && (this.parentEnabled))
				oItem.label.className = "list-item-label";
			else
				oItem.label.className = "list-item-label-disabled";
			
			if(this.align == jaswin.ALIGN_RIGHT)
				oItem.label.style.textAlign = "right";
			else if(this.align == jaswin.ALIGN_CENTER)
				oItem.label.style.textAlign = "center";
			
			if(this.style == list.STYLE_CHECKBOX)
				oItem.label.style.left = "18px";
			else
				oItem.label.style.left = "2px";
			oItem.label.style.width = iWidth + "px";
			oItem.label.innerHTML = oItem.text;
			oItem.element.appendChild(oItem.label);
			
			if((this.enabled) && (this.parentEnabled))
			{
				oItem.element.addEventListener("mousemove", listBoxMove, false);
				oItem.element.addEventListener("mouseout", listBoxOut, false);
				oItem.element.addEventListener("mousedown", listBoxMouseDown, false);
			}
			
			this.element.appendChild(oItem.element);
			this.items[this.items.length] = oItem;
			
			if(bSelected)
				this.selectList(this.items.length - 1, false);
		},
		
		//==================================================
		//	Remove item from this listbox
		//==================================================
		removeItem: function(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"list.js", "jaswin.list.ListBox.removeItem");
			if((iIndex < 0) || (iIndex >= this.items.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"list.js", "jaswin.list.ListBox.removeItem");
			// Remove an item
			this.element.removeChild(this.items[iIndex].element);
			this.items.splice(iIndex, 1);
			
			// Adjust list width and height
			iWidth = this.rect.width - 2 - (1 * 2) - (1 * 2);	// 2:padding 1*2:border(frame) 1*2:border(item)
			if(this.style == list.STYLE_CHECKBOX)
				iWidth -= 17;									// 17:checkbox
			
			if((this.items.length * 20) > this.rect.height - 2)
				iWidth -= 17;									// 17:scrollbar
			
			if(this.style == list.STYLE_POPUP)
			{
				// Height when popup
				if((this.items.length - 1) * 20 < this.rect.height)
					this.element.style.height = (this.items.length - 1) * 20 + "px";
				else
					this.element.style.height = this.rect.height + "px";
			}
			
			// Width of label and selection
			this.selectedIndexes = new Array();
			this.selectedIndex = -1;
			this.currentIndex = -1;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				this.items[iCounter].label.style.width = iWidth + "px";
				if(this.items[iCounter].selected)
				{
					if(this.style == list.STYLE_CHECKBOX)
						this.selectedIndexes[this.selectedIndexes.length] = iCounter;
					else
						this.selectedIndexes[0] = iCounter;
					this.selectedIndex = iCounter;
				}
			}
		},
		
		//==================================================
		//	Set list item of this listbox hover
		//==================================================
		setHover: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].element.contains(oElement))
				{
					this.currentIndex = iCounter;
					if(!this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item-hover";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX_HOVER;
					}
					break;
				}
			}
		},
		
		//==================================================
		//	Set list item of this listbox mouse out
		//==================================================
		setOut: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].element.contains(oElement))
				{
					this.currentIndex = -1;
					if(!this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX;
					}
					break;
				}
			}
		},
		
		//==================================================
		//	Set list item of this listbox focused
		//==================================================
		setFocusItem: function(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"list.js", "jaswin.list.ListBox.setFocusItem");
			
			var iCounter = 0;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(iCounter == iIndex)
				{
					this.currentIndex = iCounter;
					if(!this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item-hover";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX_HOVER;
					}
				}
				else
				{
					if(!this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX;
					}
				}
			}
			
			if(iIndex != -1)
			{
				if(iIndex * 20 < this.element.scrollTop)
					this.element.scrollTop = iIndex * 20;
				else if((iIndex + 1) * 20 > (this.rect.height - 2) + this.element.scrollTop)
					this.element.scrollTop = (iIndex + 1) * 20 - ((this.rect.height - 2) + this.element.scrollTop);
			}
		},
		
		//==================================================
		//	Click list item of this listbox
		//==================================================
		clickList: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].element.contains(oElement))
				{
					this.selectList(iCounter, true);
					break;
				}
			}
		},
		
		//==================================================
		//	Select list item of this listbox
		//==================================================
		selectList: function(iIndex, bExecFunc)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"list.js", "jaswin.list.ListBox.selectList");
			if((iIndex < 0) || (iIndex >= this.items.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"list.js", "jaswin.list.ListBox.selectList");
			
			var bExec = true;
			
			if((typeof bExecFunc != "undefined") && (bExecFunc != null))
			{
				if(typeof bExecFunc != "boolean")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_EXEC_FUNC, this.id),
												"list.js", "jaswin.list.ListBox.selectList");
				bExec = bExecFunc;
			}
			
			if((this.style != list.STYLE_CHECKBOX)
			&& (iIndex == this.selectedIndex))
				return;
			
			var iCounter;
			
			this.selectedIndexes = new Array();
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.style == list.STYLE_CHECKBOX)
				{
					if(iCounter == iIndex)
					{
						if(this.items[iIndex].selected)
						{
							if((this.enabled) && (this.parentEnabled))
							{
								this.items[iIndex].element.className = "list-item-hover";
								this.items[iIndex].box.className = jaswin.ICON_CHECKBOX_HOVER;
							}
							else
							{
								this.items[iIndex].element.className = "list-item-disabled";
								this.items[iIndex].box.className = jaswin.ICON_CHECKBOX;
							}
							this.items[iIndex].selected = false;
						}
						else
						{
							if((this.enabled) && (this.parentEnabled))
							{
								this.items[iIndex].element.className = "list-item-selected";
								this.items[iIndex].box.className = jaswin.ICON_CHECKBOX_CHECKED;
							}
							else
							{
								this.items[iIndex].element.className = "list-item-selected-disabled";
								this.items[iIndex].box.className = jaswin.ICON_CHECKBOX_CHECKED_DISABLED;
							}
							this.items[iIndex].selected = true;
							this.selectedIndexes[this.selectedIndexes.length] = iIndex;
						}
					}
					else if(this.items[iCounter].selected)
						this.selectedIndexes[this.selectedIndexes.length] = iCounter;
				}
				else
				{
					if(iCounter == iIndex)
					{
						if((this.enabled) && (this.parentEnabled))
							this.items[iIndex].element.className = "list-item-selected";
						else
							this.items[iIndex].element.className = "list-item-selected-disabled";
						this.items[iIndex].selected = true;
						this.selectedIndex = iIndex;
					}
					else
					{
						if((this.enabled) && (this.parentEnabled))
							this.items[iCounter].element.className = "list-item";
						else
							this.items[iCounter].element.className = "list-item-disabled";
						this.items[iCounter].selected = false;
					}
				}
			}
			
			if(this.style != list.STYLE_CHECKBOX)
			{
				this.selectedIndex = iIndex;
				this.selectedIndexes[0] = iIndex;
			}
			
			if((this.onChange) && (this.enabled) && (this.parentEnabled)
			&& (bExec) && (iIndex != -1))
				this.onChange(this.items[iIndex]);
		},
		
		//==================================================
		//	Key down on this listbox
		//==================================================
		keyDownList: function(e)
		{
			if((e.keyCode == 13) || (e.keyCode == 32))
			{
				if(this.currentIndex != -1)
				{
					this.selectList(this.currentIndex, true);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
			else if(e.keyCode == 40)
			{
				// down
				if(this.currentIndex < this.items.length - 1)
					this.setFocusItem(this.currentIndex + 1);
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			else if(e.keyCode == 38)
			{
				// up
				if(this.currentIndex > 0)
					this.setFocusItem(this.currentIndex - 1);
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			return true;
		},
		
		//==================================================
		//	Blur from this listbox
		//==================================================
		blur: function()
		{
			this.currentIndex = -1;
			this.setFocusItem(-1);
		},
		
		//==================================================
		//	Clear list of this listbox
		//==================================================
		clear: function()
		{
			this.selectedIndex = -1;
			this.selectedIndexes = new Array();
			this.items = new Array();
			this.element.innerHTML = "";
		},
		
		//==================================================
		//	Get selected items
		//==================================================
		getSelectedItems: function()
		{
			var oaItems;
			var iCounter;
			
			oaItems = new Array();
			for(iCounter = 0; iCounter < this.selectedIndexes.length; iCounter++)
			{
				oaItems[oaItems.length] = this.items[this.selectedIndexes[iCounter]];
			}
			
			return oaItems;
		},
		
		//==================================================
		//	Get text by value
		//==================================================
		getTextByValue: function(sValue)
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].value == sValue)
					return this.items[iCounter].text;
			}
			
			return null;
		},
		
		//==================================================
		//	Contain item or not
		//==================================================
		contains: function(oItem)
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter] == oItem)
					return true;
			}
			
			return false;
		},
		
		//==================================================
		//	Sort
		//==================================================
		sort: function(iSortBy, iAscDesc)
		{
			if(typeof iSortBy != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_SORT_BY, this.id),
											"list.js", "jaswin.list.ListBox.sort");
			if(typeof iAscDesc != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_SORT_TYPE, this.id),
											"list.js", "jaswin.list.ListBox.sort");
			if(!this.items.length)
				return;
			
			// Execute sort
			if(iSortBy == 0)
			{
				if(iAscDesc == 0)
				{
					this.items.sort(function(oItemA, oItemB){
										if(oItemA.text < oItemB.text) return -1;
										if(oItemA.text > oItemB.text) return 1;
										return 0;
									});
				}
				else
				{
					this.items.sort(function(oItemA, oItemB){
										if(oItemA.text > oItemB.text) return -1;
										if(oItemA.text < oItemB.text) return 1;
										return 0;
									});
				}
			}
			else
			{
				if(iAscDesc == 0)
				{
					this.items.sort(function(oItemA, oItemB){
										if(oItemA.value < oItemB.value) return -1;
										if(oItemA.value > oItemB.value) return 1;
										return 0;
									});
				}
				else
				{
					this.items.sort(function(oItemA, oItemB){
										if(oItemA.value > oItemB.value) return -1;
										if(oItemA.value < oItemB.value) return 1;
										return 0;
									});
				}
			}
			
			// Replace elements
			var iCounter;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
				this.items[iCounter].element.parentNode.removeChild(this.items[iCounter].element);
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
				this.element.appendChild(this.items[iCounter].element);
		},
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip: function(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"list.js", "jaswin.list.ListBox.setTooltip");
			this.element.title = sTooltip;
		},
		
		//==================================================
		//	Set change event
		//==================================================
		setChangeEvent: function(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, this.id),
											"list.js", "jaswin.list.ListBox.setChangeEvent");
			
			this.onChange = fpChange;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetListBox
	//
	//	Summary		:	Get target listbox element from event
	//
	//==========================================================
	function getTargetListBox(e)
	{
		var oElement = e.target || e.srcElement;
		var oListBox = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "ul")
			|| (oElement.tagName == "UL"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oListBox = list.listBoxes[oElement.id];
				if(!oListBox)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oListBox;
	}
	
	
	//==========================================================
	//
	//	Function	:	listBoxMove
	//
	//	Summary		:	Event handler of mouse move a listbox
	//
	//==========================================================
	function listBoxMove(e)
	{
		var oListBox = null;
		
		oListBox = getTargetListBox(e);
		
		if(oListBox)
			oListBox.setHover(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	listBoxOut
	//
	//	Summary		:	Event handler of mouse out of a listbox
	//
	//==========================================================
	function listBoxOut(e)
	{
		var oListBox = null;
		
		oListBox = getTargetListBox(e);
		
		if(oListBox)
			oListBox.setOut(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	listBoxMouseDown
	//
	//	Summary		:	Event handler of mouse down of a listbox
	//
	//==========================================================
	function listBoxMouseDown(e)
	{
		var oListBox = null;
		
		oListBox = getTargetListBox(e);
		
		if(oListBox)
			oListBox.clickList(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	listBoxKeyEvent
	//
	//	Summary		:	Event handler of key down on a listbox
	//
	//==========================================================
	function listBoxKeyEvent(e)
	{
		var oListBox = null;
		
		oListBox = getTargetListBox(e);
		
		if(oListBox)
			return oListBox.keyDownList(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	listBoxBlurEvent
	//
	//	Summary		:	Event handler of blur from a listbox
	//
	//==========================================================
	function listBoxBlurEvent(e)
	{
		var oListBox = null;
		
		oListBox = getTargetListBox(e);
		
		if(oListBox)
			return oListBox.blur(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	listBoxCancel
	//
	//	Summary		:	Event handler of cancel a listbox
	//
	//==========================================================
	function listBoxCancel(e)
	{
		if(!oPopUp)
			return;
		
		oPopUp.hide();
		return;
	}
	
	
	//==========================================================
	//
	//	Class		:	ComboBox
	//
	//	Summary		:	Class of combobox
	//
	//==========================================================
	list.ComboBox = function(sID,			// * ID[String]
							oRect,			// * Size and position[FixedRect]
							iStyle,			// * Combobox style[int]
							bEnabled,		// * Enabled or not[boolean]
							iMaxDropdown,	// Number of drop down items[int]
							sTooltip,		// Tool tip text[String]
							iTabIndex,		// Tab index[int]
							iAlign,			// Text align[int]
							fpChange)		// Function on change[function](ListItem)
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"list.js", "jaswin.list.ComboBox.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"list.js", "jaswin.list.ComboBox.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"list.js", "jaswin.list.ComboBox.constructor");
		
		if((iStyle != list.STYLE_DEFAULT)
		&& (iStyle != list.STYLE_CHECKBOX)
		&& (iStyle != list.STYLE_EDITABLE))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sID),
										"list.js", "jaswin.list.ComboBox.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"list.js", "jaswin.list.ComboBox.constructor");
		
		if(iMaxDropdown)
		{
			if(typeof iMaxDropdown != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MAX_DROP, sID),
											"list.js", "jaswin.list.ComboBox.constructor");
		}
		
		if(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
											"list.js", "jaswin.list.ComboBox.constructor");
		}
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"list.js", "jaswin.list.ComboBox.constructor");
		}
		
		if(iAlign)
		{
			if(typeof iAlign != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ALIGN, sID),
											"list.js", "jaswin.list.ComboBox.constructor");
			if((iAlign != jaswin.ALIGN_LEFT)
			&& (iAlign != jaswin.ALIGN_RIGHT)
			&& (iAlign != jaswin.ALIGN_CENTER))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_ALIGN, sID),
											"list.js", "jaswin.list.ComboBox.constructor");
		}
		
		if(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
											"list.js", "jaswin.list.ComboBox.constructor");
		}
		
		// Set properties
		this.id = sID;							// ID[String]
		this.rect = oRect;						// Size and position[FixedRect]
		this.style = iStyle;					// Combobox style[int]
		if(iMaxDropdown)
			this.maxDropdown = iMaxDropdown;	// Number of drop down items[int]
		else
			this.maxDropdown = 8;
		this.enabled = bEnabled;				// Enabled or not[boolean]
		this.parentEnabled = true;				// Enabled or not[boolean]
		this.onChange = fpChange;				// Function on change[function](ListItem)
		this.element = null;					// This element[DOM Element]
		this.focusElement = null;				// Focus element[DOM Element]
		this.box = null;						// Box element[DOM Element]
		this.text = null;						// Textbox or label element[DOM Element]
		this.arrow = null;						// Arrow button element[DOM Element]
		this.list = null;						// Content element for list[DOM Element]
		this.tabIndex = -1;						// Tab index[int]
		this.parent = null;						// Parent class[global / window / panel class]
		this.items = new Array();				// List items[Array of ListItem]
		this.dropDown = false;					// Drop down or not[boolean]
		this.selectedIndex = -1;				// Selected index[int]
		this.selectedIndexes = new Array();		// Selected indexes[Array of int]
		this.currentIndex = -1;					// Current index of item on focus[int]
		if(iAlign)								// Text align of label[int]
			this.align = iAlign;
		else
			this.align = jaswin.ALIGN_LEFT;
		
		// Create element
		this.element = document.createElement("div");
		this.element.style.position = "absolute";
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		this.element.style.width = this.rect.width + "px";
		this.element.style.height = "20px";
		
		// Box part
		this.box = document.createElement("div");
		this.box.className = "combobox";
		
		if(this.style == list.STYLE_EDITABLE)
		{
			// Editable
			this.text = document.createElement("input");
			this.text.type = "text";
			this.text.className = "combobox-text";
			this.text.style.width = this.rect.width - 2 - 3 - 1 - 17 + "px";	// 2:border 3:left 1:arrow border 17:arrow
			
			if(this.enabled)
				this.text.className = "combobox-text";
			else
				this.text.disabled = true;
			
			if(this.align == jaswin.ALIGN_RIGHT)
				this.text.style.textAlign = "right";
			else if(this.align == jaswin.ALIGN_CENTER)
				this.text.style.textAlign = "center";
			
			this.box.appendChild(this.text);
		}
		else
		{
			this.text = document.createElement("span");
			this.text.style.width = this.rect.width - 2 - 3 - 17 + "px";	// 2:border 3:left 17:arrow
			if(this.enabled)
				this.text.className = "combobox-label";
			else
				this.text.className = "combobox-label-disabled";
			
			if(this.align == jaswin.ALIGN_RIGHT)
				this.text.style.textAlign = "right";
			else if(this.align == jaswin.ALIGN_CENTER)
				this.text.style.textAlign = "center";
			
			this.box.appendChild(this.text);
		}
		
		var oTriangle;
		this.arrow = document.createElement("div");
		oTriangle = document.createElement("span");
		this.arrow.appendChild(oTriangle);
		if(this.enabled)
		{
			this.arrow.className = "combobox-arrow";
			oTriangle.className = jaswin.ICON_TRIANGLE_BOTTOM;
		}
		else
		{
			this.arrow.className = "combobox-arrow-disabled";
			oTriangle.className = jaswin.ICON_TRIANGLE_BOTTOM_DISABLED;
		}
		oTriangle.style.left = "4px";
		oTriangle.style.top = "6px";
		
		this.box.appendChild(this.arrow);
		this.element.appendChild(this.box);
		
		// List part
		this.list = document.createElement("ul");
		this.list.className = "combobox-list";
		this.list.style.width = this.rect.width - 2 + "px";				// 2:border
		this.list.style.height = "0px";
		this.element.appendChild(this.list);
		
		// Tooltip
		if((sTooltip) && (sTooltip.length > 0))
			this.element.title = sTooltip;
		
		// Tab index
		if(this.style == list.STYLE_EDITABLE)
			this.focusElement = this.text;
		else
			this.focusElement = this.element;
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Event binding
		if(this.enabled)
		{
			if(this.style == list.STYLE_EDITABLE)
			{
				this.text.addEventListener("focus", comboTextFocus, false);
				this.text.addEventListener("blur", comboTextBlur, false);
				this.text.addEventListener("input", comboTextInput, false);
			}
			else
			{
				this.element.addEventListener("focus", comboBoxFocus, false);
				this.element.addEventListener("blur", comboBoxBlur, false);
				this.text.addEventListener("click", comboBoxClick, false);
			}
			
			this.arrow.addEventListener("click", comboBoxClick, false);
			this.element.addEventListener("mousemove", comboBoxMove);
			this.element.addEventListener("mouseout", comboBoxOut);
			this.element.addEventListener("keydown", comboBoxKeyDown, false);
			this.list.addEventListener("mousedown", comboBoxMouseDown, false);
			this.list.addEventListener("mouseup", comboBoxMouseUp, false);
		}
	}
	list.ComboBox.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			delete list.comboBoxes[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (list.comboBoxes[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"list.js", "jaswin.list.ComboBox.setParent");
			this.element.id = sID;
			list.comboBoxes[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this combobox
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.focusElement.tabIndex = this.tabIndex;
				
				this.arrow.className = "combobox-arrow";
				this.arrow.addEventListener("click", comboBoxClick, false);
				this.arrow.childNodes[0].className = jaswin.ICON_TRIANGLE_BOTTOM;
				
				if(this.style == list.STYLE_EDITABLE)
				{
					this.text.className = "combobox-text";
					this.text.disabled = false;
					this.text.addEventListener("focus", comboTextFocus, false);
					this.text.addEventListener("blur", comboTextBlur, false);
					this.text.addEventListener("input", comboTextInput, false);
				}
				else
				{
					this.text.className = "combobox-label";
					this.text.addEventListener("click", comboBoxClick, false);
					this.element.addEventListener("focus", comboBoxFocus, false);
					this.element.addEventListener("blur", comboBoxBlur, false);
				}
				this.element.addEventListener("mousemove", comboBoxMove, false);
				this.element.addEventListener("mouseout", comboBoxOut, false);
				this.element.addEventListener("keydown", comboBoxKeyDown, false);
				this.list.addEventListener("mousedown", comboBoxMouseDown, false);
				this.list.addEventListener("mouseup", comboBoxMouseUp, false);
			}
		},
		
		//==================================================
		//	Disable this combobox
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.element.tabIndex = -1;
			if(this.style == list.STYLE_EDITABLE)
				this.text.tabIndex = -1;
			
			this.list.style.display = "none";
			this.dropDown = false;
			this.box.className = "combobox";
			oDropDown = null;
			global.element.removeEventListener("click", comboBoxCancel, false);
			
			this.arrow.className = "combobox-arrow-disabled";
			this.arrow.removeEventListener("click", comboBoxClick, false);
			this.arrow.childNodes[0].className = jaswin.ICON_TRIANGLE_BOTTOM_DISABLED;
			
			if(this.style == list.STYLE_EDITABLE)
			{
				this.text.disabled = true;
				this.text.removeEventListener("focus", comboTextFocus, false);
				this.text.removeEventListener("blur", comboTextBlur, false);
				this.text.removeEventListener("input", comboTextInput, false);
			}
			else
			{
				this.text.className = "combobox-label-disabled";
				this.text.removeEventListener("click", comboBoxClick, false);
				this.element.removeEventListener("focus", comboBoxFocus, false);
				this.element.removeEventListener("blur", comboBoxBlur, false);
			}
			this.element.removeEventListener("mousemove", comboBoxMove, false);
			this.element.removeEventListener("mouseout", comboBoxOut, false);
			this.element.removeEventListener("keydown", comboBoxKeyDown, false);
			this.list.removeEventListener("mousedown", comboBoxMouseDown, false);
			this.list.removeEventListener("mouseup", comboBoxMouseUp, false);
		},
		
		//==================================================
		//	Show this combobox
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this combobox
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Set this combobox hover
		//==================================================
		setHover: function()
		{
			this.box.className = "combobox-focus";
			this.arrow.className = "combobox-arrow-focus";
		},
		
		//==================================================
		//	Set this combobox mouse out
		//==================================================
		setOut: function()
		{
			if(this.style == list.STYLE_EDITABLE)
			{
				if(document.activeElement == this.text)
					return;
			}
			
			if(!this.dropDown)
			{
				this.box.className = "combobox";
				this.arrow.className = "combobox-arrow";
			}
		},
		
		//==================================================
		//	Blur from this combobox
		//==================================================
		blur: function()
		{
			if(this.dropDown)
			{
				this.list.style.display = "none";
				this.dropDown = false;
				oDropDown = null;
				global.element.removeEventListener("click", comboBoxCancel, false);
			}
			this.box.className = "combobox";
			this.arrow.className = "combobox-arrow";
			
			this.currentIndex = -1;
			this.setFocusItem(-1);
		},
		
		//==================================================
		//	Click this combobox
		//==================================================
		click: function()
		{
			if(this.dropDown)
			{
				// Dropped down to hidden
				this.list.style.display = "none";
				this.dropDown = false;
				oDropDown = null;
				global.element.removeEventListener("click", comboBoxCancel, false);
				
				this.box.className = "combobox";
				this.arrow.className = "combobox-arrow";
				this.currentIndex = -1;
				
				var iCounter;
				
				for(iCounter = 0; iCounter < this.items.length; iCounter++)
				{
					if(!this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX;
					}
				}
				this.list.scrollTop = 0;
			}
			else
			{
				// Hidden to dropped down
				var oWindow = jaswin.findParentWindow(this);
				var iListHeight;
				var bBellow = true;
				var oThisRect;
				var oParentRect;
				
				// Which show list bellow or above?
				if(this.items.length >= this.maxDropdown)
					iListHeight = 20 * this.maxDropdown;
				else
					iListHeight = 20 * this.items.length;
				oThisRect = this.element.getBoundingClientRect();
				if(oWindow == global)
				{
					oParentRect = global.container.getBoundingClientRect();
					if(oThisRect.top - oParentRect.top < iListHeight)
						bBellow = true;
					else if(oThisRect.top + 20 + iListHeight + 4 > oParentRect.bottom)	// 4:border of box and list
						bBellow = false;
				}
				else
				{
					oParentRect = oWindow.parent.container.getBoundingClientRect();
					if(oThisRect.top - oParentRect.top < iListHeight)
						bBellow = true;
					else if(oThisRect.top + 20 + iListHeight + 4 > oParentRect.bottom)	// 4:border of box and list
						bBellow = false;
				}
				if(bBellow)
					this.list.style.top = "20px";
				else
					this.list.style.top = (1 + iListHeight) * (-1) + "px";
				
				if((jaswin.frame) && (jaswin.frame.windowCount))
				{
					if(oWindow != global)
						this.list.style.zIndex = 100 + jaswin.frame.windowCount + oWindow.modalLevel;
					else
						this.list.style.zIndex = 100 + jaswin.frame.windowCount;
				}
				else
					this.list.style.zIndex = 100;
				this.list.style.display = "block";
				this.dropDown = true;
				oDropDown = this;
				
				if(this.style == list.STYLE_CHECKBOX)
				{
					if(this.selectedIndexes.length > 0)
						this.currentIndex = this.selectedIndexes[0];
					else
						this.currentIndex = -1;
				}
				else
					this.currentIndex = this.selectedIndex;
				
				if(this.currentIndex < 0)
					this.list.scrollTop = 0;
				else
				{
					if(this.currentIndex * 20 < this.list.scrollTop)
						this.list.scrollTop = this.currentIndex * 20;
					else if((this.currentIndex + 1) * 20 > this.maxDropdown * 20 + this.list.scrollTop)
						this.list.scrollTop = (this.currentIndex + 1) * 20 - (this.maxDropdown * 20 + this.list.scrollTop);
				}
				
				global.element.addEventListener("click", comboBoxCancel, false);
			}
		},
		
		//==================================================
		//	Focus textbox of this combobox
		//==================================================
		focusText: function()
		{
			if(this.style != list.STYLE_EDITABLE)
				return;
			
			this.text.select();
			this.setHover();
		},
		
		//==================================================
		//	Blur textbox of this combobox
		//==================================================
		blurText: function()
		{
			if(this.style != list.STYLE_EDITABLE)
				return;
			
			this.list.style.display = "none";
			this.dropDown = false;
			oDropDown = null;
			global.element.removeEventListener("click", comboBoxCancel, false);
			
			this.box.className = "combobox";
			this.arrow.className = "combobox-arrow";
			
			this.setOut();
			this.currentIndex = -1;
			this.setFocusItem(-1);
		},
		
		//==================================================
		//	Input textbox of this combobox
		//==================================================
		inputText: function()
		{
			if(this.style != list.STYLE_EDITABLE)
				return;
			
			if(this.text.value == "")
			{
				this.selectList(-1, false);
				this.list.style.display = "none";
				this.dropDown = false;
				oDropDown = null;
				this.list.scrollTop = 0;
				this.currentIndex = -1;
				global.element.removeEventListener("click", comboBoxCancel, false);
				return;
			}
			
			var iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.text.value == this.items[iCounter].text)
				{
					this.selectList(iCounter, true);
					this.list.scrollTop = 20 * iCounter;
					return;
				}
			}
			
			this.selectList(-1, false);
			this.currentIndex = -1;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].text.indexOf(this.text.value) == 0)
				{
					if(iCounter * 20 < this.list.scrollTop)
						this.list.scrollTop = this.iCounter * 20;
					else if((iCounter + 1) * 20 > this.maxDropdown * 20 + this.list.scrollTop)
						this.list.scrollTop = (iCounter + 1) * 20 - (this.maxDropdown * 20 + this.list.scrollTop);
					return;
				}
			}
		},
		
		//==================================================
		//	Add item to this combobox
		//==================================================
		addItem: function(sText, sVal, bSelected)
		{
			if(typeof sText != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"list.js", "jaswin.list.ComboBox.addItem");
			
			if(typeof sVal != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"list.js", "jaswin.list.ComboBox.addItem");
			
			var oItem = new list.ListItem();
			var iCounter;
			var iWidth;
			
			// Adjust list width and height
			iWidth = this.rect.width - 2 - (1 * 2) - (1 * 2);	// 2:padding 1*2:border(frame) 1*2:border(item)
			if(this.style == list.STYLE_CHECKBOX)
				iWidth -= 17;									// 17:checkbox
			
			if(this.items.length + 1 <= this.maxDropdown)
			{
				// Height of list
				this.list.style.height = (this.items.length + 1) * 20 + "px";
			}
			else
			{
				// Width of label
				iWidth -= 17;									// 17:scrollbar
				for(iCounter = 0; iCounter < this.items.length; iCounter++)
					this.items[iCounter].label.style.width = iWidth + "px";
			}
			
			// Make a new item
			oItem.text = sText;
			oItem.value = sVal;
			oItem.element = document.createElement("li");
			oItem.element.className = "list-item";
			
			if(this.style == list.STYLE_CHECKBOX)
			{
				// Checkbox
				oItem.box = document.createElement("span");
				oItem.box.className = jaswin.ICON_CHECKBOX;
				oItem.box.style.left = "2px";
				oItem.box.style.top = "3px";
				oItem.element.appendChild(oItem.box);
			}
			
			// Label
			oItem.label = document.createElement("span");
			oItem.label.className = "list-item-label";
			
			if(this.align == jaswin.ALIGN_RIGHT)
				oItem.label.style.textAlign = "right";
			else if(this.align == jaswin.ALIGN_CENTER)
				oItem.label.style.textAlign = "center";
			
			if(this.style == list.STYLE_CHECKBOX)
				oItem.label.style.left = "18px";
			else
				oItem.label.style.left = "2px";
			oItem.label.style.width = iWidth + "px";
			oItem.label.innerHTML = oItem.text;
			oItem.element.appendChild(oItem.label);
			
			oItem.element.addEventListener("mousemove", comboListMove, false);
			oItem.element.addEventListener("mouseout", comboListOut, false);
			oItem.element.addEventListener("mousedown", comboListMouseDown, false);
			
			this.list.appendChild(oItem.element);
			this.items[this.items.length] = oItem;
			
			if(bSelected)
				this.selectList(this.items.length - 1, false);
		},
		
		//==================================================
		//	Remove item from this combobox
		//==================================================
		removeItem: function(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"list.js", "jaswin.list.ComboBox.removeItem");
			if((iIndex < 0) || (iIndex >= this.items.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"list.js", "jaswin.list.ComboBox.removeItem");
			// Remove an item
			this.list.removeChild(this.items[iIndex].element);
			this.items.splice(iIndex, 1);
			
			// Adjust list width and height
			iWidth = this.rect.width - 2 - (1 * 2) - (1 * 2);	// 2:padding 1*2:border(frame) 1*2:border(item)
			if(this.style == list.STYLE_CHECKBOX)
				iWidth -= 17;									// 17:checkbox
			
			if(this.items.length <= this.maxDropdown)
				this.list.style.height = this.items.length * 20 + "px";
			else
			{
				this.list.style.height = this.maxDropdown * 20 + "px";
				iWidth -= 17;									// 17:scrollbar
			}
			
			// Width of label and selection
			var sLabelText = "";
			this.selectedIndexes = new Array();
			this.selectedIndex = -1;
			this.currentIndex = -1;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				this.items[iCounter].label.style.width = iWidth + "px";
				if(this.items[iCounter].selected)
				{
					if(this.style == list.STYLE_CHECKBOX)
					{
						this.selectedIndexes[this.selectedIndexes.length] = iCounter;
						if(sLabelText.length > 0)
							sLabelText += ",";
						sLabelText += this.items[iCounter].text;
					}
					else
					{
						this.selectedIndexes[0] = iCounter;
						sLabelText = this.items[iCounter].text;
					}
					this.selectedIndex = iCounter;
				}
			}
			if(this.style == list.STYLE_EDITABLE)
				this.text.value = sLabelText;
			else
				this.text.innerHTML = sLabelText;
		},
		
		//==================================================
		//	Set list of this combobox hover
		//==================================================
		setHoverList: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].element.contains(oElement))
				{
					this.currentIndex = iCounter;
					if(!this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item-hover";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX_HOVER;
					}
					break;
				}
			}
		},
		
		//==================================================
		//	Set list item of this combobox mouse out
		//==================================================
		setOutList: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].element.contains(oElement))
				{
					this.currentIndex = -1;
					if(!this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX;
					}
					break;
				}
			}
		},
		
		//==================================================
		//	Set list item of this combobox focused
		//==================================================
		setFocusItem:function(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"list.js", "jaswin.list.ComboBox.setFocusItem");
			var iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(iCounter == iIndex)
				{
					this.currentIndex = iCounter;
					if(!this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item-hover";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX_HOVER;
					}
				}
				else
				{
					if(!this.items[iCounter].selected)
					{
						this.items[iCounter].element.className = "list-item";
						if(this.style == list.STYLE_CHECKBOX)
							this.items[iCounter].box.className = jaswin.ICON_CHECKBOX;
					}
				}
			}
			
			if(iIndex == -1)
				this.list.scrollTop = 0;
			else
			{
				if(iIndex * 20 < this.list.scrollTop)
					this.list.scrollTop = iIndex * 20;
				else if((iIndex + 1) * 20 > this.maxDropdown * 20 + this.list.scrollTop)
					this.list.scrollTop = (iIndex + 1) * 20 - (this.maxDropdown * 20 + this.list.scrollTop);
			}
		},
		
		//==================================================
		//	Click list item of this combobox
		//==================================================
		clickList: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].element.contains(oElement))
				{
					this.selectList(iCounter, true);
					break;
				}
			}
			
			if(this.style == list.STYLE_CHECKBOX)
				bDropDownKeep = true;
			else
			{
				this.list.style.display = "none";
				this.dropDown = false;
				oDropDown = null;
				global.element.removeEventListener("click", comboBoxCancel, false);
				
				this.box.className = "combobox";
				this.arrow.className = "combobox-arrow";
			}
		},
		
		//==================================================
		//	Select list item of this combobox
		//==================================================
		selectList: function(iIndex, bExecFunc)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"list.js", "jaswin.list.ComboBox.selectList");
			if(iIndex >= this.items.length)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"list.js", "jaswin.list.ComboBox.selectList");
			
			var bExec = true;
			
			if((typeof bExecFunc != "undefined") && (bExecFunc != null))
			{
				if(typeof bExecFunc != "boolean")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_EXEC_FUNC, this.id),
												"list.js", "jaswin.list.ComboBox.selectList");
				bExec = bExecFunc;
			}
			
			if((this.style != list.STYLE_CHECKBOX)
			&& (iIndex == this.selectedIndex))
				return;
			
			var iCounter;
			var sLabelText = "";
			
			this.selectedIndexes = new Array();
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.style == list.STYLE_CHECKBOX)
				{
					if(iCounter == iIndex)
					{
						if(this.items[iIndex].selected)
						{
							if((this.enabled) && (this.parentEnabled))
							{
								this.items[iIndex].element.className = "list-item";
								this.items[iIndex].box.className = jaswin.ICON_CHECKBOX;
							}
							this.items[iIndex].selected = false;
						}
						else
						{
							if((this.enabled) && (this.parentEnabled))
							{
								this.items[iIndex].element.className = "list-item-selected";
								this.items[iIndex].box.className = jaswin.ICON_CHECKBOX_CHECKED;
							}
							this.items[iIndex].selected = true;
							this.selectedIndexes[this.selectedIndexes.length] = iIndex;
							
							if(sLabelText.length > 0)
								sLabelText += ",";
							sLabelText += this.items[iIndex].text;
						}
					}
					else if(this.items[iCounter].selected)
					{
						this.selectedIndexes[this.selectedIndexes.length] = iCounter;
						if(sLabelText.length > 0)
							sLabelText += ",";
						sLabelText += this.items[iCounter].text;
					}
				}
				else
				{
					if(iCounter == iIndex)
					{
						if((this.enabled) && (this.parentEnabled))
							this.items[iIndex].element.className = "list-item-selected";
						this.items[iIndex].selected = true;
						this.selectedIndex = iIndex;
						sLabelText = this.items[iIndex].text;
					}
					else
					{
						if((this.enabled) && (this.parentEnabled))
							this.items[iCounter].element.className = "list-item";
						this.items[iCounter].selected = false;
					}
				}
			}
			
			if(this.style != list.STYLE_CHECKBOX)
			{
				this.selectedIndex = iIndex;
				this.selectedIndexes[0] = iIndex;
				this.currentIndex = iIndex;
			}
			
			if(this.style == list.STYLE_EDITABLE)
			{
				if((iIndex != -1)
				&& (this.text.value != sLabelText))
					this.text.value = sLabelText;
			}
			else
				this.text.innerHTML = sLabelText;
			
			if((this.onChange) && (this.enabled) && (this.parentEnabled)
			&& (bExec) && (iIndex != -1))
				this.onChange(this.items[iIndex]);
		},
		
		//==================================================
		//	Select list item of this combobox by value
		//==================================================
		selectListByValue: function(sValue, bExecFunc)
		{
			if(typeof sValue != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"list.js", "jaswin.list.ComboBox.selectListByValue");
			
			var bExec = true;
			
			if((typeof bExecFunc != "undefined") && (bExecFunc != null))
			{
				if(typeof bExecFunc != "boolean")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_EXEC_FUNC, this.id),
												"list.js", "jaswin.list.ComboBox.selectListByValue");
				bExec = bExecFunc;
			}
			
			var iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].value == sValue)
				{
					this.selectList(iCounter, bExec);
					return;
				}
			}
			
			this.selectList(-1, bExec);
		},
		
		//==================================================
		//	Key down on this combobox
		//==================================================
		keyDownCombo: function(e)
		{
			if(e.keyCode == 13)
			{
				// Enter
				if(this.currentIndex != -1)
				{
					if(this.style != list.STYLE_CHECKBOX)
						this.selectList(this.currentIndex, true);
					this.click();
					this.setHover();
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
			else if(e.keyCode == 27)
			{
				// Esc
				if(this.dropDown)
				{
					this.click();
					this.setHover();
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
			else if(e.keyCode == 32)
			{
				// Space
				if(this.style != list.STYLE_EDITABLE)
				{
					if(this.style == list.STYLE_CHECKBOX)
					{
						if(!this.dropDown)
							this.click();
						else
							this.selectList(this.currentIndex, true);
					}
					else
					{
						if((oDropDown) && (oDropDown != this))
							oDropDown.click();
						this.click();
						this.setHover();
					}
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
			else if(e.keyCode == 40)
			{
				// down
				if(!this.dropDown)
				{
					this.click();
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				
				if(this.currentIndex < this.items.length - 1)
					this.setFocusItem(this.currentIndex + 1);
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			else if(e.keyCode == 38)
			{
				// up
				if(this.currentIndex <= 0)
				{
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				
				if(this.dropDown)
				{
					this.setFocusItem(this.currentIndex - 1);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
			else if((e.keyCode == 9)
				&& (this.style == list.STYLE_EDITABLE))
			{
				this.text.selectionStart = 0;
				this.text.selectionEnd = 0;
				window.getSelection().removeAllRanges();
			}
			return true;
		},
		
		//==================================================
		//	Clear list of this combobox
		//==================================================
		clear: function()
		{
			this.currentIndex = -1;
			this.selectedIndex = -1;
			this.selectedIndexes = new Array();
			this.items = new Array();
			this.list.innerHTML = "";
			this.list.style.height = "0px";
			if(this.style == list.STYLE_EDITABLE)
				this.text.value = "";
			else
				this.text.innerHTML = "";
		},
		
		//==================================================
		//	Get selected value
		//==================================================
		selectedValue: function()
		{
			if(this.style == list.STYLE_CHECKBOX)
			{
				if(this.selectedIndexes.length == 0)
					return null;
				
				var sRet = "";
				var iCounter;
				
				for(iCounter = 0; iCounter < this.selectedIndexes.length; iCounter++)
				{
					if(iCounter != 0)
						sRet += ",";
					sRet += this.items[this.selectedIndexes[iCounter]].value;
				}
				
				return sRet;
			}
			else
			{
				if(this.selectedIndex == -1)
					return null;
				
				return this.items[this.selectedIndex].value;
			}
		},
		
		//==================================================
		//	Get selected text
		//==================================================
		selectedText: function()
		{
			if(this.style == list.STYLE_EDITABLE)
				return this.text.value;
			
			if(this.style == list.STYLE_CHECKBOX)
			{
				if(this.selectedIndexes.length == 0)
					return null;
				else
					return this.text.innerHTML;
			}
			else
			{
				if(this.selectedIndex == -1)
					return null;
				else
					return this.text.innerHTML;
			}
		},
		
		//==================================================
		//	Get selected items
		//==================================================
		getSelectedItems: function()
		{
			var oaItems;
			var iCounter;
			
			oaItems = new Array();
			for(iCounter = 0; iCounter < this.selectedIndexes.length; iCounter++)
			{
				oaItems[oaItems.length] = this.items[this.selectedIndexes[iCounter]];
			}
			
			return oaItems;
		},
		
		//==================================================
		//	Get text by value
		//==================================================
		getTextByValue: function(sValue)
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].value == sValue)
					return this.items[iCounter].text;
			}
			
			return null;
		},
		
		//==================================================
		//	Contain item or not
		//==================================================
		contains: function(oItem)
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter] == oItem)
					return true;
			}
			
			return false;
		},
		
		//==================================================
		//	Sort
		//==================================================
		sort: function(iSortBy, iAscDesc)
		{
			if(typeof iSortBy != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_SORT_BY, this.id),
											"list.js", "jaswin.list.ComboBox.sort");
			if(typeof iAscDesc != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_SORT_TYPE, this.id),
											"list.js", "jaswin.list.ComboBox.sort");
			if(!this.items.length)
				return;
			
			// Execute sort
			if(iSortBy == 0)
			{
				if(iAscDesc == 0)
				{
					this.items.sort(function(oItemA, oItemB){
										if(oItemA.text < oItemB.text) return -1;
										if(oItemA.text > oItemB.text) return 1;
										return 0;
									});
				}
				else
				{
					this.items.sort(function(oItemA, oItemB){
										if(oItemA.text > oItemB.text) return -1;
										if(oItemA.text < oItemB.text) return 1;
										return 0;
									});
				}
			}
			else
			{
				if(iAscDesc == 0)
				{
					this.items.sort(function(oItemA, oItemB){
										if(oItemA.value < oItemB.value) return -1;
										if(oItemA.value > oItemB.value) return 1;
										return 0;
									});
				}
				else
				{
					this.items.sort(function(oItemA, oItemB){
										if(oItemA.value > oItemB.value) return -1;
										if(oItemA.value < oItemB.value) return 1;
										return 0;
									});
				}
			}
			
			// Replace elements
			var iCounter;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
				this.items[iCounter].element.parentNode.removeChild(this.items[iCounter].element);
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
				this.list.appendChild(this.items[iCounter].element);
		},
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip: function(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"list.js", "jaswin.list.ComboBox.setTooltip");
			this.element.title = sTooltip;
		},
		
		//==================================================
		//	Set change event
		//==================================================
		setChangeEvent: function(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, this.id),
											"list.js", "jaswin.list.ComboBox.setChangeEvent");
			
			this.onChange = fpChange;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetComboBox
	//
	//	Summary		:	Get target combobox element from event
	//
	//==========================================================
	function getTargetComboBox(e)
	{
		var oElement = e.target || e.srcElement;
		var oComboBox = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oComboBox = list.comboBoxes[oElement.id];
				if(!oComboBox)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oComboBox;
	}
	
	
	//==========================================================
	//
	//	Function	:	comboBoxMove
	//
	//	Summary		:	Event handler of mouse move a combobox
	//
	//==========================================================
	function comboBoxMove(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(oCmbBox)
			oCmbBox.setHover();
	}
	
	
	//==========================================================
	//
	//	Function	:	comboBoxOut
	//
	//	Summary		:	Event handler of mouse out a combobox
	//
	//==========================================================
	function comboBoxOut(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(oCmbBox)
			oCmbBox.setOut();
	}
	
	
	//==========================================================
	//
	//	Function	:	comboBoxMouseDown
	//
	//	Summary		:	Event handler of mousedown on a combobox
	//
	//==========================================================
	function comboBoxMouseDown(e)
	{
		var oCmbBox = null;
		var oElement = e.target || e.srcElement;
		var iCounter;
		
		oCmbBox = getTargetComboBox(e);
		
		for(iCounter = 0; iCounter < oCmbBox.items.length; iCounter++)
		{
			if(oCmbBox.items[iCounter].element.contains(oElement))
				return;
		}
		
		oCurrentMouseDown = oCmbBox;
	}
	
	
	//==========================================================
	//
	//	Function	:	comboBoxMouseUp
	//
	//	Summary		:	Event handler of mouseup on a combobox
	//
	//==========================================================
	function comboBoxMouseUp(e)
	{
		oCurrentMouseDown = null;
	}
	
	
	//==========================================================
	//
	//	Function	:	comboBoxFocus
	//
	//	Summary		:	Event handler of focus to a combobox
	//
	//==========================================================
	function comboBoxFocus(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(!oCmbBox)
			return;
		
		if((!oCmbBox.enabled) || (!oCmbBox.parentEnabled))
		{
			e.preventDefault();
			return;
		}
		
		oCmbBox.setHover();
	}
	
	
	//==========================================================
	//
	//	Function	:	comboBoxBlur
	//
	//	Summary		:	Event handler of blur from a combobox
	//
	//==========================================================
	function comboBoxBlur(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(!oCmbBox)
			return;
		
		if(bDropDownKeep)
		{
			bDropDownKeep = false;
			return;
		}
		
		if(oCmbBox == oCurrentMouseDown)
			return;
		
		if((!oCmbBox.enabled) || (!oCmbBox.parentEnabled))
		{
			e.preventDefault();
			return;
		}
		
		oCmbBox.blur();
	}
	
	
	//==========================================================
	//
	//	Function	:	comboBoxClick
	//
	//	Summary		:	Event handler of click a combobox
	//
	//==========================================================
	function comboBoxClick(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(oCmbBox)
		{
			if((oDropDown) && (oDropDown != oCmbBox))
				oDropDown.click();
			oCmbBox.click();
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	comboBoxCancel
	//
	//	Summary		:	Event handler of cancel a combobox
	//
	//==========================================================
	function comboBoxCancel(e)
	{
		if(!oDropDown)
			return;
		
		var oElement = e.target || e.srcElement;
		
		if(!oDropDown.element.contains(oElement))
		{
			oDropDown.click();
			return;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	comboBoxKeyDown
	//
	//	Summary		:	Event handler of key down a combobox
	//
	//==========================================================
	function comboBoxKeyDown(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		if(oCmbBox)
			return oCmbBox.keyDownCombo(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	comboTextFocus
	//
	//	Summary		:	Event handler of focus to a textbox
	//
	//==========================================================
	function comboTextFocus(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(oCmbBox)
			oCmbBox.focusText();
	}
	
	
	//==========================================================
	//
	//	Function	:	comboTextBlur
	//
	//	Summary		:	Event handler of blur from a textbox
	//
	//==========================================================
	function comboTextBlur(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(oCmbBox)
			oCmbBox.blurText();
	}
	
	
	//==========================================================
	//
	//	Function	:	comboTextInput
	//
	//	Summary		:	Event handler of input to a textbox
	//
	//==========================================================
	function comboTextInput(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(oCmbBox)
			oCmbBox.inputText();
	}
	
	
	//==========================================================
	//
	//	Function	:	comboListMove
	//
	//	Summary		:	Event handler of mouse move a list
	//
	//==========================================================
	function comboListMove(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(oCmbBox)
			oCmbBox.setHoverList(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	comboListOut
	//
	//	Summary		:	Event handler of mouse out a list
	//
	//==========================================================
	function comboListOut(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(oCmbBox)
			oCmbBox.setOutList(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	comboListMouseDown
	//
	//	Summary		:	Event handler of mouse down to a list
	//
	//==========================================================
	function comboListMouseDown(e)
	{
		var oCmbBox = null;
		
		oCmbBox = getTargetComboBox(e);
		
		if(oCmbBox)
			oCmbBox.clickList(e);
	}
	
})(jaswin.list = jaswin.list || {});
