//==================================================================
//
//	File name	:	listview.js
//
//	Summary		:	Listview classes
//
//	Ceated		:	2014/10/05
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (listview) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	listview.STYLE_NONE = 0;
	listview.STYLE_DATAGRID = 1;
	listview.STYLE_SUBHEAD = 2;
	listview.COL_SORT_NONE = 0;
	listview.COL_SORT_ASC = 1;
	listview.COL_SORT_DESC = 2;
	listview.COL_SORT_BOTH = 3;
	listview.COL_ITEM_LABEL = 0;
	listview.COL_ITEM_TEXTBOX = 1;
	listview.COL_ITEM_COMBOBOX = 2;
	
	// Grobal variables
	listview.listViews = {};
	
	// Private variables
	var oResizingCol = null;
	var iResizingColX = 0;
	var oCurrentListView = null;
	
	
	//==========================================================
	//
	//	Class		:	Column
	//
	//	Summary		:	Class of column
	//
	//==========================================================
	listview.Column = function(sLabel,				// * Label text[String]
								iWidth,				// * Column width[int]
								iAlign,				// * Text align[int]
								iSortType,			// * Sort type[int]
								bResizable,			// * Resizable or not
								fpSort)				// Function on sort[function]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sLabel != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, ""),
										"listview.js", "jaswin.listview.Column.constructor");
		
		if(typeof iWidth != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_WIDTH, ""),
										"listview.js", "jaswin.listview.Column.constructor");
		if(iWidth < 0)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_WIDTH, ""),
										"listview.js", "jaswin.listview.Column.constructor");
		
		if(typeof iAlign != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ALIGN, ""),
										"listview.js", "jaswin.listview.Column.constructor");
		
		if((iAlign != jaswin.ALIGN_LEFT)
		&& (iAlign != jaswin.ALIGN_RIGHT)
		&& (iAlign != jaswin.ALIGN_CENTER))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_ALIGN, ""),
										"listview.js", "jaswin.listview.Column.constructor");
		
		if(typeof iSortType != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_SORT_TYPE,
										"listview.js", "jaswin.listview.Column.constructor");
		if((iSortType != listview.COL_SORT_NONE)
		&& (iSortType != listview.COL_SORT_ASC)
		&& (iSortType != listview.COL_SORT_DESC)
		&& (iSortType != listview.COL_SORT_BOTH))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_VALUE_SORT_TYPE,
										"listview.js", "jaswin.listview.Column.constructor");
		
		if(typeof bResizable != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_RESIZABLE,
										"listview.js", "jaswin.listview.Column.constructor");
		
		if(fpSort)
		{
			if(typeof fpSort != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ON_SORT,
											"listview.js", "jaswin.listview.Column.constructor");
		}
		
		// Set properties
		this.width = iWidth;					// Column width[int]
		this.align = iAlign;					// Text align[int]
		this.sortType = iSortType;				// Sort type[int]
		this.sort = listview.COL_SORT_NONE;		// Current sort[int]
		this.resizable = bResizable;			// Resizable or not[boolean]
		this.onSort = fpSort;					// Function on sort[function]
		this.element = null;					// This element[DOM Element]
		this.label = null;						// Label element[DOM Element]
		this.sortIcon = null;					// Sort icon element[DOM Element]
		this.delimiter = null;					// Delimiter element[DOM Element]
		this.listView = null;					// Parent list view[ListView class]
		
		// Create element
		this.element = document.createElement("div");
		this.element.className = "listview-header-col";
		this.element.style.width = (this.width - 1) + "px";
		this.label = document.createElement("span");
		this.label.className = "listview-header-col-label";
		this.label.innerHTML = sLabel;
		this.element.appendChild(this.label);
		if(this.sortType != listview.COL_SORT_NONE)
		{
			this.element.style.cursor = "pointer";
			this.sortIcon = document.createElement("span");
			this.element.appendChild(this.sortIcon);
		}
		
		this.delimiter = document.createElement("div");
		this.delimiter.className = "listview-header-delimiter";
	}
	listview.Column.prototype =
	{
		//==================================================
		//	Add column layout
		//==================================================
		setLabel: function(sLabel)		// Label text[string]
		{
			if(typeof sLabel != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, ""),
											"listview.js", "jaswin.listview.Column.setLabel");
			this.label.innerHTML = sLabel;
		}
	}
	
	
	//==========================================================
	//
	//	Class		:	ColumnLayout
	//
	//	Summary		:	Class of column layout
	//
	//==========================================================
	listview.ColumnLayout = function(sName,			// * Item name[String]
									oRect,			// * Size and position[FixedRect]
									iType,			// * Item type[int]
									oInput,			// Input class[TextBox or ComboBox]
									sDefault,		// Default value[String]
									iAlign,			// Align[int]
									iFontSize,		// Font size by point[number]
									sColor,			// Font color[String]
									bBold,			// Bold font or not[boolean]
									bFocus)			// Focus or not[boolean]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sName != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_NAME,
										"listview.js", "jaswin.listview.ColumnLayout.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, ""),
										"listview.js", "jaswin.listview.ColumnLayout.constructor");
		
		if(typeof iType != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ITEM_TYPE,
										"listview.js", "jaswin.listview.ColumnLayout.constructor");
		if((iType != listview.COL_ITEM_LABEL)
		&& (iType != listview.COL_ITEM_TEXTBOX)
		&& (iType != listview.COL_ITEM_COMBOBOX))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_VALUE_ITEM_TYPE,
										"listview.js", "jaswin.listview.ColumnLayout.constructor");
		
		if(iType == listview.COL_ITEM_TEXTBOX)
		{
			if(!jaswin.textbox)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "textbox.js"),
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
			if((typeof oInput != "object")
			|| (!oInput instanceof jaswin.textbox.TextBox))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_INPUT,
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
		}
		else if(iType == listview.COL_ITEM_COMBOBOX)
		{
			if(!jaswin.list)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "list.js"),
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
			if((typeof oInput != "object")
			|| (!oInput instanceof jaswin.list.ComboBox))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_INPUT,
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
		}
		
		if(sDefault)
		{
			if(typeof sDefault != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_DEFAULT,
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
		}
		
		if(iAlign)
		{
			if(typeof iAlign != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ALIGN, ""),
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
			if((iAlign != jaswin.ALIGN_LEFT)
			&& (iAlign != jaswin.ALIGN_RIGHT)
			&& (iAlign != jaswin.ALIGN_CENTER))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_ALIGN, ""),
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
		}
		
		if(iFontSize)
		{
			if(typeof iFontSize != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_FONT_SIZE,
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
		}
		
		if(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_FONT_COLOR,
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
		}
		
		if(bBold)
		{
			if(typeof bBold != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_BOLD,
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
		}
		
		if(bFocus)
		{
			if(typeof bFocus != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_FOCUS,
											"listview.js", "jaswin.listview.ColumnLayout.constructor");
		}
		
		// Set properties
		this.name = sName;
		this.rect = oRect;
		this.type = iType;
		this.input = oInput;
		this.focus = false;
		
		if(sDefault)
			this.defaultValue = sDefault;
		else
			this.defaultValue = "";
		
		if(iAlign)
			this.align = iAlign;
		else
			this.align = jaswin.ALIGN_LEFT;
		if(iFontSize)
			this.fontSize = iFontSize;
		else
			this.fontSize = 9;
		
		if(sColor)
			this.color = sColor;
		else
			this.color = "#000000";
		
		if(bBold)
			this.bold = true;
		else
			this.bold = false;
		
		if(this.input)
		{
			this.input.hide();
			if(bFocus)
				this.focus = true;
		}
	}
	
	
	//==========================================================
	//
	//	Class		:	DataBind
	//
	//	Summary		:	Class of databind to listview
	//
	//==========================================================
	listview.DataBind = function(iColNumber)	// Number of columns[int]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof iColNumber != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_COL_NUMBER,
										"listview.js", "jaswin.listview.DataBind.constructor");
		if(iColNumber <= 0)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_VALUE_COL_NUMBER,
										"listview.js", "jaswin.listview.DataBind.constructor");
		
		// Set properties
		var iCounter;
		
		this.columnLayouts = new Array(iColNumber);
		for(iCounter = 0; iCounter < iColNumber; iCounter++)
			this.columnLayouts[iCounter] = new Array();
		this.hiddenParameters = new Array();
	}
	listview.DataBind.prototype =
	{
		//==================================================
		//	Add column layout
		//==================================================
		addLayout: function(oLayout,	// Column layout[ColumnLayout]
							iIndex)		// Column index[int]
		{
			if((typeof oLayout != "object")
			|| (!oLayout instanceof listview.ColumnLayout))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_COL_LAYOUT,
											"listview.js", "jaswin.listview.DataBind.addLayout");
			
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, ""),
											"listview.js", "jaswin.listview.DataBind.addLayout");
			if((iIndex < 0) || (iIndex >= this.columnLayouts.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, ""),
											"listview.js", "jaswin.listview.DataBind.addLayout");
			
			this.columnLayouts[iIndex][this.columnLayouts[iIndex].length] = oLayout;
		},
		
		//==================================================
		//	Add hidden parameter
		//==================================================
		addHidden: function(sHiddenName)
		{
			if(typeof sHiddenName != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_PARAM, ""),
											"listview.js", "jaswin.listview.DataBind.addHidden");
			
			this.hiddenParameters[this.hiddenParameters.length] = sHiddenName;
		}
	}
	
	
	//==========================================================
	//
	//	Class		:	Row
	//
	//	Summary		:	Class of row
	//
	//==========================================================
	listview.Row = function(iWidth,		// * Row width[int]
							iHeight,	// * Row height[int]
							sSubhead)	// Subhead[string]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof iWidth != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_WIDTH, ""),
										"listview.js", "jaswin.listview.Row.constructor");
		if(iWidth < 0)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_WIDTH, ""),
										"listview.js", "jaswin.listview.Row.constructor");
		
		if(typeof iHeight != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_HEIGHT, ""),
										"listview.js", "jaswin.listview.Row.constructor");
		if(iHeight < 0)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_HEIGHT, ""),
										"listview.js", "jaswin.listview.Row.constructor");
		
		if(sSubhead)
		{
			if(typeof sSubhead != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_HEIGHT, ""),
											"listview.js", "jaswin.listview.Row.constructor");
		}
		
		// Set properties
		this.element = null;			// This element[DOM Element]
		this.height = iHeight;			// Row height[number]
		this.checkBox = null;			// Check box element[DOM Element]
		this.subheadBox = null;			// Subhead box element[DOM Element]
		this.columns = null;			// Array of column elements[DOM Element]
		this.selected = false;			// Selected or not[boolean]
		this.checked = false;			// Checked or not[boolean]
		this.visible = true;			// Visible or not[boolean]
		this.data = {};					// Data bound to this row[Hashtable]
		this.listView = null;			// Parent list view[ListView class]
		this.controls = new Array();	// Controls on this row[Array of Control classes]
		if(sSubhead)
			this.subhead = true;
		else
			this.subhead = false;		// Subhead or not[boolean]
		this.subheadOpen = true;		// Subhead items opened or not[boolean]
		
		// Create element
		this.element = document.createElement("li");
		if(this.subhead)
		{
			this.element.className = "listview-subhead";
			this.element.style.width = (iWidth - 1) + "px";
			this.element.style.height = iHeight + "px";
			
			this.subheadBox = document.createElement("span");
			this.subheadBox.className = "listview-subhead-open";
			this.element.appendChild(this.subheadBox);
			var oSpan = document.createElement("span");
			oSpan.className = "listview-subhead-label";
			oSpan.innerHTML = sSubhead;
			this.element.appendChild(oSpan);
		}
		else
		{
			this.element.className = "listview-row";
			this.element.style.width = iWidth + "px";
			this.element.style.height = iHeight + "px";
			this.columns = new Array();
		}
	}
	listview.Row.prototype =
	{
		//==================================================
		//	Select this row
		//==================================================
		select: function(bSelect)
		{
			if(bSelect)
				this.element.className = "listview-row-focus";
			else
				this.element.className = "listview-row";
			
			this.selected = bSelect;
		},
		
		//==================================================
		//	Check this row
		//==================================================
		check: function()
		{
			if(!this.checkBox)
				return;
			
			if(this.checked)
			{
				this.checkBox.childNodes[0].className = jaswin.ICON_CHECKBOX;
				this.checked = false;
			}
			else
			{
				this.checkBox.childNodes[0].className = jaswin.ICON_CHECKBOX_CHECKED;
				this.checked = true;
			}
		},
		
		//==================================================
		//	Hover checkbox
		//==================================================
		hoverCheckBox: function()
		{
			if(!this.checked)
				this.checkBox.childNodes[0].className = jaswin.ICON_CHECKBOX_HOVER;
		},
		
		//==================================================
		//	Mouse out checkbox
		//==================================================
		outCheckBox: function()
		{
			if(!this.checked)
				this.checkBox.childNodes[0].className = jaswin.ICON_CHECKBOX;
		},
		
		//==================================================
		//	Change opened or closed
		//==================================================
		changeOpen: function()
		{
			if(this.subheadOpen)
			{
				this.subheadBox.className = "listview-subhead-close";
				this.subheadOpen = false;
			}
			else
			{
				this.subheadBox.className = "listview-subhead-open";
				this.subheadOpen = true;
			}
			
			return this.subheadOpen;
		},
		
		//==================================================
		//	Set row height
		//==================================================
		setHeight: function(iHeight)
		{
			if(typeof iHeight != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_HEIGHT, ""),
											"listview.js", "jaswin.panel.Row.setHeight");
			
			if((!this.columns) && (!this.subheadBox))
				return;
			
			this.height = iHeight;
			this.element.style.height = this.height + "px";
			
			if(this.checkBox)
			{
				var oSpan = this.checkBox.getElementsByTagName("span")[0];
				this.checkBox.style.height = this.height + "px";
				oSpan.style.top = Math.floor((this.height - 11) / 2) + "px";
			}
			
			if(this.columns)
			{
				let iCounter;
				for(iCounter = 0; iCounter < this.columns.length; iCounter++)
					this.columns[iCounter].style.height = this.height + "px";
			}
		},
		
		//==================================================
		//	Set label text
		//==================================================
		setLabel: function(iCol, sLabel)
		{
			if(typeof iCol != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_COL_NUMBER,
											"listview.js", "jaswin.listview.Row.setLabel");
			if(typeof sLabel != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, ""),
											"listview.js", "jaswin.listview.Row.setLabel");
			
			var oDivLabel;
			var oSpan;
			
			oDivLabel = document.createElement("div");
			oDivLabel.className = "listview-row-col-label";
			this.columns[iCol].appendChild(oDivLabel);
			oSpan = document.createElement("span");
			oSpan.style.position = "absolute";
			oSpan.style.left = "1px";
			oSpan.style.right = "1px";
			oSpan.style.top = "1px";
			oSpan.style.bottom = "1px";
			oSpan.style.lineHeight = this.height - 2 + "px";
			oSpan.innerHTML = sLabel;
			oDivLabel.appendChild(oSpan);
		},
		
		//==================================================
		//	Set control
		//==================================================
		setControl: function(iCol, oControl)
		{
			if(typeof iCol != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_COL_NUMBER,
											"listview.js", "jaswin.listview.Row.setControl");
			if(typeof oControl != "object")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_CONTROL, ""),
											"listview.js", "jaswin.listview.Row.setControl");
			
			if((!this.listView) || (!this.listView.parent))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, ""),
											"listview.js", "jaswin.listview.Row.setControl");
			
			var oDivInput;
			
			oDivInput = document.createElement("div");
			oDivInput.className = "listview-row-col-input";
			this.columns[iCol].appendChild(oDivInput);
			oDivInput.appendChild(oControl.element);
			oControl.setParent(this.listView);
			this.controls[this.controls.length] = oControl;
		},
		
		//==================================================
		//	Add data to row
		//==================================================
		addData: function(sKey, sValue)
		{
			if(typeof sKey != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_KEY,
											"listview.js", "jaswin.listview.Row.addData");
			if(typeof sValue != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_VALUE,
											"listview.js", "jaswin.listview.Row.addData");
			
			this.data[sKey] = sValue;
		}
	}
	
	
	//==========================================================
	//
	//	Class		:	ListView
	//
	//	Summary		:	Class of listview
	//
	//==========================================================
	listview.ListView = function(sID,			// * ID[string]
								oRect,			// * Size and position[ExpandableRect]
								iStyle,			// * Item style[int]
								oaColumns,		// * Columns[Array of Column]
								iRowHeight,		// * Row height[int]
								bEnabled,		// * Enabled or not[boolean]
								bShowHeader,	// * Show header or not[boolean]
								bCheckbox,		// * Has checkbox or not[boolean]
								iTabIndex,		// Tab index[int]
								oDataBind,		// Databind[DataBind]
								fpChange,		// On selected index changed[function]
								fpDblClick)		// On double click function[function]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		var iCnt;
		var iCounter;
		
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"listview.js", "jaswin.listview.ListView.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.ExpandableRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"listview.js", "jaswin.listview.ListView.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"listview.js", "jaswin.listview.ListView.constructor");
		
		if((iStyle != listview.STYLE_NONE)
		&& (iStyle != listview.STYLE_DATAGRID)
		&& (iStyle != listview.STYLE_SUBHEAD))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sID),
										"listview.js", "jaswin.listview.ListView.constructor");
		
		if(!Array.isArray(oaColumns))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_COLUMNS, sID),
										"listview.js", "jaswin.listview.ListView.constructor");
		iCnt = oaColumns.length;
		for(iCounter = 0; iCounter < iCnt; iCounter++)
		{
			if((typeof oaColumns[iCounter] != "object")
			|| (!oaColumns[iCounter] instanceof listview.Column))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_COLUMNS, sID),
											"listview.js", "jaswin.listview.ListView.constructor");
		}
		
		if(typeof iRowHeight != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ROW_HEIGHT, sID),
										"listview.js", "jaswin.listview.ListView.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"listview.js", "jaswin.listview.ListView.constructor");
		
		if(typeof bShowHeader != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_SHOW_HEADER, sID),
										"listview.js", "jaswin.listview.ListView.constructor");
		
		if(typeof bCheckbox != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_CHECKBOX, sID),
										"listview.js", "jaswin.listview.ListView.constructor");
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"listview.js", "jaswin.listview.ListView.constructor");
		}
		
		if(iStyle == listview.STYLE_DATAGRID)
		{
			if((typeof oDataBind != "object")
			|| (!oDataBind instanceof listview.DataBind))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_DATABIND, sID),
											"listview.js", "jaswin.listview.ListView.constructor");
			if(oDataBind.columnLayouts.length != oaColumns.length)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_COL_NUMBER, sID),
											"listview.js", "jaswin.listview.ListView.constructor");
		}
		
		if(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
											"listview.js", "jaswin.listview.ListView.constructor");
		}
		
		if(fpDblClick)
		{
			if(typeof fpDblClick != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_DBLCLICK, sID),
											"listview.js", "jaswin.listview.ListView.constructor");
		}
		
		// Set properties
		this.id = sID;							// ID[String]
		this.rect = oRect;						// Size and position[ExpandableRect]
		this.style = iStyle;					// ListView style[int]
		this.columns = oaColumns;				// Columns[Array of Column]
		this.rowWidth = 0;						// Row width[int]
		this.rowHeight = iRowHeight;			// Row height[int]
		this.enabled = bEnabled;				// Enabled or not[boolean]
		this.parentEnabled = true;				// Enabled or not[boolean]
		if(this.style == listview.STYLE_SUBHEAD)
			this.checkBox = false;
		else
			this.checkBox = bCheckbox;			// Has checkbox or not[boolean]
		this.element = null;					// This element[DOM Element]
		this.focusElement = null;				// Focus element[DOM Element]
		this.header = null;						// Header element[DOM Element]
		this.checkHeader = null;				// Checkbox header[DOM Element]
		this.container = null;					// Data container element[DOM Element]
		this.rows = new Array();				// Rows[Array of Row]
		this.tabIndex = -1;						// Tab index[int]
		this.parent = null;						// Parent class[global / window / panel class]
		this.selectedIndex = -1;				// Index of selected row[int]
		this.sortIndex = -1;					// Index of sorted column[int]
		this.sortType = listview.COL_SORT_NONE;	// Sort type[int]
		this.dataBind = oDataBind;				// Databind[DataBind]
		this.onChange = fpChange;				// On selected index changed[function]
		this.onDoubleClick = fpDblClick;		// On double click function[function]
		this.contextMenu = null;				// Context menu on this panel[Menu class]
		this.onContextMenu = null;				// Function on right click[function(Argument:Row class)]
		
		// Create element
		this.element = document.createElement("div");
		this.element.className = "listview";
		if(!bShowHeader)
			this.element.style.borderRadius = "0px";
		this.focusElement = this.element;
		
		// Horizontal rect
		if(this.rect.anchorX == jaswin.ANCHOR_BOTH)
		{
			this.element.style.left = this.rect.left + "px";
			this.element.style.right = this.rect.right + "px";
		}
		else
		{
			if(this.rect.width < this.minWidth)
				this.rect.width = this.minWidth;
			
			if(this.rect.anchorX == jaswin.ANCHOR_LEFT)
				this.element.style.left = this.rect.left + "px";
			else if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.right + "px";
			this.element.style.width = this.rect.width - 2 + "px";		// 2:border
		}
		
		// Vertical rect
		if(this.rect.anchorY == jaswin.ANCHOR_BOTH)
		{
			this.element.style.top = this.rect.top + "px";
			this.element.style.bottom = this.rect.bottom + "px";
		}
		else
		{
			if(this.rect.anchorY == jaswin.ANCHOR_TOP)
				this.element.style.top = this.rect.top + "px";
			else if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.bottom + "px";
			this.element.style.height = this.rect.height - 2 + "px";	// 2:border
		}
		
		// Create header
		var sBrowser = jaswin.BROWSER;
		var bResizable;
		var iLoop;
		var oDiv;
		
		iCnt = this.columns.length;
		this.header = document.createElement("div");
		this.header.className = "listview-header";
		
		if(this.checkBox)
		{
			// Header for checkbox
			this.checkHeader = document.createElement("div");
			this.checkHeader.className = "listview-header-col";
			this.checkHeader.style.width = "14px";
			this.checkHeader.style.cursor = "pointer";
			this.header.appendChild(this.checkHeader);
			
			if(this.enabled)
				this.checkHeader.addEventListener("click", clickCheckboxHeaderEvent, false);
			
			oDiv = document.createElement("div");
			oDiv.className = "listview-header-delimiter";
			oDiv.style.cursor = "default";
			this.header.appendChild(oDiv);
			
			this.rowWidth += 15 + 1;
		}
		if(this.style == listview.STYLE_SUBHEAD)
		{
			// Header for brank
			this.checkHeader = document.createElement("div");
			this.checkHeader.className = "listview-header-col";
			this.checkHeader.style.width = "14px";
			this.header.appendChild(this.checkHeader);
			
			oDiv = document.createElement("div");
			oDiv.className = "listview-header-delimiter";
			oDiv.style.cursor = "default";
			this.header.appendChild(oDiv);
			
			this.rowWidth += 15 + 1;
		}
		
		for(iCounter = 0; iCounter < iCnt; iCounter++)
		{
			this.header.appendChild(this.columns[iCounter].element);
			this.header.appendChild(this.columns[iCounter].delimiter);
			this.columns[iCounter].listView = this;
			
			if(this.enabled)
			{
				if(this.columns[iCounter].sortType != listview.COL_SORT_NONE)
				{
					this.columns[iCounter].element.addEventListener("click", clickHeaderEvent, false);
					this.columns[iCounter].element.style.cursor = "pointer";
				}
				
				if(this.style == listview.STYLE_DATAGRID)
				{
					for(iLoop = 0; iLoop < this.dataBind.columnLayouts[iCounter].length; iLoop++)
					{
						if((this.dataBind.columnLayouts[iCounter][iLoop].type == listview.COL_ITEM_TEXTBOX)
						|| (this.dataBind.columnLayouts[iCounter][iLoop].type == listview.COL_ITEM_COMBOBOX))
						{
							this.columns[iCounter].resizable = false;
							this.dataBind.columnLayouts[iCounter][iLoop].input.setChangeEvent(changeListViewInputEvent);
							break;
						}
					}
				}
				if(this.columns[iCounter].resizable)
					this.columns[iCounter].delimiter.addEventListener("mousedown", startResizeCol, false);
				else
					this.columns[iCounter].delimiter.style.cursor = "default";
			}
			else
				this.columns[iCounter].delimiter.style.cursor = "default";
			
			this.rowWidth += this.columns[iCounter].width + 1;
		}
		oDiv = document.createElement("div");
		oDiv.style.clear = "both";
		this.header.appendChild(oDiv);
		
		this.header.style.left = "0px";
		this.header.style.right = "0px";
		this.header.style.top = "0px";
		if(sBrowser == "ie")
			this.header.style.minWidth = (this.rowWidth + 1) + "px";
		else
			this.header.style.minWidth = (this.rowWidth + 18) + "px";
		this.element.appendChild(this.header);
		
		if(!bShowHeader)
			this.header.style.display = "none";
		
		// Create data area
		this.container = document.createElement("ul");
		this.container.className = "listview-data";
		this.container.style.left = "0px";
		this.container.style.right = "0px";
		if(bShowHeader)
			this.container.style.top = "17px";
		else
			this.container.style.top = "0px";
		this.container.style.bottom = "0px";
		if(sBrowser == "ie")
			this.container.style.minWidth = (this.rowWidth + 1) + "px";
		else
			this.container.style.minWidth = (this.rowWidth + 18) + "px";
		this.element.appendChild(this.container);
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Event binding
		if(this.enabled)
		{
			this.element.addEventListener("keydown", keyEvent, false);
			this.container.addEventListener("click", clickContainerEvent, false);
			
			if(this.checkBox)
				this.checkHeader.addEventListener("click", clickCheckboxHeaderEvent, false);
		}
	}
	listview.ListView.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var iCnt;
			var iCounter;
			var iLoop;
			var oParentWnd = jaswin.findParentWindow(this);
			
			if(this.style == listview.STYLE_DATAGRID)
			{
				for(iCounter = 0; iCounter < this.columns.length; iCounter++)
				{
					iCnt = this.dataBind.columnLayouts[iCounter].length;
					for(iLoop = 0; iLoop < iCnt; iLoop++)
					{
						if(this.dataBind.columnLayouts[iCounter][iLoop].type != listview.COL_ITEM_LABEL)
							this.dataBind.columnLayouts[iCounter][iLoop].input.destroy();
					}
				}
			}
			
			this.clear();
			
			if(this.contextMenu)
				this.contextMenu.destroy();
			
			delete listview.listViews[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (listview.listViews[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"listview.js", "jaswin.listview.ListView.setParent");
			
			this.element.id = sID;
			listview.listViews[this.element.id] = this;
			oParentWnd[this.id] = this;
			
			// Input controls
			if(this.style == listview.STYLE_DATAGRID)
			{
				var iCounter;
				var iCnt;
				var iLoop;
				
				for(iCounter = 0; iCounter < this.columns.length; iCounter++)
				{
					iCnt = this.dataBind.columnLayouts[iCounter].length;
					for(iLoop = 0; iLoop < iCnt; iLoop++)
					{
						if(this.dataBind.columnLayouts[iCounter][iLoop].type != listview.COL_ITEM_LABEL)
						{
							this.element.appendChild(this.dataBind.columnLayouts[iCounter][iLoop].input.element);
							this.dataBind.columnLayouts[iCounter][iLoop].input.setParent(this);
						}
					}
				}
			}
		},
		
		//==================================================
		//	Enable this listview
		//==================================================
		enable: function(bParent)
		{
			var iCounter;
			
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.element.tabIndex = -1;
				
				this.element.addEventListener("keydown", keyEvent, false);
				this.container.addEventListener("click", clickContainerEvent, false);
				
				if(this.checkBox)
				{
					this.checkHeader.style.cursor = "pointer";
					this.checkHeader.addEventListener("click", clickCheckboxHeaderEvent, false);
				}
				
				for(iCounter = 0; iCounter < this.columns.length; iCounter++)
				{
					this.columns[iCounter].label.className = "listview-header-col-label";
					
					if(this.columns[iCounter].sort == listview.COL_SORT_ASC)
					{
						this.columns[iCounter].sortIcon.className = "listview-header-sort-asc";
						this.columns[iCounter].element.className = "listview-header-col-sorted";
						this.columns[iCounter].delimiter.className = "listview-header-delimiter-sorted";
					}
					else if(this.columns[iCounter].sort == listview.COL_SORT_DESC)
					{
						this.columns[iCounter].sortIcon.className = "listview-header-sort-desc";
						this.columns[iCounter].element.className = "listview-header-col-sorted";
						this.columns[iCounter].delimiter.className = "listview-header-delimiter-sorted";
					}
					
					if(this.columns[iCounter].sortType != listview.COL_SORT_NONE)
					{
						this.columns[iCounter].element.addEventListener("click", clickHeaderEvent, false);
						this.columns[iCounter].element.style.cursor = "pointer";
					}
					
					if(this.columns[iCounter].resizable)
					{
						this.columns[iCounter].delimiter.style.cursor = "col-resize";
						this.columns[iCounter].delimiter.addEventListener("mousedown", startResizeCol, false);
					}
				}
				
				for(iCounter = 0; iCounter < this.rows.length; iCounter++)
				{
					if(this.checkBox)
					{
						if(this.rows[iCounter].checked)
							this.rows[iCounter].checkBox.childNodes[0].className = jaswin.ICON_CHECKBOX_CHECKED;
						this.rows[iCounter].checkBox.childNodes[0].style.cursor = "pointer";
						this.rows[iCounter].checkBox.addEventListener("click", clickRowCheckEvent, false);
						this.rows[iCounter].checkBox.addEventListener("mousemove", mouseMoveCheckEvent, false);
						this.rows[iCounter].checkBox.addEventListener("mouseout", mouseOutCheckEvent, false);
					}
					if(this.rows[iCounter].subhead)
					{
						if(this.rows[iCounter].subheadOpen)
							this.rows[iCounter].subheadBox.className = "listview-subhead-open";
						else
							this.rows[iCounter].subheadBox.className = "listview-subhead-close";
						this.rows[iCounter].subheadBox.addEventListener("click", clickRowSubheadEvent, false);
					}
					else
					{
						this.rows[iCounter].element.addEventListener("click", clickRowEvent, false);
						this.rows[iCounter].element.addEventListener("contextmenu", contextMenuRowEvent, false);
						this.rows[iCounter].element.addEventListener("dblclick", dblClickRowEvent, false);
					}
				}
			}
		},
		
		//==================================================
		//	Disable this listview
		//==================================================
		disable: function(bParent)
		{
			var iCounter;
			var bResizable;
			
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.element.tabIndex = -1;
			
			this.element.removeEventListener("keydown", keyEvent, false);
			this.container.removeEventListener("click", clickContainerEvent, false);
			
			if(this.checkBox)
			{
				this.checkHeader.style.cursor = "default";
				this.checkHeader.removeEventListener("click", clickCheckboxHeaderEvent, false);
			}
			
			for(iCounter = 0; iCounter < this.columns.length; iCounter++)
			{
				this.columns[iCounter].label.className = "listview-header-col-label-disabled";
				this.columns[iCounter].element.className = "listview-header-col";
				this.columns[iCounter].delimiter.className = "listview-header-delimiter";
				if(this.columns[iCounter].sortType != listview.COL_SORT_NONE)
				{
					this.columns[iCounter].sortIcon.className = null;
					
					this.columns[iCounter].element.removeEventListener("click", clickHeaderEvent, false);
					this.columns[iCounter].element.style.cursor = "default";
				}
				
				if(this.columns[iCounter].resizable)
					this.columns[iCounter].delimiter.removeEventListener("mousedown", startResizeCol, false);
				
				this.columns[iCounter].delimiter.style.cursor = "default";
			}
			
			this.selectRow(-1);
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(this.checkBox)
				{
					if(this.rows[iCounter].checked)
						this.rows[iCounter].checkBox.childNodes[0].className = jaswin.ICON_CHECKBOX_CHECKED_DISABLED;
					this.rows[iCounter].checkBox.childNodes[0].style.cursor = "default";
					this.rows[iCounter].checkBox.removeEventListener("click", clickRowCheckEvent, false);
					this.rows[iCounter].checkBox.removeEventListener("mousemove", mouseMoveCheckEvent, false);
					this.rows[iCounter].checkBox.removeEventListener("mouseout", mouseOutCheckEvent, false);
				}
				if(this.rows[iCounter].subhead)
				{
					if(this.rows[iCounter].subheadOpen)
							this.rows[iCounter].subheadBox.className = "listview-subhead-open-disabled";
						else
							this.rows[iCounter].subheadBox.className = "listview-subhead-close-disabled";
					this.rows[iCounter].subheadBox.removeEventListener("click", clickRowSubheadEvent, false);
				}
				else
				{
					this.rows[iCounter].element.removeEventListener("click", clickRowEvent, false);
					this.rows[iCounter].element.removeEventListener("contextmenu", contextMenuRowEvent, false);
					this.rows[iCounter].element.removeEventListener("dblclick", dblClickRowEvent, false);
				}
			}
		},
		
		//==================================================
		//	Show this listview
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this listview
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Blur from this listview
		//==================================================
		blur: function()
		{
			global.element.removeEventListener("click", blurEvent, false);
			
			if(this.style == listview.STYLE_DATAGRID)
			{
				var iCounter;
				var iLoop;
				
				for(iCounter = 0; iCounter < this.rows.length; iCounter++)
				{
					for(iLoop = 0; iLoop < this.columns.length; iLoop++)
					{
						if(this.rows[iCounter].columns[iLoop].childNodes.length == 2)
							this.rows[iCounter].columns[iLoop].childNodes[1].style.display = "none";
					}
				}
			}
		},
		
		//==================================================
		//	Set column label
		//==================================================
		setColumnLabel: function(iCol, sLabel)
		{
			if(typeof iCol != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_COL_NUMBER,
											"listview.js", "jaswin.listview.ListView.setColumnLabel");
			if(typeof sLabel != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"listview.js", "jaswin.listview.ListView.setColumnLabel");
			
			this.columns[iCol].setLabel(sLabel);
		},
		
		//==================================================
		//	Add empty row
		//==================================================
		addRow: function(sSubhead)
		{
			if(sSubhead)
			{
				if(this.style != listview.STYLE_SUBHEAD)
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, this.id),
												"listview.js", "jaswin.listview.ListView.addRow");
				
				if(typeof sSubhead != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_HEIGHT, this.id),
												"listview.js", "jaswin.listview.ListView.addRow");
			}
			
			var iCnt = this.columns.length;
			var iCounter;
			var oSpan;
			var oRow;
			var oDiv;
			
			if(sSubhead)
				oRow = new listview.Row(this.rowWidth, this.rowHeight, sSubhead);
			else
				oRow = new listview.Row(this.rowWidth, this.rowHeight);
			
			if(this.checkBox)
			{
				// Checkbox
				oRow.checkBox = document.createElement("div");
				oRow.checkBox.className = "listview-row-col";
				oRow.checkBox.style.width = "15px";
				oRow.checkBox.style.height = this.rowHeight + "px";
				
				oSpan = document.createElement("span");
				oSpan.className = jaswin.ICON_CHECKBOX;
				oSpan.style.left = "2px";
				oSpan.style.top = Math.floor((this.rowHeight - 11) / 2) + "px";
				if(!this.enabled)
					oSpan.style.cursor = "default";
				oRow.checkBox.appendChild(oSpan);
				oRow.element.appendChild(oRow.checkBox);
				
				if(this.enabled)
				{
					oRow.checkBox.addEventListener("click", clickRowCheckEvent, false);
					oRow.checkBox.addEventListener("mousemove", mouseMoveCheckEvent, false);
					oRow.checkBox.addEventListener("mouseout", mouseOutCheckEvent, false);
				}
			}
			
			if(this.style == listview.STYLE_SUBHEAD)
			{
				if(sSubhead)
				{
					if(this.enabled)
						oRow.subheadBox.addEventListener("click", clickRowSubheadEvent, false);
				}
				else
				{
					oDiv = document.createElement("div");
					oDiv.className = "listview-subhead-col";
					oDiv.style.height = this.rowHeight + "px";
					oRow.element.appendChild(oDiv);
				}
			}
			
			if(!sSubhead)
			{
				for(iCounter = 0; iCounter < iCnt; iCounter++)
				{
					// Make each column
					oRow.columns[iCounter] = document.createElement("div");
					oRow.columns[iCounter].className = "listview-row-col";
					oRow.columns[iCounter].style.width = this.columns[iCounter].width + "px";
					oRow.columns[iCounter].style.height = this.rowHeight + "px";
					if(this.columns[iCounter].align == jaswin.ALIGN_RIGHT)
						oRow.columns[iCounter].style.textAlign = "right";
					else if(this.columns[iCounter].align == jaswin.ALIGN_CENTER)
						oRow.columns[iCounter].style.textAlign = "center";
					oRow.element.appendChild(oRow.columns[iCounter]);
				}
				
				// Event binding
				if((this.enabled) && (this.parentEnabled))
				{
					oRow.element.addEventListener("click", clickRowEvent, false);
					oRow.element.addEventListener("contextmenu", contextMenuRowEvent, false);
					oRow.element.addEventListener("dblclick", dblClickRowEvent, false);
				}
			}
			oDiv = document.createElement("div");
			oDiv.style.clear = "both";
			oRow.element.appendChild(oDiv);
			
			this.rows[this.rows.length] = oRow;
			this.container.appendChild(oRow.element);
			oRow.listView = this;
			
			return oRow;
		},
		
		//==================================================
		//	Add data row to data area
		//==================================================
		addTextData: function(saData)		// * Data array[Array of string]
		{
			if((this.style != listview.STYLE_NONE)
			&& (this.style != listview.STYLE_SUBHEAD))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, this.id),
											"listview.js", "jaswin.listview.ListView.addTextData");
			if(!Array.isArray(saData))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_DATA, this.id),
											"listview.js", "jaswin.listview.ListView.addTextData");
			var iCnt;
			var iCounter;
			
			iCnt = saData.length;
			if(iCnt != this.columns.length)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_NUMBER_DATA, this.id),
											"listview.js", "jaswin.listview.ListView.addTextData");
			
			for(iCounter = 0; iCounter < iCnt; iCounter++)
			{
				if(typeof saData[iCounter] != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_DATA, this.id),
												"listview.js", "jaswin.listview.ListView.addTextData");
			}
			
			var oRow = this.addRow();
			
			for(iCounter = 0; iCounter < iCnt; iCounter++)
			{
				// Set text data
				oRow.setLabel(iCounter, saData[iCounter]);
			}
			
			return oRow;
		},
		
		//==================================================
		//	Add data row to data area by XML
		//==================================================
		addXmlData: function(oXML)
		{
			if(this.style != listview.STYLE_DATAGRID)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, this.id),
											"listview.js", "jaswin.listview.ListView.addXmlData");
			if(!oXML.getElementsByTagName)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_DATA, this.id),
											"listview.js", "jaswin.listview.ListView.addXmlData");
			
			var iCnt;
			var iCounter;
			var iLoop;
			var oRow = this.addRow();
			var oDivLabel;
			var oDivInput;
			var oSpan;
			var oLayout;
			var oaItems;
			var sValue;
			var sText;
			var bChecked = false;
			
			if(this.checkBox)
			{
				// Checkbox
				oaItems = oXML.getElementsByTagName("Checked")
				if((oaItems) && (oaItems.length > 0))
				{
					if(oaItems[0].textContent.toLowerCase() == "true")
						bChecked = true;
				}
				if(bChecked)
				{
					if(this.enabled)
						oRow.checkBox.childNodes[0].className = jaswin.ICON_CHECKBOX_CHECKED;
					else
						oRow.checkBox.childNodes[0].className = jaswin.ICON_RADIO_CHECKED_DISABLED;
					
					oRow.checked = true;
				}
			}
			
			// Set data to columns with layout
			for(iCounter = 0; iCounter < this.columns.length; iCounter++)
			{
				// Column data
				oDivLabel = document.createElement("div");
				oDivLabel.className = "listview-row-col-label";
				oRow.columns[iCounter].appendChild(oDivLabel);
				
				oDivInput = null;
				
				iCnt = this.dataBind.columnLayouts[iCounter].length;
				for(iLoop = 0; iLoop < iCnt; iLoop++)
				{
					oLayout = this.dataBind.columnLayouts[iCounter][iLoop];
					
					oaItems = oXML.getElementsByTagName(oLayout.name)
					if((!oaItems) || (oaItems.length == 0))
						sValue = oLayout.defaultValue;
					else
						sValue = oaItems[0].textContent;
					
					oSpan = document.createElement("span");
					oSpan.style.position = "absolute";
					
					if(oLayout.align == jaswin.ALIGN_CENTER)
					{
						oSpan.style.left = oLayout.rect.posX + "px";
						oSpan.style.right = oLayout.rect.posX + "px";
					}
					else
					{
						if(oLayout.rect.anchorX == jaswin.ANCHOR_LEFT)
							oSpan.style.left = oLayout.rect.posX + "px";
						else
							oSpan.style.right = oLayout.rect.posX + "px";
						oSpan.style.width = oLayout.rect.width + "px";
					}
					if(oLayout.rect.anchorY == jaswin.ANCHOR_TOP)
						oSpan.style.top = oLayout.rect.posY + "px";
					else
						oSpan.style.bottom = oLayout.rect.posY + "px";
					
					oSpan.style.height = oLayout.rect.height + "px";
					oSpan.style.fontSize = oLayout.fontSize + "pt";
					oSpan.style.color = oLayout.color;
					if(oLayout.bold)
						oSpan.style.fontWeight = "bold";
					
					if(oLayout.type == listview.COL_ITEM_COMBOBOX)
						sText = oLayout.input.getTextByValue(sValue);
					else
						sText = sValue;
					
					oSpan.innerHTML = sText;
					oDivLabel.appendChild(oSpan);
					
					if(oLayout.type != listview.COL_ITEM_LABEL)
					{
						if(!oDivInput)
						{
							oDivInput = document.createElement("div");
							oDivInput.className = "listview-row-col-input";
							oDivInput.style.display = "none";
							oRow.columns[iCounter].appendChild(oDivInput);
						}
					}
					
					oRow.addData(oLayout.name, sValue);
				}
			}
			
			// Hidden parameters
			var sHidden;
			
			iCnt = this.dataBind.hiddenParameters.length;
			for(iCounter = 0; iCounter < iCnt; iCounter++)
			{
				sHidden = this.dataBind.hiddenParameters[iCounter];
				oaItems = oXML.getElementsByTagName(sHidden);
				if((!oaItems) || (oaItems.length == 0))
					sValue = "";
				else
					sValue = oaItems[0].textContent;
				
				oRow.addData(sHidden, sValue);
			}
			
			return oRow;
		},
		
		//==================================================
		//	Add data rows to data area by XML
		//==================================================
		setXmlData: function(oXML, sRowTag)
		{
			if(!this.enabled)
				return;
			
			if(!oXML.getElementsByTagName)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_DATA, this.id),
											"listview.js", "jaswin.listview.ListView.setXmlData");
			
			if(typeof sRowTag != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ROW_TAG, this.id),
											"listview.js", "jaswin.listview.ListView.setXmlData");
			
			var oaItems = oXML.getElementsByTagName(sRowTag)
			var iCounter;
			
			if(!oaItems)
				return;
			
			for(iCounter = 0; iCounter < oaItems.length; iCounter++)
				this.addXmlData(oaItems[iCounter]);
		},
		
		//==================================================
		//	Remove data row from data area
		//==================================================
		removeData: function(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"listview.js", "jaswin.listview.ListView.removeData");
			
			if((iIndex < 0) || (iIndex >= this.rows.length))
				return;
			
			var iCounter;
			var iCnt;
			
			// Remove row on focus
			if(this.selectedIndex == iIndex)
			{
				this.selectedIndex = -1;
				if(this.style == listview.STYLE_DATAGRID)
				{
					// Datagrid
					var iLoop;
					var oLayout;
					
					for(iCounter = 0; iCounter < this.columns.length; iCounter++)
					{
						iCnt = this.dataBind.columnLayouts[iCounter].length;
						for(iLoop = 0; iLoop < iCnt; iLoop++)
						{
							oLayout = this.dataBind.columnLayouts[iCounter][iLoop];
							if(oLayout.type != listview.COL_ITEM_LABEL)
							{
								oLayout.input.hide();
								if(oLayout.input.element.parentNode != this.element)
								{
									// Save input element to Listview top
									oLayout.input.element.parentNode.style.display = "none";
									this.element.appendChild(oLayout.input.element);
								}
							}
						}
					}
				}
			}
			else if(this.selectedIndex > iIndex)
				this.selectedIndex--;
			
			var oRow = this.rows[iIndex];
			
			// Remove direct bound controls
			iCnt = oRow.controls.length;
			for(iCounter = 0; iCounter < iCnt; iCounter++)
				oRow.controls[iCounter].destroy();
			
			this.rows.splice(iIndex, 1);
			this.container.removeChild(oRow.element);
		},
		
		//==================================================
		//	Click column header
		//==================================================
		clickHeader: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			var iIndex = -1;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.columns.length; iCounter++)
			{
				if(this.columns[iCounter].element.contains(oElement))
				{
					iIndex = iCounter;
					
					if((this.columns[iCounter].sortType == listview.COL_SORT_ASC)
					&& (this.columns[iCounter].sort == listview.COL_SORT_ASC))
						return;
					
					if((this.columns[iCounter].sortType == listview.COL_SORT_DESC)
					&& (this.columns[iCounter].sort == listview.COL_SORT_DESC))
						return;
					
					if(this.columns[iCounter].sortType == listview.COL_SORT_ASC)
					{
						this.columns[iCounter].sort = listview.COL_SORT_ASC;
						this.columns[iCounter].sortIcon.className = "listview-header-sort-asc";
						this.columns[iCounter].element.className = "listview-header-col-sorted";
						this.columns[iCounter].delimiter.className = "listview-header-delimiter-sorted";
					}
					else if(this.columns[iCounter].sortType == listview.COL_SORT_DESC)
					{
						this.columns[iCounter].sort = listview.COL_SORT_DESC;
						this.columns[iCounter].sortIcon.className = "listview-header-sort-desc";
						this.columns[iCounter].element.className = "listview-header-col-sorted";
						this.columns[iCounter].delimiter.className = "listview-header-delimiter-sorted";
					}
					else
					{
						if(this.columns[iCounter].sort == listview.COL_SORT_ASC)
						{
							this.columns[iCounter].sort = listview.COL_SORT_DESC;
							this.columns[iCounter].sortIcon.className = "listview-header-sort-desc";
							this.columns[iCounter].element.className = "listview-header-col-sorted";
							this.columns[iCounter].delimiter.className = "listview-header-delimiter-sorted";
						}
						else
						{
							this.columns[iCounter].sort = listview.COL_SORT_ASC;
							this.columns[iCounter].sortIcon.className = "listview-header-sort-asc";
							this.columns[iCounter].element.className = "listview-header-col-sorted";
							this.columns[iCounter].delimiter.className = "listview-header-delimiter-sorted";
						}
					}
				}
				else if(this.columns[iCounter].sortType)
				{
					this.columns[iCounter].sort = listview.COL_SORT_NONE;
					this.columns[iCounter].sortIcon.className = null;
					this.columns[iCounter].element.className = "listview-header-col";
					this.columns[iCounter].delimiter.className = "listview-header-delimiter";
				}
			}
			
			if(iIndex != -1)
			{
				this.sortIndex = iIndex;
				this.sortType = this.columns[iIndex].sort;
				
				if(this.columns[iIndex].onSort)
				{
					this.columns[iIndex].onSort(this.columns[iIndex]);
					this.redraw();
				}
			}
		},
		
		//==================================================
		//	Click container space
		//==================================================
		clickContainer: function(e)
		{
			var oElement = e.target || e.srcElement;
			
			if(oElement != this.container)
				return;
			if(this.rows.length == 0)
				return;
			
			this.selectRow(-1);
		},
		
		//==================================================
		//	Click data row
		//==================================================
		clickRow: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(this.rows[iCounter].element.contains(oElement))
				{
					if(this.rows[iCounter].subhead)
						this.selectRow(-1);
					else
						this.selectRow(iCounter);
					break;
				}
			}
		},
		
		//==================================================
		//	Select data row
		//==================================================
		selectRow: function(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"listview.js", "jaswin.listview.ListView.selectRow");
			if((iIndex != -1)
			&& (this.rows[iIndex].subhead))
				return;
			
			var iCounter;
			var iLoop;
			
			if(iIndex == this.selectedIndex)
			{
				// Selected the same row
				if(this.style == listview.STYLE_DATAGRID)
				{
					var iCounter;
					var iLoop;
					
					for(iCounter = 0; iCounter < this.rows.length; iCounter++)
					{
						for(iLoop = 0; iLoop < this.columns.length; iLoop++)
						{
							if(this.rows[iCounter].columns[iLoop].childNodes.length == 2)
								this.rows[iCounter].columns[iLoop].childNodes[1].style.display = "block";
						}
					}
				}
				
				oCurrentListView = this;
				global.element.addEventListener("click", blurEvent, false);
				
				return;
			}
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				// Row selection
				if(iCounter == iIndex)
					this.rows[iCounter].select(true);
				else
				{
					if(!this.rows[iCounter].subhead)
						this.rows[iCounter].select(false);
				}
			}
			this.selectedIndex = iIndex;
			
			if(this.selectedIndex != -1)
			{
				var iTop = 0;
				var iBottom = 0;
				
				for(iCounter = 0; iCounter < iIndex; iCounter++)
				{
					if(this.rows[iCounter].visible)
						iTop += this.rows[iCounter].height + 1;
				}
				iBottom = iTop + this.rows[iIndex].height + 1;
				if(this.container.scrollTop > iTop)
					this.container.scrollTop = iTop;
				else if(this.container.clientHeight < iBottom - this.container.scrollTop)
					this.container.scrollTop = iBottom - this.container.clientHeight;
			}
			
			if(this.style == listview.STYLE_DATAGRID)
			{
				// Input control
				var iCnt;
				var oLayout;
				var sValue;
				var oFocusItem = null;
				
				for(iCounter = 0; iCounter < this.columns.length; iCounter++)
				{
					iCnt = this.dataBind.columnLayouts[iCounter].length;
					for(iLoop = 0; iLoop < iCnt; iLoop++)
					{
						oLayout = this.dataBind.columnLayouts[iCounter][iLoop];
						if(oLayout.type != listview.COL_ITEM_LABEL)
						{
							if(this.selectedIndex == -1)
							{
								oLayout.input.hide();
								if(oLayout.input.element.parentNode != this.element)
								{
									// Save input element to Listview top
									oLayout.input.element.parentNode.style.display = "none";
									this.element.appendChild(oLayout.input.element);
								}
							}
							else
							{
								this.rows[this.selectedIndex].columns[iCounter].childNodes[1].appendChild(oLayout.input.element);
								
								sValue = this.rows[this.selectedIndex].data[oLayout.name];
								if(oLayout.type == listview.COL_ITEM_COMBOBOX)
									oLayout.input.selectListByValue(sValue);
								else if(oLayout.type == listview.COL_ITEM_TEXTBOX)
									oLayout.input.setValue(sValue);
								
								oLayout.input.show();
								this.rows[this.selectedIndex].columns[iCounter].childNodes[1].style.display = "block";
								
								if((!oFocusItem) && (oLayout.focus))
									oFocusItem = oLayout.input.element;
							}
						}
					}
					
					for(iLoop = 0; iLoop < this.rows.length; iLoop++)
					{
						if((iLoop != this.selectedIndex)
						&& (this.rows[iLoop].columns[iCounter].childNodes.length == 2))
							this.rows[iLoop].columns[iCounter].childNodes[1].style.display = "none";
					}
				}
				if(oFocusItem)
					oFocusItem.focus();
			}
			
			// Event function
			if(this.selectedIndex == -1)
			{
				if(this.onChange)
					this.onChange(null);
			}
			else
			{
				oCurrentListView = this;
				global.element.addEventListener("click", blurEvent, false);
				
				if(this.onChange)
					this.onChange(this.rows[this.selectedIndex]);
			}
		},
		
		//==================================================
		//	Double click row
		//==================================================
		doubleClickRow: function()
		{
			if(!this.enabled)
				return;
			if(this.selectedIndex == -1)
				return;
			
			if(this.onDoubleClick)
				this.onDoubleClick(this.rows[this.selectedIndex]);
		},
		
		//==================================================
		//	Set context menu to this listview
		//==================================================
		setContextMenu: function(oMenu, fpContext)
		{
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, this.id),
											"listview.js", "jaswin.listview.ListView.setContextMenu");
			
			if(!jaswin.menu)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "menu.js"),
											"listview.js", "jaswin.listview.ListView.setContextMenu");
			
			if((typeof oMenu != "object")
			|| (!oMenu instanceof jaswin.menu.Menu))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU, this.id),
											"listview.js", "jaswin.listview.ListView.setContextMenu");
			
			if(fpContext)
			{
				if(typeof fpContext != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CONTEXTMENU, this.id),
												"listview.js", "jaswin.listview.ListView.setContextMenu");
			}
			
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
			}
			
			var oParentWnd = jaswin.findParentWindow(this);
			oParentWnd.container.appendChild(oMenu.element);
			
			this.contextMenu = oMenu;
			oMenu.setParent(this, 0);
			
			if(fpContext)
				this.onContextMenu = fpContext;
		},
		
		//==================================================
		//	Remove context menu from this listview
		//==================================================
		removeContextMenu: function()
		{
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
			}
		},
		
		//==================================================
		//	Context menu to data row
		//==================================================
		contextMenuRow: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(this.rows[iCounter].element.contains(oElement))
				{
					if(iCounter != this.selectedIndex)
						this.selectRow(iCounter);
					
					if(this.contextMenu)
					{
						if(this.onContextMenu)
						{
							if(!this.onContextMenu(this.rows[iCounter]))
							{
								jaswin.menu.menuCancel(null);
								break;
							}
						}
						this.focusElement.focus();
						this.contextMenu.showContext(e);
					}
					break;
				}
			}
		},
		
		//==================================================
		//	Click checkbox of data row
		//==================================================
		clickCheckboxHeader: function()
		{
			var iCounter;
			var bCheck = true;
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(this.rows[iCounter].checked)
				{
					bCheck = false;
					break;
				}
			}
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(this.rows[iCounter].checked != bCheck)
					this.rows[iCounter].check();
			}
		},
		
		//==================================================
		//	Click checkbox of data row
		//==================================================
		clickRowCheck: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(this.rows[iCounter].element.contains(oElement))
				{
					this.rows[iCounter].check();
					break;
				}
			}
		},
		
		//==================================================
		//	Mouse move checkbox of data row
		//==================================================
		mouseMoveCheck: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(this.rows[iCounter].checkBox.contains(oElement))
				{
					this.rows[iCounter].hoverCheckBox();
					break;
				}
			}
		},
		
		//==================================================
		//	Mouse out checkbox of data row
		//==================================================
		mouseOutCheck: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(this.rows[iCounter].checkBox.contains(oElement))
				{
					this.rows[iCounter].outCheckBox();
					break;
				}
			}
		},
		
		//==================================================
		//	Click subhead box of data row
		//==================================================
		clickRowSubhead: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			var bOpen = false;
			var bStart = false;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(bStart)
				{
					if(this.rows[iCounter].subhead)
						break;
					if(bOpen)
					{
						this.rows[iCounter].element.style.display = "block";
						this.rows[iCounter].visible = true;
					}
					else
					{
						this.rows[iCounter].element.style.display = "none";
						this.rows[iCounter].visible = false;
						if(iCounter == this.selectedIndex)
							this.selectRow(-1);
					}
				}
				else
				{
					if(this.rows[iCounter].element.contains(oElement))
					{
						bOpen = this.rows[iCounter].changeOpen();
						bStart = true;
					}
				}
			}
		},
		
		//==================================================
		//	Clear rows
		//==================================================
		clear: function()
		{
			var iCounter;
			var iCnt = this.rows.length;
			
			for(iCounter = 0; iCounter < iCnt; iCounter++)
				this.removeData(0);
			
			for(iCounter = 0; iCounter < this.columns.length; iCounter++)
			{
				if(this.columns[iCounter].sortType)
				{
					this.columns[iCounter].sort = listview.COL_SORT_NONE;
					this.columns[iCounter].sortIcon.className = null;
					this.columns[iCounter].element.className = "listview-header-col";
					this.columns[iCounter].delimiter.className = "listview-header-delimiter";
				}
			}
			this.container.scrollTop = 0;
		},
		
		//==================================================
		//	Clear rows
		//==================================================
		redraw: function()
		{
			var iCounter;
			var oElem;
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				oElem = this.container.childNodes[iCounter];
				if(oElem != this.rows[iCounter].element)
					this.container.insertBefore(this.rows[iCounter].element, oElem);
			}
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				if(this.rows[iCounter].selected)
				{
					this.selectedIndex = iCounter;
					return;
				}
			}
			
			this.selectedIndex = -1;
		},
		
		//==================================================
		//	Change input
		//==================================================
		changeInput: function(oInput)
		{
			if(this.style != listview.STYLE_DATAGRID)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, this.id),
											"listview.js", "jaswin.listview.ListView.changeInput");
			var iCounter;
			var iCnt;
			var iLoop;
			var oLayout;
			var sValue;
			var sText;
			
			for(iCounter = 0; iCounter < this.columns.length; iCounter++)
			{
				iCnt = this.dataBind.columnLayouts[iCounter].length;
				for(iLoop = 0; iLoop < iCnt; iLoop++)
				{
					oLayout = this.dataBind.columnLayouts[iCounter][iLoop];
					if(oLayout.type == listview.COL_ITEM_LABEL)
						continue;
					
					if(oLayout.type == listview.COL_ITEM_TEXTBOX)
					{
						if(oLayout.input == oInput)
						{
							sValue = oInput.getValue();
							sText = oInput.getText();
							this.rows[this.selectedIndex].data[oLayout.name] = sValue;
							this.rows[this.selectedIndex].columns[iCounter].childNodes[0].childNodes[iLoop].innerHTML = sText;
							return;
						}
					}
					else if(oLayout.type == listview.COL_ITEM_COMBOBOX)
					{
						if(oLayout.input.contains(oInput))
						{
							sValue = oInput.value;
							sText = oInput.text;
							if(!sText)
								sText = "";
							
							this.rows[this.selectedIndex].data[oLayout.name] = sValue;
							this.rows[this.selectedIndex].columns[iCounter].childNodes[0].childNodes[iLoop].innerHTML = sText;
							return;
						}
					}
				}
			}
		},
		
		//==================================================
		//	Blur from row
		//==================================================
		blurRow: function()
		{
			if(this.selectedIndex == -1)
				return;
			
			if(this.style == listview.STYLE_DATAGRID)
			{
				var iCounter;
				var iCnt;
				var iLoop;
				var oLayout;
				
				for(iCounter = 0; iCounter < this.columns.length; iCounter++)
				{
					iCnt = this.dataBind.columnLayouts[iCounter].length;
					for(iLoop = 0; iLoop < iCnt; iLoop++)
					{
						oLayout = this.dataBind.columnLayouts[iCounter][iLoop];
						if(oLayout.type == listview.COL_ITEM_LABEL)
							continue;
						
						if(oLayout.type == listview.COL_ITEM_TEXTBOX)
						{
							sValue = oLayout.input.getValue();
							sText = oLayout.input.getText();
							this.rows[this.selectedIndex].data[oLayout.name] = sValue;
							this.rows[this.selectedIndex].columns[iCounter].childNodes[0].childNodes[iLoop].innerHTML = sText;
							
							if(jaswin.BROWSER == "chrome")
								oLayout.input.element.blur();
						}
					}
				}
			}
		},
		
		//==================================================
		//	Resize column
		//==================================================
		resizeColumn: function(iGap)
		{
			var iCounter;
			var iIndex = -1;
			var iCurWidth;
			var sBrowser = jaswin.BROWSER;
			
			for(iCounter = 0; iCounter < this.columns.length; iCounter++)
			{
				if(this.columns[iCounter].delimiter == oResizingCol)
				{
					iIndex = iCounter;
					break;
				}
			}
			
			if(iIndex == -1)
				return;
			
			iCurWidth = parseInt(this.columns[iIndex].element.style.width);
			if(iCurWidth + iGap < 0)
				return;
			
			this.columns[iIndex].width = iCurWidth + iGap + 1;
			this.columns[iIndex].element.style.width = iCurWidth + iGap + "px";
			this.rowWidth += iGap;
			
			for(iCounter = 0; iCounter < this.rows.length; iCounter++)
			{
				this.rows[iCounter].element.style.width = parseInt(this.rows[iCounter].element.style.width) + iGap + "px";
				if(!this.rows[iCounter].subhead)
					this.rows[iCounter].columns[iIndex].style.width = iCurWidth + iGap + 1 + "px";
			}
			if(sBrowser == "ie")
			{
				this.header.style.minWidth = (this.rowWidth + 1) + "px";
				this.container.style.minWidth = (this.rowWidth + 1) + "px";
			}
			else
			{
				this.header.style.minWidth = (this.rowWidth + 18) + "px";
				this.container.style.minWidth = (this.rowWidth + 18) + "px";
			}
		},
		
		//==================================================
		//	Key down on this listview
		//==================================================
		keyDownList: function(e)
		{
			if((this.contextMenu) && (this.contextMenu.isVisible()))
				return this.contextMenu.menuKey(e);
			
			var iCounter;
			
			if(e.keyCode == 13)
			{
				// Enter
				if(this.selectedIndex == - 1)
					return true;
				
				this.doubleClickRow();
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			else if(e.keyCode == 40)
			{
				// down
				if(this.selectedIndex != this.rows.length - 1)
				{
					if(this.selectedIndex != -1)
						this.blurRow();
					
					for(iCounter = this.selectedIndex + 1; iCounter < this.rows.length; iCounter++)
					{
						if((!this.rows[iCounter].subhead)
						&& (this.rows[iCounter].visible))
						{
							this.selectRow(iCounter);
							break;
						}
					}
				}
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			else if(e.keyCode == 38)
			{
				// up
				if(this.selectedIndex > 0)
				{
					if(this.selectedIndex != -1)
						this.blurRow();
					
					for(iCounter = this.selectedIndex - 1; iCounter >= 0; iCounter--)
					{
						if((!this.rows[iCounter].subhead)
						&& (this.rows[iCounter].visible))
						{
							this.selectRow(iCounter);
							break;
						}
					}
				}
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			else if(e.keyCode == 36)
			{
				// Home
				if(this.rows.length > 0)
				{
					if(this.selectedIndex != -1)
						this.blurRow();
					
					for(iCounter = 0; iCounter < this.rows.length; iCounter++)
					{
						if((!this.rows[iCounter].subhead)
						&& (this.rows[iCounter].visible))
						{
							this.selectRow(iCounter);
							break;
						}
					}
				}
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			else if(e.keyCode == 35)
			{
				// End
				if(this.rows.length > 0)
				{
					if(this.selectedIndex != -1)
						this.blurRow();
					
					for(iCounter = this.rows.length - 1; iCounter >= 0; iCounter--)
					{
						if((!this.rows[iCounter].subhead)
						&& (this.rows[iCounter].visible))
						{
							this.selectRow(iCounter);
							break;
						}
					}
				}
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			else if(e.keyCode == 32)
			{
				// space
				if(!this.checkBox)
					return true;
				if(this.selectedIndex == -1)
					return true;
				
				this.rows[this.selectedIndex].check();
				e.preventDefault();
				e.stopPropagation();
				return false;
			}
			return true;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetListView
	//
	//	Summary		:	Get target listview element from event
	//
	//==========================================================
	function getTargetListView(e)
	{
		var oElement = e.target || e.srcElement;
		var oList = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oList = listview.listViews[oElement.id];
				if(!oList)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oList;
	}
	
	
	//==========================================================
	//
	//	Function	:	clickHeaderEvent
	//
	//	Summary		:	Event handler of click a list header
	//
	//==========================================================
	function clickHeaderEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.clickHeader(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	clickContainerEvent
	//
	//	Summary		:	Event handler of click a list container
	//
	//==========================================================
	function clickContainerEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.clickContainer(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	clickRowEvent
	//
	//	Summary		:	Event handler of click a list row
	//
	//==========================================================
	function clickRowEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.clickRow(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	contextMenuRowEvent
	//
	//	Summary		:	Event handler of context menu to a list row
	//
	//==========================================================
	function contextMenuRowEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.contextMenuRow(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	dblClickRowEvent
	//
	//	Summary		:	Event handler of double click a row
	//
	//==========================================================
	function dblClickRowEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.doubleClickRow();
	}
	
	
	//==========================================================
	//
	//	Function	:	clickCheckboxHeaderEvent
	//
	//	Summary		:	Event handler of click checkbox header
	//
	//==========================================================
	function clickCheckboxHeaderEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.clickCheckboxHeader();
	}
	
	
	//==========================================================
	//
	//	Function	:	clickRowCheckEvent
	//
	//	Summary		:	Event handler of click a row checkbox
	//
	//==========================================================
	function clickRowCheckEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.clickRowCheck(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseMoveCheckEvent
	//
	//	Summary		:	Event handler of mouse move row checkbox
	//
	//==========================================================
	function mouseMoveCheckEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.mouseMoveCheck(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseOutCheckEvent
	//
	//	Summary		:	Event handler of mouse out row checkbox
	//
	//==========================================================
	function mouseOutCheckEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.mouseOutCheck(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	clickRowSubheadEvent
	//
	//	Summary		:	Event handler of click a row subhead box
	//
	//==========================================================
	function clickRowSubheadEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			oList.clickRowSubhead(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	startResizeCol
	//
	//	Summary		:	Event handler of start resizing a column
	//
	//==========================================================
	function startResizeCol(e)
	{
		var oList = null;
		
		oResizingCol = e.target || e.srcElement;
		
		// Target not found
		if(!oResizingCol)
			return;
		
		oList = getTargetListView(e);
		iResizingColX = e.clientX;
		
		oList.header.addEventListener("mouseup", endResizeCol, false);
		oList.header.addEventListener("mouseout", outHeader, false);
		oList.header.addEventListener("mousemove", resizeCol, false);
	}
	
	
	//==========================================================
	//
	//	Function	:	resizeCol
	//
	//	Summary		:	Event handler of resizing a column
	//
	//==========================================================
	function resizeCol(e)
	{
		var oElement = e.target || e.srcElement;
		var iCurX = e.clientX;
		var oList = null;
		var oSelection = window.getSelection();
		
		oSelection.removeAllRanges();
		
		oList = getTargetListView(e);
		
		if((!oResizingCol) || (!oList.header.contains(oElement)))
		{
			oList.header.removeEventListener("mouseup", endResizeCol, false);
			oList.header.removeEventListener("mouseout", outHeader, false);
			oList.header.removeEventListener("mousemove", resizeCol, false);
			oResizingCol = null;
			return;
		}
		
		oList.resizeColumn(iCurX - iResizingColX);
		iResizingColX = iCurX;
	}
	
	
	//==========================================================
	//
	//	Function	:	outHeader
	//
	//	Summary		:	Event handler of mouse out header
	//
	//==========================================================
	function outHeader(e)
	{
		var oElement = e.target || e.srcElement;
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oElement == oList.header)
		{
			oList.header.removeEventListener("mouseup", endResizeCol, false);
			oList.header.removeEventListener("mouseout", outHeader, false);
			oList.header.removeEventListener("mousemove", resizeCol, false);
			oResizingCol = null;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	endResizeCol
	//
	//	Summary		:	Event handler of end resizing a column
	//
	//==========================================================
	function endResizeCol(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		oList.header.removeEventListener("mouseup", endResizeCol, false);
		oList.header.removeEventListener("mouseout", outHeader, false);
		oList.header.removeEventListener("mousemove", resizeCol, false);
		
		oResizingCol = null;
	}
	
	
	//==========================================================
	//
	//	Function	:	keyEvent
	//
	//	Summary		:	Event handler of key down to a listview
	//
	//==========================================================
	function keyEvent(e)
	{
		var oList = null;
		
		oList = getTargetListView(e);
		
		if(oList)
			return oList.keyDownList(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	blurEvent
	//
	//	Summary		:	Event handler of blur from a listview
	//
	//==========================================================
	function blurEvent(e)
	{
		if(!oCurrentListView)
		{
			global.element.removeEventListener("click", blurEvent, false);
			return;
		}
		
		var oElement = e.target || e.srcElement;
		if(!oCurrentListView.element.contains(oElement))
			oCurrentListView.blur();
	}
	
	
	//==========================================================
	//
	//	Function	:	changeListViewInputEvent
	//
	//	Summary		:	Event handler of change a input
	//
	//==========================================================
	function changeListViewInputEvent(oInput)
	{
		var oElement = oInput.element.parentNode;
		var oList = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oList = listview.listViews[oElement.id];
				if(!oList)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		if(oList)
			return oList.changeInput(oInput);
	}
	
})(jaswin.listview = jaswin.listview || {});
