//==================================================================
//
//	File name	:	net.js
//
//	Summary		:	Ajax, Download hole classes
//
//	Ceated		:	2014/11/15
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (net) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	net.METHOD_GET = 0;
	net.METHOD_POST = 1;
	net.STATUS_READY = 0;
	net.STATUS_REQUEST = 1;
	net.STATUS_RESPONSE = 2;
	net.STATUS_TIMEOUT = 3;
	net.STATUS_ABORT = 4;
	net.STATUS_ERROR = 9;
	net.RESPONSE_TEXT = 0;
	net.RESPONSE_DOCUMENT = 1;
	net.RESPONSE_ARRAYBUFFER = 2;
	net.RESPONSE_BLOB = 3;
	net.RESPONSE_JSON = 4;
	
	// Grobal variables
	net.ajaxes = {};
	net.downloadHoles = {};
	net.fileUploads = {};
	
	
	//==========================================================
	//
	//	Class		:	Ajax
	//
	//	Summary		:	Class of Ajax
	//
	//==========================================================
	net.Ajax = function(sID,			// * ID[string]
						sURL,			// * Target URL[string]
						iMethod,		// * Method[int]
						iTimeout,		// Timeout[int]
						fpOk,			// Function on OK[function]
						fpNg,			// Function on NG[function]
						fpTimeout,		// Function on Timeout[function]
						fpAbort,		// Function on Abort[function]
						fpProgress,		// Function on Progress[function]
						bMultipart,		// Multipart or not[boolean]
						fpUpProgress)	// Function on Upload Progress[function]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"net.js", "jaswin.net.Ajax.constructor");
		
		if(net.ajaxes[sID])
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
										"net.js", "jaswin.net.Ajax.constructor");
		
		if(typeof sURL != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_URL, sID),
										"net.js", "jaswin.net.Ajax.constructor");
		
		if(typeof iMethod != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_METHOD, sID),
										"net.js", "jaswin.net.Ajax.constructor");
		
		if((iMethod != net.METHOD_GET) && (iMethod != net.METHOD_POST))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_METHOD, sID),
										"net.js", "jaswin.net.Ajax.constructor");
		
		if(iTimeout)
		{
			if(typeof iTimeout != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TIMEOUT, sID),
											"net.js", "jaswin.net.Ajax.constructor");
		}
		
		if(fpOk)
		{
			if(typeof fpOk != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_OK, sID),
											"net.js", "jaswin.net.Ajax.constructor");
		}
		
		if(fpNg)
		{
			if(typeof fpNg != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_NG, sID),
											"net.js", "jaswin.net.Ajax.constructor");
		}
		
		if(fpTimeout)
		{
			if(typeof fpTimeout != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_TIMEOUT, sID),
											"net.js", "jaswin.net.Ajax.constructor");
		}
		
		if(fpAbort)
		{
			if(typeof fpAbort != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_ABORT, sID),
											"net.js", "jaswin.net.Ajax.constructor");
		}
		
		if(fpProgress)
		{
			if(typeof fpProgress != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_PROGRESS, sID),
											"net.js", "jaswin.net.Ajax.constructor");
		}
		
		if((bMultipart) && (iMethod == net.METHOD_GET))
		{
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_METHOD, sID),
										"net.js", "jaswin.net.Ajax.constructor");
		}
		
		if(fpUpProgress)
		{
			if(typeof fpUpProgress != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_UPLOAD_PROGRESS, sID),
											"net.js", "jaswin.net.Ajax.constructor");
		}
		
		// Set properties
		this.id = sID;							// ID[String]
		this.url = sURL;						// Target URL[string]
		this.method = iMethod;					// Method[int]
		this.contentType = null;				// Content type header[string]
		this.data = null;						// Data[object]
		this.timeout = iTimeout;				// Timeout[int]
		this.onOk = fpOk;						// Function on OK[function]
		this.onNG = fpNg;						// Function on NG[function]
		this.onTimeout = fpTimeout;				// Function on Timeout[function]
		this.onAbort = fpAbort;					// Function on Abort[function]
		this.onProgress = fpProgress;			// Function on Progress[function]
		this.onUpProgress = fpUpProgress;		// Function on Upload Progress[function]
		this.httpRequest = null;				// Request object[XMLHttpRequest]
		this.status = net.STATUS_READY;			// Status[int]
		this.loaded = 0;						// Loaded bytes[int]
		this.total = 0;							// Total bytes[int]
		this.responseType = net.RESPONSE_TEXT;	// Response type[int]
		this.multipart = false;					// Multipart or not[boolean]
		
		if(bMultipart)
		{
			this.data = new FormData();
			this.multipart = true;
			this.contentType = "multipart/form-data";
		}
		
		net.ajaxes[this.id] = this;
	}
	net.Ajax.prototype =
	{
		//==================================================
		//	Delete this
		//==================================================
		destroy: function()
		{
			delete net.ajaxes[this.id];
		},
		
		//==================================================
		//	Set URL
		//==================================================
		setUrl: function(sURL)
		{
			if(typeof sURL != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_URL, this.id),
											"net.js", "jaswin.net.Ajax.setUrl");
			
			this.url = sURL;
		},
		
		//==================================================
		//	Set method
		//==================================================
		setMethod: function(iMethod)
		{
			if(typeof iMethod != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_METHOD, this.id),
											"net.js", "jaswin.net.Ajax.setMethod");
			
			if((iMethod != net.METHOD_GET) && (iMethod != net.METHOD_POST))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_METHOD, this.id),
											"net.js", "jaswin.net.Ajax.setMethod");
			
			if((typeof this.data == "object")
			&& (this.multipart)
			&& (iMethod != net.METHOD_POST))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_METHOD, this.id),
											"net.js", "jaswin.net.Ajax.setMethod");
			
			this.method = iMethod;
		},
		
		//==================================================
		//	Set timeout
		//==================================================
		setTimeout: function(iTimeout)
		{
			if(typeof iTimeout != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TIMEOUT, this.id),
											"net.js", "jaswin.net.Ajax.setTimeout");
			
			this.timeout = iTimeout;
		},
		
		//==================================================
		//	Set OK event
		//==================================================
		setOkEvent: function(fpOk)
		{
			if(typeof fpOk != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_OK, this.id),
											"net.js", "jaswin.net.Ajax.setOkEvent");
			this.onOk = fpOk;
		},
		
		//==================================================
		//	Set NG event
		//==================================================
		setNgEvent: function(fpNg)
		{
			if(typeof fpNg != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_NG, this.id),
											"net.js", "jaswin.net.Ajax.setNgEvent");
			this.onNG = fpNg;
		},
		
		//==================================================
		//	Set Timeout event
		//==================================================
		setTimeoutEvent: function(fpTimeout)
		{
			if(typeof fpTimeout != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_TIMEOUT, this.id),
											"net.js", "jaswin.net.Ajax.setTimeoutEvent");
			this.onTimeout = fpTimeout;
		},
		
		//==================================================
		//	Set Abort event
		//==================================================
		setAbortEvent: function(fpAbort)
		{
			if(typeof fpAbort != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_ABORT, this.id),
											"net.js", "jaswin.net.Ajax.setAbortEvent");
			this.onAbort = fpAbort;
		},
		
		//==================================================
		//	Set Progress event
		//==================================================
		setProgressEvent: function(fpProgress)
		{
			if(typeof fpProgress != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_PROGRESS, this.id),
											"net.js", "jaswin.net.Ajax.setProgressEvent");
			this.onProgress = fpProgress;
		},
		
		//==================================================
		//	Set Upload progress event
		//==================================================
		setUploadProgressEvent: function(fpUpProgress)
		{
			if(typeof fpUpProgress != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_UPLOAD_PROGRESS, this.id),
											"net.js", "jaswin.net.Ajax.setUploadProgressEvent");
			this.onUpProgress = fpUpProgress;
		},
		
		//==================================================
		//	Set data
		//==================================================
		setData: function(sContentType, oData)
		{
			if(typeof sContentType != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_CONTENT_TYPE, this.id),
											"net.js", "jaswin.net.Ajax.setData");
			
			if(this.multipart)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CANNOT_IF_MULTIPART, this.id),
											"net.js", "jaswin.net.Ajax.setData");
			
			if((sContentType.toUpperCase() != "APPLICATION/X-WWW-FORM-URLENCODED")
			&& (this.method != net.METHOD_POST))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_METHOD, this.id),
											"net.js", "jaswin.net.Ajax.setData");
			
			this.contentType = sContentType;
			this.data = oData;
		},
		
		//==================================================
		//	Set parameter
		//==================================================
		setParams: function(saKey, oaVal)
		{
			if(!Array.isArray(saKey))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_KEY, this.id),
											"net.js", "jaswin.net.Ajax.setParams");
			
			if(!Array.isArray(oaVal))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"net.js", "jaswin.net.Ajax.setParams");
			
			if(saKey.length != oaVal.length)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_NUMBER_DATA, this.id),
											"net.js", "jaswin.net.Ajax.setParams");
			
			var iCounter;
			var sData = "";
			
			for(iCounter = 0; iCounter < saKey.length; iCounter++)
			{
				if(typeof saKey[iCounter] != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_KEY, this.id),
												"net.js", "jaswin.net.Ajax.setParams");
				
				if((!this.contentType) || (this.contentType.toUpperCase() != "MULTIPART/FORM-DATA"))
				{
					if(typeof oaVal[iCounter] != "string")
						throw new jaswin.JaswinError(jaswin.ERR_ALERT,
													jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
													"net.js", "jaswin.net.Ajax.setParams");
				}
				
				if(this.multipart)
					this.data.append(saKey[iCounter], oaVal[iCounter]);
				else
				{
					if(iCounter != 0)
						sData += "&";
					sData += encodeURIComponent(saKey[iCounter]) + "=" + encodeURIComponent(oaVal[iCounter]);
				}
			}
			
			if((!this.contentType) || (!this.multipart))
			{
				this.contentType = "application/x-www-form-urlencoded";
				this.data = sData;
			}
		},
		
		//==================================================
		//	Add parameter
		//==================================================
		addParam: function(sKey, oVal)
		{
			if(typeof sKey != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_KEY, this.id),
											"net.js", "jaswin.net.Ajax.addParam");
			
			if((!this.contentType) || (this.contentType.toUpperCase() != "MULTIPART/FORM-DATA"))
			{
				if(typeof oVal != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
												"net.js", "jaswin.net.Ajax.addParam");
			}
			
			if(this.contentType)
			{
				if((this.contentType.toUpperCase() != "APPLICATION/X-WWW-FORM-URLENCODED")
				&& (this.contentType.toUpperCase() != "MULTIPART/FORM-DATA"))
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_VALUE_CONTENT_TYPE, this.id),
												"net.js", "jaswin.net.Ajax.addParam");
			}
			else
				this.contentType = "application/x-www-form-urlencoded";
			
			if(this.multipart)
			{
				if(!this.data)
					this.data = new FormData();
				this.data.append(sKey, oVal);
			}
			else
			{
				if(this.data == null)
					this.data = "";
				
				if(this.data.length > 0)
					this.data += "&";
				
				this.data += encodeURIComponent(sKey) + "=" + encodeURIComponent(oVal);
			}
		},
		
		//==================================================
		//	Send request
		//==================================================
		request: function(iResponseType)
		{
			if(iResponseType)
			{
				if((iResponseType != net.RESPONSE_TEXT)
				&& (iResponseType != net.RESPONSE_DOCUMENT)
				&& (iResponseType != net.RESPONSE_ARRAYBUFFER)
				&& (iResponseType != net.RESPONSE_BLOB)
				&& (iResponseType != net.RESPONSE_JSON))
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_VALUE_RESPONSE_TYPE, this.id),
												"net.js", "jaswin.net.Ajax.request");
				
				this.responseType = iResponseType;
			}
			
			this.status = net.STATUS_READY;
			this.httpRequest = new XMLHttpRequest();
			var oSendData;
			
			if(this.onAbort)
			{
				// Abort event
				this.httpRequest.addEventListener("abort", abortEvent, false);
			}
			
			if(this.onProgress)
			{
				// Progress event
				this.httpRequest.addEventListener("progress", progressEvent, false);
			}
			
			if((this.multipart) && (this.onUpProgress))
			{
				// Upload progress event
				this.httpRequest.upload.addEventListener("progress", upProgressEvent, false);
			}
			
			if(this.method == net.METHOD_GET)
			{
				// GET method
				var sURL = this.url;
				
				if((this.data) && (this.data.length > 0))
					sURL + "?" + this.data;
				
				this.httpRequest.open("GET", sURL, true);
				
				if(this.timeout)
				{
					// Time out setting
					this.httpRequest.timeout = this.timeout * 1000;
					this.httpRequest.addEventListener("timeout", timeoutEvent, false);
				}
				
				if(this.responseType)
				{
					if(this.responseType == net.RESPONSE_TEXT)
						this.httpRequest.responseType = "text";
					else if(this.responseType == net.RESPONSE_DOCUMENT)
						this.httpRequest.responseType = "document";
					else if(this.responseType == net.RESPONSE_ARRAYBUFFER)
						this.httpRequest.responseType = "arraybuffer";
					else if(this.responseType == net.RESPONSE_BLOB)
						this.httpRequest.responseType = "blob";
					else if(this.responseType == net.RESPONSE_JSON)
					{
						if(jaswin.BROWSER == "ie")
							this.httpRequest.responseType = "text";
						else
							this.httpRequest.responseType = "json";
					}
				}
				this.httpRequest.setRequestHeader("Pragma", "no-cache");
				this.httpRequest.setRequestHeader("Cache-Control", "no-cache");
				oSendData = null;
			}
			else
			{
				// POST method
				this.httpRequest.open("POST", this.url, true);
				
				if(this.timeout)
				{
					// Time out setting
					this.httpRequest.timeout = this.timeout * 1000;
					this.httpRequest.addEventListener("timeout", timeoutEvent, false);
				}
				
				if(this.responseType)
				{
					if(this.responseType == net.RESPONSE_TEXT)
						this.httpRequest.responseType = "text";
					else if(this.responseType == net.RESPONSE_DOCUMENT)
						this.httpRequest.responseType = "document";
					else if(this.responseType == net.RESPONSE_ARRAYBUFFER)
						this.httpRequest.responseType = "arraybuffer";
					else if(this.responseType == net.RESPONSE_BLOB)
						this.httpRequest.responseType = "blob";
					else if(this.responseType == net.RESPONSE_JSON)
					{
						if(jaswin.BROWSER == "ie")
							this.httpRequest.responseType = "text";
						else
							this.httpRequest.responseType = "json";
					}
				}
				this.httpRequest.setRequestHeader("Content-Type", this.contentType);
				oSendData = this.data;
			}
			
			this.httpRequest.addEventListener("loadend", loadEndEvent, false);
			
			// Send data
			this.httpRequest.send(oSendData);
			this.status = net.STATUS_REQUEST;
		},
		
		//==================================================
		//	Abort request
		//==================================================
		abort: function()
		{
			this.httpRequest.abort();
		},
		
		//==================================================
		//	Progress
		//==================================================
		progress: function(iLoaded, iTotal)
		{
			this.loaded = iLoaded;
			this.total = iTotal;
			
			if(this.onProgress)
			{
				this.onProgress(this);
			}
			else if(this.onUpProgress)
			{
				this.onUpProgress(this);
			}
		},
		
		//==================================================
		//	Receive response
		//==================================================
		response: function()
		{
			if((this.httpRequest.status == 200)
			|| (this.httpRequest.status == 304))
			{
				// Response OK(200, 304)
				this.status = net.STATUS_RESPONSE;
				if(this.onOk)
					this.onOk(this);
			}
			else if(this.status == net.STATUS_TIMEOUT)
			{
				// Timeout
				if(this.onTimeout)
					this.onTimeout(this);
				else if(this.onNG)
					this.onNG(this);
			}
			else if(this.status == net.STATUS_ABORT)
			{
				// Abort
				if(this.onAbort)
					this.onAbort(this);
			}
			else
			{
				// Response NG(404, 500, ...)
				this.status = net.STATUS_ERROR;
				if(this.onNG)
					this.onNG(this);
			}
		},
		
		//==================================================
		//	Get HTTP status
		//==================================================
		getHttpStatus: function()
		{
			if(!this.httpRequest)
				return null;
			
			return this.httpRequest.status;
		},
		
		//==================================================
		//	Get returned response
		//==================================================
		getResponse: function()
		{
			if(!this.httpRequest.response)
				return null;
			
			if((this.responseType)
			&& (this.responseType == net.RESPONSE_JSON)
			&& (jaswin.BROWSER == "ie"))
			{
				return JSON.parse(this.httpRequest.responseText);
			}
			else
				return this.httpRequest.response;
		},
		
		//==================================================
		//	Get returned response by text
		//==================================================
		responseText: function()
		{
			if(!this.httpRequest)
				return null;
			
			return this.httpRequest.responseText;
		},
		
		//==================================================
		//	Get returned response by XML
		//==================================================
		responseXML: function()
		{
			if(!this.httpRequest)
				return null;
			
			return this.httpRequest.responseXML;
		},
		
		//==================================================
		//	Show save file dialog
		//==================================================
		showSaveDialog: function(sFileName)
		{
			if((!this.responseType)
			|| (this.responseType != net.RESPONSE_BLOB))
			{
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_VALUE_RESPONSE_TYPE, this.id),
												"net.js", "jaswin.net.Ajax.showSaveDialog");
			}
			
			if(typeof sFileName != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_FILE_NAME, this.id),
												"net.js", "jaswin.net.Ajax.showSaveDialog");
			
			var oContents = this.httpRequest.response;
			
			if(jaswin.BROWSER == "ie")
			{
				window.navigator.msSaveBlob(oContents, sFileName);
			}
			else
			{
				var oLink = document.createElement("a");
				var oUrl = window.URL.createObjectURL(new Blob([oContents], {type: oContents.type}));
				
				document.body.appendChild(oLink);
				oLink.style.display = "none";
				oLink.href = oUrl;
				oLink.download = sFileName;
				oLink.click();
				document.body.removeChild(oLink);
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	loadEndEvent
	//
	//	Summary		:	Load end event
	//
	//==========================================================
	function loadEndEvent(e)
	{
		var oRequest = e.target || e.srcElement;
		
		for(var sKey in net.ajaxes)
		{
			if(oRequest == net.ajaxes[sKey].httpRequest)
			{
				net.ajaxes[sKey].response();
				return;
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	timeoutEvent
	//
	//	Summary		:	Time out event
	//
	//==========================================================
	function timeoutEvent(e)
	{
		var oRequest = e.target || e.srcElement;
		
		for(var sKey in net.ajaxes)
		{
			if(oRequest == net.ajaxes[sKey].httpRequest)
			{
				net.ajaxes[sKey].status = net.STATUS_TIMEOUT;
				return;
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	abortEvent
	//
	//	Summary		:	Abort event
	//
	//==========================================================
	function abortEvent(e)
	{
		var oRequest = e.target || e.srcElement;
		
		for(var sKey in net.ajaxes)
		{
			if(oRequest == net.ajaxes[sKey].httpRequest)
			{
				net.ajaxes[sKey].status = net.STATUS_ABORT;
				return;
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	progressEvent
	//
	//	Summary		:	Progress event
	//
	//==========================================================
	function progressEvent(e)
	{
		var oRequest = e.target || e.srcElement;
		
		for(var sKey in net.ajaxes)
		{
			if(oRequest == net.ajaxes[sKey].httpRequest)
			{
				net.ajaxes[sKey].progress(e.loaded, e.total);
				return;
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	upProgressEvent
	//
	//	Summary		:	Upload progress event
	//
	//==========================================================
	function upProgressEvent(e)
	{
		var oUpload = e.target || e.srcElement;
		
		for(var sKey in net.ajaxes)
		{
			if(oUpload == net.ajaxes[sKey].httpRequest.upload)
			{
				net.ajaxes[sKey].progress(e.loaded, e.total);
				return;
			}
		}
	}
	
	
	//==========================================================
	//
	//	Class		:	DownloadHole
	//
	//	Summary		:	Class of download hole
	//
	//==========================================================
	net.DownloadHole = function(sID,		// * ID[string]
								sURL,		// * Target URL[string]
								iMethod)	// * Method[int]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"net.js", "jaswin.net.DownloadHole.constructor");
		
		if(net.downloadHoles[sID])
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
										"net.js", "jaswin.net.DownloadHole.constructor");
		
		if(typeof sURL != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_URL, sID),
										"net.js", "jaswin.net.DownloadHole.constructor");
		
		if(typeof iMethod != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_METHOD, sID),
										"net.js", "jaswin.net.DownloadHole.constructor");
		
		if((iMethod != net.METHOD_GET) && (iMethod != net.METHOD_POST))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_METHOD, sID),
										"net.js", "jaswin.net.DownloadHole.constructor");
		
		// Set properties
		this.id = sID;					// ID[String]
		this.url = sURL;				// Target URL[string]
		this.method = iMethod;			// Method[int]
		this.keys = new Array();		// Key of datas[Array of string]
		this.vals = new Array();		// Value of datas[Array of string]
		this.element = null				// This element[DOM Element]
		this.parent = null				// Parent class[Global class]
		
		// Create element
		this.element = document.createElement("iframe");
		this.element.id = this.id;
		this.element.width = 0;
		this.element.height = 0;
		this.element.style.border = "0px none";
		this.element.name = "IFR_DOWNLOAD_HOLE";
		net.downloadHoles[this.id] = this;
	}
	net.DownloadHole.prototype =
	{
		//==================================================
		//	Delete this
		//==================================================
		destroy: function()
		{
			delete net.downloadHoles[this.id];
			this.element.parentNode.removeChild(this.element);
			delete global[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function()
		{
			this.parent = global;
			global[this.id] = this;
		},
		
		//==================================================
		//	Set URL
		//==================================================
		setUrl: function(sURL)
		{
			if(typeof sURL != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_URL, this.id),
											"net.js", "jaswin.net.Ajax.setUrl");
			
			this.url = sURL;
		},
		
		//==================================================
		//	Set method
		//==================================================
		setMethod: function(iMethod)
		{
			if(typeof iMethod != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_METHOD, this.id),
											"net.js", "jaswin.net.DownloadHole.setMethod");
			
			if((iMethod != net.METHOD_GET) && (iMethod != net.METHOD_POST))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_METHOD, this.id),
											"net.js", "jaswin.net.DownloadHole.setMethod");
			
			this.method = iMethod;
		},
		
		//==================================================
		//	Set data
		//==================================================
		setData: function(sData)
		{
			if(typeof sData != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_DATA, this.id),
											"net.js", "jaswin.net.DownloadHole.setData");
			
			this.keys = new Array();
			this.vals = new Array();
			
			if(sData == "")
				return;
			
			var sNewData;
			var saParams;
			var iCounter;
			var saElements;
			var sKey;
			var sVal;
			
			if(sData.charAt(0) == "?")
				sNewData = sData.substring(1);
			else
				sNewData = sData;
			
			saParams = sNewData.split("&");
			for(iCounter = 0; iCounter < saParams.length; iCounter++)
			{
				if(saParams[iCounter] == "")
					continue;
				saElements = saParams[iCounter].split("=");
				if(saElements.length != 2)
					continue;
				if(saElements[0] == "")
					continue;
				this.keys[this.keys.length] = decodeURIComponent(saElements[0]);
				this.vals[this.vals.length] = decodeURIComponent(saElements[1]);
			}
		},
		
		//==================================================
		//	Set parameter
		//==================================================
		setParams: function(saKey, saVal)
		{
			if(!Array.isArray(saKey))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_KEY, this.id),
											"net.js", "jaswin.net.DownloadHole.setParams");
			
			if(!Array.isArray(saVal))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"net.js", "jaswin.net.DownloadHole.setParams");
			
			if(saKey.length != saVal.length)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_NUMBER_DATA, this.id),
											"net.js", "jaswin.net.DownloadHole.setParams");
			
			var iCounter;
			var sData = "";
			
			for(iCounter = 0; iCounter < saKey.length; iCounter++)
			{
				if(typeof saKey[iCounter] != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_KEY, this.id),
												"net.js", "jaswin.net.DownloadHole.setParams");
				
				if(typeof saVal[iCounter] != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
												"net.js", "jaswin.net.DownloadHole.setParams");
			}
			
			this.keys = saKey
			this.vals = saVal;
		},
		
		//==================================================
		//	Add parameter
		//==================================================
		addParam: function(sKey, sVal)
		{
			if(typeof sKey != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_KEY, this.id),
											"net.js", "jaswin.net.DownloadHole.addParam");
			
			if(typeof sVal != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"net.js", "jaswin.net.DownloadHole.addParam");
			
			this.keys[this.keys.length] = sKey;
			this.vals[this.vals.length] = sVal;
		},
		
		//==================================================
		//	Send request
		//==================================================
		request: function()
		{
			var iCounter;
			
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_ALLOCATE, this.id),
											"net.js", "jaswin.net.DownloadHole.request");
			
			// GET method
			if(this.method == net.METHOD_GET)
			{
				var sURL = this.url;
				var sData = "";
				
				for(iCounter = 0; iCounter < this.keys.length; iCounter++)
				{
					if(iCounter != 0)
						sData += "&";
					sData += encodeURIComponent(this.keys[iCounter])
							+ "="
							+ encodeURIComponent(this.vals[iCounter]);
				}
				if(sData.length > 0)
					sURL += "?" + sData;
				
				this.element.src = sURL;
				return;
			}
			
			// POST method
			var oDoc = this.element.contentWindow.document;
			var oForm = null;
			
			oDoc.open();
			oDoc.write("<html><head></head><body></body></html>");
			oDoc.close();
			oForm = oDoc.createElement("form");
			oForm.enctype = "application/x-www-form-urlencoded";
			oForm.action = this.url;
			oForm.method = "post";
			oDoc.body.appendChild(oForm);
			
			for(iCounter = 0; iCounter < this.keys.length; iCounter++)
			{
				var oHidden = document.createElement("input");
				oHidden.type = "hidden";
				oHidden.name = this.keys[iCounter];
				oHidden.value = this.vals[iCounter];
				oForm.appendChild(oHidden);
			}
			
			oForm.submit();
		}
	}
	
	
	//==========================================================
	//
	//	Class		:	FileUpload
	//
	//	Summary		:	Class of File upload
	//
	//==========================================================
	net.FileUpload = function(sID,				// * ID[string]
								oRect,			// * Size and position[FixedRect]
								bEnabled,		// * Enabled or not[boolean]
								sURL,			// * Upload URL[string]
								sParamName,		// * Parameter name[string]
								bMultiple,		// * Multi files or not[boolean]
								sTooltip,		// Tool tip text[string]
								iTabIndex,		// Tab index[int]
								fpCallBack,		// Function of call back[function]
								bSelectionOnly)	// File selection only[boolean]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"net.js", "jaswin.net.FileUpload.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"net.js", "jaswin.net.FileUpload.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"net.js", "jaswin.net.FileUpload.constructor");
		
		if(!bSelectionOnly)
		{
			if(typeof sURL != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_URL, sID),
											"net.js", "jaswin.net.FileUpload.constructor");
		}
		
		if(typeof sParamName != "string")
		throw new jaswin.JaswinError(jaswin.ERR_ALERT,
									jaswin.formatMessage(jaswin.ALERT_TYPE_PARAM, sID),
									"net.js", "jaswin.net.FileUpload.constructor");
		
		if(typeof bMultiple != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_MULTIPLE, sID),
										"net.js", "jaswin.net.FileUpload.constructor");
		
		if(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
											"net.js", "jaswin.net.FileUpload.constructor");
		}
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"net.js", "jaswin.net.FileUpload.constructor");
		}
		
		if(fpCallBack)
		{
			if(typeof fpCallBack != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_CALLBACK, sID),
											"net.js", "jaswin.net.FileUpload.constructor");
		}
		
		if(!jaswin.button)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_INCLUDE, "button.js"),
										"net.js", "jaswin.net.FileUpload.constructor");
		
		// Set properties
		this.id = sID;					// ID[String]
		this.rect = oRect;				// Size and position[FixedRect]
		this.enabled = bEnabled;		// Enabled or not[boolean]
		this.parentEnabled = true;		// Enabled or not[boolean]
		this.element = null;			// This element[DOM Element]
		this.focusElement = null;		// Focus element[DOM Element]
		this.content = null;			// Content element for file selection[DOM Element]
		this.fileElement = null;		// File element[DOM Element]
		this.fileName = null;			// File name element[DOM Element]
		this.button = null;				// Upload or selection button[Button class]
		this.responseFrame = null;		// IFrame element for response[DOM Element]
		this.tabIndex = -1;				// Tab index[int]
		this.parent = null;				// Parent class[global / window / panel class]
		this.callBack = fpCallBack;		// Function of call back[function]
		this.keys = new Array();		// Keys of parameters[String]
		this.vals = new Array();		// Values of parameters[String]
		this.selectionOnly = false;		// File selection only[boolean]
		
		if(bSelectionOnly)
			this.selectionOnly = true;
		
		// Create element
		this.element = document.createElement("div");
		this.element.style.position = "absolute";
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		
		this.element.style.width = this.rect.width + "px";
		this.element.style.height = "20px";
		this.focusElement = this.element;
		
		// File selection part
		this.content = document.createElement("form");
		this.content.className = "file-upload";
		this.content.action = sURL;
		this.content.method = "post";
		this.content.enctype = "multipart/form-data";
		if(!this.enabled)
			this.content.style.cursor = "default";
		this.element.appendChild(this.content);
		
		this.fileElement = document.createElement("input");
		this.fileElement.type = "file";
		this.fileElement.multiple = bMultiple;
		this.fileElement.name = sParamName;
		this.fileElement.style.width = "0px";
		this.fileElement.style.height = "0px";
		this.content.appendChild(this.fileElement);
		
		this.fileName = document.createElement("div");
		if(this.enabled)
			this.fileName.className = "file-upload-filename";
		else
			this.fileName.className = "file-upload-filename-disabled";
		this.content.appendChild(this.fileName);
		
		if(this.selectionOnly)
		{
			// Selection button
			var oButtonRect = new jaswin.FixedRect(jaswin.ANCHOR_RIGHT, jaswin.ANCHOR_TOP,
													0, 0, 20, 20);
			this.button = new jaswin.button.Button("BTN_" + this.id, oButtonRect,
													jaswin.button.STYLE_DEFAULT | jaswin.button.STYLE_ICON,
													true,
													"icon-open", null,
													-1, clickSelectionEvent);
			this.button.element.style.borderRadius = "0px 2px 2px 0px";
			this.element.appendChild(this.button.element);
		}
		else
		{
			// Upload button
			var oButtonRect = new jaswin.FixedRect(jaswin.ANCHOR_RIGHT, jaswin.ANCHOR_TOP,
													0, 0, 20, 20);
			this.button = new jaswin.button.Button("BTN_" + this.id, oButtonRect,
													jaswin.button.STYLE_DEFAULT | jaswin.button.STYLE_ICON,
													false,
													"icon-file-upload", null,
													-1, clickUploadEvent);
			this.button.element.style.borderRadius = "0px 2px 2px 0px";
			this.element.appendChild(this.button.element);
			
			// IFrame
			this.responseFrame = document.createElement("iframe");
			this.responseFrame.width = 0;
			this.responseFrame.height = 0;
			this.responseFrame.name = "IFR_" + this.id;
			this.responseFrame.sandbox = "allow-same-origin";
			this.element.appendChild(this.responseFrame);
			this.content.target = this.responseFrame.name;
		}
		
		// Tooltip
		if((sTooltip) && (sTooltip.length > 0))
			this.element.title = sTooltip;
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Event binding
		if(this.enabled)
		{
			this.content.addEventListener("mousemove", mouseMoveEvent, false);
			this.content.addEventListener("mouseout", mouseOutEvent, false);
			this.content.addEventListener("click", clickEvent, false);
			this.element.addEventListener("keyup", keyUpEvent, false);
			this.fileElement.addEventListener("change", changeEvent, false);
		}
	}
	net.FileUpload.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			this.button.destroy();
			
			delete net.fileUploads[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			this.button.setParent(oParent);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (net.fileUploads[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"net.js", "jaswin.net.FileUpload.setParent");
			this.element.id = sID;
			net.fileUploads[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this file upload
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.element.tabIndex = this.tabIndex;
				this.content.style.cursor = "pointer";
				this.fileName.className = "file-upload-filename";
				
				if(this.selectionOnly)
					this.button.enable(bParent);
				else if((bParent) && (this.fileElement.files.length > 0))
					this.button.enable(bParent);
				
				this.content.addEventListener("mousemove", mouseMoveEvent, false);
				this.content.addEventListener("mouseout", mouseOutEvent, false);
				this.content.addEventListener("click", clickEvent, false);
				this.element.addEventListener("keyup", keyUpEvent, false);
				this.fileElement.addEventListener("change", changeEvent, false);
			}
		},
		
		//==================================================
		//	Disable this file upload
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.element.tabIndex = -1;
			this.content.style.cursor = "default";
			this.fileName.className = "file-upload-filename-disabled";
			this.button.disable(bParent);
			
			this.content.removeEventListener("mousemove", mouseMoveEvent, false);
			this.content.removeEventListener("mouseout", mouseOutEvent, false);
			this.content.removeEventListener("click", clickEvent, false);
			this.element.removeEventListener("keyup", keyUpEvent, false);
			this.fileElement.removeEventListener("change", changeEvent, false);
		},
		
		//==================================================
		//	Show this file upload
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this file upload
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Set this file upload hover
		//==================================================
		setHover: function()
		{
			this.content.className = "file-upload-focus";
		},
		
		//==================================================
		//	Set this file upload mouse out
		//==================================================
		setOut: function()
		{
			this.content.className = "file-upload";
		},
		
		//==================================================
		//	Click this file upload
		//==================================================
		click: function()
		{
			if(!this.enabled)
				return;
			
			this.fileElement.click();
		},
		
		//==================================================
		//	Change this file upload
		//==================================================
		change: function()
		{
			var sFileName = "";
			var iCounter;
			
			for(iCounter = 0; iCounter < this.fileElement.files.length; iCounter++)
			{
				if(iCounter > 0)
					sFileName += ", ";
				sFileName += this.fileElement.files[iCounter].name;
			}
			
			this.fileName.innerHTML = sFileName;
			
			if((!this.selectionOnly) && (!this.selectionOnly) && (sFileName.length > 0))
				this.button.enable();
		},
		
		//==================================================
		//	Set URL
		//==================================================
		setUrl: function(sURL)
		{
			if(typeof sURL != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_URL, this.id),
											"net.js", "jaswin.net.FileUpload.setUrl");
			
			this.content.action = sURL;
		},
		
		//==================================================
		//	Get selected files
		//==================================================
		getFiles: function()
		{
			return this.fileElement.files;
		},
		
		//==================================================
		//	Execute upload file
		//==================================================
		upload: function()
		{
			if(!this.enabled)
				return;
			if(this.fileElement.files.length == 0)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_NO_FILE, this.id),
											"net.js", "jaswin.net.FileUpload.upload");
			
			var iCounter;
			
			for(iCounter = 0; iCounter < this.keys.length; iCounter++)
			{
				var oHidden = document.createElement("input");
				oHidden.type = "hidden";
				oHidden.name = this.keys[iCounter];
				oHidden.value = this.vals[iCounter];
				this.content.appendChild(oHidden);
			}
			
			this.responseFrame.addEventListener("load", responseEvent);
			this.content.submit();
		},
		
		//==================================================
		//	Add hidden parameter
		//==================================================
		addParam: function(sKey, sVal)
		{
			if(typeof sKey != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_KEY, this.id),
											"net.js", "jaswin.net.FileUpload.addParam");
			
			if(typeof sVal != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"net.js", "jaswin.net.FileUpload.addParam");
			
			this.keys[this.keys.length] = sKey;
			this.vals[this.vals.length] = sVal;
		},
		
		//==================================================
		//	Clear contents
		//==================================================
		clear: function()
		{
			if(!this.selectionOnly)
				this.button.disable();
			
			var sParamName = this.fileElement.name;
			var bMultiple = this.fileElement.multiple;
			var iCounter;
			for(iCounter = this.content.childNodes.length - 1; iCounter >= 0; iCounter--)
				this.content.removeChild(this.content.childNodes[iCounter]);
			
			this.fileElement = document.createElement("input");
			this.fileElement.type = "file";
			this.fileElement.multiple = bMultiple;
			this.fileElement.name = sParamName;
			this.fileElement.style.width = "0px";
			this.fileElement.style.height = "0px";
			this.content.appendChild(this.fileElement);
			if(this.enabled)
				this.fileElement.addEventListener("change", changeEvent, false);
			
			this.fileName = document.createElement("div");
			if(this.enabled)
				this.fileName.className = "file-upload-filename";
			else
				this.fileName.className = "file-upload-filename-disabled";
			this.content.appendChild(this.fileName);
			
			this.keys = new Array();
			this.vals = new Array();
		},
		
		//==================================================
		//	Response of upload
		//==================================================
		response: function()
		{
			this.clear();
			if(this.callBack)
				this.callBack(this);
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetFileUpload
	//
	//	Summary		:	Get target file upload from event
	//
	//==========================================================
	function getTargetFileUpload(e)
	{
		var oElement = e.target || e.srcElement;
		var oFileUpload = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oFileUpload = net.fileUploads[oElement.id];
				if(!oFileUpload)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oFileUpload;
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseMoveEvent
	//
	//	Summary		:	Event handler of mouse move
	//
	//==========================================================
	function mouseMoveEvent(e)
	{
		var oFileUpload = null;
		
		oFileUpload = getTargetFileUpload(e);
		
		if(oFileUpload)
			oFileUpload.setHover();
	}
	
	
	//==========================================================
	//
	//	Function	:	mouseOutEvent
	//
	//	Summary		:	Event handler of mouse out
	//
	//==========================================================
	function mouseOutEvent(e)
	{
		var oFileUpload = null;
		
		oFileUpload = getTargetFileUpload(e);
		
		if(oFileUpload)
			oFileUpload.setOut();
	}
	
	
	//==========================================================
	//
	//	Function	:	clickEvent
	//
	//	Summary		:	Event handler of click
	//
	//==========================================================
	function clickEvent(e)
	{
		var oFileUpload = null;
		
		oFileUpload = getTargetFileUpload(e);
		
		if(oFileUpload)
			oFileUpload.click();
	}
	
	
	//==========================================================
	//
	//	Function	:	keyUpEvent
	//
	//	Summary		:	Event handler of key up
	//
	//==========================================================
	function keyUpEvent(e)
	{
		var oFileUpload = null;
		
		if((e.keyCode != 13) && (e.keyCode != 32))
			return true;
		
		oFileUpload = getTargetFileUpload(e);
		if(!oFileUpload)
			return true;
		
		if(e.keyCode == 13)
			oFileUpload.upload();
		else if(e.keyCode == 32)
			oFileUpload.click();
	}
	
	
	//==========================================================
	//
	//	Function	:	changeEvent
	//
	//	Summary		:	Event handler of change file
	//
	//==========================================================
	function changeEvent(e)
	{
		var oFileUpload = null;
		
		oFileUpload = getTargetFileUpload(e);
		
		if(oFileUpload)
			oFileUpload.change();
	}
	
	
	//==========================================================
	//
	//	Function	:	clickSelectionEvent
	//
	//	Summary		:	Event handler of click selection button
	//
	//==========================================================
	function clickSelectionEvent(oButton)
	{
		var oFileUpload = null;
		
		for(var sKey in net.fileUploads)
		{
			if(net.fileUploads[sKey].button == oButton)
			{
				oFileUpload = net.fileUploads[sKey];
				break;
			}
		}
		
		if(oFileUpload)
			oFileUpload.click();
	}
	
	
	//==========================================================
	//
	//	Function	:	clickUploadEvent
	//
	//	Summary		:	Event handler of click upload button
	//
	//==========================================================
	function clickUploadEvent(oButton)	// Button class
	{
		var oFileUpload = null;
		
		for(var sKey in net.fileUploads)
		{
			if(net.fileUploads[sKey].button == oButton)
			{
				oFileUpload = net.fileUploads[sKey];
				break;
			}
		}
		
		if(oFileUpload)
			oFileUpload.upload();
	}
	
	
	//==========================================================
	//
	//	Function	:	responseEvent
	//
	//	Summary		:	Event handler of upload response
	//
	//==========================================================
	function responseEvent(e)
	{
		var oFileUpload = null;
		
		oFileUpload = getTargetFileUpload(e);
		
		oFileUpload.response();
	}
	
})(jaswin.net = jaswin.net || {});
