//==================================================================
//
//	File name	:	panel.js
//
//	Summary		:	Panel, GroupBox, Tab classes
//
//	Ceated		:	2014/09/19
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (panel) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	panel.STYLE_DEFAULT = 0;
	panel.STYLE_INSET = 1;
	panel.STYLE_OUTSET = 2;
	panel.OVERFLOW_DEFAULT = 0;
	panel.OVERFLOW_HIDDEN = 1;
	panel.OVERFLOW_SCROLL = 2;
	panel.TYPE_H_SPLIT = 0;
	panel.TYPE_V_SPLIT = 1;
	panel.STYLE_FIXED = 0;
	panel.STYLE_VARIABLE = 1;
	panel.STYLE_ALIGN_RIGHT = 2;
	
	// Grobal variables
	panel.panels = {};
	panel.groupboxes = {};
	panel.tabs = {};
	
	// Private variables
	var oResizingDelimiter = null;
	var iResizingPos = 0;
	
	
	//==========================================================
	//
	//	Class		:	Panel
	//
	//	Summary		:	Class of panel
	//
	//==========================================================
	panel.Panel = function(sID,				// * ID[String]
							oRect,			// * Size and position[ExpandableRect]
							iStyle,			// * Panel style[int]
							bEnabled,		// * Enabled[boolean]
							iOverflow,		// * Overflow[int]
							bPopup)			// Pop up or not
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"panel.js", "jaswin.panel.Panel.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.ExpandableRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"panel.js", "jaswin.panel.Panel.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"panel.js", "jaswin.panel.Panel.constructor");
		
		if((iStyle != panel.STYLE_DEFAULT)
		&& (iStyle != panel.STYLE_INSET)
		&& (iStyle != panel.STYLE_OUTSET))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sID),
										"panel.js", "jaswin.panel.Panel.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"panel.js", "jaswin.panel.Panel.constructor");
		
		if(typeof iOverflow != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_OVERFLOW, sID),
										"panel.js", "jaswin.panel.Panel.constructor");
		
		if((iOverflow != panel.OVERFLOW_DEFAULT)
		&& (iOverflow != panel.OVERFLOW_HIDDEN)
		&& (iOverflow != panel.OVERFLOW_SCROLL))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_OVERFLOW, sID),
										"panel.js", "jaswin.panel.Panel.constructor");
		
		if(bPopup)
		{
			if(typeof bPopup != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_POPUP, sID),
											"panel.js", "jaswin.panel.Panel.constructor");
		}
		
		// Set properties
		this.id = sID;					// ID[String]
		this.rect = oRect;				// Size and position[ExpandableRect]
		this.style = iStyle;			// Panel style[int]
		this.element = null;			// Window element[DOM Element]
		this.container = null;			// Container for client area[DOM Element]
		this.controls = new Array();	// Controls on this panel[Array of Control classes]
		this.contextMenu = null;		// Context menu on this panel[Menu class]
		this.onContextMenu = null;		// Function on right click[function(Argument:Panel class)]
		this.parent = null;				// Parent class[global / window / panel class]
		this.enabled = bEnabled;		// Enabled or not[boolean]
		this.parentEnabled = true;		// Enabled or not[boolean]
		this.popup = bPopup;			// Popup or not[boolean]
		
		// Create element
		this.element = document.createElement("div");
		if(iStyle == panel.STYLE_DEFAULT)
			this.element.className = "panel";
		else if(iStyle == panel.STYLE_INSET)
			this.element.className = "panel-inset";
		else if(iStyle == panel.STYLE_OUTSET)
			this.element.className = "panel-outset";
		
		if(this.popup)
			this.element.style.boxShadow = "2px 2px 2px #a0a0a0";
		
		// Horizontal rect
		if(this.rect.anchorX == jaswin.ANCHOR_BOTH)
		{
			this.element.style.left = this.rect.left + "px";
			this.element.style.right = this.rect.right + "px";
		}
		else
		{
			if(this.rect.anchorX == jaswin.ANCHOR_LEFT)
				this.element.style.left = this.rect.left + "px";
			else if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.right + "px";
			
			if(this.style == panel.STYLE_DEFAULT)
				this.element.style.width = this.rect.width + "px";
			else
				this.element.style.width = this.rect.width - 2 + "px";
		}
		
		// Vertical rect
		if(this.rect.anchorY == jaswin.ANCHOR_BOTH)
		{
			this.element.style.top = this.rect.top + "px";
			this.element.style.bottom = this.rect.bottom + "px";
		}
		else
		{
			if(this.rect.anchorY == jaswin.ANCHOR_TOP)
				this.element.style.top = this.rect.top + "px";
			else if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.bottom + "px";
			
			if(this.style == panel.STYLE_DEFAULT)
				this.element.style.height = this.rect.height + "px";
			else
				this.element.style.height = this.rect.height - 2 + "px";
		}
		
		// Overflow
		if(iOverflow == panel.OVERFLOW_HIDDEN)
			this.element.style.overflow = "hidden";
		else if(iOverflow == panel.OVERFLOW_SCROLL)
			this.element.style.overflow = "auto";
		
		// Container of Panel is itself;
		this.container = this.element;
	}
	panel.Panel.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var iCounter;
			var oParentWnd = jaswin.findParentWindow(this);
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
				this.controls[iCounter].destroy();
			
			if(this.contextMenu)
				this.contextMenu.destroy();
			
			delete panel.panels[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
			{
				sID = oParentWnd.id + "_" + this.id;
				if(this.popup)
					this.element.style.zIndex = 50 + jaswin.frame.windowCount + oParentWnd.modalLevel;
			}
			
			// Duplication check
			if((document.getElementById(sID)) || (panel.panels[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"panel.js", "jaswin.panel.Panel.setParent");
			this.element.id = sID;
			panel.panels[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this panel
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				var iCounter;
				
				for(iCounter = 0; iCounter < this.controls.length; iCounter++)
					this.controls[iCounter].enable(true);
			}
		},
		
		//==================================================
		//	Disable this panel
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			var iCounter;
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
			{
				if(jaswin.frame)
				{
					if(this.controls[iCounter] instanceof jaswin.frame.Window)
						continue;
					else
						this.controls[iCounter].disable(true);
				}
				else
					this.controls[iCounter].disable(true);
			}
		},
		
		//==================================================
		//	Show this panel
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this panel
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Visible or not
		//==================================================
		isVisible: function()
		{
			if(this.element.style.display == "none")
				return false;
			else
				return true;
		},
		
		//==================================================
		//	Set background color
		//==================================================
		setBackgroundColor: function(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BACKGROUND, this.id),
											"panel.js", "jaswin.panel.Panel.setBackgroundColor");
			
			this.container.style.backgroundColor = sColor;
		},
		
		//==================================================
		//	Set size
		//==================================================
		setSize: function(iWidth, iHeight)
		{
			if(typeof iWidth != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_WIDTH, this.id),
											"panel.js", "jaswin.panel.Panel.setSize");
			if(typeof iHeight != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_HEIGHT, this.id),
											"panel.js", "jaswin.panel.Panel.setSize");
			
			if(this.rect.anchorX != jaswin.ANCHOR_BOTH)
			{
				this.rect.width = iWidth;
				if(this.style == panel.STYLE_DEFAULT)
					this.element.style.width = this.rect.width + "px";
				else
					this.element.style.width = this.rect.width - 2 + "px";
			}
			if(this.rect.anchorY != jaswin.ANCHOR_BOTH)
			{
				this.rect.height = iHeight;
				if(this.style == panel.STYLE_DEFAULT)
					this.element.style.height = this.rect.height + "px";
				else
					this.element.style.height = this.rect.height - 2 + "px";
			}
		},
		
		//==================================================
		//	Add control to this panel
		//==================================================
		addControl: function(oControl, oParentElement)
		{
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, this.id),
											"panel.js", "jaswin.panel.Panel.addControl");
			
			var oParentWnd = jaswin.findParentWindow(this);
			
			if((jaswin.frame) && (oControl instanceof jaswin.frame.Window))
			{
				if(oParentWnd != global)
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_WINDOW_ALLOCATION, this.id),
												"panel.js", "jaswin.panel.Panel.addControl");
			}
			
			if(oParentElement)
			{
				if(!oParentElement.getElementsByTagName)
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_PARENT_ELEMENT, this.id),
												"panel.js", "jaswin.panel.Panel.addControl");
				
				if(!this.element.contains(oParentElement))
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_VALUE_PARENT_ELEMENT, this.id),
												"panel.js", "jaswin.panel.Panel.addControl");
			}
			
			// Append element and set parent
			if(oParentElement)
				oParentElement.appendChild(oControl.element);
			else
				this.container.appendChild(oControl.element);
			this.controls[this.controls.length] = oControl;
			oControl.setParent(this);
			if(jaswin.frame)
			{
				if((oControl instanceof jaswin.frame.Window)
				&& (oControl.isVisible()))
					oControl.setActive();
			}
			
			// Set tabstop
			if((oControl.tabIndex) && (oControl.tabIndex > 0))
			{
				oParentWnd.addTabstop(oControl);
				
				if((jaswin.frame) && (jaswin.frame.currentWindow))
				{
					if(oParentWnd != jaswin.frame.currentWindow)
						oControl.focusElement.tabIndex = -1;
				}
				else
				{
					if(oParentWnd != global)
						oControl.focusElement.tabIndex = -1;
				}
			}
		},
		
		//==================================================
		//	Remove control from this panel
		//==================================================
		removeControl: function(oControl)
		{
			var iCounter;
			
			if((oControl.tabIndex) && (oControl.tabIndex > 0))
			{
				var oParentWnd = jaswin.findParentWindow(this);
				
				oParentWnd.removeTabstop(oControl);
			}
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
			{
				if(this.controls[iCounter] == oControl)
				{
					this.controls.splice(iCounter, 1);
					oControl.destroy();
					return;
				}
			}
		},
		
		//==================================================
		//	Notify destroy window
		//==================================================
		notifyDestroyWindow: function(oWindow)
		{
			if((typeof oWindow != "object")
			|| (!oWindow instanceof jaswin.frame.Window))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_WINDOW, this.id),
											"panel.js", "jaswin.panel.Panel.notifyDestroyWindow");
			
			var iCounter;
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
			{
				if(this.controls[iCounter] == oWindow)
				{
					this.controls.splice(iCounter, 1);
					return;
				}
			}
		},
		
		//==================================================
		//	Set context menu to this panel
		//==================================================
		setContextMenu: function(oMenu, fpContext)
		{
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, this.id),
											"panel.js", "jaswin.panel.Panel.setContextMenu");
			
			if(!jaswin.menu)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "menu.js"),
											"panel.js", "jaswin.panel.Panel.setContextMenu");
			
			if((typeof oMenu != "object")
			|| (!oMenu instanceof jaswin.menu.Menu))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU, this.id),
											"panel.js", "jaswin.panel.Panel.setContextMenu");
			
			if(fpContext)
			{
				if(typeof fpContext != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CONTEXTMENU, this.id),
												"panel.js", "jaswin.panel.Panel.setContextMenu");
			}
			
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
			}
			
			var oParentWnd = jaswin.findParentWindow(this);
			oParentWnd.container.appendChild(oMenu.element);
			
			this.contextMenu = oMenu;
			oMenu.setParent(this, 0);
			
			if(fpContext)
				this.onContextMenu = fpContext;
			
			this.container.addEventListener("contextmenu", panelContextMenu, false);
		},
		
		//==================================================
		//	Remove context menu from this panel
		//==================================================
		removeContextMenu: function()
		{
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
				this.container.removeEventListener("contextmenu", panelContextMenu, false);
			}
		},
		
		//==================================================
		//	Show context menu
		//==================================================
		showContextMenu: function(e)
		{
			var oElement = e.target || e.srcElement;
			
			if(oElement != this.container)
				return;
			
			if(this.onContextMenu)
			{
				if(!this.onContextMenu(this))
					return;
			}
			
			this.contextMenu.showContext(e);
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetPanel
	//
	//	Summary		:	Get target panel from event
	//
	//==========================================================
	function getTargetPanel(e)
	{
		var oElement = e.target || e.srcElement;
		var oPanel = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oPanel = panel.panels[oElement.id];
				if(!oPanel)
				{
					oElement = oElement.parentNode;
					continue;
				}
				
				if(!oPanel instanceof panel.Panel)
				{
					oElement = oElement.parentNode;
					continue;
				}
				
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oPanel;
	}
	
	
	//==========================================================
	//
	//	Function	:	panelContextMenu
	//
	//	Summary		:	Event handler of context menu
	//
	//==========================================================
	function panelContextMenu(e)
	{
		var oPanel = null;
		
		oPanel = getTargetPanel(e);
		
		if(oPanel)
			oPanel.showContextMenu(e);
		
		return false;
	}
	
	
	//==========================================================
	//
	//	Class		:	SplitPanel
	//
	//	Summary		:	Class of SplitPanel
	//
	//==========================================================
	panel.SplitPanel = function(sID,			// * ID[String]
								oRect,			// * Size and position[ExpandableRect]
								iType,			// * Type of separation[int]
								bEnabled,		// * Enabled[boolean]
								iDelimiterPos,	// * Position of delimiter[int]
								bResizable,		// * Resizable or not[boolean]
								iaStyles,		// * Panel style[Array of int]
								iaOverflows,	// * Overflow[int]
								iMinPos,		// Minimum position of delimiter[int]
								iMaxPos)		// Maximum position of delimiter[int]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.ExpandableRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		
		if(typeof iType != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_SPLIT, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		if((iType != panel.TYPE_H_SPLIT)
		&& (iType != panel.TYPE_V_SPLIT))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_SPLIT, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		
		if(typeof iDelimiterPos != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_DELIMITER, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		
		if(typeof bResizable != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RESIZABLE, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		
		if(!Array.isArray(iaStyles))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		if(iaStyles.length < 2)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_NUMBER_STYLE, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		if(typeof iaStyles[0] != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID + "_0"),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		if(typeof iaStyles[1] != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID + "_1"),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		
		if(!Array.isArray(iaOverflows))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_OVERFLOW, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		if(iaOverflows.length < 2)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_NUMBER_OVERFLOW, sID),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		if(typeof iaOverflows[0] != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_OVERFLOW, sID + "_0"),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		if(typeof iaOverflows[1] != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_OVERFLOW, sID + "_1"),
										"panel.js", "jaswin.panel.SplitPanel.constructor");
		
		if(iMinPos)
		{
			if(typeof iMinPos != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MIN_POSITION, sID),
											"panel.js", "jaswin.panel.SplitPanel.constructor");
		}
		
		if(iMaxPos)
		{
			if(typeof iMaxPos != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MAX_POSITION, sID),
											"panel.js", "jaswin.panel.SplitPanel.constructor");
		}
		
		// Set properties
		this.id = sID;						// ID[String]
		this.rect = oRect;					// Size and position[ExpandableRect]
		this.type = iType;					// Separation type[int]
		this.delimiterPos = iDelimiterPos;	// Position of delimiter[int]
		this.minPos = -1;					// Minimum position of delimiter[int]
		this.maxPos = -1;					// Maximum position of delimiter[int]
		this.element = null;				// Window element[DOM Element]
		this.container = null;				// Container for client area[DOM Element]
		this.controls = new Array();		// Controls on this panel[Array of Control classes]
		this.delimiter = null;				// Delemiter element[DOM Element]
		this.parent = null;					// Parent class[global / window / panel class]
		this.enabled = bEnabled;			// Enabled or not[boolean]
		this.parentEnabled = true;			// Enabled or not[boolean]
		this.resizable = bResizable;		// Resizable or not[boolean]
		
		if(iMinPos) this.minPos = iMinPos;
		if(iMaxPos) this.maxPos = iMaxPos;
		
		// Create element
		this.element = document.createElement("div");
		this.element.className = "panel";
		
		// Horizontal rect
		if(this.rect.anchorX == jaswin.ANCHOR_BOTH)
		{
			this.element.style.left = this.rect.left + "px";
			this.element.style.right = this.rect.right + "px";
		}
		else
		{
			if(this.rect.anchorX == jaswin.ANCHOR_LEFT)
				this.element.style.left = this.rect.left + "px";
			else if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.right + "px";
			this.element.style.width = this.rect.width + "px";
		}
		
		// Vertical rect
		if(this.rect.anchorY == jaswin.ANCHOR_BOTH)
		{
			this.element.style.top = this.rect.top + "px";
			this.element.style.bottom = this.rect.bottom + "px";
		}
		else
		{
			if(this.rect.anchorY == jaswin.ANCHOR_TOP)
				this.element.style.top = this.rect.top + "px";
			else if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.bottom + "px";
			this.element.style.height = this.rect.height + "px";
		}
		
		// Container of Panel is itself;
		this.container = this.element;
		
		// Make first panel
		var oExpRect;
		var oPanel;
		
		if(this.type == panel.TYPE_H_SPLIT)
		{
			oExpRect = new jaswin.ExpandableRect(jaswin.ANCHOR_LEFT, jaswin.ANCHOR_BOTH,
												0, 0,
												0, 0,
												this.delimiterPos, 0);
		}
		else if(this.type == panel.TYPE_V_SPLIT)
		{
			oExpRect = new jaswin.ExpandableRect(jaswin.ANCHOR_BOTH, jaswin.ANCHOR_TOP,
												0, 0,
												0, 0,
												0, this.delimiterPos);
		}
		oPanel = new panel.Panel(this.id + "_0", oExpRect, iaStyles[0], true, iaOverflows[0]);
		this.container.appendChild(oPanel.element);
		this.controls[0] = oPanel;
		
		// Make delimiter
		this.delimiter = document.createElement("div");
		if(this.type == panel.TYPE_H_SPLIT)
		{
			this.delimiter.className = "panel-h-delimiter";
			this.delimiter.style.left = this.delimiterPos + "px";
		}
		else if(this.type == panel.TYPE_V_SPLIT)
		{
			this.delimiter.className = "panel-v-delimiter";
			this.delimiter.style.top = this.delimiterPos + "px";
		}
		this.container.appendChild(this.delimiter);
		
		if(this.enabled && this.resizable)
			this.delimiter.addEventListener("mousedown", startResizePanel, false);
		
		// Make second panel
		if(this.type == panel.TYPE_H_SPLIT)
		{
			oExpRect = new jaswin.ExpandableRect(jaswin.ANCHOR_BOTH, jaswin.ANCHOR_BOTH,
												this.delimiterPos + 2, 0,
												0, 0,
												0, 0);
		}
		else if(this.type == panel.TYPE_V_SPLIT)
		{
			oExpRect = new jaswin.ExpandableRect(jaswin.ANCHOR_BOTH, jaswin.ANCHOR_BOTH,
												0, 0,
												this.delimiterPos + 2, 0,
												0, 0);
		}
		oPanel = new panel.Panel(this.id + "_1", oExpRect, iaStyles[1], true, iaOverflows[1]);
		this.container.appendChild(oPanel.element);
		this.controls[1] = oPanel;
	}
	panel.SplitPanel.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			this.controls[0].destroy();
			this.controls[1].destroy();
			
			delete panel.panels[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (panel.panels[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"panel.js", "jaswin.panel.SplitPanel.setParent");
			if((document.getElementById(sID + "_0")) || (panel.panels[sID + "_0"]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID + "_0"),
											"panel.js", "jaswin.panel.SplitPanel.setParent");
			if((document.getElementById(sID + "_1")) || (panel.panels[sID + "_1"]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID + "_1"),
											"panel.js", "jaswin.panel.SplitPanel.setParent");
			
			this.element.id = sID;
			panel.panels[this.element.id] = this;
			oParentWnd[this.id] = this;
			this.controls[0].setParent(this);
			this.controls[1].setParent(this);
		},
		
		//==================================================
		//	Enable this panel
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				if(this.resizable)
					this.delimiter.addEventListener("mousedown", startResizePanel, false);
				
				var iCounter;
				
				for(iCounter = 0; iCounter < this.controls.length; iCounter++)
					this.controls[iCounter].enable(true);
			}
		},
		
		//==================================================
		//	Disable this panel
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.delimiter.removeEventListener("mousedown", startResizePanel, false);
			
			var iCounter;
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
				this.controls[iCounter].disable(true);
		},
		
		//==================================================
		//	Show this panel
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this panel
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Resizable or not
		//==================================================
		setResizable: function(bResizable)
		{
			if(typeof bResizable != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"panel.js", "jaswin.panel.SplitPanel.setResizable");
			if(this.resizable == bResizable)
				return;
			
			this.resizable = bResizable;
			if((this.resizable) && (this.enabled))
				this.delimiter.addEventListener("mousedown", startResizePanel, false);
			else if((!this.resizable) && (this.enabled))
				this.delimiter.removeEventListener("mousedown", startResizePanel, false);
		},
		
		//==================================================
		//	Get child panel
		//==================================================
		getChildPanel: function(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"panel.js", "jaswin.panel.SplitPanel.getChildPanel");
			if((iIndex != 0) && (iIndex != 1))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"panel.js", "jaswin.panel.SplitPanel.getChildPanel");
			
			return this.controls[iIndex];
		},
		
		//==================================================
		//	Resize panels
		//==================================================
		resizePanel: function(iGap)
		{
			if(this.delimiterPos + iGap < 0)
			{
				// Under 0
				this.element.removeEventListener("mouseup", endResizePanel, false);
				this.element.removeEventListener("mousemove", resizePanel, false);
				if(this.type == panel.TYPE_H_SPLIT)
					this.delimiter.className = "panel-h-delimiter";
				else if(this.type == panel.TYPE_V_SPLIT)
					this.delimiter.className = "panel-v-delimiter";
				oResizingDelimiter = null;
				return;
			}
			
			if(this.type == panel.TYPE_H_SPLIT)
			{
				if(this.delimiterPos + iGap + 2 > this.element.scrollWidth)
				{
					// Over size
					this.element.removeEventListener("mouseup", endResizePanel, false);
					this.element.removeEventListener("mousemove", resizePanel, false);
					this.delimiter.className = "panel-h-delimiter";
					oResizingDelimiter = null;
					return;
				}
			}
			else if(this.type == panel.TYPE_V_SPLIT)
			{
				if(this.delimiterPos + iGap + 2 > this.element.scrollHeight)
				{
					// Maximum
					this.element.removeEventListener("mouseup", endResizePanel, false);
					this.element.removeEventListener("mousemove", resizePanel, false);
					this.delimiter.className = "panel-v-delimiter";
					oResizingDelimiter = null;
					return;
				}
			}
			
			this.delimiterPos += iGap;
			
			if(this.type == panel.TYPE_H_SPLIT)
			{
				this.controls[0].rect.width = this.delimiterPos - 2;
				if(this.controls[0].style == panel.STYLE_DEFAULT)
					this.controls[0].element.style.width = this.delimiterPos + "px";
				else
					this.controls[0].element.style.width = this.delimiterPos - 2 + "px";
				this.delimiter.style.left = this.delimiterPos + "px";
				this.controls[1].rect.left = this.delimiterPos + 2;
				this.controls[1].element.style.left = (this.delimiterPos + 2) + "px";
			}
			else if(this.type == panel.TYPE_V_SPLIT)
			{
				this.controls[0].rect.height = this.delimiterPos - 2;
				if(this.controls[0].style == panel.STYLE_DEFAULT)
					this.controls[0].element.style.height = this.delimiterPos + "px";
				else
					this.controls[0].element.style.height = this.delimiterPos - 2 + "px";
				this.delimiter.style.top = this.delimiterPos + "px";
				this.controls[1].rect.top = this.delimiterPos + 2;
				this.controls[1].element.style.top = (this.delimiterPos + 2) + "px";
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetSplitPanel
	//
	//	Summary		:	Get target split panel from event
	//
	//==========================================================
	function getTargetSplitPanel(e)
	{
		var oElement = e.target || e.srcElement;
		var oPanel = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oPanel = panel.panels[oElement.id];
				if(!oPanel)
				{
					oElement = oElement.parentNode;
					continue;
				}
				
				if(!oPanel instanceof panel.SplitPanel)
				{
					oElement = oElement.parentNode;
					continue;
				}
				
				if(oPanel.delimiter != oResizingDelimiter)
				{
					oElement = oElement.parentNode;
					continue;
				}
				
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oPanel;
	}
	
	
	//==========================================================
	//
	//	Function	:	startResizePanel
	//
	//	Summary		:	Event handler of start resizing panel
	//
	//==========================================================
	function startResizePanel(e)
	{
		var oPanel = null;
		
		oResizingDelimiter = e.target || e.srcElement;
		
		// Target not found
		if(!oResizingDelimiter) return;
		
		oPanel = getTargetSplitPanel(e);
		if(!oPanel) return;
		
		if(oPanel.type == panel.TYPE_H_SPLIT)
		{
			iResizingPos = e.clientX;
			oResizingDelimiter.className = "panel-h-delimiter-on";
		}
		else if(oPanel.type == panel.TYPE_V_SPLIT)
		{
			iResizingPos = e.clientY;
			oResizingDelimiter.className = "panel-v-delimiter-on";
		}
		
		oPanel.element.addEventListener("mouseup", endResizePanel, false);
		oPanel.element.addEventListener("mousemove", resizePanel, false);
	}
	
	
	//==========================================================
	//
	//	Function	:	resizePanel
	//
	//	Summary		:	Event handler of resizing panel
	//
	//==========================================================
	function resizePanel(e)
	{
		var oElement = e.target || e.srcElement;
		var iCurPos;
		var oPanel = null;
		var oSelection = window.getSelection();
		
		oSelection.removeAllRanges();
		
		oPanel = getTargetSplitPanel(e);
		if((!oResizingDelimiter) || (!oPanel.element.contains(oElement)))
		{
			oPanel.element.removeEventListener("mouseup", endResizePanel, false);
			oPanel.element.removeEventListener("mousemove", resizePanel, false);
			
			if(oPanel.type == panel.TYPE_H_SPLIT)
				oResizingDelimiter.className = "panel-h-delimiter";
			else if(oPanel.type == panel.TYPE_V_SPLIT)
				oResizingDelimiter.className = "panel-v-delimiter";
			
			oResizingDelimiter = null;
			return;
		}
		
		if(oPanel.type == panel.TYPE_H_SPLIT)
			iCurPos = e.clientX;
		else if(oPanel.type == panel.TYPE_V_SPLIT)
			iCurPos = e.clientY;
		
		if((oPanel.minPos >= 0) && (iCurPos - iResizingPos < 0))
		{
			if(oPanel.delimiterPos + iCurPos - iResizingPos < oPanel.minPos)
			{
				// Under minimum position
				oPanel.element.removeEventListener("mouseup", endResizePanel, false);
				oPanel.element.removeEventListener("mousemove", resizePanel, false);
				
				if(oPanel.type == panel.TYPE_H_SPLIT)
					oResizingDelimiter.className = "panel-h-delimiter";
				else if(oPanel.type == panel.TYPE_V_SPLIT)
					oResizingDelimiter.className = "panel-v-delimiter";
				oResizingDelimiter = null;
				return;
			}
		}
		else if((oPanel.maxPos >= 0) && (iCurPos - iResizingPos > 0))
		{
			if(oPanel.delimiterPos + iCurPos - iResizingPos > oPanel.maxPos)
			{
				// Over maximum position
				oPanel.element.removeEventListener("mouseup", endResizePanel, false);
				oPanel.element.removeEventListener("mousemove", resizePanel, false);
				
				if(oPanel.type == panel.TYPE_H_SPLIT)
					oResizingDelimiter.className = "panel-h-delimiter";
				else if(oPanel.type == panel.TYPE_V_SPLIT)
					oResizingDelimiter.className = "panel-v-delimiter";
				oResizingDelimiter = null;
				return;
			}
		}
		
		oPanel.resizePanel(iCurPos - iResizingPos);
		iResizingPos = iCurPos;
	}
	
	
	//==========================================================
	//
	//	Function	:	endResizePanel
	//
	//	Summary		:	Event handler of end resizing panel
	//
	//==========================================================
	function endResizePanel(e)
	{
		var oPanel = null;
		
		oPanel = getTargetSplitPanel(e);
		oPanel.element.removeEventListener("mouseup", endResizePanel, false);
		oPanel.element.removeEventListener("mousemove", resizePanel, false);
		
		if(oPanel.type == panel.TYPE_H_SPLIT)
			oResizingDelimiter.className = "panel-h-delimiter";
		else if(oPanel.type == panel.TYPE_V_SPLIT)
			oResizingDelimiter.className = "panel-v-delimiter";
		
		oResizingDelimiter = null;
	}
	
	
	//==========================================================
	//
	//	Class		:	GroupBox
	//
	//	Summary		:	Class of GroupBox
	//
	//==========================================================
	panel.GroupBox = function(sID,				// * ID[String]
								oRect,			// * Size and position[ExpandableRect]
								sTitle,			// * Title[String]
								bEnabled)		// * Enabled[boolean]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"panel.js", "jaswin.panel.GroupBox.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.ExpandableRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"panel.js", "jaswin.panel.GroupBox.constructor");
		
		if(typeof sTitle != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_TITLE, sID),
										"panel.js", "jaswin.panel.GroupBox.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"panel.js", "jaswin.panel.GroupBox.constructor");
		
		// Set properties
		this.id = sID;					// ID[String]
		this.rect = oRect;				// Size and position[ExpandableRect]
		this.element = null;			// Window element[DOM Element]
		this.label = null;				// Title label[DOM Element]
		this.container = null;			// Container for client area[DOM Element]
		this.controls = new Array();	// Controls on this panel[Array of Control classes]
		this.contextMenu = null;		// Context menu on this panel[Menu class]
		this.onContextMenu = null;		// Function on right click[function(Argument:GroupBox class)]
		this.parent = null;				// Parent class[global / window / panel class]
		this.enabled = bEnabled;		// Enabled or not[boolean]
		this.parentEnabled = true;		// Enabled or not[boolean]
		
		// Create element
		this.element = document.createElement("div");
		this.element.className = "groupbox";
		
		// Horizontal rect
		if(this.rect.anchorX == jaswin.ANCHOR_BOTH)
		{
			this.element.style.left = this.rect.left + "px";
			this.element.style.right = this.rect.right + "px";
		}
		else
		{
			if(this.rect.anchorX == jaswin.ANCHOR_LEFT)
				this.element.style.left = this.rect.left + "px";
			else if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.right + "px";
			this.element.style.width = this.rect.width + "px";
		}
		
		// Vertical rect
		if(this.rect.anchorY == jaswin.ANCHOR_BOTH)
		{
			this.element.style.top = this.rect.top + "px";
			this.element.style.bottom = this.rect.bottom + "px";
		}
		else
		{
			if(this.rect.anchorY == jaswin.ANCHOR_TOP)
				this.element.style.top = this.rect.top + "px";
			else if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.bottom + "px";
			this.element.style.height = this.rect.height + "px";
		}
		
		var oLine = document.createElement("div");
		oLine.className = "groupbox-line";
		this.element.appendChild(oLine);
		
		this.label = document.createElement("span");
		this.label.className = "groupbox-title";
		this.label.innerHTML = sTitle;
		this.element.appendChild(this.label);
		
		this.container = document.createElement("div");
		this.container.className = "groupbox-container";
		this.element.appendChild(this.container);
	}
	panel.GroupBox.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var iCounter;
			var oParentWnd = jaswin.findParentWindow(this);
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
				this.controls[iCounter].destroy();
			
			if(this.contextMenu)
				this.contextMenu.destroy();
			
			delete panel.groupboxes[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (panel.groupboxes[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"panel.js", "jaswin.panel.GroupBox.setParent");
			this.element.id = sID;
			panel.groupboxes[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this groupbox
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.label.className = "groupbox-title";
				
				var iCounter;
				
				for(iCounter = 0; iCounter < this.controls.length; iCounter++)
					this.controls[iCounter].enable(true);
			}
		},
		
		//==================================================
		//	Disable this groupbox
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.label.className = "groupbox-title-disabled";
			
			var iCounter;
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
				this.controls[iCounter].disable(true);
		},
		
		//==================================================
		//	Show this groupbox
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this groupbox
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Add control to this groupbox
		//==================================================
		addControl: function(oControl)
		{
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, this.id),
											"panel.js", "jaswin.panel.GroupBox.addControl");
			
			if((jaswin.frame) && (oControl instanceof jaswin.frame.Window))
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_WINDOW_ALLOCATION, this.id),
												"panel.js", "jaswin.panel.GroupBox.addControl");
			
			this.container.appendChild(oControl.element);
			this.controls[this.controls.length] = oControl;
			oControl.setParent(this);
			
			if((oControl.tabIndex) && (oControl.tabIndex > 0))
			{
				var oParentWnd = jaswin.findParentWindow(this);
				
				oParentWnd.addTabstop(oControl);
				
				if((jaswin.frame) && (jaswin.frame.currentWindow))
				{
					if(oParentWnd != jaswin.frame.currentWindow)
						oControl.focusElement.tabIndex = -1;
				}
				else
				{
					if(oParentWnd != global)
						oControl.focusElement.tabIndex = -1;
				}
			}
		},
		
		//==================================================
		//	Remove control to this groupbox
		//==================================================
		removeControl: function(oControl)
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
			{
				if(this.controls[iCounter] == oControl)
				{
					this.controls.splice(iCounter, 1);
					oControl.destroy();
					return;
				}
			}
		},
		
		//==================================================
		//	Set context menu to this groupbox
		//==================================================
		setContextMenu: function(oMenu, fpContext)
		{
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, this.id),
											"panel.js", "jaswin.panel.GroupBox.setContextMenu");
			
			if(!jaswin.menu)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "menu.js"),
											"panel.js", "jaswin.panel.GroupBox.setContextMenu");
			
			if((typeof oMenu != "object")
			|| (!oMenu instanceof jaswin.menu.Menu))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU, this.id),
											"panel.js", "jaswin.panel.GroupBox.setContextMenu");
			
			if(fpContext)
			{
				if(typeof fpContext != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CONTEXTMENU, this.id),
												"panel.js", "jaswin.panel.GroupBox.setContextMenu");
			}
			
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
			}
			
			var oParentWnd = jaswin.findParentWindow(this);
			oParentWnd.container.appendChild(oMenu.element);
			
			this.contextMenu = oMenu;
			oMenu.setParent(this, 0);
			
			if(fpContext)
				this.onContextMenu = fpContext;
			
			this.container.addEventListener("contextmenu", groupBoxContextMenu, false);
		},
		
		//==================================================
		//	Remove context menu from this groupbox
		//==================================================
		removeContextMenu: function()
		{
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
				this.container.removeEventListener("contextmenu", groupBoxContextMenu, false);
			}
		},
		
		//==================================================
		//	Show context menu
		//==================================================
		showContextMenu: function(e)
		{
			var oElement = e.target || e.srcElement;
			
			if(oElement != this.container)
				return;
			
			if(this.onContextMenu)
			{
				if(!this.onContextMenu(this))
					return;
			}
			
			this.contextMenu.showContext(e);
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetGroupBox
	//
	//	Summary		:	Get target group box from event
	//
	//==========================================================
	function getTargetGroupBox(e)
	{
		var oElement = e.target || e.srcElement;
		var oGroupBox = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oGroupBox = panel.groupboxes[oElement.id];
				if(!oGroupBox)
				{
					oElement = oElement.parentNode;
					continue;
				}
				
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oGroupBox;
	}
	
	
	//==========================================================
	//
	//	Function	:	groupBoxContextMenu
	//
	//	Summary		:	Event handler of context menu
	//
	//==========================================================
	function groupBoxContextMenu(e)
	{
		var oGroupBox = null;
		
		oGroupBox = getTargetGroupBox(e);
		
		if(oGroupBox)
			oGroupBox.showContextMenu(e);
		
		return false;
	}
	
	
	//==========================================================
	//
	//	Class		:	Tab
	//
	//	Summary		:	Class of tab
	//
	//==========================================================
	panel.Tab = function(sID,				// * ID[String]
						oRect,				// * Size and position[ExpandableRect]
						iStyle,				// * Tab style[int]
						bEnabled,			// * Enabled[boolean]
						saLabels,			// Index labels[Array of string]
						iStartIndex,		// Start index of active tab[int]
						fpSelect)			// Function on select[function]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		var iCounter;
		var iCnt = 0;
		var iIndex = 0;
		
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"panel.js", "jaswin.panel.Tab.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.ExpandableRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"panel.js", "jaswin.panel.Tab.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"panel.js", "jaswin.panel.Tab.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"panel.js", "jaswin.panel.Tab.constructor");
		
		if((!iStyle & panel.STYLE_VARIABLE) && (saLabels == null))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, sID),
										"panel.js", "jaswin.panel.Tab.constructor");
		if(saLabels)
		{
			if(!Array.isArray(saLabels))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, sID),
											"panel.js", "jaswin.panel.Tab.constructor");
			if((!iStyle & panel.STYLE_VARIABLE) && (saLabels.length == 0))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_NUMBER_LABEL, sID),
											"panel.js", "jaswin.panel.Tab.constructor");
			iCnt = saLabels.length;
			for(iCounter = 0; iCounter < iCnt; iCounter++)
			{
				if(typeof saLabels[iCounter] != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, sID + "_1"),
												"panel.js", "jaswin.panel.Tab.constructor");
			}
		}
		
		if(iCnt)
		{
			if(iStartIndex)
			{
				if(typeof iStartIndex != "number")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, sID),
												"panel.js", "jaswin.panel.Tab.constructor");
				
				if((iStartIndex < 0) || (iStartIndex >= iCnt))
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, sID),
												"panel.js", "jaswin.panel.Tab.constructor");
				
				iIndex = iStartIndex;
			}
			else
				iIndex = 0;
		}
		else
			iIndex = 0;
		
		if(fpSelect)
		{
			if(typeof fpSelect != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CLICK, sID),
											"panel.js", "jaswin.panel.Tab.constructor");
		}
		
		// Set properties
		this.id = sID;						// ID[String]
		this.rect = oRect;					// Size and position[ExpandableRect]
		this.style = iStyle;				// Tab style[int]
		this.element = null;				// Window element[DOM Element]
		this.container = null;				// Container for client area[DOM Element]
		this.index = null;					// Element for indexes[DOM Element]
		this.indexContainer = null;			// Container for indexes[DOM Element]
		this.indexInner = null;				// Container inner for indexes[DOM Element]
		this.indexes = new Array();			// Indexes of tab[DOM Element]
		this.controls = new Array();		// Controls on this tab[Array of Panel classes]
		this.bgColors = new Array();		// Back ground color of tab items[String]
		this.selectedIndex = iIndex;		// Selected index[int]
		this.parent = null;					// Parent class[global / window / panel class]
		this.enabled = bEnabled;			// Enabled or not[boolean]
		this.parentEnabled = true;			// Enabled or not[boolean]
		this.scrollLeftButton = null;		// Scroll left index button[boolean]
		this.scrollRightButton = null;		// Scroll right index button[boolean]
		this.adjustIndexHeight = false;		// Adjust index height or not[boolean]
		this.onSelect = fpSelect;			// Function on click[function(Argument:int)]
		
		// Create element
		this.element = document.createElement("div");
		this.element.style.position = "absolute";
		
		// Horizontal rect
		if(this.rect.anchorX == jaswin.ANCHOR_BOTH)
		{
			this.element.style.left = this.rect.left + "px";
			this.element.style.right = this.rect.right + "px";
		}
		else
		{
			if(this.rect.anchorX == jaswin.ANCHOR_LEFT)
				this.element.style.left = this.rect.left + "px";
			else if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.right + "px";
			this.element.style.width = this.rect.width + "px";
		}
		
		// Vertical rect
		if(this.rect.anchorY == jaswin.ANCHOR_BOTH)
		{
			this.element.style.top = this.rect.top + "px";
			this.element.style.bottom = this.rect.bottom + "px";
		}
		else
		{
			if(this.rect.anchorY == jaswin.ANCHOR_TOP)
				this.element.style.top = this.rect.top + "px";
			else if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.bottom + "px";
			this.element.style.height = this.rect.height + "px";
		}
		
		// Create contents
		var oPanel;
		var oExpRect;
		
		this.container = document.createElement("div");
		this.container.className = "tab-content";
		this.element.appendChild(this.container);
		
		for(iCounter = 0; iCounter < iCnt; iCounter++)
		{
			oExpRect = new jaswin.ExpandableRect(jaswin.ANCHOR_BOTH, jaswin.ANCHOR_BOTH,
												0, 0,
												0, 0,
												0, 0);
			oPanel = new panel.Panel(this.id + "_" + iCounter, oExpRect,
									panel.STYLE_DEFAULT, true,
									panel.OVERFLOW_DEFAULT);
			
			if(iCounter != this.selectedIndex)
				oPanel.hide();
			this.container.appendChild(oPanel.element);
			this.controls[iCounter] = oPanel;
			this.bgColors[iCounter] = "";
		}
		
		// Create indexes
		this.index = document.createElement("div");
		this.index.className = "tab-index";
		this.indexContainer = document.createElement("div");
		this.indexContainer.className = "tab-index-container";
		this.index.appendChild(this.indexContainer);
		this.indexInner = document.createElement("div");
		this.indexInner.className = "tab-index-inner";
		if(this.style & panel.STYLE_ALIGN_RIGHT)
			this.indexInner.style.right = "0px";
		else
			this.indexInner.style.left = "0px";
		
		this.indexContainer.appendChild(this.indexInner);
		
		for(iCounter = 0; iCounter < iCnt; iCounter++)
		{
			this.indexes[iCounter] = document.createElement("span");
			if(iCounter == this.selectedIndex)
			{
				if(this.enabled)
					this.indexes[iCounter].className = "tab-index-on";
				else
					this.indexes[iCounter].className = "tab-index-on-disabled";
			}
			else
			{
				if(this.enabled)
					this.indexes[iCounter].className = "tab-index-off";
				else
					this.indexes[iCounter].className = "tab-index-off-disabled";
			}
			this.indexes[iCounter].innerHTML = saLabels[iCounter];
			
			if(this.style & panel.STYLE_VARIABLE)
			{
				var oCloseSpan = document.createElement("span");
				if(this.enabled)
					oCloseSpan.className = "tab-index-close";
				else
					oCloseSpan.className = "tab-index-close-disabled";
				oCloseSpan.innerHTML = "x";
				this.indexes[iCounter].appendChild(oCloseSpan);
				
				if(this.enabled)
					oCloseSpan.addEventListener("click", closeTabEvent, false);
			}
			
			if(this.enabled)
				this.indexes[iCounter].addEventListener("click", selectTabEvent, false);
			
			this.indexInner.appendChild(this.indexes[iCounter]);
		}
		
		// Scroll button
		if(jaswin.button)
		{
			var oRect = new jaswin.FixedRect(jaswin.ANCHOR_LEFT, jaswin.ANCHOR_TOP,
											0, 4, 12, 18);
			this.scrollLeftButton = new jaswin.button.Button("Btn" + this.id + "_IndexLeft", oRect,
															jaswin.button.STYLE_DEFAULT | jaswin.button.STYLE_ICON,
															true,
															jaswin.ICON_TRIANGLE_LEFT, null,
															0, tabIndexScrollLeft);
			this.scrollLeftButton.hide();
			this.scrollLeftButton.content.childNodes[0].style.left = "-4px";
			this.index.appendChild(this.scrollLeftButton.element);
			
			oRect = new jaswin.FixedRect(jaswin.ANCHOR_RIGHT, jaswin.ANCHOR_TOP,
										0, 4, 12, 18);
			this.scrollRightButton = new jaswin.button.Button("Btn" + this.id + "_IndexRight", oRect,
															jaswin.button.STYLE_DEFAULT | jaswin.button.STYLE_ICON,
															true,
															jaswin.ICON_TRIANGLE_RIGHT, null,
															0, tabIndexScrollRight);
			this.scrollRightButton.hide();
			this.scrollRightButton.content.childNodes[0].style.left = "2px";
			this.index.appendChild(this.scrollRightButton.element);
			
			if(this.enabled)
			{
				this.index.addEventListener("mouseover", tabIndexMouseMove, false);
				this.index.addEventListener("mouseout", tabIndexMouseOut, false);
			}
		}
		this.element.appendChild(this.index);
	}
	panel.Tab.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var iCounter;
			var oParentWnd = jaswin.findParentWindow(this);
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
				this.controls[iCounter].destroy();
			
			if(this.scrollLeftButton)
				this.scrollLeftButton.destroy();
			if(this.scrollRightButton)
				this.scrollRightButton.destroy();
			
			delete panel.tabs[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (panel.tabs[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"panel.js", "jaswin.panel.Tab.setParent");
			this.element.id = sID;
			panel.tabs[this.element.id] = this;
			oParentWnd[this.id] = this;
			
			// Adjust index padding
			var iCounter;
			var iHeight;
			var iPadding;
			for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
			{
				if(iCounter == this.selectedIndex)
				{
					iHeight = this.indexes[iCounter].offsetHeight;
					iPadding = Math.floor((22 - iHeight) / 2);
					if(iPadding > 0)
					{
						this.indexes[iCounter].style.paddingTop = iPadding + 2 + "px";
						this.indexes[iCounter].style.paddingBottom = iPadding + "px";
						this.adjustIndexHeight = true;
					}
				}
				else
				{
					iHeight = this.indexes[iCounter].offsetHeight;
					iPadding = Math.floor((20 - iHeight) / 2);
					if(iPadding > 0)
					{
						this.indexes[iCounter].style.paddingTop = iPadding + "px";
						this.indexes[iCounter].style.paddingBottom = iPadding + "px";
						this.adjustIndexHeight = true;
					}
				}
				
				// Set parent
				this.controls[iCounter].setParent(this);
			}
			this.indexInner.style.top = "3px";
			
			if(this.scrollLeftButton)
				this.scrollLeftButton.setParent(this);
			if(this.scrollRightButton)
				this.scrollRightButton.setParent(this);
			
			// Adjust container position
			if(this.indexes.length == 0)
			{
				var oDummy = document.createElement("span");
				oDummy.className = "tab-index-on";
				oDummy.innerHTML = " ";
				this.indexInner.appendChild(oDummy);
				this.container.style.top = oDummy.offsetHeight - 1 + "px";
				this.indexInner.removeChild(oDummy);
			}
			else
				this.container.style.top = this.indexes[this.selectedIndex].offsetHeight - 1 + "px";
			
			// Set index scroll button
			var iInnerWidth = 0;
			
			for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
				iInnerWidth += this.indexes[iCounter].offsetWidth;
			
			this.indexInner.style.width = (iInnerWidth + 4) + "px";
		},
		
		//==================================================
		//	Enable this tab
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				var iCounter;
				
				for(iCounter = 0; iCounter < this.controls.length; iCounter++)
				{
					this.indexes[iCounter].addEventListener("click", selectTabEvent, false);
					this.controls[iCounter].enable(true);
					
					if(iCounter == this.selectedIndex)
						this.indexes[iCounter].className = "tab-index-on";
					else
						this.indexes[iCounter].className = "tab-index-off";
					
					if(this.style & panel.STYLE_VARIABLE)
					{
						var oCloseSpan = this.indexes[iCounter].getElementsByTagName("span")[0];
						
						oCloseSpan.className = "tab-index-close";
						oCloseSpan.addEventListener("click", closeTabEvent, false);
					}
				}
				
				if(jaswin.button)
				{
					this.index.addEventListener("mouseover", tabIndexMouseMove, false);
					this.index.addEventListener("mouseout", tabIndexMouseOut, false);
				}
			}
		},
		
		//==================================================
		//	Disable this tab
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			var iCounter;
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
			{
				this.indexes[iCounter].removeEventListener("click", selectTabEvent, false);
				this.controls[iCounter].disable(true);
				
				if(iCounter == this.selectedIndex)
					this.indexes[iCounter].className = "tab-index-on-disabled";
				else
					this.indexes[iCounter].className = "tab-index-off-disabled";
				
				if(this.style & panel.STYLE_VARIABLE)
				{
					var oCloseSpan = this.indexes[iCounter].getElementsByTagName("span")[0];
					
					oCloseSpan.className = "tab-index-close-disabled";
					oCloseSpan.removeEventListener("click", closeTabEvent, false);
				}
			}
			
			if(jaswin.button)
			{
				this.index.removeEventListener("mouseover", tabIndexMouseMove, false);
				this.index.removeEventListener("mouseout", tabIndexMouseOut, false);
			}
		},
		
		//==================================================
		//	Show this tab
		//==================================================
		show: function()
		{
			this.element.style.visibility = "visible";
		},
		
		//==================================================
		//	Hide this tab
		//==================================================
		hide: function()
		{
			this.element.style.visibility = "hidden";
		},
		
		//==================================================
		//	Get child panel
		//==================================================
		getChildPanel: function(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"panel.js", "jaswin.panel.Tab.getChildPanel");
			if((iIndex < 0) || (iIndex >= this.indexes.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"panel.js", "jaswin.panel.Tab.getChildPanel");
			
			return this.controls[iIndex];
		},
		
		//==================================================
		//	Set background color
		//==================================================
		setBackgroundColor: function(iIndex, sColor)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"panel.js", "jaswin.panel.Tab.setBgColor");
			if((iIndex < 0) || (iIndex >= this.indexes.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"panel.js", "jaswin.panel.Tab.setBgColor");
			
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BACKGROUND, this.id),
											"panel.js", "jaswin.panel.Tab.setBgColor");
			
			this.bgColors[iIndex] = sColor;
			if(iIndex == this.selectedIndex)
			{
				this.indexes[iIndex].style.backgroundColor = sColor;
				this.indexes[iIndex].style.borderBottomColor = sColor;
			}
			this.controls[iIndex].element.style.backgroundColor = sColor;
		},
		
		//==================================================
		//	Select a tab index
		//==================================================
		select: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter = 0;
			var iIndex = -1;
			
			if(!oElement)
				return;
			
			for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
			{
				if(this.indexes[iCounter].contains(oElement))
				{
					iIndex = iCounter;
					break;
				}
			}
			
			if(iIndex != -1)
				this.selectByIndex(iIndex);
		},
		
		//==================================================
		//	Select a tab by index
		//==================================================
		selectByIndex: function(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"panel.js", "jaswin.panel.Tab.getChildPanel");
			if((iIndex < 0) || (iIndex >= this.indexes.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"panel.js", "jaswin.panel.Tab.getChildPanel");
			
			var iCounter = 0;
			var iOrgIndex = this.selectedIndex;
			var bFound = false;
			var iLeft = 0;
			var iRight = 0;
			var iScroll = 0;
			
			for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
			{
				if(iCounter == iIndex)
				{
					this.indexes[iCounter].className = "tab-index-on";
					if(this.bgColors[iCounter] != "")
					{
						this.indexes[iCounter].style.backgroundColor = this.bgColors[iCounter];
						this.indexes[iCounter].style.borderBottom = "solid 1px " + this.bgColors[iCounter];
					}
					if(this.adjustIndexHeight && (iCounter != iOrgIndex))
						this.indexes[iCounter].style.paddingTop = parseInt(this.indexes[iCounter].style.paddingTop) + 2 + "px";
					
					this.controls[iCounter].element.style.display = "block";
					this.selectedIndex = iCounter;
					bFound = true;
				}
				else
				{
					this.indexes[iCounter].className = "tab-index-off";
					
					if(this.adjustIndexHeight && (iCounter == iOrgIndex))
						this.indexes[iCounter].style.paddingTop = parseInt(this.indexes[iCounter].style.paddingTop) - 2 + "px";
					
					if(this.bgColors[iCounter] != "")
						this.indexes[iCounter].style.borderBottom = "none";
					
					this.controls[iCounter].element.style.display = "none";
					if(!bFound)
						iLeft += this.indexes[iCounter].offsetWidth;
				}
			}
			
			iRight = iLeft + this.indexes[iIndex].offsetWidth;
			if(iLeft - this.indexContainer.scrollLeft < 0)
			{
				iScroll = iLeft;
				this.indexContainer.scrollLeft = iScroll;
			}
			else if(iRight - this.indexContainer.scrollLeft > this.index.offsetWidth)
			{
				iScroll = iRight - this.index.offsetWidth + 2;
				this.indexContainer.scrollLeft = iScroll;
			}
			
			if(this.onSelect)
				this.onSelect(this.selectedIndex);
		},
		
		//==================================================
		//	Find tab index with label
		//==================================================
		findIndex: function(sLabel)
		{
			var iCounter = 0;
			
			for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
			{
				if(this.indexes[iCounter].childNodes[0].nodeValue == sLabel)
					return iCounter;
			}
			
			return -1;
		},
		
		//==================================================
		//	Mouse move a tab index
		//==================================================
		hoverIndex: function()
		{
			var iLastScroll = 0;
			var iCurScroll = this.indexContainer.scrollLeft;
			var iIndexWidth = this.index.offsetWidth;
			var iInnerWidth = 0;
			var iCounter;
			
			for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
				iInnerWidth += this.indexes[iCounter].offsetWidth;
			iLastScroll = iInnerWidth - iIndexWidth + 2;
			
			if(iCurScroll)
				this.scrollLeftButton.show();
			if(iInnerWidth > iIndexWidth)
			{
				if(iCurScroll != iLastScroll)
					this.scrollRightButton.show();
			}
		},
		
		//==================================================
		//	Mouse out a tab index
		//==================================================
		outIndex: function(e)
		{
			var oElement = e.target || e.srcElement;
			var oRect = this.index.getBoundingClientRect();
			
			if((e.clientX < oRect.left)
			|| (e.clientX > oRect.right)
			|| (e.clientY < oRect.top)
			|| (e.clientY > oRect.bottom)
			|| (this.indexInner.contains(oElement)))
			{
				this.scrollLeftButton.hide();
				this.scrollRightButton.hide();
			}
		},
		
		//==================================================
		//	Scroll left a tab index
		//==================================================
		scrollLeft: function()
		{
			var iCurScroll = this.indexContainer.scrollLeft;
			
			if(iCurScroll - 64 < 0)
				this.indexContainer.scrollLeft = 0;
			else
				this.indexContainer.scrollLeft = iCurScroll - 64;
		},
		
		//==================================================
		//	Scroll right a tab index
		//==================================================
		scrollRight: function()
		{
			var iLastScroll = 0;
			var iCurScroll = this.indexContainer.scrollLeft;
			var iIndexWidth = this.index.offsetWidth;
			var iInnerWidth = 0;
			var iCounter;
			
			for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
				iInnerWidth += this.indexes[iCounter].offsetWidth;
			iLastScroll = iInnerWidth - iIndexWidth + 2;
			
			if(iCurScroll + 64 > iLastScroll)
				this.indexContainer.scrollLeft = iLastScroll;
			else
				this.indexContainer.scrollLeft += 64;
		},
		
		//==================================================
		//	Add tab
		//==================================================
		addTab: function(sLabel)
		{
			// Check parameter
			if(!this.style & panel.STYLE_VARIABLE)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, this.id),
											"panel.js", "jaswin.panel.Tab.addTab");
			
			if(typeof sLabel != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"panel.js", "jaswin.panel.Tab.addTab");
			
			// Get ID
			var iTmp = 0;
			var sID = null;
			
			while(true)
			{
				sID = this.id + "_" + iTmp;
				if(!panel.panels[sID])
					break;
				
				iTmp++;
			}
			
			// Create content
			var oExpRect = new jaswin.ExpandableRect(jaswin.ANCHOR_BOTH, jaswin.ANCHOR_BOTH,
													0, 0,
													0, 0,
													0, 0);
			var oPanel = new panel.Panel(sID, oExpRect,
										panel.STYLE_DEFAULT, true,
										panel.OVERFLOW_DEFAULT);
			if(this.indexes.length == 0)
				this.selectedIndex = 0;
			else
				oPanel.hide();
			this.container.appendChild(oPanel.element);
			this.controls[this.controls.length] = oPanel;
			oPanel.setParent(this);
			
			// Create index
			var oSpan = document.createElement("span");
			this.indexes[this.indexes.length] = oSpan;
			if(this.indexes.length == 1)
			{
				if(this.enabled)
					oSpan.className = "tab-index-on";
				else
					oSpan.className = "tab-index-on-disabled";
			}
			else
			{
				if(this.enabled)
					oSpan.className = "tab-index-off";
				else
					oSpan.className = "tab-index-off-disabled";
			}
			oSpan.innerHTML = sLabel;
			
			var oCloseSpan = document.createElement("span");
			if(this.enabled)
				oCloseSpan.className = "tab-index-close";
			else
				oCloseSpan.className = "tab-index-close-disabled";
			oCloseSpan.innerHTML = "x";
			oSpan.appendChild(oCloseSpan);
			
			if(this.enabled)
				oCloseSpan.addEventListener("click", closeTabEvent, false);
			
			if(this.enabled)
				oSpan.addEventListener("click", selectTabEvent, false);
			
			this.indexInner.appendChild(oSpan);
			
			if(this.parent)
			{
				// Set index scroll button
				var iInnerWidth = 0;
				var iCounter;
				
				for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
					iInnerWidth += this.indexes[iCounter].offsetWidth;
				
				this.indexInner.style.width = (iInnerWidth + 4) + "px";
			}
			
			this.bgColors[this.bgColors.length] = "";
		},
		//==================================================
		//	Close tab
		//==================================================
		closeTab: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter;
			var iIndex;
			
			// Find close tab
			for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
			{
				if(this.indexes[iCounter].contains(oElement))
				{
					this.indexInner.removeChild(this.indexInner.childNodes[iCounter]);
					this.indexes.splice(iCounter, 1);
					
					this.controls[iCounter].destroy();
					this.controls.splice(iCounter, 1);
					this.bgColors.splice(iCounter, 1);
					
					iIndex = iCounter;
					break;
				}
			}
			
			// Set index scroll button
			var iInnerWidth = 0;
			
			for(iCounter = 0; iCounter < this.indexes.length; iCounter++)
				iInnerWidth += this.indexes[iCounter].offsetWidth;
			
			this.indexInner.style.width = (iInnerWidth + 4) + "px";
			
			// Reset selected tab
			if((iIndex == this.selectedIndex) && (this.indexes.length > 0))
			{
				if(iIndex >= this.indexes.length)
					iIndex--;
				
				this.indexes[iIndex].className = "tab-index-on";
				
				if(this.adjustIndexHeight)
					this.indexes[iIndex].style.paddingTop = parseInt(this.indexes[iIndex].style.paddingTop) + 2 + "px";
				
				this.controls[iIndex].element.style.display = "block";
				this.selectedIndex = iIndex;
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetTab
	//
	//	Summary		:	Get target tab element from event
	//
	//==========================================================
	function getTargetTab(e)
	{
		var oElement = e.target || e.srcElement;
		
		if(oElement == null)
			return null;
		
		for(var sKey in panel.tabs)
		{
			if(panel.tabs[sKey].index.contains(oElement))
				return panel.tabs[sKey];
		}
		
		return null;
	}
	
	
	//==========================================================
	//
	//	Function	:	selectTabEvent
	//
	//	Summary		:	Event handler of selecting a tab index
	//
	//==========================================================
	function selectTabEvent(e)
	{
		var oTab = null;
		
		oTab = getTargetTab(e);
		
		if(oTab)
			oTab.select(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	tabIndexMouseMove
	//
	//	Summary		:	Event handler of mouse move a tab index
	//
	//==========================================================
	function tabIndexMouseMove(e)
	{
		var oTab = null;
		
		oTab = getTargetTab(e);
		
		if(oTab)
			oTab.hoverIndex();
	}
	
	
	//==========================================================
	//
	//	Function	:	tabIndexMouseOut
	//
	//	Summary		:	Event handler of mouse out a tab index
	//
	//==========================================================
	function tabIndexMouseOut(e)
	{
		var oTab = null;
		
		oTab = getTargetTab(e);
		
		if(oTab)
			oTab.outIndex(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	tabIndexScrollLeft
	//
	//	Summary		:	Event handler of scroll left a index
	//
	//==========================================================
	function tabIndexScrollLeft(oBtn)
	{
		oBtn.parent.scrollLeft();
	}
	
	
	//==========================================================
	//
	//	Function	:	tabIndexScrollRight
	//
	//	Summary		:	Event handler of scroll right a index
	//
	//==========================================================
	function tabIndexScrollRight(oBtn)
	{
		oBtn.parent.scrollRight();
	}
	
	
	//==========================================================
	//
	//	Function	:	closeTabEvent
	//
	//	Summary		:	Event handler of click tab close
	//
	//==========================================================
	function closeTabEvent(e)
	{
		var oTab = null;
		
		oTab = getTargetTab(e);
		
		if(oTab)
			oTab.closeTab(e);
	}
	
})(jaswin.panel = jaswin.panel || {});
