//==================================================================
//
//	File name	:	progressbar.js
//
//	Summary		:	ProgressBar class
//
//	Ceated		:	2014/10/04
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (progressbar) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	progressbar.STYLE_DEFAULT = 0;
	progressbar.STYLE_RATIO = 1;
	progressbar.STYLE_DETAIL = 2;
	
	// Grobal variables
	progressbar.progressBars = {};
	
	
	//==========================================================
	//
	//	Class		:	ProgressBar
	//
	//	Summary		:	Class of progressbar
	//
	//==========================================================
	progressbar.ProgressBar = function(sID,				// * ID[String]
										oRect,			// * Size and position[FixedRect]
										iStyle,			// * Item style[int]
										bEnabled,		// * Enabled or not[boolean]
										fTotal,			// * Total value[float or int]
										iDigit,			// * Digit number[int]
										sUnit)			// Unit[String]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		if((iStyle != progressbar.STYLE_DEFAULT)
		&& (iStyle != progressbar.STYLE_RATIO)
		&& (iStyle != progressbar.STYLE_DETAIL))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sID),
										"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		
		if(typeof fTotal != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_TOTAL, sID),
										"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		
		if(typeof iDigit != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_DIGIT, sID),
										"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		if((iDigit < 0) || (iDigit > 20))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_DIGIT, sID),
										"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		
		if(iStyle == progressbar.STYLE_DETAIL)
		{
			if(!sUnit)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_MUST_UNIT, sID),
											"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		}
		if(sUnit)
		{
			if(typeof sUnit != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_UNIT, sID),
											"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
		}
		
		// Set properties
		this.id = sID;					// ID[String]
		this.rect = oRect;				// Size and position[FixedRect]
		this.style = iStyle;			// ProgressBar style[int]
		this.unit = sUnit;				// Unit[string]
		this.total = fTotal;			// Total value[float or int]
		this.value = 0;					// Current value[float or int]
		this.digit = iDigit;			// Digit number[int]
		this.enabled = bEnabled;		// Enabled or not[boolean]
		this.parentEnabled = true;		// Enabled or not[boolean]
		this.element = null;			// This element[DOM Element]
		this.complete = null;			// Element of completed[DOM Element]
		this.label = null;				// Label element for ratio or detail[DOM Element]
		
		// Create element
		this.element = document.createElement("div");
		this.element.className = "progress-bar";
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		
		this.element.style.width = this.rect.width - 2 + "px";		// 2:border
		this.element.style.height = this.rect.height - 2 + "px";	// 2:border
		
		this.complete = document.createElement("div");
		this.complete.className = "progress-on";
		this.complete.style.width = "0%";
		this.element.appendChild(this.complete);
		
		if(this.style != progressbar.STYLE_DEFAULT)
		{
			this.label = document.createElement("span");
			this.label.className = "progress-text";
			this.label.innerHTML = "0%";
			
			if(this.rect.height > 20)
				this.label.style.marginTop = (this.rect.height - 20) / 2 + "px";
			
			if(this.style == progressbar.STYLE_DETAIL)
			{
				this.label.innerHTML += " (" + this.value.toFixed(this.digit)
										+ "/" + this.total.toFixed(this.digit) + this.unit + ")";
			}
			this.element.appendChild(this.label);
		}
	}
	progressbar.ProgressBar.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			delete progressbar.progressBars[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (progressbar.progressBars[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"progressbar.js", "jaswin.progressbar.ProgressBar.setParent");
			this.element.id = sID;
			progressbar.progressBars[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this progressbar
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.complete.className = "progress-on";
				if(this.label)
					this.label.className = "progress-text";
			}
		},
		
		//==================================================
		//	Disable this progressbar
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.complete.className = "progress-on-disabled";
			if(this.label)
				this.label.className = "progress-text-disabled";
		},
		
		//==================================================
		//	Show this progressbar
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this progressbar
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Set progress value
		//==================================================
		setValue: function(fValue)
		{
			if(typeof fValue != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"progressbar.js", "jaswin.progressbar.ProgressBar.setValue");
			
			if(!this.enabled)
				return;
			
			this.value = fValue;
			if(this.value > this.total)
				this.value = this.total;
			
			var iRatio = (this.value / this.total * 100).toFixed(0);
			this.complete.style.width = iRatio + "%";
			
			if(this.style != progressbar.STYLE_DEFAULT)
			{
				this.label.innerHTML = iRatio + "%";
				
				if(this.style == progressbar.STYLE_DETAIL)
				{
					this.label.innerHTML += " (" + this.value.toFixed(this.digit)
											+ "/" + this.total.toFixed(this.digit) + this.unit + ")";
				}
			}
		},
		
		//==================================================
		//	Set total value
		//==================================================
		setTotal: function(fTotal)
		{
			if(typeof fTotal != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOTAL, this.id),
											"progressbar.js", "jaswin.progressbar.ProgressBar.setTotal");
			
			if(!this.enabled)
				return;
			
			this.total = fTotal;
			if(this.value > this.total)
				this.value = this.total;
			
			var iRatio = this.getRatio();
			this.complete.style.width = iRatio + "%";
			
			if(this.style != progressbar.STYLE_DEFAULT)
			{
				this.label.innerHTML = iRatio + "%";
				
				if(this.style == progressbar.STYLE_DETAIL)
				{
					this.label.innerHTML += " (" + this.value.toFixed(this.digit)
											+ "/" + this.total.toFixed(this.digit) + this.unit + ")";
				}
			}
		},
		
		//==================================================
		//	Get ratio
		//==================================================
		getRatio: function()
		{
			var iRatio = 0;
			if(this.total)
				iRatio = (this.value / this.total * 100).toFixed(0);
			
			return iRatio;
		}
	}
	
})(jaswin.progressbar = jaswin.progressbar || {});
