//==================================================================
//
//	File name	:	sliderbar.js
//
//	Summary		:	Sliderbar class
//
//	Ceated		:	2014/10/04
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (sliderbar) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	sliderbar.GAUGE_TYPE_SHORT = 0;
	sliderbar.GAUGE_TYPE_LONG = 1;
	
	// Grobal variables
	sliderbar.sliderBars = {};
	
	// Private variables
	var oSliding = null;
	var iSlidingPos = 0;
	var iLastGap = 0;
	
	
	//==========================================================
	//
	//	Class		:	SliderGauge
	//
	//	Summary		:	Class of slider gauge
	//
	//==========================================================
	function SliderGauge(iType,		// * Type of gauge[int]
						iPoint,		// * Point of gauge[int]
						iPos)		// * Position of gauge[int]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof iType != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_GAUGE,
										"sliderbar.js", "jaswin.sliderbar.SliderGauge.constructor");
		if((iType != sliderbar.GAUGE_TYPE_SHORT)
		&& (iType != sliderbar.GAUGE_TYPE_LONG))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_VALUE_GAUGE,
										"sliderbar.js", "jaswin.sliderbar.SliderGauge.constructor");
		
		if(typeof iPoint != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_POINT,
										"sliderbar.js", "jaswin.sliderbar.SliderGauge.constructor");
		
		if(typeof iPos != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_POSITION,
										"sliderbar.js", "jaswin.sliderbar.SliderGauge.constructor");
		
		this.type = iType;
		this.point = iPoint;
		this.pos = iPos;
	}
	
	
	//==========================================================
	//
	//	Class		:	SliderBar
	//
	//	Summary		:	Class of SliderBar
	//
	//==========================================================
	sliderbar.SliderBar = function(sID,				// * ID[String]
									oRect,			// * Size and position[FixedRect],
									iMinPoint,		// * Minimum point[int]
									iMaxPoint,		// * Maximum point[int]
									iStartPoint,	// * Start point[int]
									bEnabled,		// * Enabled or not[boolean]
									bGauge,			// * Show gauge or not[boolean]
									iShortGauge,	// Short gauge unit[int]
									iLongGauge,		// Long gauge unit[int]
									bBindGauge,		// Bind pointer to gauge or not[boolean]
									iTabIndex,		// Tab index[int]
									fpChange)		// Function on change
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		
		if(typeof iMinPoint != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_MIN_POINT, sID),
										"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		
		if(typeof iMaxPoint != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_MAX_POINT, sID),
										"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		
		if(iMinPoint >= iMaxPoint)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_MIN_MAX, sID),
										"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		
		if(typeof iStartPoint != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_START_POINT, sID),
										"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		
		if(typeof bGauge != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_SHOW_GAUGE, sID),
										"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		
		if(bGauge)
		{
			if(typeof iShortGauge != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_SHORT_GAUGE, sID),
											"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
			if(iShortGauge < 0)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_SHORT_GAUGE, sID),
											"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
			
			if(typeof iLongGauge != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LONG_GAUGE, sID),
											"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
			if(iLongGauge < 0)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_LONG_GAUGE, sID),
											"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
			
			if((iShortGauge > 0)
			&& (iLongGauge > 0))
			{
				if(iLongGauge % iShortGauge)
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_VALUE_GAUGE_UNIT, sID),
												"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
			}
			
			if(typeof bBindGauge != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BIND_GAUGE, sID),
											"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		}
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		}
		
		if(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
											"sliderbar.js", "jaswin.sliderbar.SliderBar.constructor");
		}
		
		// Set properties
		this.id = sID;					// ID[String]
		this.rect = oRect;				// Size and position[FixedRect]
		this.minPoint = iMinPoint;		// Minimum point[int]
		this.maxPoint = iMaxPoint;		// Maximum point[int]
		this.point = iStartPoint;		// Start point[int]
		this.bindGauge = bBindGauge;	// Bind pointer to gauge or not[boolean]
		this.enabled = bEnabled;		// Enabled or not[boolean]
		this.parentEnabled = true;		// Enabled or not[boolean]
		this.tabIndex = -1;				// Tab index[int]
		this.onChange = fpChange;		// Function on click[function(Argument:SliderBar class)]
		this.element = null;			// This element[DOM Element]
		this.focusElement = null;		// Focus element[DOM Element]
		this.bar = null;				// Bar element[DOM Element]
		this.pointer = null;			// Pointer element[DOM Element]
		this.gauges = new Array();		// Array of gauge class
		this.parent = null;				// Parent class[global / window / panel class]
		
		this.getPosition = function()
		{
			// Get position of pointer by point
			return Math.round((this.point - this.minPoint) / (this.maxPoint - this.minPoint) * (this.rect.width - 2 - 9));
		}
		
		// Create element
		this.element = document.createElement("div");
		this.element.style.position = "absolute";
		this.focusElement = this.element;
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		
		this.element.style.width = this.rect.width - 2 + "px";		// 2:border
		
		// Create gauge
		if(bGauge)
		{
			this.element.style.height = "20px";
			
			var iCounter;
			var iCnt;
			var oGauge;
			var oSpan;
			var iType;
			var iPoint;
			var iPos;
			
			if((iShortGauge == 0) && (iLongGauge == 0))
				iCnt = 0;
			else if(iShortGauge == 0)
				iCnt = Math.floor((this.maxPoint - this.minPoint) / iLongGauge);
			else
				iCnt = Math.floor((this.maxPoint - this.minPoint) / iShortGauge);
			
			// First gauge
			oGauge = new SliderGauge(sliderbar.GAUGE_TYPE_LONG, this.minPoint, 0);
			oSpan = document.createElement("span");
			oSpan.className = "v-line-inset";
			oSpan.style.position = "absolute";
			oSpan.style.left = "4px";
			oSpan.style.top = "0px";
			oSpan.style.height = "8px";
			this.element.appendChild(oSpan);
			this.gauges[0] = oGauge;
			
			for(iCounter = 1; iCounter < iCnt; iCounter++)
			{
				if(iShortGauge != 0)
				{
					if(iCounter * iShortGauge + this.minPoint >= this.maxPoint)
						break;
				}
				else if(iLongGauge != 0)
				{
					if(iCounter * iLongGauge + this.minPoint >= this.maxPoint)
						break;
				}
				
				if(iLongGauge == 0)
					iType = sliderbar.GAUGE_TYPE_SHORT;
				else if(iShortGauge == 0)
					iType = sliderbar.GAUGE_TYPE_LONG;
				else if((iCounter * iShortGauge) % iLongGauge == 0)
					iType = sliderbar.GAUGE_TYPE_LONG;
				else
					iType = sliderbar.GAUGE_TYPE_SHORT;
				
				if(iShortGauge != 0)
					iPoint = this.minPoint + (iCounter * iShortGauge);
				else if(iLongGauge != 0)
					iPoint = this.minPoint + (iCounter * iLongGauge);
				
				iPos = Math.round((iPoint - this.minPoint) / (this.maxPoint - this.minPoint) * (this.rect.width - 2 - 9));
				
				oGauge = new SliderGauge(iType, iPoint, iPos);
				
				oSpan = document.createElement("span");
				oSpan.className = "v-line-inset";
				oSpan.style.position = "absolute";
				oSpan.style.left = oGauge.pos + 4 + "px";
				if(oGauge.type == sliderbar.GAUGE_TYPE_LONG)
				{
					oSpan.style.top = "0px";
					oSpan.style.height = "8px";
				}
				else
				{
					oSpan.style.top = "4px";
					oSpan.style.height = "4px";
				}
				this.element.appendChild(oSpan);
				this.gauges[iCounter] = oGauge;
			}
			
			// Last gauge
			oGauge = new SliderGauge(sliderbar.GAUGE_TYPE_LONG, this.maxPoint, this.rect.width - 2 - 9);
			oSpan = document.createElement("span");
			oSpan.className = "v-line-inset";
			oSpan.style.position = "absolute";
			oSpan.style.left = oGauge.pos + 4 + "px";
			oSpan.style.top = "0px";
			oSpan.style.height = "8px";
			this.element.appendChild(oSpan);
			this.gauges[this.gauges.length] = oGauge;
		}
		else
			this.element.style.height = "11px";
		
		// Create bar
		this.bar = document.createElement("div");
		this.bar.className = "sliderbar";
		this.bar.style.left = "3px";
		if(bGauge)
			this.bar.style.top = "13px";
		else
			this.bar.style.top = "4px";
		this.bar.style.width = this.rect.width - 2 - 8 + "px";
		this.element.appendChild(this.bar);
		
		// Create pointer
		this.pointer = document.createElement("span");
		if(this.enabled)
			this.pointer.className = "sliderbar-pointer";
		else
			this.pointer.className = "sliderbar-pointer-disabled";
		
		this.pointer.style.left = this.getPosition() + "px";
		if(bGauge)
			this.pointer.style.top = "9px";
		else
			this.pointer.style.top = "0px";
		this.element.appendChild(this.pointer);
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Event binding
		if(this.enabled)
		{
			this.bar.addEventListener("click", clickSliderBar, false);
			this.pointer.addEventListener("mousedown", startSlidePointer, false);
			this.element.addEventListener("keydown", keyEvent, false);
		}
	}
	sliderbar.SliderBar.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			delete sliderbar.sliderBars[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (sliderbar.sliderBars[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"sliderbar.js", "jaswin.sliderbar.SliderBar.setParent");
			this.element.id = sID;
			sliderbar.sliderBars[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this sliderbar
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.element.tabIndex = this.tabIndex;
				this.pointer.className = "sliderbar-pointer";
				this.bar.addEventListener("click", clickSliderBar, false);
				this.pointer.addEventListener("mousedown", startSlidePointer, false);
				this.element.addEventListener("keydown", keyEvent, false);
			}
		},
		
		//==================================================
		//	Disable this sliderbar
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.element.tabIndex = -1;
			this.pointer.className = "sliderbar-pointer-disabled";
			this.bar.removeEventListener("click", clickSliderBar, false);
			this.pointer.removeEventListener("mousedown", startSlidePointer, false);
			this.element.removeEventListener("keydown", keyEvent, false);
		},
		
		//==================================================
		//	Show this sliderbar
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this sliderbar
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Slide pointer of this sliderbar
		//==================================================
		slide: function(iGap)
		{
			var iTmpPos;
			
			iTmpPos = parseInt(this.pointer.style.left) + iGap;
			if(iTmpPos < 0)
				iTmpPos = 0;
			else if(iTmpPos > this.rect.width - 2 - 9)
				iTmpPos = this.rect.width - 2 - 9;
			
			this.pointer.style.left = iTmpPos + "px";
			this.point = Math.round(iTmpPos / (this.rect.width - 2 - 9) * (this.maxPoint - this.minPoint)) + this.minPoint;
			
			if(this.onChange) this.onChange(this);
		},
		
		//==================================================
		//	Set point directly
		//==================================================
		setPoint: function(iValue)
		{
			if(typeof iValue != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"sliderbar.js", "jaswin.sliderbar.SliderBar.setPoint");
			
			if(iValue < this.minPoint)
				this.point = this.minPoint;
			else if(iValue > this.maxPoint)
				this.point = this.maxPoint;
			else
				this.point = iValue;
			
			this.pointer.style.left = this.getPosition() + "px";
			if(this.onChange) this.onChange(this);
		},
		
		//==================================================
		//	Click sliderbar
		//==================================================
		clickBar: function(e)
		{
			var oRect;
			var iClickPos;
			var iPointerPos;
			var bToLeft = true;
			var iCounter = 0;
			var iIndex = 0;
			
			oRect = this.bar.getBoundingClientRect();
			iClickPos = e.clientX - oRect.left - 3;
			iPointerPos = parseInt(this.pointer.style.left);
			if(iClickPos < iPointerPos)
				bToLeft = true;
			else if(iClickPos >= iPointerPos + 9)
				bToLeft = false;
			else
				return;
			
			if(this.gauges.length == 0)
			{
				if(bToLeft)
				{
					this.point--;
					if(this.point < this.minPoint)
						this.point = this.minPoint;
				}
				else
				{
					this.point++;
					if(this.point > this.maxPoint)
						this.point = this.maxPoint;
				}
			}
			else
			{
				if(bToLeft)
					iIndex = 0;
				else
					iIndex = this.gauges.length - 1;
				
				for(iCounter = 0; iCounter < this.gauges.length; iCounter++)
				{
					if(bToLeft)
					{
						if(this.gauges[iCounter].point < this.point)
							iIndex = iCounter;
						else
							break;
					}
					else
					{
						if(this.gauges[iCounter].point > this.point)
						{
							iIndex = iCounter;
							break;
						}
					}
				}
				this.point = this.gauges[iIndex].point;
			}
			this.pointer.style.left = this.getPosition() + "px";
			if(this.onChange) this.onChange(this);
		},
		
		//==================================================
		//	End of slidingbar
		//==================================================
		endSliding: function()
		{
			if((this.gauges.length > 0) && (!this.bindGauge))
				return;
			
			var iCounter = 0;
			var iIndex = -1;
			var iIndex2 = -1;
			var iMinGap = -1;
			var iTmp = 0;
			
			for(iCounter = 0; iCounter < this.gauges.length; iCounter++)
			{
				iTmp = Math.abs(this.point - this.gauges[iCounter].point);
				if((iMinGap == -1) || (iTmp < iMinGap))
				{
					iMinGap = iTmp;
					iIndex = iCounter;
				}
				else if(iTmp == iMinGap)
				{
					iIndex2 = iCounter;
				}
			}
			if(iIndex == -1)
				return;
			
			if(iIndex2 != -1)
			{
				if(iLastGap > 0)
					iIndex = iIndex2;
			}
			
			this.point = this.gauges[iIndex].point;
			this.pointer.style.left = this.getPosition() + "px";
			if(this.onChange) this.onChange(this);
		},
		
		//==================================================
		//	Keydown on slidingbar
		//==================================================
		keyDown: function(e)
		{
			if(e.keyCode == 37)
			{
				// Left
				if(this.point <= this.minPoint)
					return true;
				
				if(this.gauges.length == 0)
				{
					// No gauge
					this.point--;
					this.pointer.style.left = this.getPosition() + "px";
					if(this.onChange) this.onChange(this);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				else
				{
					var iCounter;
					
					if(e.ctrlKey)
					{
						// With Ctrl key
						for(iCounter = this.gauges.length - 1; iCounter >= 0; iCounter--)
						{
							if(this.point > this.gauges[iCounter].point)
							{
								this.point = this.gauges[iCounter].point;
								this.pointer.style.left = this.getPosition() + "px";
								if(this.onChange) this.onChange(this);
								e.preventDefault();
								e.stopPropagation();
								return false;
							}
						}
					}
					else
					{
						this.point--;
						this.pointer.style.left = this.getPosition() + "px";
						if(this.onChange) this.onChange(this);
						e.preventDefault();
						e.stopPropagation();
						return false;
					}
				}
			}
			else if(e.keyCode == 39)
			{
				// Right
				if(this.point >= this.maxPoint)
					return true;
				
				if(this.gauges.length == 0)
				{
					// No gauge
					this.point++;
					this.pointer.style.left = this.getPosition() + "px";
					if(this.onChange) this.onChange(this);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				else
				{
					var iCounter;
					
					if(e.ctrlKey)
					{
						// With Ctrl key
						for(iCounter = 0; iCounter < this.gauges.length; iCounter++)
						{
							if(this.point < this.gauges[iCounter].point)
							{
								this.point = this.gauges[iCounter].point;
								this.pointer.style.left = this.getPosition() + "px";
								if(this.onChange) this.onChange(this);
								e.preventDefault();
								e.stopPropagation();
								return false;
							}
						}
					}
					else
					{
						this.point++;
						this.pointer.style.left = this.getPosition() + "px";
						if(this.onChange) this.onChange(this);
						e.preventDefault();
						e.stopPropagation();
						return false;
					}
				}
			}
			
			return true;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetSliderBar
	//
	//	Summary		:	Get target sliderbar element from event
	//
	//==========================================================
	function getTargetSliderBar(e)
	{
		var oElement = e.target || e.srcElement;
		var oSliderBar = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oSliderBar = sliderbar.sliderBars[oElement.id];
				if(!oSliderBar)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oSliderBar;
	}
	
	
	//==========================================================
	//
	//	Function	:	startSlidePointer
	//
	//	Summary		:	Event handler of start sliding pointer
	//
	//==========================================================
	function startSlidePointer(e)
	{
		oSliding = getTargetSliderBar(e);
		if(!oSliding)
			return;
		
		iLastGap = 0;
		
		iSlidingPos = e.clientX;
		global.element.addEventListener("mouseup", endSlidePointer, false);
		global.element.addEventListener("mouseout", outSlidePointer, false);
		global.element.addEventListener("mousemove", slidePointer, false);
	}
	
	
	//==========================================================
	//
	//	Function	:	slidePointer
	//
	//	Summary		:	Event handler of sliding pointer
	//
	//==========================================================
	function slidePointer(e)
	{
		var oTarget = null;
		var iCurX;
		var iCurY;
		var oSliderBar = null;
		var oRect = null;
		var oSelection = window.getSelection();
		
		oSelection.removeAllRanges();
		
		if(!oSliding)
		{
			endSlidePointer(e);
			return;
		}
		
		iCurX = e.clientX;
		iCurY = e.clientY;
		
		oRect = oSliding.element.getBoundingClientRect();
		if((iCurX < oRect.left)
		|| (iCurX > oRect.right)
		|| (iCurY < oRect.top)
		|| (iCurY > oRect.bottom))
		{
			endSlidePointer(e);
			return;
		}
		
		iLastGap = iCurX - iSlidingPos;
		oSliding.slide(iLastGap);
		iSlidingPos = iCurX;
	}
	
	
	//==========================================================
	//
	//	Function	:	outSlidePointer
	//
	//	Summary		:	Event handler of mouse out slider bar
	//
	//==========================================================
	function outSlidePointer(e)
	{
		var oRect = null;
		
		if(!oSliding)
			return;
		
		oRect = oSliding.element.getBoundingClientRect();
		
		var iCurX;
		var iCurY;
		
		iCurX = e.clientX;
		iCurY = e.clientY;
		
		if((iCurX < oRect.left)
		|| (iCurX > oRect.right)
		|| (iCurY < oRect.top)
		|| (iCurY > oRect.bottom))
		{
			endSlidePointer(e);
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	endSlidePointer
	//
	//	Summary		:	Event handler of end sliding pointer
	//
	//==========================================================
	function endSlidePointer(e)
	{
		global.element.removeEventListener("mouseup", endSlidePointer, false);
		global.element.removeEventListener("mouseout", outSlidePointer, false);
		global.element.removeEventListener("mousemove", slidePointer, false);
		
		if(oSliding)
			oSliding.endSliding();
		
		oSliding = null;
		iLastGap = 0;
	}
	
	
	//==========================================================
	//
	//	Function	:	clickSliderBar
	//
	//	Summary		:	Event handler of click sliderbar
	//
	//==========================================================
	function clickSliderBar(e)
	{
		var oSliderBar = null;
		
		oSliderBar = getTargetSliderBar(e);
		
		if(oSliderBar)
			oSliderBar.clickBar(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	keyEvent
	//
	//	Summary		:	Event handler of keydown sliderbar
	//
	//==========================================================
	function keyEvent(e)
	{
		var oSliderBar = null;
		
		oSliderBar = getTargetSliderBar(e);
		
		if(oSliderBar)
			return oSliderBar.keyDown(e);
	}
})(jaswin.sliderbar = jaswin.sliderbar || {});
