//==================================================================
//
//	File name	:	textbox.js
//
//	Summary		:	TextBox class
//
//	Ceated		:	2014/10/10
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (textbox) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	textbox.STYLE_DEFAULT = 0;
	textbox.STYLE_FLAT = 1;
	textbox.STYLE_NOWRAP = 2;
	textbox.STYLE_TAB_INPUT = 4;
	textbox.STYLE_LINE_NO = 8;
	textbox.STYLE_HIGHLIGHTING = 16;
	textbox.TYPE_NONE = 0;
	textbox.TYPE_NUMBER_ALPHA = 1;
	textbox.TYPE_NUMBER = 2;
	textbox.TYPE_FORMATTED_NUMBER = 3;
	textbox.TYPE_PASSWORD = 4;
	textbox.IME_AUTO = 0;
	textbox.IME_ACTIVE = 1;
	textbox.IME_INACTIVE = 2;
	textbox.IME_DISABLED = 3;
	textbox.MARK_CHARACTERS = {58:':', 59:';', 64:'@',
								186:':', 187:';', 188:',', 189:'-', 190:'.', 191:'/', 192:'@',
								219:'[', 220:'\\', 221:']', 222:'^', 226:'\\'};
	textbox.SHIFTED_CHARACTERS = {49:'!', 50:'\"', 51:'#', 52:'$', 53:'%', 54:'&', 55:'\'', 56:'(', 57:')',
									58:'*', 59:'+', 64:'`',
									186:'*', 187:'+', 188:'<', 189:'=', 190:'>', 191:'?', 192:'`',
									219:'{', 220:'|', 221:'}', 222:'~', 226:'_'};
	
	// Grobal variables
	textbox.textBoxes = {};
	textbox.editBoxes = {};
	
	
	//==========================================================
	//
	//	Class		:	TextBox
	//
	//	Summary		:	Class of textbox
	//
	//==========================================================
	textbox.TextBox = function(sID,				// * ID[String]
								oRect,			// * Size and position[FixedRect]
								iStyle,			// * Textbox style[int]
								bEnabled,		// * Enabled or not[boolean]
								bReadOnly,		// Read only or not[boolean]
								sValue,			// Initial value[String]
								iMaxLength,		// Max length[int]
								sTooltip,		// Tool tip text[String]
								iTabIndex,		// Tab index[int]
								iType,			// Text box type[int]
								iDigit,			// Number digit[int]
								iImeMode,		// IME mode[int]
								fpChange,		// Function on change[function](Textbox)
								fpFocus,		// Function on focus[function](Textbox)
								fpBlur,			// Function on blur[function](Textbox)
								fpInput)		// Function on input[function](Textbox)
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"textbox.js", "jaswin.textbox.TextBox.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.FixedRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"textbox.js", "jaswin.textbox.TextBox.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"textbox.js", "jaswin.textbox.TextBox.constructor");
		
		if((iStyle != textbox.STYLE_DEFAULT)
		&& (iStyle != textbox.STYLE_FLAT))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sID),
										"textbox.js", "jaswin.textbox.TextBox.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"textbox.js", "jaswin.textbox.TextBox.constructor");
		
		if((typeof bReadOnly != "undefined") && (bReadOnly != null))
		{
			if(typeof bReadOnly != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_READONLY, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		if(sValue)
		{
			if(typeof sValue != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		if(iMaxLength)
		{
			if(typeof iMaxLength != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MAX_LENGTH, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		if(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		if(iType)
		{
			if(typeof iType != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TYPE, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
			
			if((iType != textbox.TYPE_NONE)
			&& (iType != textbox.TYPE_NUMBER_ALPHA)
			&& (iType != textbox.TYPE_NUMBER)
			&& (iType != textbox.TYPE_FORMATTED_NUMBER)
			&& (iType != textbox.TYPE_PASSWORD))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_TYPE, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		if(iDigit)
		{
			if(iType == textbox.TYPE_FORMATTED_NUMBER)
			{
				if(typeof iDigit != "number")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_DIGIT, sID),
												"textbox.js", "jaswin.textbox.TextBox.constructor");
				if(iDigit < 0)
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_VALUE_DIGIT, sID),
												"textbox.js", "jaswin.textbox.TextBox.constructor");
			}
		}
		
		if(iImeMode)
		{
			if(typeof iImeMode != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_IME_MODE, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
			
			if((iImeMode != textbox.IME_AUTO)
			&& (iImeMode != textbox.IME_ACTIVE)
			&& (iImeMode != textbox.IME_INACTIVE)
			&& (iImeMode != textbox.IME_DISABLED))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_IME_MODE, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		if(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		if(fpFocus)
		{
			if(typeof fpFocus != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_FOCUS, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		if(fpBlur)
		{
			if(typeof fpBlur != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_BLUR, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		if(fpInput)
		{
			if(typeof fpInput != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_INPUT, sID),
											"textbox.js", "jaswin.textbox.TextBox.constructor");
		}
		
		// Set properties
		this.id = sID;							// ID[String]
		this.rect = oRect;						// Size and position[FixedRect]
		this.style = iStyle;					// TextBox style[int]
		this.maxLength = iMaxLength;			// Max length[int]
		this.readOnly = bReadOnly;				// Read only or not[boolean]
		this.enabled = bEnabled;				// Enabled or not[boolean]
		this.parentEnabled = true;				// Enabled or not[boolean]
		if(iType)								// Text box type[int]
			this.type = iType;
		else
			this.type = textbox.TYPE_NONE;
		if(iDigit)								// Number digit[int]
			this.digit = iDigit;
		else
			this.digit = 0;
		this.onChange = fpChange;				// Function on change[function](Textbox)
		this.onFocus = fpFocus;					// Function on focus[function](Textbox)
		this.onBlur = fpBlur;					// Function on blur[function](Textbox)
		this.onInput = fpInput;					// Function on input[function](Textbox)
		this.element = null;					// This element[DOM Element]
		this.focusElement = null;				// Focus element[DOM Element]
		this.tabIndex = -1;						// Tab index[int]
		this.parent = null;						// Parent class[global / window / panel class]
		this.backgroundColor = "";				// Back ground color on normal[string]
		this.focusedBackColor = "";				// Back ground color on focus[string]
		this.unusableCharacters = null;			// Unusable characters[string]
		this.suggestionList = null;				// List box for suggestion[ListBox]
		
		// Create element
		this.element = document.createElement("input");
		if(this.type == textbox.TYPE_PASSWORD)
			this.element.type = "password";
		else
			this.element.type = "text";
		if(this.maxLength)
			this.element.maxLength = this.maxLength;
		
		if(this.style == textbox.STYLE_FLAT)
			this.element.className = "textbox-flat";
		else
			this.element.className = "textbox";
		
		if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
			this.element.style.right = this.rect.posX + "px";
		else
			this.element.style.left = this.rect.posX + "px";
		
		if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
			this.element.style.bottom = this.rect.posY + "px";
		else
			this.element.style.top = this.rect.posY + "px";
		
		if(this.style == textbox.STYLE_FLAT)
			this.element.style.width = this.rect.width - 2 + "px";				// 2:padding
		else
			this.element.style.width = this.rect.width - 2 - 2 + "px";			// 2:padding 2:border
		
		this.element.style.height = "16px";
		
		if(!this.enabled)
			this.element.disabled = true;
		
		if(this.readOnly)
			this.element.readOnly = true;
		
		if((this.type == textbox.TYPE_NUMBER_ALPHA)
		|| (this.type == textbox.TYPE_NUMBER)
		|| (this.type == textbox.TYPE_FORMATTED_NUMBER)
		|| (this.type == textbox.TYPE_PASSWORD))
		{
			this.element.style.imeMode = "disabled";
			
			if((this.type == textbox.TYPE_NUMBER)
			|| (this.type == textbox.TYPE_FORMATTED_NUMBER))
				this.element.style.textAlign = "right";
		}
		else
		{
			if(iImeMode == textbox.IME_ACTIVE)
				this.element.style.imeMode = "active";
			else if(iImeMode == textbox.IME_INACTIVE)
				this.element.style.imeMode = "inactive";
			else if(iImeMode == textbox.IME_DISABLED)
				this.element.style.imeMode = "disabled";
		}
		
		// Initial value
		if(sValue)
		{
			var fValue;
			
			if((this.type == textbox.TYPE_NUMBER)
			|| (this.type == textbox.TYPE_FORMATTED_NUMBER))
			{
				if(sValue.indexOf("."))
					fValue = parseFloat(sValue);
				else
					fValue = parseInt(sValue);
				
				if(!isNaN(fValue))
				{
					if(this.type == textbox.TYPE_NUMBER)
						this.element.value = "" + fValue;
					else
						this.element.value = formatNumber("" + fValue, this.digit);
				}
			}
			else
				this.element.value = sValue;
		}
		
		this.focusElement = this.element;
		
		// Tooltip
		if((sTooltip) && (sTooltip.length > 0))
			this.element.placeholder = sTooltip;
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Event binding
		if(this.enabled)
		{
			this.focusElement.addEventListener("focus", textBoxFocusEvent, false);
			this.focusElement.addEventListener("blur", textBoxBlurEvent, false);
			this.element.addEventListener("keydown", textBoxKeyDownEvent, false);
			this.element.addEventListener("change", textBoxChangeEvent, false);
			this.element.addEventListener("input", textBoxInputEvent, false);
		}
	}
	textbox.TextBox.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			delete textbox.textBoxes[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (textbox.textBoxes[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"textbox.js", "jaswin.textbox.TextBox.setParent");
			this.element.id = sID;
			textbox.textBoxes[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this textbox
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.element.tabIndex = this.tabIndex;
				this.element.disabled = false;
				
				this.focusElement.addEventListener("focus", textBoxFocusEvent, false);
				this.focusElement.addEventListener("blur", textBoxBlurEvent, false);
				this.element.addEventListener("keydown", textBoxKeyDownEvent, false);
				this.element.addEventListener("change", textBoxChangeEvent, false);
				this.element.addEventListener("input", textBoxInputEvent, false);
			}
		},
		
		//==================================================
		//	Disable this textbox
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.element.tabIndex = -1;
			this.element.disabled = true;
			
			this.focusElement.removeEventListener("focus", textBoxFocusEvent, false);
			this.focusElement.removeEventListener("blur", textBoxBlurEvent, false);
			this.element.removeEventListener("keydown", textBoxKeyDownEvent, false);
			this.element.removeEventListener("change", textBoxChangeEvent, false);
			this.element.removeEventListener("input", textBoxInputEvent, false);
		},
		
		//==================================================
		//	Set this textbox read only or not
		//==================================================
		setReadOnly: function(bReadOnly)
		{
			if(typeof bReadOnly != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_READONLY, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setReadOnly");
			
			this.readOnly = bReadOnly;
			this.element.readOnly = bReadOnly;
		},
		
		//==================================================
		//	Show this textbox
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this textbox
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Focus to this textbox
		//==================================================
		focus: function()
		{
			if(!this.enabled)
				return;
			
			if(this.style == textbox.STYLE_DEFAULT)
				this.element.className = "textbox-focus";
			
			this.element.style.backgroundColor = this.focusedBackColor;
			
			if(this.type == textbox.TYPE_FORMATTED_NUMBER)
				this.element.value = this.element.value.replace(/,/g, "");
			
			this.element.select();
			
			if(this.onFocus)
				this.onFocus(this);
		},
		
		//==================================================
		//	Blur from this textbox
		//==================================================
		blur: function()
		{
			if(!this.enabled)
				return;
			
			if((this.type == textbox.TYPE_FORMATTED_NUMBER)
			&& (this.element.value != ""))
			{
				var sValue = formatNumber(this.element.value, this.digit);
				
				if(this.element.value != sValue)
					this.element.value = sValue;
			}
			
			if(this.style == textbox.STYLE_DEFAULT)
				this.element.className = "textbox";
			
			this.element.style.backgroundColor = this.backgroundColor;
			
			if(this.onBlur)
				this.onBlur(this);
		},
		
		//==================================================
		//	Input to this textbox
		//==================================================
		input: function()
		{
			if(this.onInput)
				this.onInput(this);
		},
		
		//==================================================
		//	Keydown this textbox
		//==================================================
		keyDown: function(e)
		{
			if((this.suggestionList)
			&& (this.suggestionList.element.style.display != "none")
			&& ((e.keyCode == 13) || (e.keyCode == 32) || (e.keyCode == 38) || (e.keyCode == 40)))
			{
				// When suggestion list is shown and key down enter/space or arrow up/down
				this.suggestionList.keyDownList(e);
			}
			
			if((this.suggestionList)
			&& (this.suggestionList.element.style.display != "none")
			&& (e.keyCode == 27))
			{
				// When suggestion list is shown and key down esc
				this.suggestionList.hide();
			}
			
			if((this.unusableCharacters)
			&& (this.unusableCharacters.length))
			{
				// Unusable characters
				var iCounter;
				var sUnusableChar;
				var sCurChar;
				
				for(iCounter = 0; iCounter < this.unusableCharacters.length; iCounter++)
				{
					sUnusableChar = this.unusableCharacters.charAt(iCounter);
					if((e.shiftKey) && !((e.keyCode >= 65) && (e.keyCode <= 90)))
					{
						// Shifted mark
						sCurChar = textbox.SHIFTED_CHARACTERS["" + e.keyCode];
						if((sCurChar) && (sCurChar == sUnusableChar))
						{
							e.preventDefault();
							e.stopPropagation();
							return false;
						}
					}
					else
					{
						if((e.keyCode >= 65) && (e.keyCode <= 90))
						{
							// [A-Z][a-z]
							sCurChar = String.fromCharCode(e.keyCode);
							if(sCurChar.toUpperCase() == sUnusableChar.toUpperCase())
							{
								e.preventDefault();
								e.stopPropagation();
								return false;
							}
						}
						else
						{
							// Others
							sCurChar = textbox.MARK_CHARACTERS["" + e.keyCode];
							if((sCurChar) && (sCurChar == sUnusableChar))
							{
								e.preventDefault();
								e.stopPropagation();
								return false;
							}
						}
					}
				}
			}
			
			if((this.type == textbox.TYPE_NUMBER)
			|| (this.type == textbox.TYPE_FORMATTED_NUMBER))
			{
				// Number only
				if(((e.keyCode < 48) || (e.keyCode > 57))
				&& ((e.keyCode < 96) || (e.keyCode > 105))
				&& ((e.keyCode < 33) || (e.keyCode > 46))
				&& (e.keyCode != 8)
				&& (e.keyCode != 9))
				{
					if(((e.keyCode == 190) && !e.shiftKey) || (e.keyCode == 110))
					{
						// Period
						if(this.element.value.indexOf(".") == -1)
							return true;
					}
					
					if(((e.keyCode == 189) && !e.shiftKey)
					|| ((e.keyCode == 173) && !e.shiftKey)
					|| (e.keyCode == 109))
					{
						// Minus
						if(((this.element.selectionEnd == 0) && (this.element.value.indexOf("-") == -1))
						|| ((this.element.selectionStart == 0) && (this.element.selectionEnd == this.element.value.length)))
							return true;
					}
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				
				if((e.keyCode >= 48) && (e.keyCode <= 57) && e.shiftKey)
				{
					// Prevent Shift on number key
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				
				if(((e.keyCode == 48) && !e.shiftKey) || (e.keyCode == 96))
				{
					// Zero
					if(this.element.value == "")
						return true;
					if((this.element.value.charAt(0) == "0")
					|| ((this.element.value.length >= 2) && (this.element.value.substring(0, 2) == "-0")))
					{
						if((this.element.value.indexOf(".") == -1)
						|| (this.element.selectionEnd <= this.element.value.indexOf(".")))
						{
							e.preventDefault();
							e.stopPropagation();
							return false;
						}
					}
				}
			}
			else if(this.type == textbox.TYPE_NUMBER_ALPHA)
			{
				// Number or Alphabet only
				if(((e.keyCode < 48) || (e.keyCode > 57))
				&& ((e.keyCode < 65) || (e.keyCode > 90))
				&& ((e.keyCode < 96) || (e.keyCode > 105))
				&& ((e.keyCode < 33) || (e.keyCode > 46))
				&& (e.keyCode != 8)
				&& (e.keyCode != 9))
				{
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				
				if((e.keyCode >= 48) && (e.keyCode <= 57) && e.shiftKey)
				{
					// Prevent Shift on number key
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
			
			if(e.keyCode == 9)
			{
				if((this.suggestionList)
				&& (this.suggestionList.element.style.display != "none"))
					this.suggestionList.hide();
				
				window.getSelection().removeAllRanges();
			}
			
			return true;
		},
		
		//==================================================
		//	Change this textbox
		//==================================================
		change: function()
		{
			if(this.onChange)
				this.onChange(this);
		},
		
		//==================================================
		//	Get text
		//==================================================
		getText: function()
		{
			var fValue;
			
			if(this.element.value == "")
				return "";
			
			if(this.type == textbox.TYPE_FORMATTED_NUMBER)
			{
				if(this.element.value.indexOf("."))
					fValue = parseFloat(this.element.value);
				else
					fValue = parseInt(this.element.value);
				
				if(!isNaN(fValue))
					return formatNumber("" + fValue, this.digit);
				else
					return "";
			}
			else
				return this.element.value;
		},
		
		//==================================================
		//	Get value
		//==================================================
		getValue: function()
		{
			if(this.type == textbox.TYPE_FORMATTED_NUMBER)
				return this.element.value.replace(/,/g, "");
			else
				return this.element.value;
		},
		
		//==================================================
		//	Set value
		//==================================================
		setValue: function(sValue)
		{
			if(typeof sValue != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setValue");
			
			var fValue;
			
			if((this.type == textbox.TYPE_NUMBER)
			|| (this.type == textbox.TYPE_FORMATTED_NUMBER))
			{
				if(sValue.indexOf("."))
					fValue = parseFloat(sValue);
				else
					fValue = parseInt(sValue);
				
				if(isNaN(fValue))
					this.element.value = "";
				else
				{
					if(this.type == textbox.TYPE_NUMBER)
						this.element.value = "" + fValue;
					else
						this.element.value = formatNumber("" + fValue, this.digit);
				}
			}
			else
				this.element.value = sValue;
		},
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip: function(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setTooltip");
			this.element.placeholder = sTooltip;
		},
		
		//==================================================
		//	Set font size
		//==================================================
		setFontSize: function(iFontPoint)
		{
			if(typeof iFontPoint != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_SIZE, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setFontSize");
			this.element.style.fontSize = iFontPoint + "pt";
		},
		
		//==================================================
		//	Set font color
		//==================================================
		setFontColor: function(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_COLOR),
											"textbox.js", "jaswin.textbox.TextBox.setFontColor");
			this.element.style.color = sColor;
		},
		
		//==================================================
		//	Set font family
		//==================================================
		setFontFamily: function(sFontFamily)
		{
			if(typeof sFontFamily != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_FAMILY, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setFontFamily");
			this.element.style.fontFamily = sFontFamily;
		},
		
		//==================================================
		//	Set background color
		//==================================================
		setBackgroundColor: function(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BACKGROUND),
											"textbox.js", "jaswin.textbox.TextBox.setBackgroundColor");
			this.backgroundColor = sColor;
			this.element.style.backgroundColor = sColor;
		},
		
		//==================================================
		//	Set background color when focused
		//==================================================
		setFocusedBackgroundColor: function(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BACKGROUND),
											"textbox.js", "jaswin.textbox.TextBox.setFocusedBackgroundColor");
			this.focusedBackColor = sColor;
		},
		
		//==================================================
		//	Set unusable characters
		//==================================================
		setUnusableCharacters: function(sUnusable)
		{
			if(typeof sUnusable != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_UNUSABLE_CHARACTERS),
											"textbox.js", "jaswin.textbox.TextBox.setUnusableCharacters");
			this.unusableCharacters = sUnusable;
		},
		
		//==================================================
		//	Set change event
		//==================================================
		setChangeEvent: function(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setChangeEvent");
			
			this.onChange = fpChange;
		},
		
		//==================================================
		//	Set focus event
		//==================================================
		setFocusEvent: function(fpFocus)
		{
			if(typeof fpFocus != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_FOCUS, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setFocusEvent");
			this.onFocus = fpFocus;
		},
		
		//==================================================
		//	Set blur event
		//==================================================
		setBlurEvent: function(fpBlur)
		{
			if(typeof fpBlur != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_BLUR, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setBlurEvent");
			this.onBlur = fpBlur;
		},
		
		//==================================================
		//	Set input event
		//==================================================
		setInputEvent: function(fpInput)
		{
			if(typeof fpInput != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_INPUT, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setInputEvent");
			this.onInput = fpInput;
		},
		
		//==================================================
		//	Set suggestion list
		//==================================================
		setSuggestionList: function(oListBox)
		{
			if(!jaswin.list)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "list.js"),
											"textbox.js", "jaswin.textbox.TextBox.setSuggestionList");
			
			if((typeof oListBox != "object")
			|| (!oListBox instanceof jaswin.list.ListBox))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LISTBOX, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setSuggestionList");
			
			this.suggestionList = oListBox;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	formatNumber
	//
	//	Summary		:	Format numeric string
	//
	//==========================================================
	function formatNumber(sSrc,			// Source text[string]
							iDigit)		// Digit[int]
	{
		if(typeof sSrc != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, ALERT_TYPE_SOURCE, "jaswin.js", "jaswin.textbox.TextBox.formatNumber");
		if(typeof iDigit != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_DIGIT, ""),
										"textbox.js", "jaswin.textbox.TextBox.formatNumber");
		if(iDigit < 0)
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_VALUE_DIGIT, ""),
										"textbox.js", "jaswin.textbox.TextBox.formatNumber");
		
		var sValue = sSrc;
		var iIndex = sValue.indexOf(".");
		var iCounter;
		var iUnder = 0;
		var iOver = 0;
		var iCnt = 0;
		var bMinus = false;
		var iStart = 0;
		var iEnd = 0;
		
		if(sValue.charAt(0) == "-")
		{
			bMinus = true;
			sValue = sValue.substring(1);
			iIndex = sValue.indexOf(".");
		}
		
		if((sValue.charAt(0) == "0") && (iIndex != 1))
		{
			if(iIndex == -1)
				iEnd = sValue.length;
			else
				iEnd = iIndex;
			
			for(iCounter = 0; iCounter < iEnd; iCounter++)
			{
				if(sValue.charAt(iCounter) != "0")
				{
					iStart = iCounter;
					break;
				}
			}
			sValue = sValue.substring(iStart);
			iIndex = sValue.indexOf(".");
		}
		
		// Fix digit
		if(iIndex == -1)
		{
			if(iDigit)
			{
				sValue += ".";
				for(iCounter = 0; iCounter < iDigit; iCounter++)
					sValue += "0";
			}
		}
		else
		{
			if(iDigit)
			{
				if(iIndex >= sValue.length - 1)
				{
					for(iCounter = 0; iCounter < iDigit; iCounter++)
						sValue += "0";
				}
				else
				{
					iUnder = sValue.substring(iIndex + 1).length;
					if(iUnder < iDigit)
					{
						for(iCounter = 0; iCounter < iDigit - iUnder; iCounter++)
							sValue += "0";
					}
					else if(iUnder > iDigit)
						sValue = sValue.substring(0, iIndex + iDigit + 1);
				}
				if(iIndex == 0)
					sValue = "0" + sValue;
			}
			else
			{
				if(iIndex == 0)
					sValue = "0";
				else
					sValue = sValue.substring(0, iIndex);
			}
		}
		
		// Comma
		iIndex = sValue.indexOf(".");
		if(iIndex == -1)
			iOver = sValue.length;
		else
			iOver = iIndex;
		if(iOver % 3)
			iCnt = Math.floor(iOver / 3);
		else
			iCnt = Math.floor(iOver / 3) - 1;
		
		for(iCounter = 0; iCounter < iCnt; iCounter++)
		{
			iIndex = sValue.lastIndexOf(",");
			if(iIndex == -1)
			{
				if(iOver % 3)
					sValue = sValue.substring(0, iOver % 3) + "," + sValue.substring(iOver % 3);
				else
					sValue = sValue.substring(0, 3) + "," + sValue.substring(3);
			}
			else
				sValue = sValue.substring(0, iIndex + 4) + "," + sValue.substring(iIndex + 4);
		}
		
		if(bMinus)
		{
			for(iCounter = 0; iCounter < sValue.length; iCounter++)
			{
				if((sValue.charAt(iCounter) != "0")
				&& (sValue.charAt(iCounter) != "."))
				{
					sValue = "-" + sValue;
					break;
				}
			}
		}
		
		return sValue;
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetTextBox
	//
	//	Summary		:	Get target textbox element from event
	//
	//==========================================================
	function getTargetTextBox(e)
	{
		var oElement = e.target || e.srcElement;
		var oTextBox = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "input")
			|| (oElement.tagName == "INPUT"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oTextBox = textbox.textBoxes[oElement.id];
				if(!oTextBox)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oTextBox;
	}
	
	
	//==========================================================
	//
	//	Function	:	textBoxFocusEvent
	//
	//	Summary		:	Event handler of focus to a textbox
	//
	//==========================================================
	function textBoxFocusEvent(e)
	{
		var oTextBox = getTargetTextBox(e);
		
		if(oTextBox)
			oTextBox.focus();
	}
	
	
	//==========================================================
	//
	//	Function	:	textBoxBlurEvent
	//
	//	Summary		:	Event handler of blur from a textbox
	//
	//==========================================================
	function textBoxBlurEvent(e)
	{
		var oTextBox = getTargetTextBox(e);
		
		if(oTextBox)
			oTextBox.blur();
	}
	
	
	//==========================================================
	//
	//	Function	:	textBoxInputEvent
	//
	//	Summary		:	Event handler of input from a textbox
	//
	//==========================================================
	function textBoxInputEvent(e)
	{
		var oTextBox = getTargetTextBox(e);
		
		if(oTextBox)
			oTextBox.input();
	}
	
	
	//==========================================================
	//
	//	Function	:	textBoxKeyDownEvent
	//
	//	Summary		:	Event handler of keydown a textbox
	//
	//==========================================================
	function textBoxKeyDownEvent(e)
	{
		var oTextBox = getTargetTextBox(e);
		
		if(oTextBox)
			return oTextBox.keyDown(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	textBoxChangeEvent
	//
	//	Summary		:	Event handler of change a textbox
	//
	//==========================================================
	function textBoxChangeEvent(e)
	{
		var oTextBox = getTargetTextBox(e);
		
		if(oTextBox)
			oTextBox.change();
	}
	
	
	//==========================================================
	//
	//	Class		:	EditBox
	//
	//	Summary		:	Class of editbox
	//
	//==========================================================
	textbox.EditBox = function(sID,				// * ID[String]
								oRect,			// * Size and position[ExpandableRect]
								iStyle,			// * Editbox style[int:bit flag]
								bEnabled,		// * Enabled or not[boolean]
								bReadOnly,		// Read only or not[boolean]
								sValue,			// Initial value[String]
								sTooltip,		// Tool tip text[String]
								iTabIndex,		// Tab index[int]
								iImeMode,		// IME mode[int]
								fpChange,		// Function on change[function](EditBox)
								fpFocus,		// Function on focus[function](EditBox)
								fpBlur,			// Function on blur[function](EditBox)
								fpInput,		// Function on input[function](EditBox)
								fpKeyDown)		// Function on keydown[function](EditBox, event)
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"textbox.js", "jaswin.textbox.EditBox.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.ExpandableRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"textbox.js", "jaswin.textbox.EditBox.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
										"textbox.js", "jaswin.textbox.EditBox.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"textbox.js", "jaswin.textbox.EditBox.constructor");
		
		if((typeof bReadOnly != "undefined") && (bReadOnly != null))
		{
			if(typeof bReadOnly != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_READONLY, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		if(sValue)
		{
			if(typeof sValue != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		if(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		if(iImeMode)
		{
			if(typeof iImeMode != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_IME_MODE, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
			
			if((iImeMode != textbox.IME_AUTO)
			&& (iImeMode != textbox.IME_ACTIVE)
			&& (iImeMode != textbox.IME_INACTIVE)
			&& (iImeMode != textbox.IME_DISABLED))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_IME_MODE, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		if(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		if(fpFocus)
		{
			if(typeof fpFocus != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_FOCUS, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		if(fpBlur)
		{
			if(typeof fpBlur != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_BLUR, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		if(fpInput)
		{
			if(typeof fpInput != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_INPUT, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		if(fpKeyDown)
		{
			if(typeof fpKeyDown != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_KEYDOWN, sID),
											"textbox.js", "jaswin.textbox.EditBox.constructor");
		}
		
		// Set properties
		this.id = sID;					// ID[String]
		this.rect = oRect;				// Size and position[ExpandableRect]
		this.style = iStyle;			// TextBox style[int]
		this.readOnly = bReadOnly;		// Read only or not[boolean]
		this.enabled = bEnabled;		// Enabled or not[boolean]
		this.parentEnabled = true;		// Enabled or not[boolean]
		this.onChange = fpChange;		// Function on change[function](EditBox)
		this.onFocus = fpFocus;			// Function on focus[function](EditBox)
		this.onBlur = fpBlur;			// Function on blur[function](EditBox)
		this.onInput = fpInput;			// Function on input[function](EditBox)
		this.onKeyDown = fpKeyDown;		// Function on keydown[function](EditBox, e)
		this.onHighlight = null;		// Function for highlighting[function](string) returns string
		this.element = null;			// This element[DOM Element]
		this.textElement = null;		// Element of textarea[DOM Element]
		this.focusElement = null;		// Focus element[DOM Element]
		this.tabIndex = -1;				// Tab index[int]
		this.parent = null;				// Parent class[global / window / panel class]
		this.suggestionList = null;		// List box for suggestion[ListBox]
		this.searchWord = "";			// Search word[string]
		this.searchIgnoreCase = false;	// Serach word with ignore case option[boolean]
		this.searchRegExp = false;		// Serach word with regular expression option[boolean]
		
		// Create element
		this.element = document.createElement("div");
		if(this.style & textbox.STYLE_FLAT)
			this.element.className = "editbox-flat";
		else
			this.element.className = "editbox";
		
		// Horizontal rect
		if(this.rect.anchorX == jaswin.ANCHOR_BOTH)
		{
			this.element.style.left = this.rect.left + "px";
			this.element.style.right = this.rect.right + "px";
		}
		else
		{
			if(this.rect.anchorX == jaswin.ANCHOR_LEFT)
				this.element.style.left = this.rect.left + "px";
			else if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.right + "px";
			
			if(this.style & textbox.STYLE_FLAT)
				this.element.style.width = this.rect.width + "px";
			else
				this.element.style.width = this.rect.width - 2 + "px";
		}
		
		// Vertical rect
		if(this.rect.anchorY == jaswin.ANCHOR_BOTH)
		{
			this.element.style.top = this.rect.top + "px";
			this.element.style.bottom = this.rect.bottom + "px";
		}
		else
		{
			if(this.rect.anchorY == jaswin.ANCHOR_TOP)
				this.element.style.top = this.rect.top + "px";
			else if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.bottom + "px";
			
			if(this.style & textbox.STYLE_FLAT)
				this.element.style.height = this.rect.height + "px";
			else
				this.element.style.height = this.rect.height - 2 + "px";
		}
		
		if(this.style & textbox.STYLE_LINE_NO)
		{
			this.lineNoElement = document.createElement("div");
			this.lineNoElement.className = "editbox-line-no";
			this.lineNoElement.innerHTML = "1";
			this.element.appendChild(this.lineNoElement);
		}
		
		var oDiv = document.createElement("div");
		oDiv.className = "editbox-inner";
		if((jaswin.BROWSER == "chrome")
		|| (jaswin.BROWSER == "opera"))
		{
			oDiv.style.right = "4px";
			oDiv.style.bottom = "4px";
		}
		if(this.style & textbox.STYLE_LINE_NO)
			oDiv.style.left = "46px";
		this.element.appendChild(oDiv);
		
		if(this.style & textbox.STYLE_HIGHLIGHTING)
		{
			this.highlightElement = document.createElement("pre");
			this.highlightElement.className = "editbox-highlight";
			
			if(jaswin.BROWSER == "gecko")
				this.highlightElement.style.left = "1px";
			else if(jaswin.BROWSER == "ie")
			{
				this.highlightElement.style.left = "1px";
				this.highlightElement.style.top = "1px";
			}
			else if((jaswin.BROWSER == "chrome")
			|| (jaswin.BROWSER == "opera")
			|| (jaswin.BROWSER == "safari"))
			{
				this.highlightElement.style.left = "2px";
				this.highlightElement.style.top = "2px";
			}
			
			oDiv.appendChild(this.highlightElement);
		}
		
		this.textElement = document.createElement("textarea");
		this.textElement.className = "editbox-text";
		if(this.style & textbox.STYLE_HIGHLIGHTING)
			this.textElement.style.color = "transparent";
		
		if(this.style & textbox.STYLE_NOWRAP)
			this.textElement.wrap = "off";
		if(this.textElement.spellcheck)
			this.textElement.spellcheck = false;
		oDiv.appendChild(this.textElement);
		this.focusElement = this.textElement;
		
		if(!this.enabled)
			this.textElement.disabled = true;
		
		if(this.readOnly)
			this.textElement.readOnly = true;
		
		if(iImeMode == textbox.IME_ACTIVE)
			this.textElement.style.imeMode = "active";
		else if(iImeMode == textbox.IME_INACTIVE)
			this.textElement.style.imeMode = "inactive";
		else if(iImeMode == textbox.IME_DISABLED)
			this.textElement.style.imeMode = "disabled";
		
		if(sValue)
			this.textElement.value = sValue;
		
		// Tooltip
		if((sTooltip) && (sTooltip.length > 0))
			this.textElement.placeholder = sTooltip;
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Event binding
		if(this.enabled)
		{
			this.focusElement.addEventListener("focus", editBoxFocusEvent, false);
			this.focusElement.addEventListener("blur", editBoxBlurEvent, false);
			this.textElement.addEventListener("keydown", editBoxKeyDownEvent, false);
			this.textElement.addEventListener("change", editBoxChangeEvent, false);
			this.textElement.addEventListener("input", editBoxInputEvent, false);
			this.textElement.addEventListener("scroll", editBoxScrollEvent, false);
		}
	}
	textbox.EditBox.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			delete textbox.editBoxes[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (textbox.editBoxes[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"textbox.js", "jaswin.textbox.EditBox.setParent");
			this.element.id = sID;
			textbox.editBoxes[this.element.id] = this;
			oParentWnd[this.id] = this;
			
			if(this.highlightElement)
			{
				if((jaswin.frame) && (jaswin.frame.windowCount))
				{
					if(oParentWnd != global)
					{
						this.highlightElement.style.zIndex = oParentWnd.zIndex + oParentWnd.modalLevel - 1;
						this.textElement.style.zIndex = oParentWnd.zIndex + oParentWnd.modalLevel;
					}
					else
					{
						this.highlightElement.style.zIndex = -1;
						this.textElement.style.zIndex = 0;
					}
				}
				else
				{
					this.highlightElement.style.zIndex = -1;
					this.textElement.style.zIndex = 0;
				}
			}
		},
		
		//==================================================
		//	Enable this editbox
		//==================================================
		enable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.focusElement.tabIndex = this.tabIndex;
				this.textElement.disabled = false;
				this.textElement.style.backgroundColor = "transparent";
				this.textElement.style.opacity = null;
				
				this.focusElement.addEventListener("focus", editBoxFocusEvent, false);
				this.focusElement.addEventListener("blur", editBoxBlurEvent, false);
				this.textElement.addEventListener("keydown", editBoxKeyDownEvent, false);
				this.textElement.addEventListener("change", editBoxChangeEvent, false);
				this.textElement.addEventListener("input", editBoxInputEvent, false);
				this.textElement.addEventListener("scroll", editBoxScrollEvent, false);
			}
		},
		
		//==================================================
		//	Disable this editbox
		//==================================================
		disable: function(bParent)
		{
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.focusElement.tabIndex = -1;
			this.textElement.disabled = true;
			this.textElement.style.backgroundColor = "#c0c0c0";
			this.textElement.style.opacity = "0.4";
			
			this.focusElement.removeEventListener("focus", editBoxFocusEvent, false);
			this.focusElement.removeEventListener("blur", editBoxBlurEvent, false);
			this.textElement.removeEventListener("keydown", editBoxKeyDownEvent, false);
			this.textElement.removeEventListener("change", editBoxChangeEvent, false);
			this.textElement.removeEventListener("input", editBoxInputEvent, false);
			this.textElement.removeEventListener("scroll", editBoxScrollEvent, false);
		},
		
		//==================================================
		//	Set this editbox read only or not
		//==================================================
		setReadOnly: function(bReadOnly)
		{
			if(typeof bReadOnly != "boolean")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_READONLY, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setReadOnly");
			
			this.readOnly = bReadOnly;
			this.textElement.readOnly = bReadOnly;
		},
		
		//==================================================
		//	Show this textbox
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this textbox
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Focus to this editbox
		//==================================================
		focus: function()
		{
			if(!this.enabled)
				return;
			
			if(!this.style & textbox.STYLE_FLAT)
				this.element.className = "editbox-focus";
			
			if(this.onFocus)
				this.onFocus(this);
		},
		
		//==================================================
		//	Blur from this editbox
		//==================================================
		blur: function()
		{
			if(!this.enabled)
				return;
			
			if(!this.style & textbox.STYLE_FLAT)
				this.element.className = "editbox";
			
			if(this.onBlur)
				this.onBlur(this);
		},
		
		//==================================================
		//	Input to this editbox
		//==================================================
		input: function()
		{
			if(this.lineNoElement)
			{
				var iCnt;
				var iCounter;
				var sTmp = "";
				
				if(!this.textElement.value.match(/\n/g))
					iCnt = 1;
				else
					iCnt = this.textElement.value.match(/\n/g).length + 1;
				
				for(iCounter = 1; iCounter <= iCnt; iCounter++)
					sTmp += iCounter + "<br />";
				
				this.lineNoElement.innerHTML = sTmp;
			}
			
			if(this.highlightElement)
			{
				var sDoc = this.textElement.value;
				
				sDoc = sDoc.replace(/&/g, "&amp;");
				sDoc = sDoc.replace(/</g, "&lt;");
				sDoc = sDoc.replace(/>/g, "&gt;");
				
				if(this.onHighlight)
					sDoc = this.onHighlight(sDoc);
				
				sDoc = sDoc.replace(/\t/g, "<span class=\"editbox-tab\">\t</span>");
				sDoc = sDoc.replace(/　/g, "<span class=\"editbox-widespace\">　</span>");
				sDoc = sDoc.replace(/\n/g, "<span class=\"editbox-enter\"> </span>\n");
				if((this.searchWord) && (this.searchWord.length))
				{
					var oRegEx;
					var sSearchWord;
					
					if(this.searchRegExp)
						sSearchWord = this.searchWord;
					else
					{
						sSearchWord = this.searchWord.replace(/</g, "&lt;");
						sSearchWord = sSearchWord.replace(/>/g, "&gt;");
						sSearchWord = jaswin.escapeRegExp(sSearchWord);
					}
					if(this.searchIgnoreCase)
						oRegEx = new RegExp(sSearchWord, "gi");
					else
						oRegEx = new RegExp(sSearchWord, "g");
					
					sDoc = sDoc.replace(oRegEx,
										function(){
											var iTagStart;
											var iTagEnd;
											iTagStart = arguments[arguments.length - 1].indexOf("<", arguments[arguments.length - 2] + arguments[0].length);
											iTagEnd = arguments[arguments.length - 1].indexOf(">", arguments[arguments.length - 2] + arguments[0].length);
											if((iTagEnd == -1) || (iTagStart < iTagEnd))
												return "<span class=\"editbox-search\">" + arguments[0] + "</span>";
											return arguments[0];
										});
				}
				this.highlightElement.innerHTML = sDoc;
			}
			
			if(this.onInput)
				this.onInput(this);
		},
		
		//==================================================
		//	Scrolled this editbox
		//==================================================
		scroll: function()
		{
			if(this.lineNoElement)
				this.lineNoElement.style.top = this.textElement.scrollTop * (-1) + "px";
			
			if(this.highlightElement)
			{
				if(jaswin.BROWSER == "gecko")
				{
					this.highlightElement.style.top = this.textElement.scrollTop * (-1) + "px";
					this.highlightElement.style.left = this.textElement.scrollLeft * (-1) + 1 + "px";
				}
				else if(jaswin.BROWSER == "ie")
				{
					this.highlightElement.style.top = this.textElement.scrollTop * (-1) + 1 + "px";
					this.highlightElement.style.left = this.textElement.scrollLeft * (-1) + 1 + "px";
				}
				else if((jaswin.BROWSER == "chrome")
				|| (jaswin.BROWSER == "opera")
				|| (jaswin.BROWSER == "safari"))
				{
					this.highlightElement.style.top = this.textElement.scrollTop * (-1) + 2 + "px";
					this.highlightElement.style.left = this.textElement.scrollLeft * (-1) + 2 + "px";
				}
				else
				{
					this.highlightElement.style.top = this.textElement.scrollTop * (-1) + "px";
					this.highlightElement.style.left = this.textElement.scrollLeft * (-1) + "px";
				}
			}
		},
		
		//==================================================
		//	Keydown this editbox
		//==================================================
		keyDown: function(e)
		{
			if((this.suggestionList)
			&& (this.suggestionList.element.style.display != "none")
			&& ((e.keyCode == 13) || (e.keyCode == 32) || (e.keyCode == 38) || (e.keyCode == 40)))
			{
				// When suggestion list is shown and key down enter/space or arrow up/down
				this.suggestionList.keyDownList(e);
			}
			
			if((this.suggestionList)
			&& (this.suggestionList.element.style.display != "none")
			&& (e.keyCode == 27))
			{
				// When suggestion list is shown and key down esc
				this.suggestionList.hide();
			}
			
			if(e.keyCode == 9)
			{
				if(this.style & textbox.STYLE_TAB_INPUT)
				{
					// Input a tab indent
					var iStart = this.textElement.selectionStart;
					var iEnd = this.textElement.selectionEnd;
					var sValue = this.textElement.value;
					var sSelected = this.textElement.value.substring(iStart, iEnd);
					var sBefore = this.textElement.value.substring(0, iStart);
					var sAfter = this.textElement.value.substring(iEnd);
					var iLineStart;
					var iLineEnd;
					var iCounter;
					
					if(iStart == iEnd)
					{
						this.textElement.value = "" + sBefore + "\t" + sAfter;
						this.textElement.selectionStart = this.textElement.selectionEnd = iStart + 1;
						e.preventDefault();
						e.stopPropagation();
						
						this.input();
						return false;
					}
					else
					{
						if(sSelected.indexOf("\n") == -1)
						{
							// selected one line
							iLineStart = sBefore.lastIndexOf("\n") + 1;
							iLineEnd = sValue.indexOf("\n", iEnd);
							if(iLineEnd == -1)
								iLineEnd = sValue.length;
							if(sSelected.length != iLineEnd - iLineStart)
							{
								this.textElement.value = "" + sBefore + "\t" + sAfter;
								this.textElement.selectionStart = this.textElement.selectionEnd = iStart + 1;
								e.preventDefault();
								e.stopPropagation();
								
								this.input();
								return false;
							}
						}
					}
					
					iStart = sBefore.lastIndexOf("\n") + 1;
					iEnd = sValue.indexOf("\n", iEnd);
					if(iEnd == -1)
						iEnd = sValue.length;
					sSelected = this.textElement.value.substring(iStart, iEnd);
					
					var saLines = sSelected.split("\n");
					iLineStart = iStart;
					for(iCounter = 0; iCounter < saLines.length; iCounter++)
					{
						this.textElement.value = this.textElement.value.substring(0, iLineStart)
												+ "\t" + this.textElement.value.substring(iLineStart);
						iLineStart = iLineStart + 1 + saLines[iCounter].length + 1;		// 1px:TAB 1px:LF
					}
					this.textElement.selectionStart = iStart;
					this.textElement.selectionEnd = iEnd + saLines.length;
					
					e.preventDefault();
					e.stopPropagation();
					
					this.input();
					return false;
				}
			}
			
			if(this.onKeyDown)
				return this.onKeyDown(this, e);
			
			return true;
		},
		
		//==================================================
		//	Change this editbox
		//==================================================
		change: function()
		{
			if(this.onChange)
				this.onChange(this);
		},
		
		//==================================================
		//	Get value
		//==================================================
		getValue: function()
		{
			return this.textElement.value;
		},
		
		//==================================================
		//	Set value
		//==================================================
		setValue: function(sValue)
		{
			if(typeof sValue != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setValue");
			this.textElement.value = sValue;
			this.input();
			this.textElement.scrollTop = 0;
			this.textElement.selectionStart = 0;
			this.textElement.selectionEnd = 0;
		},
		
		//==================================================
		//	Replace selected string to upper case
		//==================================================
		toUpperCase: function()
		{
			if(this.textElement.selectionStart == this.textElement.selectionEnd)
				return;
			
			var iStart = this.textElement.selectionStart;
			var iEnd = this.textElement.selectionEnd;
			this.textElement.value = this.textElement.value.substring(0, iStart)
								+ this.textElement.value.substring(iStart, iEnd).toUpperCase()
								+ this.textElement.value.substring(iEnd);
			this.input();
			this.textElement.selectionStart = iStart;
			this.textElement.selectionEnd = iEnd;
		},
		
		//==================================================
		//	Replace selected string to lower case
		//==================================================
		toLowerCase: function()
		{
			if(this.textElement.selectionStart == this.textElement.selectionEnd)
				return;
			
			var iStart = this.textElement.selectionStart;
			var iEnd = this.textElement.selectionEnd;
			this.textElement.value = this.textElement.value.substring(0, iStart)
								+ this.textElement.value.substring(iStart, iEnd).toLowerCase()
								+ this.textElement.value.substring(iEnd);
			this.input();
			this.textElement.selectionStart = iStart;
			this.textElement.selectionEnd = iEnd;
		},
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip: function(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setTooltip");
			this.textElement.placeholder = sTooltip;
		},
		
		//==================================================
		//	Set font size
		//==================================================
		setFontSize: function(iSize)
		{
			if(typeof iSize != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_SIZE, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setFontSize");
			this.textElement.style.fontSize = iSize + "pt";
			if(this.lineNoElement)
				this.lineNoElement.style.fontSize = iSize + "pt";
			if(this.highlightElement)
				this.highlightElement.style.fontSize = iSize + "pt";
		},
		
		//==================================================
		//	Set font color
		//==================================================
		setFontColor: function(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_COLOR),
											"textbox.js", "jaswin.textbox.EditBox.setFontColor");
			this.textElement.style.color = sColor;
		},
		
		//==================================================
		//	Set font family
		//==================================================
		setFontFamily: function(sFontFamily)
		{
			if(typeof sFontFamily != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_FONT_FAMILY, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setFontFamily");
			this.textElement.style.fontFamily = sFontFamily;
			if(this.lineNoElement)
				this.lineNoElement.style.fontFamily = sFontFamily;
			if(this.highlightElement)
				this.highlightElement.style.fontFamily = sFontFamily;
		},
		
		//==================================================
		//	Set tab indent size
		//==================================================
		setTabSize: function(iTabSize)
		{
			if(typeof iTabSize != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TAB_SIZE, this.id),
											"textbox.js", "jaswin.textbox.TextBox.setFontSize");
			this.textElement.style.tabSize = "" + iTabSize;
			this.textElement.style.setProperty("-moz-tab-size", "" + iTabSize);
			if(this.highlightElement)
			{
				this.highlightElement.style.tabSize = "" + iTabSize;
				this.highlightElement.style.setProperty("-moz-tab-size", "" + iTabSize);
			}
		},
		
		//==================================================
		//	Set background color
		//==================================================
		setBackgroundColor: function(sColor)
		{
			if(typeof sColor != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_BACKGROUND),
											"textbox.js", "jaswin.textbox.EditBox.setBackgroundColor");
			this.backgroundColor = sColor;
			this.element.style.backgroundColor = sColor;
		},
		
		//==================================================
		//	Set search word
		//==================================================
		setSearchWord: function(sWord, bIgnoreCase, bRegExp)
		{
			if(!this.style & textbox.STYLE_HIGHLIGHTING)
				return;
			
			if(typeof sWord != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_SEARCH_WORD, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setSearchWord");
			
			this.searchWord = sWord;
			this.searchIgnoreCase = bIgnoreCase;
			this.searchRegExp = bRegExp;
			
			this.input();
			
			if(this.searchWord.length)
			{
				this.textElement.selectionEnd = this.textElement.selectionStart;
				this.searchNext();
			}
		},
		
		//==================================================
		//	Search previous
		//==================================================
		searchPrev: function()
		{
			if((!this.searchWord) || (!this.searchWord.length))
				return;
			
			var iStart = 0;
			var iEnd = this.textElement.selectionStart;
			var oRegEx;
			var iIndex;
			var iMax = -1;
			var sSearchWord;
			var sMatched;
			
			if(this.searchRegExp)
				sSearchWord = this.searchWord;
			else
				sSearchWord = jaswin.escapeRegExp(this.searchWord);
			
			if(this.searchIgnoreCase)
				oRegEx = new RegExp(sSearchWord, "i");
			else
				oRegEx = new RegExp(sSearchWord);
			
			if(oRegEx)
			{
				while(true)
				{
					iIndex = this.textElement.value.substring(iStart, iEnd).search(oRegEx);
					if(iIndex == -1)
						break;
					
					saMatched = this.textElement.value.substring(iStart, iEnd).match(oRegEx);
					iMax = iStart + iIndex;
					iStart = iMax + 1;
				}
				
				if(iMax != -1)
				{
					this.textElement.selectionStart = iMax;
					this.textElement.selectionEnd = iMax
												+ saMatched[saMatched.length - 1].length;
					
					if((jaswin.BROWSER == "chrome")
					|| (jaswin.BROWSER == "safari")
					|| (jaswin.BROWSER == "opera"))
					{
						var iCnt;
						var iLineHeight = Math.ceil(parseInt(this.textElement.style.fontSize) * 4 / 3 * 1.1);
						var iHeight = this.textElement.clientHeight;
						if(!this.textElement.value.substring(0, this.textElement.selectionStart).match(/\n/g))
							iCnt = 1;
						else
							iCnt = this.textElement.value.substring(0, this.textElement.selectionStart).match(/\n/g).length + 1;
						
						if((iCnt + 1) * iLineHeight > iHeight)
						{
							this.textElement.scrollTop = (iCnt + 1) * iLineHeight - iHeight;
						}
					}
				}
			}
		},
		
		//==================================================
		//	Search next
		//==================================================
		searchNext: function()
		{
			if((!this.searchWord) || (!this.searchWord.length))
				return;
			
			var iStart = this.textElement.selectionEnd;
			var oRegEx;
			var iIndex;
			var sSearchWord;
			
			if(this.searchRegExp)
				sSearchWord = this.searchWord;
			else
				sSearchWord = jaswin.escapeRegExp(this.searchWord);
			
			if(this.searchIgnoreCase)
				oRegEx = new RegExp(sSearchWord, "i");
			else
				oRegEx = new RegExp(sSearchWord);
			
			if(oRegEx)
			{
				iIndex = this.textElement.value.substring(iStart).search(oRegEx);
				if(iIndex != -1)
				{
					this.textElement.selectionStart = iStart + iIndex;
					this.textElement.selectionEnd = iStart + iIndex
													+ this.textElement.value.substring(iStart).match(oRegEx)[0].length;
					
					if((jaswin.BROWSER == "chrome")
					|| (jaswin.BROWSER == "safari")
					|| (jaswin.BROWSER == "opera"))
					{
						var iCnt;
						var iLineHeight = Math.ceil(parseInt(this.textElement.style.fontSize) * 4 / 3 * 1.1);
						var iHeight = this.textElement.clientHeight;
						if(!this.textElement.value.substring(0, this.textElement.selectionStart).match(/\n/g))
							iCnt = 1;
						else
							iCnt = this.textElement.value.substring(0, this.textElement.selectionStart).match(/\n/g).length + 1;
						
						if((iCnt + 1) * iLineHeight > iHeight)
						{
							this.textElement.scrollTop = (iCnt + 1) * iLineHeight - iHeight;
						}
					}
				}
			}
		},
		
		//==================================================
		//	Set change event
		//==================================================
		setChangeEvent: function(fpChange)
		{
			if(typeof fpChange != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CHANGE, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setChangeEvent");
			
			this.onChange = fpChange;
		},
		
		//==================================================
		//	Set focus event
		//==================================================
		setFocusEvent: function(fpFocus)
		{
			if(typeof fpFocus != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_FOCUS, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setFocusEvent");
			this.onFocus = fpFocus;
		},
		
		//==================================================
		//	Set blur event
		//==================================================
		setBlurEvent: function(fpBlur)
		{
			if(typeof fpBlur != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_BLUR, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setBlurEvent");
			this.onBlur = fpBlur;
		},
		
		//==================================================
		//	Set input event
		//==================================================
		setInputEvent: function(fpInput)
		{
			if(typeof fpInput != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_INPUT, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setInputEvent");
			this.onInput = fpInput;
		},
		
		//==================================================
		//	Set keydown event
		//==================================================
		setKeyDownEvent: function(fpKeyDown)
		{
			if(typeof fpKeyDown != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_KEYDOWN, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setKeyDownEvent");
			this.onKeyDown = fpKeyDown;
		},
		
		//==================================================
		//	Set highlighting function
		//==================================================
		setHighlighting: function(fpHighlighting)
		{
			if(fpHighlighting)
			{
				if(typeof fpHighlighting != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_HIGHLIGHT, this.id),
												"textbox.js", "jaswin.textbox.EditBox.setHighlighting");
			}
			this.onHighlight = fpHighlighting;
			this.input();
		},
		
		//==================================================
		//	Set suggestion list
		//==================================================
		setSuggestionList: function(oListBox)
		{
			if(!jaswin.list)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "list.js"),
											"textbox.js", "jaswin.textbox.EditBox.setSuggestionList");
			
			if((typeof oListBox != "object")
			|| (!oListBox instanceof jaswin.list.ListBox))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LISTBOX, this.id),
											"textbox.js", "jaswin.textbox.EditBox.setSuggestionList");
			
			this.suggestionList = oListBox;
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetEditBox
	//
	//	Summary		:	Get target editbox element from event
	//
	//==========================================================
	function getTargetEditBox(e)
	{
		var oElement = e.target || e.srcElement;
		var oEditBox = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "textarea")
			|| (oElement.tagName == "TEXTAREA"))
			{
				oElement = oElement.parentNode.parentNode;
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oEditBox = textbox.editBoxes[oElement.id];
				if(!oEditBox)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oEditBox;
	}
	
	
	//==========================================================
	//
	//	Function	:	editBoxFocusEvent
	//
	//	Summary		:	Event handler of focus to a editbox
	//
	//==========================================================
	function editBoxFocusEvent(e)
	{
		var oEditBox = getTargetEditBox(e);
		
		if(oEditBox)
			oEditBox.focus();
	}
	
	
	//==========================================================
	//
	//	Function	:	editBoxBlurEvent
	//
	//	Summary		:	Event handler of blur from a editbox
	//
	//==========================================================
	function editBoxBlurEvent(e)
	{
		var oEditBox = getTargetEditBox(e);
		
		if(oEditBox)
			oEditBox.blur();
	}
	
	
	//==========================================================
	//
	//	Function	:	editBoxInputEvent
	//
	//	Summary		:	Event handler of input from a editbox
	//
	//==========================================================
	function editBoxInputEvent(e)
	{
		var oEditBox = getTargetEditBox(e);
		
		if(oEditBox)
			oEditBox.input();
	}
	
	
	//==========================================================
	//
	//	Function	:	editBoxKeyDownEvent
	//
	//	Summary		:	Event handler of keydown a editbox
	//
	//==========================================================
	function editBoxKeyDownEvent(e)
	{
		var oEditBox = getTargetEditBox(e);
		
		if(oEditBox)
			return oEditBox.keyDown(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	editBoxChangeEvent
	//
	//	Summary		:	Event handler of change a editbox
	//
	//==========================================================
	function editBoxChangeEvent(e)
	{
		var oEditBox = getTargetEditBox(e);
		
		if(oEditBox)
			oEditBox.change();
	}
	
	
	//==========================================================
	//
	//	Function	:	editBoxScrollEvent
	//
	//	Summary		:	Event handler of scroll a editbox
	//
	//==========================================================
	function editBoxScrollEvent(e)
	{
		var oEditBox = getTargetEditBox(e);
		
		if(oEditBox)
			oEditBox.scroll();
	}
})(jaswin.textbox = jaswin.textbox || {});
