//==================================================================
//
//	File name	:	toolbar.js
//
//	Summary		:	ToolBar class
//
//	Ceated		:	2014/10/15
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (toolbar) {
	//==========================================================
	//
	//	Class		:	ToolBarDelimiter
	//
	//	Summary		:	Class of ToolBarDelimiter
	//
	//==========================================================
	toolbar.ToolBarDelimiter = function()
	{
		//==================================================
		//	Constructor
		//==================================================
		// Create element
		this.element = document.createElement("div");
		this.element.style.width = "4px";
		this.element.style.height = "20px";
		this.element.style.cssFloat = "left";
		this.tabIndex = -1;
		
		var oSpan = document.createElement("span");
		oSpan.className = "v-line-inset";
		oSpan.style.height = "18px";
		oSpan.style.margin = "1px";
		
		this.element.appendChild(oSpan);
	}
	toolbar.ToolBarDelimiter.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			this.element.parentNode.removeChild(this.element);
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			return;
		},
		
		//==================================================
		//	Enable this
		//==================================================
		enable: function(bParent)
		{
			return;
		},
		
		//==================================================
		//	Disable this
		//==================================================
		disable: function(bParent)
		{
			return;
		}
	}
	
	
	//==========================================================
	//
	//	Class		:	ToolBar
	//
	//	Summary		:	Class of toolbar
	//
	//==========================================================
	toolbar.ToolBar = function()
	{
		//==================================================
		//	Constructor
		//==================================================
		// Set properties
		this.controls = new Array();		// Controls on this toolbar[Array of classes]
		this.enabled = true;				// Enabled or not[boolean]
		this.element = null;				// Tool bar element[DOM Element]
		this.parent = null;					// Parent window or global
		
		// Create element
		this.element = document.createElement("div");
		this.element.className = "toolbar";
	}
	toolbar.ToolBar.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
				this.controls[iCounter].destroy();
			
			this.element.parentNode.removeChild(this.element);
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			var iCounter;
			
			this.parent = oParent;
			for(iCounter = 0; iCounter < this.controls.length; iCounter++)
				this.controls[iCounter].setParent(this);
		},
		
		//==================================================
		//	Visible or not
		//==================================================
		isVisible: function()
		{
			if(this.element.style.display == "none")
				return false;
			else
				return true;
		},
		
		//==================================================
		//	Show this button
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this button
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Add control to this tool bar
		//==================================================
		addControl: function(oControl)
		{
			if(typeof oControl != "object")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_CONTROL,
											"toolbar.js", "jaswin.toolbar.ToolBar.addControl");
			
			if((!oControl instanceof jaswin.button.Button)
			&& (!oControl instanceof jaswin.checkbox.CheckBox)
			&& (!oControl instanceof jaswin.list.ComboBox)
			&& (!oControl instanceof toolbar.ToolBarDelimiter))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_CONTROL,
											"toolbar.js", "jaswin.toolbar.ToolBar.addControl");
			
			oControl.element.style.position = "relative";
			oControl.element.style.cssFloat = "left";
			oControl.element.style.left = "";
			oControl.element.style.top = "";
			
			if(oControl.tabIndex)
			{
				oControl.tabIndex = -1;
				if(oControl.focusElement)
					oControl.focusElement.tabIndex = -1;
			}
			
			this.element.appendChild(oControl.element);
			if(this.parent)
				oControl.setParent(this);
			this.controls[this.controls.length] = oControl;
		},
		
		//==================================================
		//	Add label to this tool bar
		//==================================================
		addLabel: function(sLabel)
		{
			if(typeof sLabel != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, ""),
											"toolbar.js", "jaswin.toolbar.ToolBar.addLabel");
			var oSpan;
			
			oSpan = document.createElement("span");
			oSpan.className = "label";
			oSpan.style.position = "relative";
			oSpan.style.cssFloat = "left";
			oSpan.style.lineHeight = "20px";
			oSpan.style.paddingLeft = "2px";
			oSpan.style.paddingRight = "2px";
			oSpan.innerHTML = sLabel;
			
			this.element.appendChild(oSpan);
		}
	}
	
})(jaswin.toolbar = jaswin.toolbar || {});
