//==================================================================
//
//	File name	:	treeview.js
//
//	Summary		:	TreeView class
//
//	Ceated		:	2014/10/06
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================

(function (treeview) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	treeview.STYLE_ICON = 0;
	treeview.STYLE_IMAGE = 1;
	treeview.STYLE_EDITABLE = 2;
	
	// Grobal variables
	treeview.treeViews = {};
	
	
	//==========================================================
	//
	//	Class		:	Node
	//
	//	Summary		:	Class of treeview node
	//
	//==========================================================
	treeview.Node = function(sLabel,			// * Label text[String]
							sValue,				// * Node value[String]
							iStyle,				// * Node style[int]
							sContents,			// Icon name or image URL[String]
							fpClick,			// Function on click[function]
							fpDblClick)			// Function on double click[function]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sLabel != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, ""),
										"treeview.js", "jaswin.treeview.Node.constructor");
		
		if(typeof sValue != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, ""),
										"treeview.js", "jaswin.treeview.Node.constructor");
		
		if(typeof iStyle != "number")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, ""),
										"treeview.js", "jaswin.treeview.Node.constructor");
		if(iStyle & treeview.STYLE_EDITABLE)
		{
			if(!jaswin.textbox)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "textbox.js"),
											"treeview.js", "jaswin.treeview.Node.constructor");
		}
		
		if(sContents)
		{
			if(typeof sContents != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_CONTENTS, ""),
											"treeview.js", "jaswin.treeview.Node.constructor");
		}
		
		if(fpClick)
		{
			if(typeof fpClick != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CLICK, ""),
											"treeview.js", "jaswin.treeview.Node.constructor");
		}
		
		if(fpDblClick)
		{
			if(typeof fpDblClick != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CLICK, ""),
											"treeview.js", "jaswin.treeview.Node.constructor");
		}
		
		// Set properties
		this.value = sValue;					// Node value[String]
		this.style = iStyle;					// Node style[int]
		this.onClick = fpClick;					// Function on click[function]
		this.onDblClick = fpDblClick;			// Function on double click[function]
		this.element = null;					// This element[DOM Element]
		this.nodeItem = null;					// Node item element[DOM Element]
		this.label = null;						// Label element[DOM Element]
		this.box = null;						// Icon box of open or close[DOM Element]
		this.openClose = null;					// Icon of open or close[DOM Element]
		this.icon = null;						// Icon element[DOM Element]
		this.opened = false;					// Open or not[boolean]
		this.selected = false;					// Selected or not[boolean]
		this.container = null;					// Container element[DOM Element]
		this.childNodes = new Array();			// Child nodes[Array of Node]
		this.parentNode = null;					// ParentNode[Node]
		this.treeView = null;					// Treeview[TreeView class]
		this.editable = false;					// Label is editable or not[boolean]
		
		// Create element
		this.element = document.createElement("li");
		this.nodeItem = document.createElement("div");
		this.nodeItem.className = "tree-node-item";
		this.element.appendChild(this.nodeItem);
		
		// Box
		this.box = document.createElement("div");
		this.box.className = "tree-node-box";
		this.openClose = document.createElement("span");
		this.openClose.className = "tree-icon-close";
		this.box.appendChild(this.openClose);
		this.nodeItem.appendChild(this.box);
		
		// Icon
		this.icon = document.createElement("div");
		this.icon.className = "tree-node-icon";
		
		if(!sContents)
		{
			var oSpan = document.createElement("span");
			oSpan.className = jaswin.ICON_FOLDER;
			this.icon.appendChild(oSpan);
		}
		else
		{
			if(iStyle & treeview.STYLE_IMAGE)
			{
				var oImg = document.createElement("img");
				oImg.src = sContents;
				oImg.width = 16;
				oImg.height = 16;
				this.icon.appendChild(oImg);
			}
			else
			{
				var oSpan = document.createElement("span");
				oSpan.className = sContents;
				this.icon.appendChild(oSpan);
			}
		}
		this.nodeItem.appendChild(this.icon);
		if(iStyle & treeview.STYLE_EDITABLE)
			this.editable = true;
		
		// Label
		this.label = document.createElement("span");
		this.label.className = "tree-node-label";
		this.label.innerHTML = sLabel;
		this.nodeItem.appendChild(this.label);
		
		var oDiv = document.createElement("div");
		oDiv.style.clear = "both";
		this.nodeItem.appendChild(oDiv);
	}
	treeview.Node.prototype =
	{
		//==================================================
		//	Set parent treeview
		//==================================================
		setTreeView: function(oTreeView)
		{
			if((typeof oTreeView != "object")
			|| (!oTreeView instanceof treeview.TreeView))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_TREEVIEW,
											"treeview.js", "jaswin.treeview.Node.setTreeView");
			
			this.treeView = oTreeView;
			if(this.treeView)
			{
				if(this.treeView.enabled)
					this.enable();
				else
					this.disable();
				
				var iMinWidth = this.label.offsetWidth + 10 + 16 + 2;	// 10:Box 16:Icon 2:border
				this.nodeItem.style.minWidth = iMinWidth + "px";
			}
			
			if(this.container)
			{
				var iCounter;
				
				for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
					this.childNodes[iCounter].setTreeView(oTreeView);
			}
		},
		
		//==================================================
		//	Enable this tree node
		//==================================================
		enable: function()
		{
			this.nodeItem.className = "tree-node-item";
			this.icon.style.cursor = "pointer";
			this.label.style.cursor = "pointer";
			this.box.addEventListener("click", treeBoxClick, false);
			this.icon.addEventListener("click", treeNodeClick, false);
			this.icon.addEventListener("dblclick", treeNodeDblClick, false);
			this.icon.addEventListener("contextmenu", treeNodeContextMenu, false);
			this.label.addEventListener("click", treeNodeClick, false);
			this.label.addEventListener("dblclick", treeNodeDblClick, false);
			this.label.addEventListener("contextmenu", treeNodeContextMenu, false);
			
			if(this.container)
			{
				var iCounter;
				
				for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
					this.childNodes[iCounter].enable();
			}
		},
		
		//==================================================
		//	Disable this tree node
		//==================================================
		disable: function()
		{
			this.nodeItem.className = "tree-node-item-disabled";
			this.icon.style.cursor = "default";
			this.label.style.cursor = "default";
			this.box.removeEventListener("click", treeBoxClick, false);
			this.icon.removeEventListener("click", treeNodeClick, false);
			this.icon.removeEventListener("dblclick", treeNodeDblClick, false);
			this.icon.removeEventListener("contextmenu", treeNodeContextMenu, false);
			this.label.removeEventListener("click", treeNodeClick, false);
			this.label.removeEventListener("dblclick", treeNodeDblClick, false);
			this.label.removeEventListener("contextmenu", treeNodeContextMenu, false);
			
			if(this.container)
			{
				var iCounter;
				
				for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
					this.childNodes[iCounter].disable();
			}
		},
		
		//==================================================
		//	Add child node to this node
		//==================================================
		addNode: function(oChildNode)
		{
			if((typeof oChildNode != "object")
			|| (!oChildNode instanceof treeview.Node))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_NODE, ""),
											"treeview.js", "jaswin.treeview.Node.addNode");
			
			if(!this.container)
			{
				this.container = document.createElement("ul");
				this.container.className = "tree-list";
				this.container.style.display = "none";
				this.element.appendChild(this.container);
			}
			
			this.container.appendChild(oChildNode.element);
			this.childNodes[this.childNodes.length] = oChildNode;
			oChildNode.parentNode = this;
			oChildNode.setTreeView(this.treeView);
		},
		
		//==================================================
		//	Remove this node
		//==================================================
		removeNode: function()
		{
			var oParentNode = this.parentNode;
			var iCounter;
			var iIndex = -1;
			
			if(!oParentNode)
				return;
			
			for(iCounter = 0; iCounter < oParentNode.childNodes.length; iCounter++)
			{
				if(oParentNode.childNodes[iCounter] == this)
				{
					iIndex = iCounter;
					break;
				}
			}
			if(iIndex == -1)
				return;
			
			oParentNode.container.removeChild(this.element);
			oParentNode.childNodes.splice(iIndex, 1);
			if((oParentNode.childNodes.length == 0)
			&& (oParentNode instanceof treeview.Node))
			{
				oParentNode.element.removeChild(oParentNode.container);
				oParentNode.container = null;
			}
		},
		
		//==================================================
		//	Set label text
		//==================================================
		setLabel: function(sText)
		{
			if(typeof sText != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"treeview.js", "jaswin.treeview.Node.setLabel");
			
			this.label.innerHTML = sText;
			if(this.treeView)
			{
				var iMinWidth = this.label.offsetWidth + 10 + 16 + 2;	// 10:Box 16:Icon 2:border
				this.nodeItem.style.minWidth = iMinWidth + "px";
			}
		},
		
		//==================================================
		//	Set this node hover
		//==================================================
		setHover: function()
		{
			if(this.childNodes.length == 0)
				return true;
			
			this.openClose.style.visibility = "visible";
			if(this.opened)
			{
				var iCounter;
				
				for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
					this.childNodes[iCounter].setHover();
			}
		},
		
		//==================================================
		//	Set this node mouse out
		//==================================================
		setOut: function()
		{
			if(this.childNodes.length == 0)
				return true;
			
			this.openClose.style.visibility = "hidden";
			if(this.opened)
			{
				var iCounter;
				
				for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
					this.childNodes[iCounter].setOut();
			}
		},
		
		//==================================================
		//	Find target node of event
		//==================================================
		findNode: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter;
			
			if(!oElement)
				return null;
			
			if(this.childNodes.length == 0)
				return this;
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				if(this.childNodes[iCounter].element.contains(oElement))
					return this.childNodes[iCounter].findNode(e);
			}
			return this;
		},
		
		//==================================================
		//	Click event of this node box
		//==================================================
		clickBox: function()
		{
			if(this.opened)
				this.close();
			else
				this.open();
		},
		
		//==================================================
		//	Open this node
		//==================================================
		open: function()
		{
			if(!this.container)
				return;
			
			var iCounter;
			var iMinWidth;
			
			this.openClose.className = "tree-icon-open";
			this.container.style.display = "block";
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				iMinWidth = this.childNodes[iCounter].label.offsetWidth + 10 + 16 + 2;	// 10:Box 16:Icon 2:border
				this.childNodes[iCounter].nodeItem.style.minWidth = iMinWidth + "px";
			}
			this.opened = true;
		},
		
		//==================================================
		//	Close this node
		//==================================================
		close: function()
		{
			if(!this.container)
				return;
			
			this.openClose.className = "tree-icon-close";
			this.container.style.display = "none";
			this.opened = false;
			return true;
		},
		
		//==================================================
		//	Double click event of this node
		//==================================================
		dblClick: function()
		{
			if(this.onDblClick)
			{
				this.onDblClick(this);
				return;
			}
			
			if(this.childNodes.length > 0)
				this.clickBox();
		},
		
		//==================================================
		//	Select a node
		//==================================================
		selectNode: function(oTargetNode)
		{
			if((typeof oTargetNode != "object")
			|| (!oTargetNode instanceof treeview.Node))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_NODE, ""),
											"treeview.js", "jaswin.treeview.Node.selectNode");
			
			if(oTargetNode == this)
			{
				this.selected = true;
				this.nodeItem.className = "tree-node-item-focus";
				
				var oBound = this.treeView.element.getBoundingClientRect();
				var iTop = this.treeView.element.scrollTop;
				var iBottom = this.treeView.element.scrollTop + oBound.height;
				
				if(this.element.offsetTop < iTop)
					this.treeView.element.scrollTop = this.element.offsetTop;
				else if(this.element.offsetTop + 24 > iBottom)
					this.treeView.element.scrollTop += this.element.offsetTop + 24 - iBottom;
				
				if(this.onClick)
					this.onClick(this);
			}
			else
			{
				this.selected = false;
				this.nodeItem.className = "tree-node-item";
			}
			
			if(this.container)
			{
				var iCounter;
				
				for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
					this.childNodes[iCounter].selectNode(oTargetNode);
			}
			
			return true;
		},
		
		//==================================================
		//	Get selected node
		//==================================================
		getSelectedNode: function()
		{
			if(this.selected)
				return this;
			
			var iCounter;
			var oSelected = null;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				oSelected = this.childNodes[iCounter].getSelectedNode();
				if(oSelected)
					return oSelected;
			}
			
			return null;
		},
		
		//==================================================
		//	Find next node
		//==================================================
		findNext: function(oNode)
		{
			if((typeof oNode != "object")
			|| (!oNode instanceof treeview.Node))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_NODE, ""),
											"treeview.js", "jaswin.treeview.Node.findNext");
			
			if(oNode == this)
			{
				if(this.opened)
					return this.childNodes[0];
				
				return this.parentNode.findNext(this);
			}
			
			var iCounter;
			var iIndex = -1;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				if(this.childNodes[iCounter] == oNode)
				{
					iIndex = iCounter;
					break;
				}
			}
			if((iIndex == -1) || (iIndex == this.childNodes.length - 1))
				return this.parentNode.findNext(this);
			
			return this.childNodes[iIndex + 1];
		},
		
		//==================================================
		//	Find previous node
		//==================================================
		findPrevious: function(oNode)
		{
			if((typeof oNode != "object")
			|| (!oNode instanceof treeview.Node))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_NODE, ""),
											"treeview.js", "jaswin.treeview.Node.findPrevious");
			
			var iCounter;
			var iIndex = -1;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				if(this.childNodes[iCounter] == oNode)
				{
					iIndex = iCounter;
					break;
				}
			}
			
			if(iIndex <= 0)
				return this;
			
			if(this.childNodes[iIndex - 1].opened)
			{
				var iLength = this.childNodes[iIndex - 1].childNodes.length;
				
				return this.childNodes[iIndex - 1].childNodes[iLength - 1];
			}
			
			return this.childNodes[iIndex - 1];
		},
		
		//==================================================
		//	Show label text box
		//==================================================
		showLabelTextBox: function()
		{
			if(!this.treeView.labelTextBox)
				return;
			
			var iLeft;
			var iTop;
			var iWidth;
			
			if(this.treeView.labelTextBox.style == jaswin.textbox.STYLE_FLAT)
			{
				iLeft = parseInt(this.nodeItem.offsetLeft) + 32;
				iTop = parseInt(this.nodeItem.offsetTop) + 2;
				iWidth = parseInt(this.nodeItem.offsetWidth) - 35;
			}
			else
			{
				iLeft = parseInt(this.nodeItem.offsetLeft) + 31;
				iTop = parseInt(this.nodeItem.offsetTop) + 1;
				iWidth = parseInt(this.nodeItem.offsetWidth) - 36;
			}
			this.treeView.labelTextBox.element.style.left = iLeft + "px";
			this.treeView.labelTextBox.element.style.top = iTop + "px";
			this.treeView.labelTextBox.element.style.width = iWidth + "px";
			this.treeView.labelTextBox.setValue(this.label.innerHTML);
			this.treeView.labelTextBox.show();
			this.treeView.labelTextBox.element.focus();
		}
	}
	
	
	//==========================================================
	//
	//	Class		:	TreeView
	//
	//	Summary		:	Class of treeview
	//
	//==========================================================
	treeview.TreeView = function(sID,			// * ID[string]
								oRect,			// * Size and position[ExpandableRect]
								bEnabled,		// * Enabled or not[boolean]
								iTabIndex)		// Tab index[int]
	{
		//==================================================
		//	Constructor
		//==================================================
		// Check parameters
		if(typeof sID != "string")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT, jaswin.ALERT_TYPE_ID,
										"treeview.js", "jaswin.treeview.TreeView.constructor");
		
		if((typeof oRect != "object")
		|| (!oRect instanceof jaswin.ExpandableRect))
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_RECT, sID),
										"treeview.js", "jaswin.treeview.TreeView.constructor");
		
		if(typeof bEnabled != "boolean")
			throw new jaswin.JaswinError(jaswin.ERR_ALERT,
										jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, sID),
										"treeview.js", "jaswin.treeview.TreeView.constructor");
		
		if(iTabIndex)
		{
			if(typeof iTabIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TABINDEX, sID),
											"treeview.js", "jaswin.treeview.TreeView.constructor");
		}
		
		// Set properties
		this.id = sID;					// ID[String]
		this.rect = oRect;				// Size and position[ExpandableRect]
		this.enabled = bEnabled;		// Enabled or not[boolean]
		this.parentEnabled = true;		// Enabled or not[boolean]
		this.element = null;			// This element[DOM Element]
		this.focusElement = null;		// Focus element[DOM Element]
		this.container = null;			// Container element[DOM Element]
		this.childNodes = new Array();	// Child nodes[Array of Node]
		this.tabIndex = -1;				// Tab index[int]
		this.parent = null;				// Parent class[global / window / panel class]
		this.contextMenu = null;		// Context menu on this panel[Menu class]
		this.onContextMenu = null;		// Function on right click[function(Argument:Node class)]
		this.labelTextBox = null;		// TextBox for editing node label[TextBox]
		
		// Create element
		this.element = document.createElement("div");
		this.element.className = "treeview";
		this.focusElement = this.element;
		
		// Horizontal rect
		if(this.rect.anchorX == jaswin.ANCHOR_BOTH)
		{
			this.element.style.left = this.rect.left + "px";
			this.element.style.right = this.rect.right + "px";
		}
		else
		{
			if(this.rect.width < this.minWidth)
				this.rect.width = this.minWidth;
			
			if(this.rect.anchorX == jaswin.ANCHOR_LEFT)
				this.element.style.left = this.rect.left + "px";
			else if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.right + "px";
			this.element.style.width = this.rect.width - 2 + "px";		// 2:border
		}
		
		// Vertical rect
		if(this.rect.anchorY == jaswin.ANCHOR_BOTH)
		{
			this.element.style.top = this.rect.top + "px";
			this.element.style.bottom = this.rect.bottom + "px";
		}
		else
		{
			if(this.rect.anchorY == jaswin.ANCHOR_TOP)
				this.element.style.top = this.rect.top + "px";
			else if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.bottom + "px";
			this.element.style.height = this.rect.height - 2 + "px";	// 2:border
		}
		
		// Create container
		this.container = document.createElement("ul");
		this.container.className = "tree-list";
		this.element.appendChild(this.container);
		
		// Tab index
		if(iTabIndex)
		{
			if(iTabIndex > 0)
			{
				this.tabIndex = iTabIndex;
				this.focusElement.tabIndex = this.tabIndex;
			}
		}
		
		// Event binding
		if(this.enabled)
		{
			this.element.addEventListener("mousemove", treeMove, false);
			this.element.addEventListener("mouseout", treeOut, false);
			this.element.addEventListener("keydown", keyEvent, false);
		}
	}
	treeview.TreeView.prototype =
	{
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy: function()
		{
			var oParentWnd = jaswin.findParentWindow(this);
			
			if(this.contextMenu)
				this.contextMenu.destroy();
			
			if(this.labelTextBox)
				this.labelTextBox.destroy();
			
			delete treeview.treeViews[this.element.id];
			this.element.parentNode.removeChild(this.element);
			delete oParentWnd[this.id];
		},
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent: function(oParent)
		{
			this.parent = oParent;
			if((this.parent.enabled) && (!this.parentEnabled))
				this.enable(true);
			else if((!this.parent.enabled) && (this.parentEnabled))
				this.disable(true);
			
			var oParentWnd = jaswin.findParentWindow(this);
			var sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
				sID = oParentWnd.id + "_" + this.id;
			
			// Duplication check
			if((document.getElementById(sID)) || (treeview.treeViews[sID]))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, sID),
											"treeview.js", "jaswin.treeview.TreeView.setParent");
			this.element.id = sID;
			treeview.treeViews[this.element.id] = this;
			oParentWnd[this.id] = this;
		},
		
		//==================================================
		//	Enable this treeview
		//==================================================
		enable: function(bParent)
		{
			var iCounter;
			
			if(bParent)
				this.parentEnabled = true;
			else
				this.enabled = true;
			
			if(this.enabled)
			{
				this.element.tabIndex = this.tabIndex;
				
				this.element.addEventListener("mousemove", treeMove, false);
				this.element.addEventListener("mouseout", treeOut, false);
				this.element.addEventListener("keydown", keyEvent, false);
				
				for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
					this.childNodes[iCounter].enable();
			}
		},
		
		//==================================================
		//	Disable this treeview
		//==================================================
		disable: function(bParent)
		{
			var iCounter;
			
			if(bParent)
				this.parentEnabled = false;
			else
				this.enabled = false;
			
			this.element.tabIndex = -1;
			
			this.element.removeEventListener("mousemove", treeMove, false);
			this.element.removeEventListener("mouseout", treeOut, false);
			this.element.removeEventListener("keydown", keyEvent, false);
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
				this.childNodes[iCounter].disable();
		},
		
		//==================================================
		//	Show this treeview
		//==================================================
		show: function()
		{
			this.element.style.display = "block";
		},
		
		//==================================================
		//	Hide this treeview
		//==================================================
		hide: function()
		{
			this.element.style.display = "none";
		},
		
		//==================================================
		//	Add Node
		//==================================================
		addNode: function(oNode)
		{
			if((typeof oNode != "object")
			|| (!oNode instanceof treeview.Node))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_NODE, this.id),
											"treeview.js", "jaswin.treeview.TreeView.addNode");
			
			this.container.appendChild(oNode.element);
			oNode.parentNode = this;
			oNode.setTreeView(this);
			
			this.childNodes[this.childNodes.length] = oNode;
		},
		
		//==================================================
		//	Set this treeview hover
		//==================================================
		setHover: function()
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
				this.childNodes[iCounter].setHover();
		},
		
		//==================================================
		//	Set this treeview mouse out
		//==================================================
		setOut: function()
		{
			var iCounter;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
				this.childNodes[iCounter].setOut();
		},
		
		//==================================================
		//	Find target node of event
		//==================================================
		findNode: function(e)
		{
			var oElement = e.target || e.srcElement;
			var iCounter;
			
			if(!oElement)
				return null;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				if(this.childNodes[iCounter].element.contains(oElement))
					return this.childNodes[iCounter].findNode(e);
			}
		},
		
		//==================================================
		//	Click event of node box
		//==================================================
		clickBox: function(e)
		{
			var oNode;
			
			oNode = this.findNode(e);
			if(oNode)
				oNode.clickBox();
		},
		
		//==================================================
		//	Click event of node
		//==================================================
		clickNode: function(e)
		{
			var oNode;
			var iCounter;
			var oElement = e.target || e.srcElement;
			
			oNode = this.findNode(e);
			if(!oNode)
				return;
			
			if((oNode.editable) && (this.labelTextBox) && (oElement == oNode.label))
			{
				var oSelected = this.getSelectedNode();
				
				if(oNode == oSelected)
				{
					oNode.showLabelTextBox();
					return;
				}
			}
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				this.childNodes[iCounter].selectNode(oNode);
			}
		},
		
		//==================================================
		//	Click event of node
		//==================================================
		dblClickNode: function(e)
		{
			var oNode;
			
			oNode = this.findNode(e);
			if(oNode)
				oNode.dblClick();
		},
		
		//==================================================
		//	Set context menu to this listview
		//==================================================
		setContextMenu: function(oMenu, fpContext)
		{
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, this.id),
											"treeview.js", "jaswin.treeview.TreeView.setContextMenu");
			
			if(!jaswin.menu)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "menu.js"),
											"treeview.js", "jaswin.treeview.TreeView.setContextMenu");
			
			if((typeof oMenu != "object")
			|| (!oMenu instanceof jaswin.menu.Menu))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU, this.id),
											"treeview.js", "jaswin.treeview.TreeView.setContextMenu");
			
			if(fpContext)
			{
				if(typeof fpContext != "function")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CONTEXTMENU, this.id),
												"treeview.js", "jaswin.treeview.TreeView.setContextMenu");
			}
			
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
			}
			
			var oParentWnd = jaswin.findParentWindow(this);
			oParentWnd.container.appendChild(oMenu.element);
			
			this.contextMenu = oMenu;
			oMenu.setParent(this, 0);
			
			if(fpContext)
				this.onContextMenu = fpContext;
		},
		
		//==================================================
		//	Remove context menu from this listview
		//==================================================
		removeContextMenu: function()
		{
			if(this.contextMenu)
			{
				this.contextMenu.destroy();
				this.contextMenu = null;
			}
		},
		
		//==================================================
		//	Context menu event of node
		//==================================================
		contextMenuNode: function(e)
		{
			var oNode;
			var iCounter;
			
			oNode = this.findNode(e);
			if(!oNode)
				return;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				this.childNodes[iCounter].selectNode(oNode);
			}
			
			if(this.contextMenu)
			{
				if(this.onContextMenu)
				{
					if(!this.onContextMenu(oNode))
					{
						jaswin.menu.menuCancel(null);
						return;
					}
				}
				this.contextMenu.showContext(e);
			}
		},
		
		//==================================================
		//	Set text box for editing node label
		//==================================================
		setLabelTextBox: function(oTextBox)
		{
			if(!this.parent)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_CONTROL_ADD, this.id),
											"treeview.js", "jaswin.treeview.TreeView.setLabelTextBox");
			
			if(!jaswin.textbox)
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_INCLUDE, "textbox.js"),
											"treeview.js", "jaswin.treeview.TreeView.setLabelTextBox");
			
			if((typeof oTextBox != "object")
			|| (!oTextBox instanceof jaswin.textbox.TextBox))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TEXTBOX, this.id),
											"treeview.js", "jaswin.treeview.TreeView.setLabelTextBox");
			this.labelTextBox = oTextBox;
			this.labelTextBox.hide();
			this.container.appendChild(this.labelTextBox.element);
			this.labelTextBox.setParent(this);
		},
		
		//==================================================
		//	Remove text box for editing node label
		//==================================================
		removeLabelTextBox: function()
		{
			if(this.labelTextBox)
			{
				this.labelTextBox.destroy();
				this.labelTextBox = null;
			}
		},
		
		//==================================================
		//	Select a node
		//==================================================
		selectNode: function(oNode)
		{
			if((typeof oNode != "object")
			|| (!oNode instanceof treeview.Node))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_NODE, this.id),
											"treeview.js", "jaswin.treeview.TreeView.selectNode");
			
			var iCounter;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				this.childNodes[iCounter].selectNode(oNode);
			}
			
			if(this.labelTextBox)
				this.labelTextBox.hide();
		},
		
		//==================================================
		//	Get selected node
		//==================================================
		getSelectedNode: function()
		{
			var oNode = null;
			var iCounter;
			
			if(this.childNodes.length == 0)
				return null;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				oNode = this.childNodes[iCounter].getSelectedNode();
				if(oNode)
					return oNode;
			}
			
			return oNode;
		},
		
		//==================================================
		//	Find next node
		//==================================================
		findNext: function(oNode)
		{
			if((typeof oNode != "object")
			|| (!oNode instanceof treeview.Node))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_NODE, this.id),
											"treeview.js", "jaswin.treeview.TreeView.findNext");
			
			var iCounter;
			var iIndex = -1;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				if(this.childNodes[iCounter] == oNode)
				{
					iIndex = iCounter;
					break;
				}
			}
			
			if((iIndex == -1) || (iIndex == this.childNodes.length - 1))
				return null;
			
			return this.childNodes[iIndex + 1];
		},
		
		//==================================================
		//	Find previous node
		//==================================================
		findPrevious: function(oNode)
		{
			if((typeof oNode != "object")
			|| (!oNode instanceof treeview.Node))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_NODE, this.id),
											"treeview.js", "jaswin.treeview.TreeView.findPrevious");
			
			var iCounter;
			var iIndex = -1;
			
			for(iCounter = 0; iCounter < this.childNodes.length; iCounter++)
			{
				if(this.childNodes[iCounter] == oNode)
				{
					iIndex = iCounter;
					break;
				}
			}
			
			if(iIndex <= 0)
				return null;
			
			if(this.childNodes[iIndex - 1].opened)
			{
				var iLength = this.childNodes[iIndex - 1].childNodes.length;
				
				return this.childNodes[iIndex - 1].childNodes[iLength - 1];
			}
			
			return this.childNodes[iIndex - 1];
		},
		
		//==================================================
		//	Clear nodes
		//==================================================
		clear: function()
		{
			var iCounter;
			var iCnt = this.childNodes.length;
			
			for(iCounter = 0; iCounter < iCnt; iCounter++)
				this.childNodes[0].removeNode();
		},
		
		//==================================================
		//	Key down event of node box
		//==================================================
		keyDown: function(e)
		{
			if((this.contextMenu) && (this.contextMenu.isVisible()))
				return this.contextMenu.menuKey(e);
			
			if((this.labelTextBox) && (this.labelTextBox.element.style.display != "none"))
				return true;
			
			if(this.childNodes.length == 0)
				return true;
			
			var oNode = null;
			var oNext = null;
			var oPrev = null;
			
			if(e.keyCode == 40)
			{
				// down
				oNode = this.getSelectedNode();
				if(!oNode)
				{
					this.selectNode(this.childNodes[0]);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				
				if(oNode.opened)
				{
					this.selectNode(oNode.childNodes[0]);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				
				oNext = oNode.findNext(oNode);
				if(oNext)
				{
					this.selectNode(oNext);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				else
					return true;
			}
			else if(e.keyCode == 38)
			{
				// up
				oNode = this.getSelectedNode();
				if(!oNode)
					return true;
				
				oPrev = oNode.parentNode.findPrevious(oNode);
				if(oPrev)
				{
					this.selectNode(oPrev);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				else
					return true;
			}
			else if(e.keyCode == 39)
			{
				// right
				oNode = this.getSelectedNode();
				if(!oNode)
					return true;
				if(oNode.childNodes.length == 0)
					return true;
				
				if(oNode.opened)
				{
					this.selectNode(oNode.childNodes[0]);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				else
				{
					oNode.clickBox();
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
			}
			else if(e.keyCode == 37)
			{
				// left
				oNode = this.getSelectedNode();
				if(!oNode)
					return true;
				
				if(oNode.opened)
				{
					oNode.clickBox();
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				
				if(oNode.parentNode != this)
				{
					this.selectNode(oNode.parentNode);
					e.preventDefault();
					e.stopPropagation();
					return false;
				}
				
				return true;
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	getTargetTreeView
	//
	//	Summary		:	Get target treeview element from event
	//
	//==========================================================
	function getTargetTreeView(e)
	{
		var oElement = e.target || e.srcElement;
		var oTree = null;
		
		while(true)
		{
			if(oElement == global.element)
				break;
			
			if((oElement.tagName == "div")
			|| (oElement.tagName == "DIV"))
			{
				if(!oElement.id)
				{
					oElement = oElement.parentNode;
					continue;
				}
				oTree = treeview.treeViews[oElement.id];
				if(!oTree)
				{
					oElement = oElement.parentNode;
					continue;
				}
				break;
			}
			else
			{
				oElement = oElement.parentNode;
				continue;
			}
		}
		
		return oTree;
	}
	
	
	//==========================================================
	//
	//	Function	:	treeMove
	//
	//	Summary		:	Event handler of mouse move a treeview
	//
	//==========================================================
	function treeMove(e)
	{
		var oTree = null;
		
		oTree = getTargetTreeView(e);
		
		if(oTree)
			oTree.setHover();
	}
	
	
	//==========================================================
	//
	//	Function	:	treeOut
	//
	//	Summary		:	Event handler of mouse out of a treeview
	//
	//==========================================================
	function treeOut(e)
	{
		var oTree = null;
		
		oTree = getTargetTreeView(e);
		
		if(oTree)
			oTree.setOut();
	}
	
	
	//==========================================================
	//
	//	Function	:	treeBoxClick
	//
	//	Summary		:	Event handler of click of a node box
	//
	//==========================================================
	function treeBoxClick(e)
	{
		var oTree = null;
		
		oTree = getTargetTreeView(e);
		
		if(oTree)
			oTree.clickBox(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	treeNodeClick
	//
	//	Summary		:	Event handler of click of a tree node
	//
	//==========================================================
	function treeNodeClick(e)
	{
		var oTree = null;
		
		oTree = getTargetTreeView(e);
		if(oTree)
			oTree.clickNode(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	treeNodeDblClick
	//
	//	Summary		:	Event handler of double click of a node
	//
	//==========================================================
	function treeNodeDblClick(e)
	{
		var oTree = null;
		
		oTree = getTargetTreeView(e);
		if(oTree)
			oTree.dblClickNode(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	treeNodeContextMenu
	//
	//	Summary		:	Event handler of contextmenu to a node
	//
	//==========================================================
	function treeNodeContextMenu(e)
	{
		var oTree = null;
		
		oTree = getTargetTreeView(e);
		if(oTree)
			oTree.contextMenuNode(e);
	}
	
	
	//==========================================================
	//
	//	Function	:	keyEvent
	//
	//	Summary		:	Event handler of key down to a treeview
	//
	//==========================================================
	function keyEvent(e)
	{
		var oTree = null;
		
		oTree = getTargetTreeView(e);
		
		if(oTree)
			return oTree.keyDown(e);
	}
	
})(jaswin.treeview = jaswin.treeview || {});